@@ -1,17 +1,17 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
-#     
+#
 #  SLEPc is free software: you can redistribute it and/or modify it under  the
 #  terms of version 3 of the GNU Lesser General Public License as published by
 #  the Free Software Foundation.
 #
-#  SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-#  WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-#  FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+#  SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+#  WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+#  FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
 #  more details.
 #
 #  You  should have received a copy of the GNU Lesser General  Public  License
@@ -28,23 +28,22 @@ import log
 
 def LinkWithOutput(tmpdir,functions,callbacks,flags):
   code = '#include "petscksp.h"\n'
-  code += 'EXTERN_C_BEGIN\n'
   for f in functions:
-    code += 'extern int\n' + f + '();\n'
-  code += 'EXTERN_C_END\n'
-  
+    code += 'PETSC_EXTERN int\n' + f + '();\n'
+
   for c in callbacks:
     code += 'int '+ c + '() { return 0; } \n'
 
   code += 'int main() {\n'
-  code += 'PetscInitialize(PETSC_NULL,PETSC_NULL,PETSC_NULL,PETSC_NULL);\n'
-  code += 'VecCreate(PETSC_NULL,PETSC_NULL);\n'
-  code += 'MatCreate(PETSC_NULL,PETSC_NULL);\n'
-  code += 'KSPCreate(PETSC_NULL,PETSC_NULL);\n'
+  code += 'Vec v; Mat m; KSP k;\n'
+  code += 'PetscInitializeNoArguments();\n'
+  code += 'VecCreate(PETSC_COMM_WORLD,&v);\n'
+  code += 'MatCreate(PETSC_COMM_WORLD,&m);\n'
+  code += 'KSPCreate(PETSC_COMM_WORLD,&k);\n'
   for f in functions:
     code += f + '();\n'
   code += 'return 0;\n}\n'
-  
+
   cfile = open(os.sep.join([tmpdir,'checklink.c']),'w')
   cfile.write(code)
   cfile.close()
@@ -54,8 +53,8 @@ def LinkWithOutput(tmpdir,functions,callbacks,flags):
   if result:
     return (0,code + output)
   else:
-    return (1,code + output)  
- 
+    return (1,code + output)
+
 def Link(tmpdir,functions,callbacks,flags):
   (result, output) = LinkWithOutput(tmpdir,functions,callbacks,flags)
   log.write(output)
@@ -70,7 +69,7 @@ def FortranLink(tmpdir,functions,callbacks,flags):
   c = []
   for i in callbacks:
     c.append(i+'_')
-  (result, output1) = LinkWithOutput(tmpdir,f,c,flags) 
+  (result, output1) = LinkWithOutput(tmpdir,f,c,flags)
   output1 = '\n====== With underscore Fortran names\n' + output1
   if result: return ('UNDERSCORE',output1)
 
@@ -79,15 +78,15 @@ def FortranLink(tmpdir,functions,callbacks,flags):
     f.append(i.upper())
   c = []
   for i in callbacks:
-    c.append(i.upper())  
-  (result, output2) = LinkWithOutput(tmpdir,f,c,flags) 
+    c.append(i.upper())
+  (result, output2) = LinkWithOutput(tmpdir,f,c,flags)
   output2 = '\n====== With capital Fortran names\n' + output2
   if result: return ('CAPS',output2)
 
-  (result, output3) = LinkWithOutput(tmpdir,functions,callbacks,flags) 
+  (result, output3) = LinkWithOutput(tmpdir,functions,callbacks,flags)
   output3 = '\n====== With unmodified Fortran names\n' + output3
   if result: return ('STDCALL',output3)
-  
+
   return ('',output + output1 + output2 + output3)
 
 def GenerateGuesses(name):
@@ -102,7 +101,7 @@ def GenerateGuesses(name):
       dirs = dirs + [i + '/' + d]
       dirs = dirs + [i + '/' + d + '/lib']
       dirs = dirs + [i + '/lib/' + d]
-      
+
   for d in dirs[:]:
     if not os.path.exists(d):
       dirs.remove(d)
@@ -124,7 +123,7 @@ def FortranLib(tmpdir,conf,vars,cmake,name,dirs,libs,functions,callbacks = []):
       (mangling, output) = FortranLink(tmpdir,functions,callbacks,flags)
       error += output
       if mangling: break
-    if mangling: break    
+    if mangling: break
 
   if mangling:
     log.write(output)
@@ -134,7 +133,6 @@ def FortranLib(tmpdir,conf,vars,cmake,name,dirs,libs,functions,callbacks = []):
     log.Println('ERROR: In directories '+''.join([s+' ' for s in dirs]))
     log.Println('ERROR: With flags '+''.join([s+' ' for s in flags]))
     log.Exit('')
-    
 
   conf.write('#ifndef SLEPC_HAVE_' + name + '\n#define SLEPC_HAVE_' + name + ' 1\n#define SLEPC_' + name + '_HAVE_'+mangling+' 1\n#endif\n\n')
   vars.write(name + '_LIB = '+str.join(' ',flags)+'\n')
