@@ -2,17 +2,17 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
-#     
+#
 #  SLEPc is free software: you can redistribute it and/or modify it under  the
 #  terms of version 3 of the GNU Lesser General Public License as published by
 #  the Free Software Foundation.
 #
-#  SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-#  WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-#  FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+#  SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+#  WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+#  FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
 #  more details.
 #
 #  You  should have received a copy of the GNU Lesser General  Public  License
@@ -38,19 +38,20 @@ if not os.path.isdir(configDir):
 sys.path.insert(0, configDir)
 
 import petscversion
+import slepcversion
 import petscconf
 import log
 import check
 import arpack
 import blzpack
-import trlan  
+import trlan
+import feast
 import lapack
 import primme
 import blopex
-import slepc4py
 
-if not hasattr(sys, 'version_info') or not sys.version_info[1] >= 2:
-  print '**** You must have Python version 2.2 or higher to run config/configure.py ******'
+if not hasattr(sys, 'version_info') or not sys.version_info[0] == 2 or not sys.version_info[1] >= 4:
+  print '*****  You must have Python2 version 2.4 or higher to run ./configure.py   ******'
   print '*           Python is easy to install for end users or sys-admin.               *'
   print '*                   http://www.python.org/download/                             *'
   print '*                                                                               *'
@@ -58,7 +59,7 @@ if not hasattr(sys, 'version_info') or not sys.version_info[1] >= 2:
   print '*********************************************************************************'
   sys.exit(4)
 
-# support a few standard configure option types 
+# support a few standard configure option types
 for l in range(1,len(sys.argv)):
   name = sys.argv[l]
   if name.startswith('--enable'):
@@ -86,10 +87,12 @@ trlanlibs = []
 haveprimme = 0
 primmedir = ''
 primmelibs = []
+havefeast = 0
+feastdir = ''
+feastlibs = []
 getblopex = 0
 haveblopex = 0
 blopexurl = ''
-getslepc4py = 0
 prefixdir = ''
 
 for i in sys.argv[1:]:
@@ -125,12 +128,18 @@ for i in sys.argv[1:]:
     haveprimme = 1
   elif i.startswith('--with-primme'):
     haveprimme = not i.endswith('=0')
+  elif i.startswith('--with-feast-dir='):
+    feastdir = i.split('=')[1]
+    havefeast = 1
+  elif i.startswith('--with-feast-flags='):
+    feastlibs = i.split('=')[1].split(',')
+    havefeast = 1
+  elif i.startswith('--with-feast'):
+    havefeast = not i.endswith('=0')
   elif i.startswith('--download-blopex'):
     getblopex = not i.endswith('=0')
     try: blopexurl = i.split('=')[1]
     except IndexError: pass
-  elif i.startswith('--download-slepc4py'):
-    getslepc4py = not i.endswith('=0')
   elif i.startswith('--prefix='):
     prefixdir = i.split('=')[1]
   elif i.startswith('--h') or i.startswith('-h') or i.startswith('-?'):
@@ -153,10 +162,12 @@ for i in sys.argv[1:]:
     print '  --with-primme                    : Indicate if you wish to test for PRIMME'
     print '  --with-primme-dir=<dir>          : Indicate the directory for PRIMME libraries'
     print '  --with-primme-flags=<flags>      : Indicate comma-separated flags for linking PRIMME'
+    print 'FEAST:'
+    print '  --with-feast                     : Indicate if you wish to test for FEAST'
+    print '  --with-feast-dir=<dir>           : Indicate the directory for FEAST libraries'
+    print '  --with-feast-flags=<flags>       : Indicate comma-separated flags for linking FEAST'
     print 'BLOPEX:'
     print '  --download-blopex                : Download and install BLOPEX in SLEPc directory'
-    print 'slepc4py:'
-    print '  --download-slepc4py              : Download and install slepc4py in SLEPc directory'
     sys.exit(0)
   else:
     sys.exit('ERROR: Invalid argument ' + i +'. Use -h for help')
@@ -189,8 +200,9 @@ else:
 
 # Check PETSc version
 petscversion.Load(petscdir)
-if petscversion.VERSION < '3.2':
-  sys.exit('ERROR: This SLEPc version is not compatible with PETSc version '+petscversion.VERSION) 
+slepcversion.Load(slepcdir)
+if petscversion.VERSION < slepcversion.VERSION:
+  sys.exit('ERROR: This SLEPc version is not compatible with PETSc version '+petscversion.VERSION)
 
 # Check some information about PETSc configuration
 petscconf.Load(petscdir)
@@ -199,21 +211,16 @@ if not petscconf.PRECISION in ['double','single','__float128']:
 if prefixinstall and not petscconf.ISINSTALL:
   sys.exit('ERROR: SLEPc cannot be configured for non-source installation if PETSc is not configured in the same way.')
 
-# Check whether this is a working copy of the Subversion repository
-subversion = 0
-if os.path.exists(os.sep.join([slepcdir,'src','docs'])) and os.path.exists(os.sep.join([slepcdir,'.svn'])):
-  (result, output) = commands.getstatusoutput('svn info')
-  if result:
-    print 'WARNING: SLEPC_DIR appears to be a subversion working copy, but svn is not found in PATH'
+# Check whether this is a working copy of the repository
+isrepo = 0
+if os.path.exists(os.sep.join([slepcdir,'src','docs'])) and os.path.exists(os.sep.join([slepcdir,'.git'])):
+  (status, output) = commands.getstatusoutput('git rev-parse')
+  if status:
+    print 'WARNING: SLEPC_DIR appears to be a git working copy, but git is not found in PATH'
   else:
-    subversion = 1
-    svnrev = '-1'
-    svndate = '-1'
-    for line in output.split('\n'):
-      if line.startswith('Last Changed Rev: '):
-        svnrev = line.split('Rev: ')[-1]
-      if line.startswith('Last Changed Date: '):
-        svndate = line.split('Date: ')[-1]
+    isrepo = 1
+    (status, gitrev) = commands.getstatusoutput('git log -1 --pretty=format:%H')
+    (status, gitdate) = commands.getstatusoutput('git log -1 --pretty=format:%ci')
 
 # Create architecture directory and configuration files
 archdir = os.sep.join([slepcdir,petscconf.ARCH])
@@ -240,6 +247,18 @@ if not os.path.exists(libdir):
     os.mkdir(libdir)
   except:
     sys.exit('ERROR: cannot create lib directory ' + libdir)
+modulesdir = os.sep.join([libdir,'modules'])
+if not os.path.exists(modulesdir):
+  try:
+    os.mkdir(modulesdir)
+  except:
+    sys.exit('ERROR: cannot create modules directory ' + modulesdir)
+pkgconfigdir = os.sep.join([libdir,'pkgconfig'])
+if not os.path.exists(pkgconfigdir):
+  try:
+    os.mkdir(pkgconfigdir)
+  except:
+    sys.exit('ERROR: cannot create pkgconfig directory ' + pkgconfigdir)
 try:
   slepcvars = open(os.sep.join([confdir,'slepcvariables']),'w')
   if not prefixdir:
@@ -247,6 +266,8 @@ try:
   slepcvars.write('SLEPC_DESTDIR = ' + prefixdir +'\n')
   testruns = set(petscconf.TEST_RUNS.split())
   testruns = testruns.intersection(set(['C','F90','Fortran','C_NoComplex','Fortran_NoComplex']))
+  if petscconf.PRECISION != '__float128':
+    testruns = testruns.union(set(['C_NoF128']))
   slepcvars.write('TEST_RUNS = ' + ' '.join(testruns) +'\n')
 except:
   sys.exit('ERROR: cannot create configuration file in ' + confdir)
@@ -258,9 +279,9 @@ try:
   slepcconf = open(os.sep.join([incdir,'slepcconf.h']),'w')
   slepcconf.write('#if !defined(__SLEPCCONF_H)\n')
   slepcconf.write('#define __SLEPCCONF_H\n\n')
-  if subversion:
-    slepcconf.write('#ifndef SLEPC_VERSION_SVN\n#define SLEPC_VERSION_SVN ' + svnrev + '\n#endif\n\n')
-    slepcconf.write('#ifndef SLEPC_VERSION_DATE_SVN\n#define SLEPC_VERSION_DATE_SVN "' + svndate + '"\n#endif\n\n')
+  if isrepo:
+    slepcconf.write('#ifndef SLEPC_VERSION_GIT\n#define SLEPC_VERSION_GIT "' + gitrev + '"\n#endif\n\n')
+    slepcconf.write('#ifndef SLEPC_VERSION_DATE_GIT\n#define SLEPC_VERSION_DATE_GIT "' + gitdate + '"\n#endif\n\n')
   slepcconf.write('#ifndef SLEPC_LIB_DIR\n#define SLEPC_LIB_DIR "' + prefixdir + '/lib"\n#endif\n\n')
 except:
   sys.exit('ERROR: cannot create configuration header in ' + confdir)
@@ -268,6 +289,14 @@ try:
   cmake = open(os.sep.join([confdir,'SLEPcConfig.cmake']),'w')
 except:
   sys.exit('ERROR: cannot create CMake configuration file in ' + confdir)
+try:
+  modules = open(os.sep.join([modulesdir,slepcversion.LVERSION+'-'+petscconf.ARCH]),'w')
+except:
+  sys.exit('ERROR: cannot create modules file in ' + modulesdir)
+try:
+  pkgconfig = open(os.sep.join([pkgconfigdir,'SLEPc.pc']),'w')
+except:
+  sys.exit('ERROR: cannot create pkgconfig file in ' + pkgconfigdir)
 if prefixinstall and os.path.isfile(os.sep.join([prefixdir,'include','slepc.h'])):
   sys.exit('ERROR: prefix directory ' + prefixdir + ' contains files from a previous installation')
 
@@ -299,18 +328,19 @@ log.write('Python version:\n' + sys.version)
 log.write('make: ' + petscconf.MAKE)
 log.write('PETSc source directory: ' + petscdir)
 log.write('PETSc install directory: ' + petscconf.DESTDIR)
-log.write('PETSc version: ' + petscversion.VERSION)
+log.write('PETSc version: ' + petscversion.LVERSION)
 log.write('PETSc architecture: ' + petscconf.ARCH)
 log.write('SLEPc source directory: ' + slepcdir)
 log.write('SLEPc install directory: ' + prefixdir)
+log.write('SLEPc version: ' + slepcversion.LVERSION)
 log.write('='*80)
 
 # Check if PETSc is working
 log.Println('Checking PETSc installation...')
-if petscversion.VERSION > '3.2':
-  log.Println('WARNING: PETSc version '+petscversion.VERSION+' is newer than SLEPc version')
-if petscversion.RELEASE != '1':
-  log.Println('WARNING: using PETSc development version')
+if petscversion.VERSION > slepcversion.VERSION:
+  log.Println('WARNING: PETSc version '+petscversion.VERSION+' is newer than SLEPc version '+slepcversion.VERSION)
+if petscversion.RELEASE != '1' and slepcversion.RELEASE == '1':
+  log.Println('WARNING: using PETSc development version with a SLEPc release version')
 if petscconf.ISINSTALL:
   if os.path.realpath(petscconf.DESTDIR) != os.path.realpath(petscdir):
     log.Println('WARNING: PETSC_DIR does not point to PETSc installation path')
@@ -326,6 +356,8 @@ if havetrlan:
   trlanlibs = trlan.Check(slepcconf,slepcvars,cmake,tmpdir,trlandir,trlanlibs)
 if haveprimme:
   primmelibs = primme.Check(slepcconf,slepcvars,cmake,tmpdir,primmedir,primmelibs)
+if havefeast:
+  feastlibs = feast.Check(slepcconf,slepcvars,cmake,tmpdir,feastdir,feastlibs)
 if getblopex:
   blopexlibs = blopex.Install(slepcconf,slepcvars,cmake,tmpdir,blopexurl,archdir)
   haveblopex = 1
@@ -333,23 +365,29 @@ if getblopex:
 # Check for missing LAPACK functions
 missing = lapack.Check(slepcconf,slepcvars,cmake,tmpdir)
 
-# Download and install slepc4py
-if getslepc4py:
-  slepc4py.Install()
-slepc4py.addMakeRule(slepcrules,prefixdir,prefixinstall,getslepc4py)
-
 # Make Fortran stubs if necessary
-if subversion and hasattr(petscconf,'FC'):
+if isrepo and hasattr(petscconf,'FC'):
   try:
     import generatefortranstubs
-    generatefortranstubs.main(petscconf.BFORT)
+    generatefortranstubs.main(slepcdir,petscconf.BFORT,os.getcwd(),0)
+    generatefortranstubs.processf90interfaces(slepcdir,0)
   except AttributeError:
     sys.exit('ERROR: cannot generate Fortran stubs; try configuring PETSc with --download-sowing or use a mercurial version of PETSc')
 
 # CMake stuff
-cmake.write('set (SLEPC_PACKAGE_LIBS "${ARPACK_LIB}" "${BLZPACK_LIB}" "${TRLAN_LIB}" "${PRIMME_LIB}")\n')
+cmake.write('set (SLEPC_PACKAGE_LIBS "${ARPACK_LIB}" "${BLZPACK_LIB}" "${TRLAN_LIB}" "${PRIMME_LIB}" "${FEAST_LIB}" "${BLOPEX_LIB}" )\n')
 cmake.write('set (SLEPC_PACKAGE_INCLUDES "${PRIMME_INCLUDE}")\n')
 cmake.write('find_library (PETSC_LIB petsc HINTS ${PETSc_BINARY_DIR}/lib )\n')
+cmake.write('''
+if (NOT PETSC_LIB) # Interpret missing libpetsc to mean that PETSc was built --with-single-library=0
+  set (PETSC_LIB "")
+  foreach (pkg sys vec mat dm ksp snes ts)
+    string (TOUPPER ${pkg} PKG)
+    find_library(PETSC${PKG}_LIB "petsc${pkg}" HINTS ${PETSc_BINARY_DIR}/lib)
+    list (APPEND PETSC_LIB "${PETSC${PKG}_LIB}")
+  endforeach ()
+endif ()
+''')
 cmake.close()
 cmakeok = False
 if sys.version_info >= (2,5) and not petscconf.ISINSTALL and petscconf.BUILD_USING_CMAKE:
@@ -372,11 +410,37 @@ if sys.version_info >= (2,5) and not petscconf.ISINSTALL and petscconf.BUILD_USI
 if cmakeok:
   slepcvars.write('SLEPC_BUILD_USING_CMAKE = 1\n')
 
+# Modules file
+modules.write('#%Module\n\n')
+modules.write('proc ModulesHelp { } {\n')
+modules.write('    puts stderr "This module sets the path and environment variables for slepc-%s"\n' % slepcversion.LVERSION)
+modules.write('    puts stderr "     see http://www.grycap.upv.es/slepc/ for more information"\n')
+modules.write('    puts stderr ""\n}\n')
+modules.write('module-whatis "SLEPc - Scalable Library for Eigenvalue Problem Computations"\n\n')
+modules.write('module load petsc\n')
+if prefixinstall:
+  modules.write('set slepc_dir %s\n' % prefixdir)
+else:
+  modules.write('set slepc_dir %s\n' % slepcdir)
+modules.write('setenv SLEPC_DIR $slepc_dir\n')
+
+# pkg-config file
+pkgconfig.write('Name: SLEPc, the Scalable Library for Eigenvalue Problem Computations\n')
+pkgconfig.write('Description: A parallel library to compute eigenvalues and eigenvectors of large, sparse matrices with iterative methods. It is based on PETSc.\n')
+pkgconfig.write('Version: %s\n' % slepcversion.LVERSION)
+pkgconfig.write('Requires: PETSc = %s\n' % petscversion.LVERSION)
+pkgconfig.write('Cflags: -I%s/include' % prefixdir)
+if not prefixinstall:
+  pkgconfig.write(' -I%s/include' % slepcdir)
+pkgconfig.write('\nLibs: -L%s/lib -lslepc\n' % prefixdir)
+
 # Finish with configuration files
 slepcvars.close()
 slepcrules.close()
 slepcconf.write('#endif\n')
 slepcconf.close()
+modules.close()
+pkgconfig.close()
 shutil.rmtree(tmpdir)
 
 # Print summary
@@ -389,7 +453,7 @@ log.Println('SLEPc directory:')
 log.Println(' '+slepcdir)
 if archdir != prefixdir:
   log.Println('SLEPc prefix directory:')
-  log.Println(' '+prefixdir)  
+  log.Println(' '+prefixdir)
 log.Println('PETSc directory:')
 log.Println(' '+petscdir)
 log.Println('Architecture "'+petscconf.ARCH+'" with '+petscconf.PRECISION+' precision '+petscconf.SCALAR+' numbers')
@@ -405,6 +469,9 @@ if havetrlan:
 if haveprimme:
   log.Println('PRIMME library flags:')
   log.Println(' '+str.join(' ',primmelibs))
+if havefeast:
+  log.Println('FEAST library flags:')
+  log.Println(' '+str.join(' ',feastlibs))
 if haveblopex:
   log.Println('BLOPEX library flags:')
   log.Println(' '+str.join(' ',blopexlibs))
