@@ -1,17 +1,17 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
-#     
+#
 #  SLEPc is free software: you can redistribute it and/or modify it under  the
 #  terms of version 3 of the GNU Lesser General Public License as published by
 #  the Free Software Foundation.
 #
-#  SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-#  WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-#  FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+#  SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+#  WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+#  FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
 #  more details.
 #
 #  You  should have received a copy of the GNU Lesser General  Public  License
@@ -23,9 +23,9 @@ import os
 import sys
 
 def Load(petscdir):
-  global ARCH,DIR,MAKE,SCALAR,PRECISION,ISINSTALL,DESTDIR,BFORT,TEST_RUNS,CC,CC_FLAGS,FC,AR,AR_FLAGS,AR_LIB_SUFFIX,RANLIB,IND64,BUILD_USING_CMAKE
-  
-  if 'PETSC_ARCH' in os.environ:
+  global ARCH,DIR,MAKE,SCALAR,PRECISION,ISINSTALL,DESTDIR,BFORT,TEST_RUNS,CC,CC_FLAGS,FC,AR,AR_FLAGS,AR_LIB_SUFFIX,RANLIB,IND64,BUILD_USING_CMAKE,MPIUNI
+
+  if 'PETSC_ARCH' in os.environ and os.environ['PETSC_ARCH']:
     ISINSTALL = 0
     ARCH = os.environ['PETSC_ARCH']
     PETSCVARIABLES = os.sep.join([petscdir,ARCH,'conf','petscvariables'])
@@ -40,9 +40,10 @@ def Load(petscdir):
   try:
     f = open(PETSCVARIABLES)
     for l in f.readlines():
-      (k,v) = l.split('=',1)
-      k = k.strip()
-      v = v.strip()
+      r = l.split('=',1)
+      if len(r)!=2: continue
+      k = r[0].strip()
+      v = r[1].strip()
       if k == 'PETSC_SCALAR':
 	SCALAR = v
       elif k == 'PETSC_PRECISION':
@@ -73,16 +74,19 @@ def Load(petscdir):
 	BUILD_USING_CMAKE = v
     f.close()
   except:
-    sys.exit('ERROR: PETSc is not configured for architecture ' + ARCH)
+    sys.exit('ERROR: cannot process file ' +  PETSCVARIABLES)
 
   IND64 = 0
+  MPIUNI = 0
   try:
     f = open(PETSCCONF_H)
     for l in f.readlines():
       l = l.split()
       if len(l)==3 and l[0]=='#define' and l[1]=='PETSC_USE_64BIT_INDICES' and l[2]=='1':
 	IND64 = 1
+      elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_MPIUNI' and l[2]=='1':
+	MPIUNI = 1
     f.close()
   except:
-    sys.exit('ERROR: cannot open petscconf.h')
+    sys.exit('ERROR: cannot process file ' +  PETSCCONF_H)
 
