@@ -4,17 +4,17 @@
 !
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !  SLEPc - Scalable Library for Eigenvalue Problem Computations
-!  Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+!  Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 !
 !  This file is part of SLEPc.
-!     
+!
 !  SLEPc is free software: you can redistribute it and/or modify it under  the
 !  terms of version 3 of the GNU Lesser General Public License as published by
 !  the Free Software Foundation.
 !
-!  SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-!  WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-!  FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+!  SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+!  WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+!  FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
 !  more details.
 !
 !  You  should have received a copy of the GNU Lesser General  Public  License
@@ -26,7 +26,7 @@
 !  Convergence flags.
 !  They should match the flags in $SLEPC_DIR/include/slepcqep.h
 
-      PetscEnum QEP_CONVERGED_TOL        
+      PetscEnum QEP_CONVERGED_TOL
       PetscEnum QEP_DIVERGED_ITS
       PetscEnum QEP_DIVERGED_BREAKDOWN
       PetscEnum QEP_CONVERGED_ITERATING
@@ -43,13 +43,16 @@
       parameter (QEP_GENERAL                =  1)
       parameter (QEP_HERMITIAN              =  2)
       parameter (QEP_GYROSCOPIC             =  3)
-      
+
       PetscEnum QEP_LARGEST_MAGNITUDE
       PetscEnum QEP_SMALLEST_MAGNITUDE
       PetscEnum QEP_LARGEST_REAL
       PetscEnum QEP_SMALLEST_REAL
       PetscEnum QEP_LARGEST_IMAGINARY
       PetscEnum QEP_SMALLEST_IMAGINARY
+      PetscEnum QEP_TARGET_MAGNITUDE
+      PetscEnum QEP_TARGET_REAL
+      PetscEnum QEP_TARGET_IMAGINARY
 
       parameter (QEP_LARGEST_MAGNITUDE      =  1)
       parameter (QEP_SMALLEST_MAGNITUDE     =  2)
@@ -57,7 +60,10 @@
       parameter (QEP_SMALLEST_REAL          =  4)
       parameter (QEP_LARGEST_IMAGINARY      =  5)
       parameter (QEP_SMALLEST_IMAGINARY     =  6)
-       
+      parameter (QEP_TARGET_MAGNITUDE       =  7)
+      parameter (QEP_TARGET_REAL            =  8)
+      parameter (QEP_TARGET_IMAGINARY       =  9)
+
 !
 !   Possible arguments to QEPMonitorSet()
 !
