@@ -5,17 +5,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -32,24 +32,22 @@ PetscBool dvd_testconv_slepc_0(dvdDashboard *d, PetscScalar eigvr,
                                 PetscScalar eigvi, PetscReal r,
                                 PetscReal *err);
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "dvd_testconv_basic"
 PetscErrorCode dvd_testconv_basic(dvdDashboard *d, dvdBlackboard *b)
 {
   PetscErrorCode  ierr;
 
   PetscFunctionBegin;
-
   /* Setup the step */
   if (b->state >= DVD_STATE_CONF) {
-    ierr = PetscFree(d->testConv_data); CHKERRQ(ierr);
+    ierr = PetscFree(d->testConv_data);CHKERRQ(ierr);
     d->testConv = dvd_testconv_basic_0;
   }
-
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "dvd_testconv_basic_0"
 PetscBool dvd_testconv_basic_0(dvdDashboard *d, PetscScalar eigvr,
                                 PetscScalar eigvi, PetscReal r,
@@ -59,33 +57,29 @@ PetscBool dvd_testconv_basic_0(dvdDashboard *d, PetscScalar eigvr,
   PetscReal       eig_norm, errest;
 
   PetscFunctionBegin;
-
   eig_norm = SlepcAbsEigenvalue(eigvr, eigvi);
   errest = r/eig_norm;
   conv = (errest <= d->tol) ? PETSC_TRUE : PETSC_FALSE;
   if (err) *err = errest;
-
   PetscFunctionReturn(conv);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "dvd_testconv_slepc"
 PetscErrorCode dvd_testconv_slepc(dvdDashboard *d, dvdBlackboard *b)
 {
   PetscErrorCode  ierr;
 
   PetscFunctionBegin;
-
   /* Setup the step */
   if (b->state >= DVD_STATE_CONF) {
-    ierr = PetscFree(d->testConv_data); CHKERRQ(ierr);
+    ierr = PetscFree(d->testConv_data);CHKERRQ(ierr);
     d->testConv = dvd_testconv_slepc_0;
   }
-
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "dvd_testconv_slepc_0"
 PetscBool dvd_testconv_slepc_0(dvdDashboard *d, PetscScalar eigvr,
                                 PetscScalar eigvi, PetscReal r,
@@ -94,11 +88,8 @@ PetscBool dvd_testconv_slepc_0(dvdDashboard *d, PetscScalar eigvr,
   PetscErrorCode  ierr;
 
   PetscFunctionBegin;
-
-  ierr = (*d->eps->conv_func)(d->eps, eigvr, eigvi, r, err,
-                              d->eps->conv_ctx);
-  CHKERRABORT(((PetscObject)d->eps)->comm, ierr);
-
+  ierr = (*d->eps->converged)(d->eps, eigvr, eigvi, r, err, d->eps->convergedctx);
+  CHKERRABORT(PetscObjectComm((PetscObject)d->eps), ierr);
   PetscFunctionReturn(*err<d->eps->tol ? PETSC_TRUE : PETSC_FALSE);
 }
 
