@@ -1,19 +1,39 @@
 /*
-  SLEPc eigensolver: "jd"
+
+   SLEPc eigensolver: "jd"
+
+   Method: Jacobi-Davidson
+
+   Algorithm:
+
+       Jacobi-Davidson with various subspace extraction and
+       restart techniques.
+
+   References:
+
+       [1] G.L.G. Sleijpen and H.A. van der Vorst, "A Jacobi-Davidson
+           iteration method for linear eigenvalue problems", SIAM J.
+           Matrix Anal. Appl. 17(2):401-425, 1996.
+
+       [2] E. Romero and J.E. Roman, "A parallel implementation of
+           Davidson methods for large-scale eigenvalue problems in
+           SLEPc", submitted, 2013.
+
+   Last update: Jul 2012
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,14 +41,10 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/epsimpl.h>                /*I "slepceps.h" I*/
+#include <slepc-private/epsimpl.h>                /*I "slepceps.h" I*/
 #include <../src/eps/impls/davidson/common/davidson.h>
 
-PetscErrorCode EPSSetUp_JD(EPS eps);
-PetscErrorCode EPSDestroy_JD(EPS eps);
-
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetFromOptions_JD"
 PetscErrorCode EPSSetFromOptions_JD(EPS eps)
 {
@@ -37,64 +53,67 @@ PetscErrorCode EPSSetFromOptions_JD(EPS eps)
   PetscInt       opi,opi0;
   PetscReal      opf;
   KSP            ksp;
+  EPSOrthType    orth;
+  const char     *orth_list[3] = {"I","B","B_opt"};
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead("EPS Jacobi-Davidson (JD) Options");CHKERRQ(ierr);
 
   ierr = EPSJDGetKrylovStart(eps,&op);CHKERRQ(ierr);
   ierr = PetscOptionsBool("-eps_jd_krylov_start","Start the searching subspace with a krylov basis","EPSJDSetKrylovStart",op,&op,&flg);CHKERRQ(ierr);
-  if(flg) { ierr = EPSJDSetKrylovStart(eps,op);CHKERRQ(ierr); }
- 
+  if (flg) { ierr = EPSJDSetKrylovStart(eps,op);CHKERRQ(ierr); }
+
   ierr = EPSJDGetBlockSize(eps,&opi);CHKERRQ(ierr);
   ierr = PetscOptionsInt("-eps_jd_blocksize","Number vectors add to the searching subspace","EPSJDSetBlockSize",opi,&opi,&flg);CHKERRQ(ierr);
-  if(flg) { ierr = EPSJDSetBlockSize(eps,opi);CHKERRQ(ierr); }
+  if (flg) { ierr = EPSJDSetBlockSize(eps,opi);CHKERRQ(ierr); }
 
   ierr = EPSJDGetRestart(eps,&opi,&opi0);CHKERRQ(ierr);
   ierr = PetscOptionsInt("-eps_jd_minv","Set the size of the searching subspace after restarting","EPSJDSetRestart",opi,&opi,&flg);CHKERRQ(ierr);
-  if(flg) { ierr = EPSJDSetRestart(eps,opi,opi0);CHKERRQ(ierr); }
+  if (flg) { ierr = EPSJDSetRestart(eps,opi,opi0);CHKERRQ(ierr); }
 
   ierr = PetscOptionsInt("-eps_jd_plusk","Set the number of saved eigenvectors from the previous iteration when restarting","EPSJDSetRestart",opi0,&opi0,&flg);CHKERRQ(ierr);
-  if(flg) { ierr = EPSJDSetRestart(eps,opi,opi0);CHKERRQ(ierr); }
+  if (flg) { ierr = EPSJDSetRestart(eps,opi,opi0);CHKERRQ(ierr); }
 
   ierr = EPSJDGetInitialSize(eps,&opi);CHKERRQ(ierr);
   ierr = PetscOptionsInt("-eps_jd_initial_size","Set the initial size of the searching subspace","EPSJDSetInitialSize",opi,&opi,&flg);CHKERRQ(ierr);
-  if(flg) { ierr = EPSJDSetInitialSize(eps,opi);CHKERRQ(ierr); }
+  if (flg) { ierr = EPSJDSetInitialSize(eps,opi);CHKERRQ(ierr); }
 
   ierr = EPSJDGetFix(eps,&opf);CHKERRQ(ierr);
   ierr = PetscOptionsReal("-eps_jd_fix","Set the tolerance for changing the target in the correction equation","EPSJDSetFix",opf,&opf,&flg);CHKERRQ(ierr);
-  if(flg) { ierr = EPSJDSetFix(eps,opf);CHKERRQ(ierr); }
+  if (flg) { ierr = EPSJDSetFix(eps,opf);CHKERRQ(ierr); }
 
-  ierr = EPSJDGetConstantCorrectionTolerance(eps,&op);CHKERRQ(ierr);
-  ierr = PetscOptionsBool("-eps_jd_constant_correction_tolerance","Disable the dynamic stopping criterion when solving the correction equation","EPSJDSetConstantCorrectionTolerance",op,&op,&flg);CHKERRQ(ierr);
-  if(flg) { ierr = EPSJDSetConstantCorrectionTolerance(eps,op);CHKERRQ(ierr); }
+  ierr = EPSJDGetBOrth(eps,&orth);CHKERRQ(ierr);
+  ierr = PetscOptionsEList("-eps_jd_borth","orthogonalization used in the search subspace","EPSJDSetBOrth",orth_list,3,orth_list[orth-1],&opi,&flg);CHKERRQ(ierr);
+  if (flg) { ierr = EPSJDSetBOrth(eps,(EPSOrthType)(opi+1));CHKERRQ(ierr); }
+
+  ierr = EPSJDGetConstCorrectionTol(eps,&op);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-eps_jd_const_correction_tol","Disable the dynamic stopping criterion when solving the correction equation","EPSJDSetConstCorrectionTol",op,&op,&flg);CHKERRQ(ierr);
+  if (flg) { ierr = EPSJDSetConstCorrectionTol(eps,op);CHKERRQ(ierr); }
 
   ierr = EPSJDGetWindowSizes(eps,&opi,&opi0);CHKERRQ(ierr);
   ierr = PetscOptionsInt("-eps_jd_pwindow","(Experimental!) Set the number of converged vectors in the projector","EPSJDSetWindowSizes",opi,&opi,&flg);CHKERRQ(ierr);
-  if(flg) { ierr = EPSJDSetWindowSizes(eps,opi,opi0);CHKERRQ(ierr); }
+  if (flg) { ierr = EPSJDSetWindowSizes(eps,opi,opi0);CHKERRQ(ierr); }
 
   ierr = PetscOptionsInt("-eps_jd_qwindow","(Experimental!) Set the number of converged vectors in the projected problem","EPSJDSetWindowSizes",opi0,&opi0,&flg);CHKERRQ(ierr);
-  if(flg) { ierr = EPSJDSetWindowSizes(eps,opi,opi0);CHKERRQ(ierr); }
-
-  ierr = PetscOptionsTail();CHKERRQ(ierr);
+  if (flg) { ierr = EPSJDSetWindowSizes(eps,opi,opi0);CHKERRQ(ierr); }
 
   /* Set STPrecond as the default ST */
-  if (!((PetscObject)eps->OP)->type_name) {
-    ierr = STSetType(eps->OP,STPRECOND);CHKERRQ(ierr);
+  if (!((PetscObject)eps->st)->type_name) {
+    ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
   }
-  ierr = STPrecondSetKSPHasMat(eps->OP,PETSC_FALSE);CHKERRQ(ierr);
+  ierr = STPrecondSetKSPHasMat(eps->st,PETSC_FALSE);CHKERRQ(ierr);
 
   /* Set the default options of the KSP */
-  ierr = STGetKSP(eps->OP,&ksp);CHKERRQ(ierr);
+  ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
   if (!((PetscObject)ksp)->type_name) {
     ierr = KSPSetType(ksp,KSPBCGSL);CHKERRQ(ierr);
     ierr = KSPSetTolerances(ksp,1e-4,PETSC_DEFAULT,PETSC_DEFAULT,90);CHKERRQ(ierr);
   }
-  
+  ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
-}  
-EXTERN_C_END
+}
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetUp_JD"
 PetscErrorCode EPSSetUp_JD(EPS eps)
 {
@@ -103,58 +122,23 @@ PetscErrorCode EPSSetUp_JD(EPS eps)
   KSP            ksp;
 
   PetscFunctionBegin;
-
   /* Setup common for all davidson solvers */
-  ierr = EPSSetUp_Davidson(eps);CHKERRQ(ierr);
+  ierr = EPSSetUp_XD(eps);CHKERRQ(ierr);
 
   /* Set the default options of the KSP */
-  ierr = STGetKSP(eps->OP,&ksp);CHKERRQ(ierr);
+  ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
   if (!((PetscObject)ksp)->type_name) {
     ierr = KSPSetType(ksp,KSPBCGSL);CHKERRQ(ierr);
     ierr = KSPSetTolerances(ksp,1e-4,PETSC_DEFAULT,PETSC_DEFAULT,90);CHKERRQ(ierr);
   }
- 
-  /* Check some constraints */ 
-  ierr = PetscTypeCompare((PetscObject)ksp,KSPPREONLY,&t);CHKERRQ(ierr);
-  if (t) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"EPSJD does not work with KSPPREONLY");
+
+  /* Check some constraints */
+  ierr = PetscObjectTypeCompare((PetscObject)ksp,KSPPREONLY,&t);CHKERRQ(ierr);
+  if (t) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"EPSJD does not work with KSPPREONLY");
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
-#define __FUNCT__ "EPSCreate_JD"
-PetscErrorCode EPSCreate_JD(EPS eps)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  /* Load the Davidson solver */
-  ierr = EPSCreate_Davidson(eps);CHKERRQ(ierr);
-
-  /* Overload the JD properties */
-  eps->ops->setfromoptions       = EPSSetFromOptions_JD;
-  eps->ops->setup                = EPSSetUp_JD;
-  eps->ops->destroy              = EPSDestroy_JD;
-
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDSetKrylovStart_C","EPSDavidsonSetKrylovStart_Davidson",EPSDavidsonSetKrylovStart_Davidson);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDGetKrylovStart_C","EPSDavidsonGetKrylovStart_Davidson",EPSDavidsonGetKrylovStart_Davidson);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDSetBlockSize_C","EPSDavidsonSetBlockSize_Davidson",EPSDavidsonSetBlockSize_Davidson);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDGetBlockSize_C","EPSDavidsonGetBlockSize_Davidson",EPSDavidsonGetBlockSize_Davidson);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDSetRestart_C","EPSDavidsonSetRestart_Davidson",EPSDavidsonSetRestart_Davidson);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDGetRestart_C","EPSDavidsonGetRestart_Davidson",EPSDavidsonGetRestart_Davidson);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDSetInitialSize_C","EPSDavidsonSetInitialSize_Davidson",EPSDavidsonSetInitialSize_Davidson);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDGetInitialSize_C","EPSDavidsonGetInitialSize_Davidson",EPSDavidsonGetInitialSize_Davidson);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDSetFix_C","EPSDavidsonSetFix_Davidson",EPSDavidsonSetFix_Davidson);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDGetFix_C","EPSDavidsonGetFix_Davidson",EPSDavidsonGetFix_Davidson);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDSetConstantCorrectionTolerance_C","EPSDavidsonSetConstantCorrectionTolerance_Davidson",EPSDavidsonSetConstantCorrectionTolerance_Davidson);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDGetConstantCorrectionTolerance_C","EPSDavidsonGetConstantCorrectionTolerance_Davidson",EPSDavidsonGetConstantCorrectionTolerance_Davidson);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDSetWindowSizes_C","EPSDavidsonSetWindowSizes_Davidson",EPSDavidsonSetWindowSizes_Davidson);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDGetWindowSizes_C","EPSDavidsonGetWindowSizes_Davidson",EPSDavidsonGetWindowSizes_Davidson);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-EXTERN_C_END
-
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSDestroy_JD"
 PetscErrorCode EPSDestroy_JD(EPS eps)
 {
@@ -162,28 +146,30 @@ PetscErrorCode EPSDestroy_JD(EPS eps)
 
   PetscFunctionBegin;
   ierr = PetscFree(eps->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDSetKrylovStart_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDGetKrylovStart_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDSetBlockSize_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDGetBlockSize_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDSetRestart_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDGetRestart_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDSetInitialSize_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDGetInitialSize_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDSetFix_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDGetFix_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDSetConstantCorrectionTolerance_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDGetConstantCorrectionTolerance_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDSetWindowSizes_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSJDGetWindowSizes_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetKrylovStart_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetKrylovStart_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetBlockSize_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetBlockSize_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetRestart_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetRestart_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetInitialSize_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetInitialSize_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetFix_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetFix_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetConstCorrectionTol_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetConstCorrectionTol_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetWindowSizes_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetWindowSizes_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetBOrth_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetBOrth_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSJDSetKrylovStart"
 /*@
    EPSJDSetKrylovStart - Activates or deactivates starting the searching
-   subspace with a Krylov basis. 
+   subspace with a Krylov basis.
 
    Logically Collective on EPS
 
@@ -194,7 +180,7 @@ PetscErrorCode EPSDestroy_JD(EPS eps)
    Options Database Key:
 .  -eps_jd_krylov_start - Activates starting the searching subspace with a
     Krylov basis
-   
+
    Level: advanced
 
 .seealso: EPSJDGetKrylovStart()
@@ -210,7 +196,7 @@ PetscErrorCode EPSJDSetKrylovStart(EPS eps,PetscBool krylovstart)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSJDGetKrylovStart"
 /*@
    EPSJDGetKrylovStart - Returns a flag indicating if the searching subspace is started with a
@@ -240,7 +226,7 @@ PetscErrorCode EPSJDGetKrylovStart(EPS eps,PetscBool *krylovstart)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSJDSetBlockSize"
 /*@
    EPSJDSetBlockSize - Sets the number of vectors to be added to the searching space
@@ -254,7 +240,7 @@ PetscErrorCode EPSJDGetKrylovStart(EPS eps,PetscBool *krylovstart)
 
    Options Database Key:
 .  -eps_jd_blocksize - number of vectors added to the searching space every iteration
-   
+
    Level: advanced
 
 .seealso: EPSJDSetKrylovStart()
@@ -270,7 +256,7 @@ PetscErrorCode EPSJDSetBlockSize(EPS eps,PetscInt blocksize)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSJDGetBlockSize"
 /*@
    EPSJDGetBlockSize - Returns the number of vectors to be added to the searching space
@@ -299,7 +285,7 @@ PetscErrorCode EPSJDGetBlockSize(EPS eps,PetscInt *blocksize)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSJDGetRestart"
 /*@
    EPSJDGetRestart - Gets the number of vectors of the searching space after
@@ -312,7 +298,7 @@ PetscErrorCode EPSJDGetBlockSize(EPS eps,PetscInt *blocksize)
 
    Output Parameter:
 +  minv - number of vectors of the searching subspace after restarting
--  plusk - number of vectors saved from the previous iteration   
+-  plusk - number of vectors saved from the previous iteration
 
    Level: advanced
 
@@ -324,13 +310,11 @@ PetscErrorCode EPSJDGetRestart(EPS eps,PetscInt *minv,PetscInt *plusk)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidIntPointer(minv,2);
-  PetscValidIntPointer(plusk,3);
   ierr = PetscTryMethod(eps,"EPSJDGetRestart_C",(EPS,PetscInt*,PetscInt*),(eps,minv,plusk));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSJDSetRestart"
 /*@
    EPSJDSetRestart - Sets the number of vectors of the searching space after
@@ -341,12 +325,12 @@ PetscErrorCode EPSJDGetRestart(EPS eps,PetscInt *minv,PetscInt *plusk)
    Input Parameters:
 +  eps - the eigenproblem solver context
 .  minv - number of vectors of the searching subspace after restarting
--  plusk - number of vectors saved from the previous iteration   
+-  plusk - number of vectors saved from the previous iteration
 
    Options Database Keys:
 +  -eps_jd_minv - number of vectors of the searching subspace after restarting
--  -eps_jd_plusk - number of vectors saved from the previous iteration   
-   
+-  -eps_jd_plusk - number of vectors saved from the previous iteration
+
    Level: advanced
 
 .seealso: EPSJDGetRestart()
@@ -363,7 +347,7 @@ PetscErrorCode EPSJDSetRestart(EPS eps,PetscInt minv,PetscInt plusk)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSJDGetInitialSize"
 /*@
    EPSJDGetInitialSize - Returns the initial size of the searching space.
@@ -377,10 +361,10 @@ PetscErrorCode EPSJDSetRestart(EPS eps,PetscInt minv,PetscInt plusk)
 .  initialsize - number of vectors of the initial searching subspace
 
    Notes:
-   If EPSGDGetKrylovStart() is PETSC_FALSE and the user provides vectors with
+   If EPSJDGetKrylovStart() is PETSC_FALSE and the user provides vectors with
    EPSSetInitialSpace(), up to initialsize vectors will be used; and if the
    provided vectors are not enough, the solver completes the subspace with
-   random vectors. In the case of EPSGDGetKrylovStart() being PETSC_TRUE, the solver
+   random vectors. In the case of EPSJDGetKrylovStart() being PETSC_TRUE, the solver
    gets the first vector provided by the user or, if not available, a random vector,
    and expands the Krylov basis up to initialsize vectors.
 
@@ -399,7 +383,7 @@ PetscErrorCode EPSJDGetInitialSize(EPS eps,PetscInt *initialsize)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSJDSetInitialSize"
 /*@
    EPSJDSetInitialSize - Sets the initial size of the searching space.
@@ -412,12 +396,12 @@ PetscErrorCode EPSJDGetInitialSize(EPS eps,PetscInt *initialsize)
 
    Options Database Key:
 .  -eps_jd_initial_size - number of vectors of the initial searching subspace
-   
+
    Notes:
-   If EPSGDGetKrylovStart() is PETSC_FALSE and the user provides vectors with
+   If EPSJDGetKrylovStart() is PETSC_FALSE and the user provides vectors with
    EPSSetInitialSpace(), up to initialsize vectors will be used; and if the
    provided vectors are not enough, the solver completes the subspace with
-   random vectors. In the case of EPSGDGetKrylovStart() being PETSC_TRUE, the solver
+   random vectors. In the case of EPSJDGetKrylovStart() being PETSC_TRUE, the solver
    gets the first vector provided by the user or, if not available, a random vector,
    and expands the Krylov basis up to initialsize vectors.
 
@@ -436,7 +420,7 @@ PetscErrorCode EPSJDSetInitialSize(EPS eps,PetscInt initialsize)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSJDGetFix"
 /*@
    EPSJDGetFix - Returns the threshold for changing the target in the correction
@@ -470,7 +454,7 @@ PetscErrorCode EPSJDGetFix(EPS eps,PetscReal *fix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSJDSetFix"
 /*@
    EPSJDSetFix - Sets the threshold for changing the target in the correction
@@ -484,7 +468,7 @@ PetscErrorCode EPSJDGetFix(EPS eps,PetscReal *fix)
 
    Options Database Key:
 .  -eps_jd_fix - the fix value
-   
+
    Note:
    The target in the correction equation is fixed at the first iterations.
    When the norm of the residual vector is lower than the fix value,
@@ -505,10 +489,10 @@ PetscErrorCode EPSJDSetFix(EPS eps,PetscReal fix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "EPSJDSetConstantCorrectionTolerance"
+#undef __FUNCT__
+#define __FUNCT__ "EPSJDSetConstCorrectionTol"
 /*@
-   EPSJDSetConstantCorrectionTolerance - If true, deactivates the dynamic stopping criterion
+   EPSJDSetConstCorrectionTol - If true, deactivates the dynamic stopping criterion
    (also called Newton) that sets the KSP relative tolerance
    to 0.5**i, where i is the number of EPS iterations from the last converged value.
 
@@ -519,27 +503,27 @@ PetscErrorCode EPSJDSetFix(EPS eps,PetscReal fix)
 -  constant - if false, the KSP relative tolerance is set to 0.5**i.
 
    Options Database Key:
-.  -eps_jd_constant_correction_tolerance - Deactivates the dynamic stopping criterion.
-   
+.  -eps_jd_const_correction_tol - Deactivates the dynamic stopping criterion.
+
    Level: advanced
 
-.seealso: EPSJDGetConstantCorrectionTolerance()
+.seealso: EPSJDGetConstCorrectionTol()
 @*/
-PetscErrorCode EPSJDSetConstantCorrectionTolerance(EPS eps,PetscBool constant)
+PetscErrorCode EPSJDSetConstCorrectionTol(EPS eps,PetscBool constant)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveBool(eps,constant,2);
-  ierr = PetscTryMethod(eps,"EPSJDSetConstantCorrectionTolerance_C",(EPS,PetscBool),(eps,constant));CHKERRQ(ierr);
+  ierr = PetscTryMethod(eps,"EPSJDSetConstCorrectionTol_C",(EPS,PetscBool),(eps,constant));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "EPSJDGetConstantCorrectionTolerance"
+#undef __FUNCT__
+#define __FUNCT__ "EPSJDGetConstCorrectionTol"
 /*@
-   EPSJDGetConstantCorrectionTolerance - Returns a flag indicating if the dynamic stopping is being used for
+   EPSJDGetConstCorrectionTol - Returns a flag indicating if the dynamic stopping is being used for
    solving the correction equation. If the flag is false the KSP relative tolerance is set
    to 0.5**i, where i is the number of EPS iterations from the last converged value.
 
@@ -553,20 +537,20 @@ PetscErrorCode EPSJDSetConstantCorrectionTolerance(EPS eps,PetscBool constant)
 
    Level: advanced
 
-.seealso: EPSJDGetConstantCorrectionTolerance()
+.seealso: EPSJDGetConstCorrectionTol()
 @*/
-PetscErrorCode EPSJDGetConstantCorrectionTolerance(EPS eps,PetscBool *constant)
+PetscErrorCode EPSJDGetConstCorrectionTol(EPS eps,PetscBool *constant)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidPointer(constant,2);
-  ierr = PetscTryMethod(eps,"EPSJDGetConstantCorrectionTolerance",(EPS,PetscBool*),(eps,constant));CHKERRQ(ierr);
+  ierr = PetscTryMethod(eps,"EPSJDGetConstCorrectionTol",(EPS,PetscBool*),(eps,constant));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSJDGetWindowSizes"
 /*@
    EPSJDGetWindowSizes - Gets the number of converged vectors in the projected
@@ -592,13 +576,11 @@ PetscErrorCode EPSJDGetWindowSizes(EPS eps,PetscInt *pwindow,PetscInt *qwindow)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidIntPointer(pwindow,2);
-  PetscValidIntPointer(qwindow,3);
   ierr = PetscTryMethod(eps,"EPSJDGetWindowSizes_C",(EPS,PetscInt*,PetscInt*),(eps,pwindow,qwindow));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSJDSetWindowSizes"
 /*@
    EPSJDSetWindowSizes - Sets the number of converged vectors in the projected
@@ -614,8 +596,8 @@ PetscErrorCode EPSJDGetWindowSizes(EPS eps,PetscInt *pwindow,PetscInt *qwindow)
 
    Options Database Keys:
 +  -eps_jd_pwindow - set the number of converged vectors in the projector
--  -eps_jd_qwindow - set the number of converged vectors in the projected problem  
-   
+-  -eps_jd_qwindow - set the number of converged vectors in the projected problem
+
    Level: advanced
 
 .seealso: EPSJDGetWindowSizes()
@@ -631,3 +613,114 @@ PetscErrorCode EPSJDSetWindowSizes(EPS eps,PetscInt pwindow,PetscInt qwindow)
   ierr = PetscTryMethod(eps,"EPSJDSetWindowSizes_C",(EPS,PetscInt,PetscInt),(eps,pwindow,qwindow));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
+#undef __FUNCT__
+#define __FUNCT__ "EPSJDSetBOrth"
+/*@
+   EPSJDSetBOrth - Selects the orthogonalization that will be used in the search
+   subspace in case of generalized Hermitian problems.
+
+   Logically Collective on EPS
+
+   Input Parameters:
++  eps   - the eigenproblem solver context
+-  borth - the kind of orthogonalization
+
+   Possible values:
+   The parameter 'borth' can have one of these values
+
++   EPS_ORTH_I - orthogonalization of the search subspace
+.   EPS_ORTH_B - B-orthogonalization of the search subspace
+-   EPS_ORTH_BOPT - B-orthogonalization of the search subspace with an alternative method
+
+   Options Database Key:
+.  -eps_jd_borth - Set the orthogonalization used in the search subspace
+
+   Notes:
+   If borth is EPS_ORTH_B, the solver uses a variant of Gram-Schmidt (selected in
+   IP associated to the EPS) with the inner product defined by the matrix problem B.
+   If borth is EPS_ORTH_BOPT, it uses another variant of Gram-Schmidt that only performs
+   one matrix-vector product although more than one reorthogonalization would be done.
+
+   Level: advanced
+
+.seealso: EPSJDGetBOrth()
+@*/
+PetscErrorCode EPSJDSetBOrth(EPS eps,EPSOrthType borth)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidLogicalCollectiveBool(eps,borth,2);
+  ierr = PetscTryMethod(eps,"EPSJDSetBOrth_C",(EPS,EPSOrthType),(eps,borth));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "EPSJDGetBOrth"
+/*@
+   EPSJDGetBOrth - Returns the orthogonalization used in the search
+   subspace in case of generalized Hermitian problems.
+
+   Not Collective
+
+   Input Parameter:
+.  eps - the eigenproblem solver context
+
+   Output Parameters:
+.  borth - the kind of orthogonalization
+
+   Notes:
+   See EPSJDSetBOrth() for possible values of 'borth'.
+
+   Level: advanced
+
+.seealso: EPSJDSetBOrth(), EPSOrthType
+@*/
+PetscErrorCode EPSJDGetBOrth(EPS eps,EPSOrthType *borth)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidPointer(borth,2);
+  ierr = PetscTryMethod(eps,"EPSJDGetBOrth_C",(EPS,EPSOrthType*),(eps,borth));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "EPSCreate_JD"
+PETSC_EXTERN PetscErrorCode EPSCreate_JD(EPS eps)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  /* Load the Davidson solver */
+  ierr = EPSCreate_XD(eps);CHKERRQ(ierr);
+  ierr = EPSXDSetMethod(eps,DVD_METH_JD);CHKERRQ(ierr);
+
+  /* Overload the JD properties */
+  eps->ops->setfromoptions = EPSSetFromOptions_JD;
+  eps->ops->setup          = EPSSetUp_JD;
+  eps->ops->destroy        = EPSDestroy_JD;
+
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetKrylovStart_C",EPSXDSetKrylovStart_XD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetKrylovStart_C",EPSXDGetKrylovStart_XD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetBlockSize_C",EPSXDSetBlockSize_XD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetBlockSize_C",EPSXDGetBlockSize_XD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetRestart_C",EPSXDSetRestart_XD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetRestart_C",EPSXDGetRestart_XD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetInitialSize_C",EPSXDSetInitialSize_XD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetInitialSize_C",EPSXDGetInitialSize_XD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetFix_C",EPSJDSetFix_JD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetFix_C",EPSXDGetFix_XD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetConstCorrectionTol_C",EPSJDSetConstCorrectionTol_JD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetConstCorrectionTol_C",EPSJDGetConstCorrectionTol_JD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetWindowSizes_C",EPSXDSetWindowSizes_XD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetWindowSizes_C",EPSXDGetWindowSizes_XD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDSetBOrth_C",EPSXDSetBOrth_XD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSJDGetBOrth_C",EPSXDGetBOrth_XD);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
