@@ -1,19 +1,19 @@
 /*
-       This file implements a wrapper to the BLOPEX solver
+   This file implements a wrapper to the BLOPEX package
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,8 +21,8 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/stimpl.h>       /*I "slepcst.h" I*/
-#include <private/epsimpl.h>      /*I "slepceps.h" I*/
+#include <slepc-private/epsimpl.h>      /*I "slepceps.h" I*/
+#include <slepc-private/stimpl.h>       /*I "slepcst.h" I*/
 #include "slepc-interface.h"
 #include <blopex_lobpcg.h>
 #include <blopex_interpreter.h>
@@ -37,27 +37,24 @@ typedef struct {
   mv_MultiVectorPtr          eigenvectors;
   mv_MultiVectorPtr          Y;
   mv_InterfaceInterpreter    ii;
-  KSP                        ksp;
+  ST                         st;
   Vec                        w;
 } EPS_BLOPEX;
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "Precond_FnSingleVector"
 static void Precond_FnSingleVector(void *data,void *x,void *y)
 {
   PetscErrorCode ierr;
   EPS            eps = (EPS)data;
   EPS_BLOPEX     *blopex = (EPS_BLOPEX*)eps->data;
-  PetscInt       lits;
-      
+
   PetscFunctionBegin;
-  ierr = KSPSolve(blopex->ksp,(Vec)x,(Vec)y); CHKERRABORT(PETSC_COMM_WORLD,ierr);
-  ierr = KSPGetIterationNumber(blopex->ksp,&lits); CHKERRABORT(PETSC_COMM_WORLD,ierr);
-  eps->OP->lineariterations+= lits;
+  ierr = KSPSolve(blopex->st->ksp,(Vec)x,(Vec)y);CHKERRABORT(PetscObjectComm((PetscObject)eps),ierr);
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "Precond_FnMultiVector"
 static void Precond_FnMultiVector(void *data,void *x,void *y)
 {
@@ -69,7 +66,7 @@ static void Precond_FnMultiVector(void *data,void *x,void *y)
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "OperatorASingleVector"
 static void OperatorASingleVector(void *data,void *x,void *y)
 {
@@ -77,19 +74,27 @@ static void OperatorASingleVector(void *data,void *x,void *y)
   EPS            eps = (EPS)data;
   EPS_BLOPEX     *blopex = (EPS_BLOPEX*)eps->data;
   Mat            A,B;
- 
+  PetscScalar    sigma;
+  PetscInt       nmat;
+
   PetscFunctionBegin;
-  ierr = STGetOperators(eps->OP,&A,&B);CHKERRABORT(PETSC_COMM_WORLD,ierr);
-  ierr = MatMult(A,(Vec)x,(Vec)y);CHKERRABORT(PETSC_COMM_WORLD,ierr);
-  if (eps->OP->sigma != 0.0) {
-    if (B) { ierr = MatMult(B,(Vec)x,blopex->w);CHKERRABORT(PETSC_COMM_WORLD,ierr); }
-    else { ierr = VecCopy((Vec)x,blopex->w);CHKERRABORT(PETSC_COMM_WORLD,ierr); }
-    ierr = VecAXPY((Vec)y,-eps->OP->sigma,blopex->w);CHKERRABORT(PETSC_COMM_WORLD,ierr);
+  ierr = STGetNumMatrices(eps->st,&nmat);CHKERRABORT(PetscObjectComm((PetscObject)eps),ierr);
+  ierr = STGetOperators(eps->st,0,&A);CHKERRABORT(PetscObjectComm((PetscObject)eps),ierr);
+  if (nmat>1) { ierr = STGetOperators(eps->st,1,&B);CHKERRABORT(PetscObjectComm((PetscObject)eps),ierr); }
+  ierr = MatMult(A,(Vec)x,(Vec)y);CHKERRABORT(PetscObjectComm((PetscObject)eps),ierr);
+  ierr = STGetShift(eps->st,&sigma);CHKERRABORT(PetscObjectComm((PetscObject)eps),ierr);
+  if (sigma != 0.0) {
+    if (nmat>1) {
+      ierr = MatMult(B,(Vec)x,blopex->w);CHKERRABORT(PetscObjectComm((PetscObject)eps),ierr);
+    } else {
+      ierr = VecCopy((Vec)x,blopex->w);CHKERRABORT(PetscObjectComm((PetscObject)eps),ierr);
+    }
+    ierr = VecAXPY((Vec)y,-sigma,blopex->w);CHKERRABORT(PetscObjectComm((PetscObject)eps),ierr);
   }
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "OperatorAMultiVector"
 static void OperatorAMultiVector(void *data,void *x,void *y)
 {
@@ -101,21 +106,21 @@ static void OperatorAMultiVector(void *data,void *x,void *y)
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "OperatorBSingleVector"
 static void OperatorBSingleVector(void *data,void *x,void *y)
 {
   PetscErrorCode ierr;
   EPS            eps = (EPS)data;
   Mat            B;
-  
+
   PetscFunctionBegin;
-  ierr = STGetOperators(eps->OP,PETSC_NULL,&B);CHKERRABORT(PETSC_COMM_WORLD,ierr);
-  ierr = MatMult(B,(Vec)x,(Vec)y);CHKERRABORT(PETSC_COMM_WORLD,ierr);
+  ierr = STGetOperators(eps->st,1,&B);CHKERRABORT(PetscObjectComm((PetscObject)eps),ierr);
+  ierr = MatMult(B,(Vec)x,(Vec)y);CHKERRABORT(PetscObjectComm((PetscObject)eps),ierr);
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "OperatorBMultiVector"
 static void OperatorBMultiVector(void *data,void *x,void *y)
 {
@@ -127,56 +132,63 @@ static void OperatorBMultiVector(void *data,void *x,void *y)
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetUp_BLOPEX"
 PetscErrorCode EPSSetUp_BLOPEX(EPS eps)
 {
+#if defined(PETSC_MISSING_LAPACK_POTRF) || defined(PETSC_MISSING_LAPACK_SYGV)
+  PetscFunctionBegin;
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"POTRF/SYGV - Lapack routine is unavailable");
+#else
   PetscErrorCode ierr;
   PetscInt       i;
-  EPS_BLOPEX     *blopex = (EPS_BLOPEX *)eps->data;
+  EPS_BLOPEX     *blopex = (EPS_BLOPEX*)eps->data;
   PetscBool      isPrecond;
 
   PetscFunctionBegin;
-  if (!eps->ishermitian) { 
-    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"blopex only works for hermitian problems"); 
-  }
+  if (!eps->ishermitian) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"blopex only works for hermitian problems");
   if (!eps->which) eps->which = EPS_SMALLEST_REAL;
-  if (eps->which!=EPS_SMALLEST_REAL) {
-    SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->which");
-  }
+  if (eps->which!=EPS_SMALLEST_REAL) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->which");
 
   /* Change the default sigma to inf if necessary */
   if (eps->which == EPS_LARGEST_MAGNITUDE || eps->which == EPS_LARGEST_REAL ||
       eps->which == EPS_LARGEST_IMAGINARY) {
-    ierr = STSetDefaultShift(eps->OP,3e300);CHKERRQ(ierr);
+    ierr = STSetDefaultShift(eps->st,3e300);CHKERRQ(ierr);
   }
 
-  ierr = STSetUp(eps->OP);CHKERRQ(ierr);
-  ierr = PetscTypeCompare((PetscObject)eps->OP,STPRECOND,&isPrecond);CHKERRQ(ierr);
-  if (!isPrecond) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"blopex only works with STPRECOND");
-  ierr = STGetKSP(eps->OP,&blopex->ksp);CHKERRQ(ierr);
+  ierr = STSetUp(eps->st);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STPRECOND,&isPrecond);CHKERRQ(ierr);
+  if (!isPrecond) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"blopex only works with STPRECOND");
+  blopex->st = eps->st;
 
   eps->ncv = eps->nev = PetscMin(eps->nev,eps->n);
   if (eps->mpd) { ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr); }
   if (!eps->max_it) eps->max_it = PetscMax(100,2*eps->n/eps->ncv);
+  if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs not supported in this solver");
 
   ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
-  ierr = EPSDefaultGetWork(eps,1);CHKERRQ(ierr);
-  
-  blopex->tol.absolute = eps->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:eps->tol;
-  blopex->tol.relative = 1e-50;
-  
+  ierr = EPSSetWorkVecs(eps,1);CHKERRQ(ierr);
+
+  if (eps->converged == EPSConvergedEigRelative) {
+    blopex->tol.absolute = 0.0;
+    blopex->tol.relative = eps->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:eps->tol;
+  } else if (eps->converged == EPSConvergedAbsolute) {
+    blopex->tol.absolute = eps->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:eps->tol;
+    blopex->tol.relative = 0.0;
+  } else {
+    SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Convergence test not supported in this solver");
+  }
+
   SLEPCSetupInterpreter(&blopex->ii);
   blopex->eigenvectors = mv_MultiVectorCreateFromSampleVector(&blopex->ii,eps->ncv,eps->V);
   for (i=0;i<eps->ncv;i++) { ierr = PetscObjectReference((PetscObject)eps->V[i]);CHKERRQ(ierr); }
-  mv_MultiVectorSetRandom(blopex->eigenvectors,1234);
-  ierr = VecDuplicate(eps->V[0],&blopex->w);CHKERRQ(ierr);
 
+  ierr = VecDuplicate(eps->V[0],&blopex->w);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent(eps,blopex->w);CHKERRQ(ierr);
   if (eps->nds > 0) {
-    blopex->Y = mv_MultiVectorCreateFromSampleVector(&blopex->ii,eps->nds,eps->DS);
-    for (i=0;i<eps->nds;i++) { ierr = PetscObjectReference((PetscObject)eps->DS[i]);CHKERRQ(ierr); }
-  } else
-    blopex->Y = PETSC_NULL;
+    blopex->Y = mv_MultiVectorCreateFromSampleVector(&blopex->ii,eps->nds,eps->defl);
+    for (i=0;i<eps->nds;i++) { ierr = PetscObjectReference((PetscObject)eps->defl[i]);CHKERRQ(ierr); }
+  } else blopex->Y = NULL;
 
 #if defined(PETSC_USE_COMPLEX)
   blopex->blap_fn.zpotrf = PETSC_zpotrf_interface;
@@ -189,67 +201,87 @@ PetscErrorCode EPSSetUp_BLOPEX(EPS eps)
   if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
 
   /* dispatch solve method */
-  if (eps->leftvecs) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Left vectors not supported in this solver");
+  if (eps->leftvecs) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Left vectors not supported in this solver");
   eps->ops->solve = EPSSolve_BLOPEX;
   PetscFunctionReturn(0);
+#endif
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSolve_BLOPEX"
 PetscErrorCode EPSSolve_BLOPEX(EPS eps)
 {
-  EPS_BLOPEX *blopex = (EPS_BLOPEX *)eps->data;
-  int        i,j,info,its,nconv;
-  double     *lambdahist=PETSC_NULL,*residhist=PETSC_NULL;
+  EPS_BLOPEX     *blopex = (EPS_BLOPEX*)eps->data;
+  PetscScalar    sigma;
+  int            i,j,info,its,nconv;
+  double         *residhist=NULL;
   PetscErrorCode ierr;
-  
+#if defined(PETSC_USE_COMPLEX)
+  komplex        *lambdahist=NULL;
+#else
+  double         *lambdahist=NULL;
+#endif
+
   PetscFunctionBegin;
+  /* Complete the initial basis with random vectors */
+  for (i=eps->nini;i<eps->ncv;i++) {
+    ierr = SlepcVecSetRandom(eps->V[i],eps->rand);CHKERRQ(ierr);
+  }
+
   if (eps->numbermonitors>0) {
+#if defined(PETSC_USE_COMPLEX)
+    ierr = PetscMalloc(eps->ncv*(eps->max_it+1)*sizeof(komplex),&lambdahist);CHKERRQ(ierr);
+#else
     ierr = PetscMalloc(eps->ncv*(eps->max_it+1)*sizeof(double),&lambdahist);CHKERRQ(ierr);
+#endif
     ierr = PetscMalloc(eps->ncv*(eps->max_it+1)*sizeof(double),&residhist);CHKERRQ(ierr);
   }
 
 #if defined(PETSC_USE_COMPLEX)
   info = lobpcg_solve_complex(blopex->eigenvectors,eps,OperatorAMultiVector,
-        eps->isgeneralized?eps:PETSC_NULL,eps->isgeneralized?OperatorBMultiVector:PETSC_NULL,
+        eps->isgeneralized?eps:NULL,eps->isgeneralized?OperatorBMultiVector:NULL,
         eps,Precond_FnMultiVector,blopex->Y,
         blopex->blap_fn,blopex->tol,eps->max_it,0,&its,
         (komplex*)eps->eigr,lambdahist,eps->ncv,eps->errest,residhist,eps->ncv);
 #else
   info = lobpcg_solve_double(blopex->eigenvectors,eps,OperatorAMultiVector,
-        eps->isgeneralized?eps:PETSC_NULL,eps->isgeneralized?OperatorBMultiVector:PETSC_NULL,
+        eps->isgeneralized?eps:NULL,eps->isgeneralized?OperatorBMultiVector:NULL,
         eps,Precond_FnMultiVector,blopex->Y,
         blopex->blap_fn,blopex->tol,eps->max_it,0,&its,
         eps->eigr,lambdahist,eps->ncv,eps->errest,residhist,eps->ncv);
 #endif
-  if (info>0) SETERRQ1(((PetscObject)eps)->comm,PETSC_ERR_LIB,"Error in blopex (code=%d)",info); 
+  if (info>0) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"Error in blopex (code=%d)",info);
 
   if (eps->numbermonitors>0) {
     for (i=0;i<its;i++) {
       nconv = 0;
-      for (j=0;j<eps->ncv;j++) { if (residhist[j+i*eps->ncv]>eps->tol) break; else nconv++; }
-      ierr = EPSMonitor(eps,i,nconv,lambdahist+i*eps->ncv,eps->eigi,residhist+i*eps->ncv,eps->ncv);CHKERRQ(ierr);
+      for (j=0;j<eps->ncv;j++) {
+        if (residhist[j+i*eps->ncv]>eps->tol) break;
+        else nconv++;
+      }
+      ierr = EPSMonitor(eps,i,nconv,(PetscScalar*)lambdahist+i*eps->ncv,eps->eigi,residhist+i*eps->ncv,eps->ncv);CHKERRQ(ierr);
     }
-    ierr = PetscFree(lambdahist);CHKERRQ(ierr); 
-    ierr = PetscFree(residhist);CHKERRQ(ierr); 
+    ierr = PetscFree(lambdahist);CHKERRQ(ierr);
+    ierr = PetscFree(residhist);CHKERRQ(ierr);
   }
 
   eps->its = its;
   eps->nconv = eps->ncv;
-  if (eps->OP->sigma != 0.0) {
-    for (i=0;i<eps->nconv;i++) eps->eigr[i]+=eps->OP->sigma;
+  ierr = STGetShift(eps->st,&sigma);CHKERRQ(ierr);
+  if (sigma != 0.0) {
+    for (i=0;i<eps->nconv;i++) eps->eigr[i]+=sigma;
   }
   if (info==-1) eps->reason = EPS_DIVERGED_ITS;
   else eps->reason = EPS_CONVERGED_TOL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSReset_BLOPEX"
 PetscErrorCode EPSReset_BLOPEX(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_BLOPEX     *blopex = (EPS_BLOPEX *)eps->data;
+  EPS_BLOPEX     *blopex = (EPS_BLOPEX*)eps->data;
 
   PetscFunctionBegin;
   mv_MultiVectorDestroy(blopex->eigenvectors);
@@ -259,7 +291,7 @@ PetscErrorCode EPSReset_BLOPEX(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSDestroy_BLOPEX"
 PetscErrorCode EPSDestroy_BLOPEX(EPS eps)
 {
@@ -271,23 +303,35 @@ PetscErrorCode EPSDestroy_BLOPEX(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetFromOptions_BLOPEX"
 PetscErrorCode EPSSetFromOptions_BLOPEX(EPS eps)
 {
   PetscErrorCode  ierr;
+  KSP             ksp;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead("EPS BLOPEX Options");CHKERRQ(ierr);
   LOBPCG_SetFromOptionsRandomContext();
+
+  /* Set STPrecond as the default ST */
+  if (!((PetscObject)eps->st)->type_name) {
+    ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
+  }
+  ierr = STPrecondSetKSPHasMat(eps->st,PETSC_TRUE);CHKERRQ(ierr);
+
+  /* Set the default options of the KSP */
+  ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
+  if (!((PetscObject)ksp)->type_name) {
+    ierr = KSPSetType(ksp,KSPPREONLY);CHKERRQ(ierr);
+  }
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSCreate_BLOPEX"
-PetscErrorCode EPSCreate_BLOPEX(EPS eps)
+PETSC_EXTERN PetscErrorCode EPSCreate_BLOPEX(EPS eps)
 {
   PetscErrorCode ierr;
 
@@ -299,9 +343,7 @@ PetscErrorCode EPSCreate_BLOPEX(EPS eps)
   eps->ops->reset                = EPSReset_BLOPEX;
   eps->ops->backtransform        = EPSBackTransform_Default;
   eps->ops->computevectors       = EPSComputeVectors_Default;
-  ierr = STSetType(eps->OP,STPRECOND);CHKERRQ(ierr);
-  ierr = STPrecondSetKSPHasMat(eps->OP,PETSC_TRUE);CHKERRQ(ierr);
-  LOBPCG_InitRandomContext(((PetscObject)eps)->comm,eps->rand);
+  LOBPCG_InitRandomContext(PetscObjectComm((PetscObject)eps),eps->rand);
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
+
