@@ -4,17 +4,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -22,9 +22,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <petscsys.h>
 #include <petscvec.h>
-#include <assert.h>
 #include <stdlib.h>
 #include <blopex_interpreter.h>
 #include <blopex_temp_multivector.h>
@@ -39,13 +37,13 @@ static void* mv_TempMultiVectorCreateFromPETScVector(void* ii_,BlopexInt n,void*
   mv_InterfaceInterpreter* ii = (mv_InterfaceInterpreter*)ii_;
 
   x = (mv_TempMultiVector*)malloc(sizeof(mv_TempMultiVector));
-  assert(x!=NULL);
-  
+  if (!x) SETERRABORT(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Allocation for x failed");
+
   x->interpreter = ii;
   x->numVectors = n;
-  
+
   x->vector = (void**)calloc(n,sizeof(void*));
-  assert(x->vector!=NULL);
+  if (!x->vector) SETERRABORT(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Allocation for x->vector failed");
 
   x->ownsVectors = 1;
   x->mask = NULL;
@@ -61,10 +59,9 @@ static void mv_TempMultiPETSCVectorDestroy(void* x_)
 {
   mv_TempMultiVector* x = (mv_TempMultiVector*)x_;
 
-  if (x==NULL)
-    return;
+  if (!x) return;
 
-  if (x->ownsVectors && x->vector!=NULL) free(x->vector);
+  if (x->ownsVectors && x->vector) free(x->vector);
   if (x->mask && x->ownsMask) free(x->mask);
   free(x);
 }
