@@ -4,36 +4,36 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
- 
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
- 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
- 
+
    You  should have received a copy of the GNU Lesser General  Public  License
    along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 
-#ifndef SLEPC_INTERFACE_HEADER
+#if !defined(SLEPC_INTERFACE_HEADER)
 #define SLEPC_INTERFACE_HEADER
 
 #include <blopex_lobpcg.h>
 #include "petsc-interface.h"
 
 extern int
-SLEPCSetupInterpreter( mv_InterfaceInterpreter *ii );
+SLEPCSetupInterpreter(mv_InterfaceInterpreter *ii);
 
 extern void
-SLEPCSetupInterpreterForDignifiedDeath( mv_InterfaceInterpreter *i );
+SLEPCSetupInterpreterForDignifiedDeath(mv_InterfaceInterpreter *i);
 
 #endif
 
