@@ -1,19 +1,19 @@
 /*
-       This file implements a wrapper to the BLZPACK package
+   This file implements a wrapper to the BLZPACK package
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,8 +21,8 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/epsimpl.h>    /*I "slepceps.h" I*/
-#include <private/stimpl.h>     /*I "slepcst.h" I*/
+#include <slepc-private/epsimpl.h>    /*I "slepceps.h" I*/
+#include <slepc-private/stimpl.h>     /*I "slepcst.h" I*/
 #include <../src/eps/impls/external/blzpack/blzpackp.h>
 
 PetscErrorCode EPSSolve_BLZPACK(EPS);
@@ -63,47 +63,46 @@ const char* blzpack_error[33] = {
   "parallel interface error "
 };
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetUp_BLZPACK"
 PetscErrorCode EPSSetUp_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
   PetscInt       listor,lrstor,ncuv,k1,k2,k3,k4;
-  EPS_BLZPACK    *blz = (EPS_BLZPACK *)eps->data;
+  EPS_BLZPACK    *blz = (EPS_BLZPACK*)eps->data;
   PetscBool      issinv;
 
   PetscFunctionBegin;
   if (eps->ncv) {
-    if (eps->ncv < PetscMin(eps->nev+10,eps->nev*2))
-      SETERRQ(((PetscObject)eps)->comm,0,"Warning: BLZpack recommends that ncv be larger than min(nev+10,nev*2)");
-  }
-  else eps->ncv = PetscMin(eps->nev+10,eps->nev*2);
+    if (eps->ncv < PetscMin(eps->nev+10,eps->nev*2)) SETERRQ(PetscObjectComm((PetscObject)eps),0,"Warning: BLZpack recommends that ncv be larger than min(nev+10,nev*2)");
+  } else eps->ncv = PetscMin(eps->nev+10,eps->nev*2);
   if (eps->mpd) { ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr); }
   if (!eps->max_it) eps->max_it = PetscMax(1000,eps->n);
 
-  if (!eps->ishermitian)
-    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Requested method is only available for Hermitian problems");
+  if (!blz->block_size) blz->block_size = 3;
+  if (!eps->ishermitian) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Requested method is only available for Hermitian problems");
   if (eps->which==EPS_ALL) {
-    if (eps->inta==0.0 && eps->intb==0.0) SETERRQ(((PetscObject)eps)->comm,1,"Must define a computational interval when using EPS_ALL"); 
+    if (eps->inta==0.0 && eps->intb==0.0) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Must define a computational interval when using EPS_ALL");
     blz->slice = 1;
   }
-  ierr = PetscTypeCompare((PetscObject)eps->OP,STSINVERT,&issinv);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STSINVERT,&issinv);CHKERRQ(ierr);
   if (blz->slice || eps->isgeneralized) {
-    if (!issinv) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Shift-and-invert ST is needed for generalized problems or spectrum slicing");
+    if (!issinv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Shift-and-invert ST is needed for generalized problems or spectrum slicing");
   }
   if (blz->slice) {
     if (eps->intb >= PETSC_MAX_REAL) { /* right-open interval */
-      if (eps->inta <= PETSC_MIN_REAL) SETERRQ(((PetscObject)eps)->comm,1,"The defined computational interval should have at least one of their sides bounded");
-      ierr = STSetDefaultShift(eps->OP,eps->inta);CHKERRQ(ierr);
+      if (eps->inta <= PETSC_MIN_REAL) SETERRQ(PetscObjectComm((PetscObject)eps),1,"The defined computational interval should have at least one of their sides bounded");
+      ierr = STSetDefaultShift(eps->st,eps->inta);CHKERRQ(ierr);
+    } else {
+      ierr = STSetDefaultShift(eps->st,eps->intb);CHKERRQ(ierr);
     }
-    else { ierr = STSetDefaultShift(eps->OP,eps->intb);CHKERRQ(ierr); }
   }
   if (!eps->which) {
     if (issinv) eps->which = EPS_TARGET_REAL;
     else eps->which = EPS_SMALLEST_REAL;
   }
-  if ((issinv && eps->which!=EPS_TARGET_REAL && eps->which!=EPS_TARGET_MAGNITUDE && eps->which!=EPS_ALL) || (!issinv && eps->which!=EPS_SMALLEST_REAL))
-    SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->which");
+  if ((issinv && eps->which!=EPS_TARGET_REAL && eps->which!=EPS_TARGET_MAGNITUDE && eps->which!=EPS_ALL) || (!issinv && eps->which!=EPS_SMALLEST_REAL)) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->which");
+  if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs not supported in this solver");
 
   k1 = PetscMin(eps->n,180);
   k2 = blz->block_size;
@@ -113,55 +112,60 @@ PetscErrorCode EPSSetUp_BLZPACK(EPS eps)
   listor = 123+k1*12;
   ierr = PetscFree(blz->istor);CHKERRQ(ierr);
   ierr = PetscMalloc((17+listor)*sizeof(PetscBLASInt),&blz->istor);CHKERRQ(ierr);
-  blz->istor[14] = PetscBLASIntCast(listor);
+  ierr = PetscLogObjectMemory(eps,(17+listor)*sizeof(PetscBLASInt));CHKERRQ(ierr);
+  ierr = PetscBLASIntCast(listor,&blz->istor[14]);CHKERRQ(ierr);
 
   if (blz->slice) lrstor = eps->nloc*(k2*4+k1*2+k4)+k3;
   else lrstor = eps->nloc*(k2*4+k1)+k3;
 lrstor*=10;
   ierr = PetscFree(blz->rstor);CHKERRQ(ierr);
   ierr = PetscMalloc((4+lrstor)*sizeof(PetscReal),&blz->rstor);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory(eps,(4+lrstor)*sizeof(PetscReal));CHKERRQ(ierr);
   blz->rstor[3] = lrstor;
 
   ncuv = PetscMax(3,blz->block_size);
   ierr = PetscFree(blz->u);CHKERRQ(ierr);
   ierr = PetscMalloc(ncuv*eps->nloc*sizeof(PetscScalar),&blz->u);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory(eps,ncuv*eps->nloc*sizeof(PetscScalar));CHKERRQ(ierr);
   ierr = PetscFree(blz->v);CHKERRQ(ierr);
   ierr = PetscMalloc(ncuv*eps->nloc*sizeof(PetscScalar),&blz->v);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory(eps,ncuv*eps->nloc*sizeof(PetscScalar));CHKERRQ(ierr);
 
   ierr = PetscFree(blz->eig);CHKERRQ(ierr);
   ierr = PetscMalloc(2*eps->ncv*sizeof(PetscReal),&blz->eig);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory(eps,2*eps->ncv*sizeof(PetscReal));CHKERRQ(ierr);
 
   if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
 
   ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
 
   /* dispatch solve method */
-  if (eps->leftvecs) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Left vectors not supported in this solver");
+  if (eps->leftvecs) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Left vectors not supported in this solver");
   eps->ops->solve = EPSSolve_BLZPACK;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSolve_BLZPACK"
 PetscErrorCode EPSSolve_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_BLZPACK    *blz = (EPS_BLZPACK *)eps->data;
+  EPS_BLZPACK    *blz = (EPS_BLZPACK*)eps->data;
   PetscInt       nn;
-  PetscBLASInt   i,nneig,lflag,nvopu;      
-  Vec            x,y;                           
-  PetscScalar    sigma,*pV;                      
-  Mat            A;                              
-  KSP            ksp;                            
-  PC             pc;                             
-  
+  PetscBLASInt   i,nneig,lflag,nvopu;
+  Vec            x,y;
+  PetscScalar    sigma,*pV;
+  Mat            A;
+  KSP            ksp;
+  PC             pc;
+
   PetscFunctionBegin;
-  ierr = VecCreateMPIWithArray(((PetscObject)eps)->comm,eps->nloc,PETSC_DECIDE,PETSC_NULL,&x);CHKERRQ(ierr);
-  ierr = VecCreateMPIWithArray(((PetscObject)eps)->comm,eps->nloc,PETSC_DECIDE,PETSC_NULL,&y);CHKERRQ(ierr);
+  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)eps),1,eps->nloc,PETSC_DECIDE,NULL,&x);CHKERRQ(ierr);
+  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)eps),1,eps->nloc,PETSC_DECIDE,NULL,&y);CHKERRQ(ierr);
   ierr = VecGetArray(eps->V[0],&pV);CHKERRQ(ierr);
-  
-  if (eps->isgeneralized && !blz->slice) { 
-    ierr = STGetShift(eps->OP,&sigma);CHKERRQ(ierr); /* shift of origin */
+
+  if (eps->isgeneralized && !blz->slice) {
+    ierr = STGetShift(eps->st,&sigma);CHKERRQ(ierr); /* shift of origin */
     blz->rstor[0]  = sigma;        /* lower limit of eigenvalue interval */
     blz->rstor[1]  = sigma;        /* upper limit of eigenvalue interval */
   } else {
@@ -171,22 +175,22 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
   }
   nneig = 0;                       /* no. of eigs less than sigma */
 
-  blz->istor[0]  = PetscBLASIntCast(eps->nloc); /* number of rows of U, V, X*/
-  blz->istor[1]  = PetscBLASIntCast(eps->nloc); /* leading dimension of U, V, X */
-  blz->istor[2]  = PetscBLASIntCast(eps->nev); /* number of required eigenpairs */
-  blz->istor[3]  = PetscBLASIntCast(eps->ncv); /* number of working eigenpairs */
+  ierr = PetscBLASIntCast(eps->nloc,&blz->istor[0]);CHKERRQ(ierr); /* no. of rows of U, V, X */
+  ierr = PetscBLASIntCast(eps->nloc,&blz->istor[1]);CHKERRQ(ierr); /* leading dim of U, V, X */
+  ierr = PetscBLASIntCast(eps->nev,&blz->istor[2]);CHKERRQ(ierr);  /* required eigenpairs */
+  ierr = PetscBLASIntCast(eps->ncv,&blz->istor[3]);CHKERRQ(ierr);  /* working eigenpairs */
   blz->istor[4]  = blz->block_size;    /* number of vectors in a block */
-  blz->istor[5]  = blz->nsteps;  /* maximun number of steps per run */
-  blz->istor[6]  = 1;            /* number of starting vectors as input */
-  blz->istor[7]  = 0;            /* number of eigenpairs given as input */
+  blz->istor[5]  = blz->nsteps;        /* maximun number of steps per run */
+  blz->istor[6]  = 1;                  /* number of starting vectors as input */
+  blz->istor[7]  = 0;                  /* number of eigenpairs given as input */
   blz->istor[8]  = (blz->slice || eps->isgeneralized) ? 1 : 0;   /* problem type */
-  blz->istor[9]  = blz->slice;   /* spectrum slicing */
+  blz->istor[9]  = blz->slice;         /* spectrum slicing */
   blz->istor[10] = eps->isgeneralized ? 1 : 0;   /* solutions refinement (purify) */
-  blz->istor[11] = 0;            /* level of printing */
-  blz->istor[12] = 6;            /* file unit for output */
-  blz->istor[13] = PetscBLASIntCast(MPI_Comm_c2f(((PetscObject)eps)->comm)); /* communicator */
+  blz->istor[11] = 0;                  /* level of printing */
+  blz->istor[12] = 6;                  /* file unit for output */
+  ierr = PetscBLASIntCast(MPI_Comm_c2f(PetscObjectComm((PetscObject)eps)),&blz->istor[13]);CHKERRQ(ierr);
 
-  blz->rstor[2]  = eps->tol;     /* threshold for convergence */
+  blz->rstor[2]  = eps->tol;           /* threshold for convergence */
 
   lflag = 0;           /* reverse communication interface flag */
 
@@ -199,12 +203,12 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
       for (i=0;i<nvopu;i++) {
         ierr = VecPlaceArray(x,blz->u+i*eps->nloc);CHKERRQ(ierr);
         ierr = VecPlaceArray(y,blz->v+i*eps->nloc);CHKERRQ(ierr);
-        if (blz->slice || eps->isgeneralized) { 
-          ierr = STAssociatedKSPSolve(eps->OP,x,y);CHKERRQ(ierr);
+        if (blz->slice || eps->isgeneralized) {
+          ierr = STMatSolve(eps->st,1,x,y);CHKERRQ(ierr);
         } else {
-          ierr = STApply(eps->OP,x,y);CHKERRQ(ierr);
+          ierr = STApply(eps->st,x,y);CHKERRQ(ierr);
         }
-        ierr = IPOrthogonalize(eps->ip,0,PETSC_NULL,eps->nds,PETSC_NULL,eps->DS,y,PETSC_NULL,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+        ierr = IPOrthogonalize(eps->ip,0,NULL,eps->nds,NULL,eps->defl,y,NULL,NULL,NULL);CHKERRQ(ierr);
         ierr = VecResetArray(x);CHKERRQ(ierr);
         ierr = VecResetArray(y);CHKERRQ(ierr);
       }
@@ -218,7 +222,7 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
       eps->its = eps->its + 1;
       if (eps->its >= eps->max_it || eps->nconv >= eps->nev) lflag = 5;
       break;
-    case 2:  
+    case 2:
       /* compute v = B u */
       for (i=0;i<nvopu;i++) {
         ierr = VecPlaceArray(x,blz->u+i*eps->nloc);CHKERRQ(ierr);
@@ -228,24 +232,24 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
         ierr = VecResetArray(y);CHKERRQ(ierr);
       }
       break;
-    case 3:  
+    case 3:
       /* update shift */
       ierr = PetscInfo1(eps,"Factorization update (sigma=%g)\n",sigma);CHKERRQ(ierr);
-      ierr = STSetShift(eps->OP,sigma);CHKERRQ(ierr);
-      ierr = STGetKSP(eps->OP,&ksp);CHKERRQ(ierr);
+      ierr = STSetShift(eps->st,sigma);CHKERRQ(ierr);
+      ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
       ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
       ierr = PCFactorGetMatrix(pc,&A);CHKERRQ(ierr);
-      ierr = MatGetInertia(A,&nn,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
-      nneig = PetscBLASIntCast(nn);
+      ierr = MatGetInertia(A,&nn,NULL,NULL);CHKERRQ(ierr);
+      ierr = PetscBLASIntCast(nn,&nneig);CHKERRQ(ierr);
       break;
-    case 4:  
+    case 4:
       /* copy the initial vector */
       ierr = VecPlaceArray(x,blz->v);CHKERRQ(ierr);
-      ierr = EPSGetStartVector(eps,0,x,PETSC_NULL);CHKERRQ(ierr);
+      ierr = EPSGetStartVector(eps,0,x,NULL);CHKERRQ(ierr);
       ierr = VecResetArray(x);CHKERRQ(ierr);
       break;
     }
-    
+
   } while (lflag > 0);
 
   ierr = VecRestoreArray(eps->V[0],&pV);CHKERRQ(ierr);
@@ -257,24 +261,24 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
     eps->eigr[i]=blz->eig[i];
   }
 
-  if (lflag!=0) { 
+  if (lflag!=0) {
     char msg[2048] = "";
     for (i = 0; i < 33; i++) {
       if (blz->istor[15] & (1 << i)) PetscStrcat(msg,blzpack_error[i]);
     }
-    SETERRQ2(((PetscObject)eps)->comm,PETSC_ERR_LIB,"Error in BLZPACK (code=%d): '%s'",blz->istor[15],msg); 
+    SETERRQ2(PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"Error in BLZPACK (code=%d): '%s'",blz->istor[15],msg);
   }
   ierr = VecDestroy(&x);CHKERRQ(ierr);
   ierr = VecDestroy(&y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSBackTransform_BLZPACK"
 PetscErrorCode EPSBackTransform_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_BLZPACK    *blz = (EPS_BLZPACK *)eps->data;
+  EPS_BLZPACK    *blz = (EPS_BLZPACK*)eps->data;
 
   PetscFunctionBegin;
   if (!blz->slice && !eps->isgeneralized) {
@@ -283,12 +287,12 @@ PetscErrorCode EPSBackTransform_BLZPACK(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSReset_BLZPACK"
 PetscErrorCode EPSReset_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_BLZPACK    *blz = (EPS_BLZPACK *)eps->data;
+  EPS_BLZPACK    *blz = (EPS_BLZPACK*)eps->data;
 
   PetscFunctionBegin;
   ierr = PetscFree(blz->istor);CHKERRQ(ierr);
@@ -300,7 +304,7 @@ PetscErrorCode EPSReset_BLZPACK(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSDestroy_BLZPACK"
 PetscErrorCode EPSDestroy_BLZPACK(EPS eps)
 {
@@ -308,12 +312,12 @@ PetscErrorCode EPSDestroy_BLZPACK(EPS eps)
 
   PetscFunctionBegin;
   ierr = PetscFree(eps->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSBlzpackSetBlockSize_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSBlzpackSetNSteps_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSBlzpackSetBlockSize_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSBlzpackSetNSteps_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSView_BLZPACK"
 PetscErrorCode EPSView_BLZPACK(EPS eps,PetscViewer viewer)
 {
@@ -322,24 +326,23 @@ PetscErrorCode EPSView_BLZPACK(EPS eps,PetscViewer viewer)
   PetscBool      isascii;
 
   PetscFunctionBegin;
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
-  if (!isascii) {
-    SETERRQ1(((PetscObject)eps)->comm,1,"Viewer type %s not supported for EPSBLZPACK",((PetscObject)viewer)->type_name);
-  }
-  ierr = PetscViewerASCIIPrintf(viewer,"  BLZPACK: block size=%d\n",blz->block_size);CHKERRQ(ierr);
-  ierr = PetscViewerASCIIPrintf(viewer,"  BLZPACK: maximum number of steps per run=%d\n",blz->nsteps);CHKERRQ(ierr);
-  if (blz->slice) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  BLZPACK: computational interval [%f,%f]\n",eps->inta,eps->intb);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  if (isascii) {
+    ierr = PetscViewerASCIIPrintf(viewer,"  BLZPACK: block size=%d\n",blz->block_size);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  BLZPACK: maximum number of steps per run=%d\n",blz->nsteps);CHKERRQ(ierr);
+    if (blz->slice) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  BLZPACK: computational interval [%f,%f]\n",eps->inta,eps->intb);CHKERRQ(ierr);
+    }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetFromOptions_BLZPACK"
 PetscErrorCode EPSSetFromOptions_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_BLZPACK    *blz = (EPS_BLZPACK *)eps->data;
+  EPS_BLZPACK    *blz = (EPS_BLZPACK*)eps->data;
   PetscInt       bs,n;
   PetscBool      flg;
 
@@ -348,33 +351,36 @@ PetscErrorCode EPSSetFromOptions_BLZPACK(EPS eps)
 
   bs = blz->block_size;
   ierr = PetscOptionsInt("-eps_blzpack_block_size","Block size","EPSBlzpackSetBlockSize",bs,&bs,&flg);CHKERRQ(ierr);
-  if (flg) {ierr = EPSBlzpackSetBlockSize(eps,bs);CHKERRQ(ierr);}
+  if (flg) {
+    ierr = EPSBlzpackSetBlockSize(eps,bs);CHKERRQ(ierr);
+  }
 
   n = blz->nsteps;
   ierr = PetscOptionsInt("-eps_blzpack_nsteps","Number of steps","EPSBlzpackSetNSteps",n,&n,&flg);CHKERRQ(ierr);
-  if (flg) {ierr = EPSBlzpackSetNSteps(eps,n);CHKERRQ(ierr);}
+  if (flg) {
+    ierr = EPSBlzpackSetNSteps(eps,n);CHKERRQ(ierr);
+  }
 
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSBlzpackSetBlockSize_BLZPACK"
-PetscErrorCode EPSBlzpackSetBlockSize_BLZPACK(EPS eps,PetscInt bs)
+static PetscErrorCode EPSBlzpackSetBlockSize_BLZPACK(EPS eps,PetscInt bs)
 {
   EPS_BLZPACK *blz = (EPS_BLZPACK*)eps->data;;
 
   PetscFunctionBegin;
   if (bs == PETSC_DEFAULT) blz->block_size = 3;
-  else if (bs <= 0) { 
-    SETERRQ(((PetscObject)eps)->comm,1,"Incorrect block size"); 
-  } else blz->block_size = PetscBLASIntCast(bs);
+  else if (bs <= 0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Block size must be positive");
+  else {
+    ierr = PetscBLASIntCast(bs,&blz->block_size);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSBlzpackSetBlockSize"
 /*@
    EPSBlzpackSetBlockSize - Sets the block size for the BLZPACK package.
@@ -401,21 +407,21 @@ PetscErrorCode EPSBlzpackSetBlockSize(EPS eps,PetscInt bs)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSBlzpackSetNSteps_BLZPACK"
-PetscErrorCode EPSBlzpackSetNSteps_BLZPACK(EPS eps,PetscInt nsteps)
+static PetscErrorCode EPSBlzpackSetNSteps_BLZPACK(EPS eps,PetscInt nsteps)
 {
   EPS_BLZPACK *blz = (EPS_BLZPACK*)eps->data;
 
   PetscFunctionBegin;
   if (nsteps == PETSC_DEFAULT) blz->nsteps = 0;
-  else { blz->nsteps = PetscBLASIntCast(nsteps); }
+  else {
+    ierr = PetscBLASIntCast(nsteps,&blz->nsteps);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSBlzpackSetNSteps"
 /*@
    EPSBlzpackSetNSteps - Sets the maximum number of steps per run for the BLZPACK
@@ -444,10 +450,9 @@ PetscErrorCode EPSBlzpackSetNSteps(EPS eps,PetscInt nsteps)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSCreate_BLZPACK"
-PetscErrorCode EPSCreate_BLZPACK(EPS eps)
+PETSC_EXTERN PetscErrorCode EPSCreate_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_BLZPACK    *blzpack;
@@ -462,13 +467,8 @@ PetscErrorCode EPSCreate_BLZPACK(EPS eps)
   eps->ops->view                 = EPSView_BLZPACK;
   eps->ops->backtransform        = EPSBackTransform_BLZPACK;
   eps->ops->computevectors       = EPSComputeVectors_Default;
-
-  blzpack->block_size = 3;
-  blzpack->slice = 0;
-  blzpack->nsteps = 0;
-
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSBlzpackSetBlockSize_C","EPSBlzpackSetBlockSize_BLZPACK",EPSBlzpackSetBlockSize_BLZPACK);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSBlzpackSetNSteps_C","EPSBlzpackSetNSteps_BLZPACK",EPSBlzpackSetNSteps_BLZPACK);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSBlzpackSetBlockSize_C",EPSBlzpackSetBlockSize_BLZPACK);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSBlzpackSetNSteps_C",EPSBlzpackSetNSteps_BLZPACK);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
+
