@@ -3,19 +3,19 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
- 
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
- 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
- 
+
    You  should have received a copy of the GNU Lesser General  Public  License
    along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -52,27 +52,22 @@ typedef struct {
     matrices
 */
 
-#if defined(PETSC_USE_SINGLE) 
+#if defined(PETSC_USE_SINGLE)
 /*
    For these machines we must call the single precision Fortran version
 */
 #define BLZpack_ SLEPC_BLZPACK(blzdrs,BLZDRS)
-#else 
+#else
 #define BLZpack_ SLEPC_BLZPACK(blzdrd,BLZDRD)
 #endif
 
 #define BLZistorr_ SLEPC_BLZPACK(istorr,ISTORR)
 #define BLZrstorr_ SLEPC_BLZPACK(rstorr,RSTORR)
 
-EXTERN_C_BEGIN
+PETSC_EXTERN void BLZpack_(PetscBLASInt*,PetscReal*,PetscScalar*,PetscBLASInt*,PetscScalar*,PetscScalar*,PetscBLASInt*,PetscBLASInt*,PetscScalar*,PetscScalar*);
 
-extern void	 BLZpack_(PetscBLASInt*,PetscReal*,PetscScalar*,PetscBLASInt*,PetscScalar*,
-        		  PetscScalar*,PetscBLASInt*,PetscBLASInt*,PetscScalar*,PetscScalar*);
-
-extern PetscBLASInt BLZistorr_(PetscBLASInt*,const char*,int);
-extern PetscReal BLZrstorr_(PetscReal*,char*,int);
-
-EXTERN_C_END
+PETSC_EXTERN PetscBLASInt BLZistorr_(PetscBLASInt*,const char*,int);
+PETSC_EXTERN PetscReal BLZrstorr_(PetscReal*,char*,int);
 
 #endif
 
