@@ -1,4 +1,4 @@
-/*                       
+/*
 
    SLEPc eigensolver: "arnoldi"
 
@@ -10,24 +10,24 @@
 
    References:
 
-       [1] "Arnoldi Methods in SLEPc", SLEPc Technical Report STR-4, 
+       [1] "Arnoldi Methods in SLEPc", SLEPc Technical Report STR-4,
            available at http://www.grycap.upv.es/slepc.
 
    Last update: Feb 2009
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -35,7 +35,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/epsimpl.h>                /*I "slepceps.h" I*/
+#include <slepc-private/epsimpl.h>                /*I "slepceps.h" I*/
 #include <slepcblaslapack.h>
 
 PetscErrorCode EPSSolve_Arnoldi(EPS);
@@ -44,7 +44,7 @@ typedef struct {
   PetscBool delayed;
 } EPS_ARNOLDI;
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetUp_Arnoldi"
 PetscErrorCode EPSSetUp_Arnoldi(EPS eps)
 {
@@ -52,45 +52,44 @@ PetscErrorCode EPSSetUp_Arnoldi(EPS eps)
 
   PetscFunctionBegin;
   if (eps->ncv) { /* ncv set */
-    if (eps->ncv<eps->nev) SETERRQ(((PetscObject)eps)->comm,1,"The value of ncv must be at least nev"); 
-  }
-  else if (eps->mpd) { /* mpd set */
+    if (eps->ncv<eps->nev) SETERRQ(PetscObjectComm((PetscObject)eps),1,"The value of ncv must be at least nev");
+  } else if (eps->mpd) { /* mpd set */
     eps->ncv = PetscMin(eps->n,eps->nev+eps->mpd);
-  }
-  else { /* neither set: defaults depend on nev being small or large */
+  } else { /* neither set: defaults depend on nev being small or large */
     if (eps->nev<500) eps->ncv = PetscMin(eps->n,PetscMax(2*eps->nev,eps->nev+15));
-    else { eps->mpd = 500; eps->ncv = PetscMin(eps->n,eps->nev+eps->mpd); }
+    else {
+      eps->mpd = 500;
+      eps->ncv = PetscMin(eps->n,eps->nev+eps->mpd);
+    }
   }
   if (!eps->mpd) eps->mpd = eps->ncv;
-  if (eps->ncv>eps->nev+eps->mpd) SETERRQ(((PetscObject)eps)->comm,1,"The value of ncv must not be larger than nev+mpd"); 
+  if (eps->ncv>eps->nev+eps->mpd) SETERRQ(PetscObjectComm((PetscObject)eps),1,"The value of ncv must not be larger than nev+mpd");
   if (!eps->max_it) eps->max_it = PetscMax(100,2*eps->n/eps->ncv);
-  if (!eps->which) { ierr = EPSDefaultSetWhich(eps);CHKERRQ(ierr); }
-  if (eps->ishermitian && (eps->which==EPS_LARGEST_IMAGINARY || eps->which==EPS_SMALLEST_IMAGINARY))
-    SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->which");
+  if (!eps->which) { ierr = EPSSetWhichEigenpairs_Default(eps);CHKERRQ(ierr); }
+  if (eps->ishermitian && (eps->which==EPS_LARGEST_IMAGINARY || eps->which==EPS_SMALLEST_IMAGINARY)) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->which");
 
   if (!eps->extraction) {
     ierr = EPSSetExtraction(eps,EPS_RITZ);CHKERRQ(ierr);
   }
+  if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs not supported in this solver");
 
   ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
-  ierr = PetscFree(eps->T);CHKERRQ(ierr);
-  ierr = PetscMalloc(eps->ncv*eps->ncv*sizeof(PetscScalar),&eps->T);CHKERRQ(ierr);
-  if (eps->leftvecs) {
-    ierr = PetscFree(eps->Tl);CHKERRQ(ierr);
-    ierr = PetscMalloc(eps->ncv*eps->ncv*sizeof(PetscScalar),&eps->Tl);CHKERRQ(ierr);
-    ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr);
-    ierr = EPSDefaultGetWork(eps,2);CHKERRQ(ierr);
-  } else {
-    ierr = EPSDefaultGetWork(eps,1);CHKERRQ(ierr);
+  ierr = DSSetType(eps->ds,DSNHEP);CHKERRQ(ierr);
+  if (eps->extraction==EPS_REFINED || eps->extraction==EPS_REFINED_HARMONIC) {
+    ierr = DSSetRefined(eps->ds,PETSC_TRUE);CHKERRQ(ierr);
   }
+  ierr = DSSetExtraRow(eps->ds,PETSC_TRUE);CHKERRQ(ierr);
+  ierr = DSAllocate(eps->ds,eps->ncv+1);CHKERRQ(ierr);
+  ierr = EPSSetWorkVecs(eps,1);CHKERRQ(ierr);
 
   /* dispatch solve method */
-  if (eps->leftvecs) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Left vectors not supported in this solver");
+  if (eps->leftvecs) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Left vectors not supported in this solver");
+  if (eps->isgeneralized && eps->ishermitian && !eps->ispositive) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Requested method does not work for indefinite problems");
   eps->ops->solve = EPSSolve_Arnoldi;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSDelayedArnoldi"
 /*
    EPSDelayedArnoldi - This function is equivalent to EPSBasicArnoldi but
@@ -108,28 +107,30 @@ PetscErrorCode EPSDelayedArnoldi(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,Pets
 
   PetscFunctionBegin;
   if (m<=100) lhh = shh;
-  else { ierr = PetscMalloc(m*sizeof(PetscScalar),&lhh);CHKERRQ(ierr); }
+  else {
+    ierr = PetscMalloc(m*sizeof(PetscScalar),&lhh);CHKERRQ(ierr);
+  }
   ierr = VecDuplicate(f,&u);CHKERRQ(ierr);
   ierr = VecDuplicate(f,&t);CHKERRQ(ierr);
 
   for (j=k;j<m;j++) {
-    ierr = STApply(eps->OP,V[j],f);CHKERRQ(ierr);
-    ierr = IPOrthogonalize(eps->ip,0,PETSC_NULL,eps->nds,PETSC_NULL,eps->DS,f,PETSC_NULL,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+    ierr = STApply(eps->st,V[j],f);CHKERRQ(ierr);
+    ierr = IPOrthogonalize(eps->ip,0,NULL,eps->nds,NULL,eps->defl,f,NULL,NULL,NULL);CHKERRQ(ierr);
 
     ierr = IPMInnerProductBegin(eps->ip,f,j+1,V,H+ldh*j);CHKERRQ(ierr);
-    if (j>k) { 
+    if (j>k) {
       ierr = IPMInnerProductBegin(eps->ip,V[j],j,V,lhh);CHKERRQ(ierr);
-      ierr = IPInnerProductBegin(eps->ip,V[j],V[j],&dot);CHKERRQ(ierr); 
+      ierr = IPInnerProductBegin(eps->ip,V[j],V[j],&dot);CHKERRQ(ierr);
     }
     if (j>k+1) {
-      ierr = IPNormBegin(eps->ip,u,&norm2);CHKERRQ(ierr); 
+      ierr = IPNormBegin(eps->ip,u,&norm2);CHKERRQ(ierr);
       ierr = VecDotBegin(u,V[j-2],&dot2);CHKERRQ(ierr);
     }
-    
+
     ierr = IPMInnerProductEnd(eps->ip,f,j+1,V,H+ldh*j);CHKERRQ(ierr);
-    if (j>k) { 
+    if (j>k) {
       ierr = IPMInnerProductEnd(eps->ip,V[j],j,V,lhh);CHKERRQ(ierr);
-      ierr = IPInnerProductEnd(eps->ip,V[j],V[j],&dot);CHKERRQ(ierr); 
+      ierr = IPInnerProductEnd(eps->ip,V[j],V[j],&dot);CHKERRQ(ierr);
     }
     if (j>k+1) {
       ierr = IPNormEnd(eps->ip,u,&norm2);CHKERRQ(ierr);
@@ -145,13 +146,13 @@ PetscErrorCode EPSDelayedArnoldi(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,Pets
         PetscFunctionReturn(0);
       }
     }
-    
-    if (j>k) {      
+
+    if (j>k) {
       norm1 = PetscSqrtReal(PetscRealPart(dot));
       for (i=0;i<j;i++)
         H[ldh*j+i] = H[ldh*j+i]/norm1;
       H[ldh*j+j] = H[ldh*j+j]/dot;
-      
+
       ierr = VecCopy(V[j],t);CHKERRQ(ierr);
       ierr = VecScale(V[j],1.0/norm1);CHKERRQ(ierr);
       ierr = VecScale(f,1.0/norm1);CHKERRQ(ierr);
@@ -183,7 +184,7 @@ PetscErrorCode EPSDelayedArnoldi(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,Pets
   H[ldh*(m-2)+m-1] = norm2;
 
   ierr = IPMInnerProduct(eps->ip,f,m,V,lhh);CHKERRQ(ierr);
-  
+
   ierr = SlepcVecMAXPBY(f,1.0,-1.0,m,lhh,V);CHKERRQ(ierr);
   for (i=0;i<m;i++)
     H[ldh*(m-1)+i] += lhh[i];
@@ -191,14 +192,14 @@ PetscErrorCode EPSDelayedArnoldi(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,Pets
   ierr = IPNorm(eps->ip,f,beta);CHKERRQ(ierr);
   ierr = VecScale(f,1.0 / *beta);CHKERRQ(ierr);
   *breakdown = PETSC_FALSE;
-  
+
   if (m>100) { ierr = PetscFree(lhh);CHKERRQ(ierr); }
   ierr = VecDestroy(&u);CHKERRQ(ierr);
   ierr = VecDestroy(&t);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSDelayedArnoldi1"
 /*
    EPSDelayedArnoldi1 - This function is similar to EPSDelayedArnoldi1,
@@ -213,27 +214,27 @@ PetscErrorCode EPSDelayedArnoldi1(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,Pet
 
   PetscFunctionBegin;
   for (j=k;j<m;j++) {
-    ierr = STApply(eps->OP,V[j],f);CHKERRQ(ierr);
-    ierr = IPOrthogonalize(eps->ip,0,PETSC_NULL,eps->nds,PETSC_NULL,eps->DS,f,PETSC_NULL,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+    ierr = STApply(eps->st,V[j],f);CHKERRQ(ierr);
+    ierr = IPOrthogonalize(eps->ip,0,NULL,eps->nds,NULL,eps->defl,f,NULL,NULL,NULL);CHKERRQ(ierr);
 
     ierr = IPMInnerProductBegin(eps->ip,f,j+1,V,H+ldh*j);CHKERRQ(ierr);
-    if (j>k) { 
-      ierr = IPInnerProductBegin(eps->ip,V[j],V[j],&dot);CHKERRQ(ierr); 
+    if (j>k) {
+      ierr = IPInnerProductBegin(eps->ip,V[j],V[j],&dot);CHKERRQ(ierr);
     }
-    
+
     ierr = IPMInnerProductEnd(eps->ip,f,j+1,V,H+ldh*j);CHKERRQ(ierr);
-    if (j>k) { 
-      ierr = IPInnerProductEnd(eps->ip,V[j],V[j],&dot);CHKERRQ(ierr); 
+    if (j>k) {
+      ierr = IPInnerProductEnd(eps->ip,V[j],V[j],&dot);CHKERRQ(ierr);
     }
-    
-    if (j>k) {      
+
+    if (j>k) {
       norm = PetscSqrtReal(PetscRealPart(dot));
       ierr = VecScale(V[j],1.0/norm);CHKERRQ(ierr);
       H[ldh*(j-1)+j] = norm;
 
       for (i=0;i<j;i++)
         H[ldh*j+i] = H[ldh*j+i]/norm;
-      H[ldh*j+j] = H[ldh*j+j]/dot;      
+      H[ldh*j+j] = H[ldh*j+j]/dot;
       ierr = VecScale(f,1.0/norm);CHKERRQ(ierr);
     }
 
@@ -250,175 +251,72 @@ PetscErrorCode EPSDelayedArnoldi1(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,Pet
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "EPSProjectedArnoldi"
-/*
-   EPSProjectedArnoldi - Solves the projected eigenproblem.
-
-   On input:
-     S is the projected matrix (leading dimension is lds)
-
-   On output:
-     S has (real) Schur form with diagonal blocks sorted appropriately
-     Q contains the corresponding Schur vectors (order n, leading dimension n)
-*/
-PetscErrorCode EPSProjectedArnoldi(EPS eps,PetscScalar *S,PetscInt lds,PetscScalar *Q,PetscInt n)
-{
-  PetscErrorCode ierr;
-  PetscInt       i;
-
-  PetscFunctionBegin;
-  /* Initialize orthogonal matrix */
-  ierr = PetscMemzero(Q,n*n*sizeof(PetscScalar));CHKERRQ(ierr);
-  for (i=0;i<n;i++) 
-    Q[i*(n+1)] = 1.0;
-  /* Reduce S to (quasi-)triangular form, S <- Q S Q' */
-  ierr = EPSDenseSchur(n,eps->nconv,S,lds,Q,eps->eigr,eps->eigi);CHKERRQ(ierr);
-  /* Sort the remaining columns of the Schur form */
-  ierr = EPSSortDenseSchur(eps,n,eps->nconv,S,lds,Q,eps->eigr,eps->eigi);CHKERRQ(ierr);    
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
-#define __FUNCT__ "EPSUpdateVectors"
-/*
-   EPSUpdateVectors - Computes approximate Schur vectors (or eigenvectors) by
-   either Ritz extraction (U=U*Q) or refined Ritz extraction 
-
-   On input:
-     n is the size of U
-     U is the orthogonal basis of the subspace used for projecting
-     s is the index of the first vector computed
-     e+1 is the index of the last vector computed
-     Q contains the corresponding Schur vectors of the projected matrix (size n x n, leading dimension ldq)
-     H is the (extended) projected matrix (size n+1 x n, leading dimension ldh)
-
-   On output:
-     v is the resulting vector
-*/
-PetscErrorCode EPSUpdateVectors(EPS eps,PetscInt n_,Vec *U,PetscInt s,PetscInt e,PetscScalar *Q,PetscInt ldq,PetscScalar *H,PetscInt ldh_)
-{
-#if defined(PETSC_MISSING_LAPACK_GESVD) 
-  SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"GESVD - Lapack routine is unavailable.");
-#else
-  PetscErrorCode ierr;
-  PetscBool      isrefined;
-  PetscInt       i,j,k;
-  PetscBLASInt   n1,lwork,idummy=1,info,n=n_,ldh=ldh_;
-  PetscScalar    *B,sdummy,*work;
-  PetscReal      *sigma;
-
-  PetscFunctionBegin;
-  isrefined = (eps->extraction==EPS_REFINED || eps->extraction==EPS_REFINED_HARMONIC)?PETSC_TRUE:PETSC_FALSE;
-  if (isrefined) {
-    /* Refined Ritz extraction */
-    n1 = n+1;
-    ierr = PetscMalloc(n1*n*sizeof(PetscScalar),&B);CHKERRQ(ierr);
-    ierr = PetscMalloc(6*n*sizeof(PetscReal),&sigma);CHKERRQ(ierr);
-    lwork = 10*n;
-    ierr = PetscMalloc(lwork*sizeof(PetscScalar),&work);CHKERRQ(ierr);
-    
-    for (k=s;k<e;k++) {
-      /* copy H to B */
-      for (i=0;i<=n;i++) {
-        for (j=0;j<n;j++) {
-          B[i+j*n1] = H[i+j*ldh];
-        }
-      }
-      /* subtract ritz value from diagonal of B^ */
-      for (i=0;i<n;i++) {
-        B[i+i*n1] -= eps->eigr[k];  /* MISSING: complex case */
-      }
-      /* compute SVD of [H-mu*I] */
-  #if !defined(PETSC_USE_COMPLEX)
-      LAPACKgesvd_("N","O",&n1,&n,B,&n1,sigma,&sdummy,&idummy,&sdummy,&idummy,work,&lwork,&info);
-  #else
-      LAPACKgesvd_("N","O",&n1,&n,B,&n1,sigma,&sdummy,&idummy,&sdummy,&idummy,work,&lwork,sigma+n,&info);
-  #endif
-      if (info) SETERRQ1(((PetscObject)eps)->comm,PETSC_ERR_LIB,"Error in Lapack xGESVD %d",info);
-      /* the smallest singular value is the new error estimate */
-      eps->errest[k] = sigma[n-1];
-      /* update vector with right singular vector associated to smallest singular value */
-      for (i=0;i<n;i++)
-        Q[k*ldq+i] = B[n-1+i*n1];
-    }
-    /* free workspace */
-    ierr = PetscFree(B);CHKERRQ(ierr);
-    ierr = PetscFree(sigma);CHKERRQ(ierr);
-    ierr = PetscFree(work);CHKERRQ(ierr);
-  }
-  /* Ritz extraction: v = U*q */
-  ierr = SlepcUpdateVectors(n_,U,s,e,Q,ldq,PETSC_FALSE);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-#endif
-}
-
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSolve_Arnoldi"
 PetscErrorCode EPSSolve_Arnoldi(EPS eps)
 {
   PetscErrorCode     ierr;
-  PetscInt           i,k,lwork,nv;
+  PetscInt           k,nv,ld;
   Vec                f=eps->work[0];
-  PetscScalar        *H=eps->T,*U,*g,*work,*Hcopy;
-  PetscReal          beta,gnorm,corrf=1.0;
-  PetscBool          breakdown;
+  PetscScalar        *H,*U,*X;
+  PetscReal          beta,gamma=1.0;
+  PetscBool          breakdown,harmonic,refined;
   IPOrthogRefineType orthog_ref;
-  EPS_ARNOLDI        *arnoldi = (EPS_ARNOLDI *)eps->data;
+  EPS_ARNOLDI        *arnoldi = (EPS_ARNOLDI*)eps->data;
 
   PetscFunctionBegin;
-  ierr = PetscMemzero(eps->T,eps->ncv*eps->ncv*sizeof(PetscScalar));CHKERRQ(ierr);
-  ierr = PetscMalloc(eps->ncv*eps->ncv*sizeof(PetscScalar),&U);CHKERRQ(ierr);
-  lwork = PetscMax((eps->ncv+1)*eps->ncv,7*eps->ncv);
-  ierr = PetscMalloc(lwork*sizeof(PetscScalar),&work);CHKERRQ(ierr);
-  if (eps->extraction==EPS_HARMONIC || eps->extraction==EPS_REFINED_HARMONIC) {
-    ierr = PetscMalloc(eps->ncv*sizeof(PetscScalar),&g);CHKERRQ(ierr);
-  }
-  if (eps->extraction==EPS_REFINED || eps->extraction==EPS_REFINED_HARMONIC) {
-    ierr = PetscMalloc((eps->ncv+1)*eps->ncv*sizeof(PetscScalar),&Hcopy);CHKERRQ(ierr);
-  }
-  
-  ierr = IPGetOrthogonalization(eps->ip,PETSC_NULL,&orthog_ref,PETSC_NULL);CHKERRQ(ierr);
+  ierr = DSGetLeadingDimension(eps->ds,&ld);CHKERRQ(ierr);
+  ierr = DSGetRefined(eps->ds,&refined);CHKERRQ(ierr);
+  harmonic = (eps->extraction==EPS_HARMONIC || eps->extraction==EPS_REFINED_HARMONIC)?PETSC_TRUE:PETSC_FALSE;
+  ierr = IPGetOrthogonalization(eps->ip,NULL,&orthog_ref,NULL);CHKERRQ(ierr);
 
   /* Get the starting Arnoldi vector */
-  ierr = EPSGetStartVector(eps,0,eps->V[0],PETSC_NULL);CHKERRQ(ierr);
-  
+  ierr = EPSGetStartVector(eps,0,eps->V[0],NULL);CHKERRQ(ierr);
+
   /* Restart loop */
   while (eps->reason == EPS_CONVERGED_ITERATING) {
     eps->its++;
 
     /* Compute an nv-step Arnoldi factorization */
     nv = PetscMin(eps->nconv+eps->mpd,eps->ncv);
+    ierr = DSSetDimensions(eps->ds,nv,0,eps->nconv,0);CHKERRQ(ierr);
+    ierr = DSGetArray(eps->ds,DS_MAT_A,&H);CHKERRQ(ierr);
     if (!arnoldi->delayed) {
-      ierr = EPSBasicArnoldi(eps,PETSC_FALSE,H,eps->ncv,eps->V,eps->nconv,&nv,f,&beta,&breakdown);CHKERRQ(ierr);
+      ierr = EPSBasicArnoldi(eps,PETSC_FALSE,H,ld,eps->V,eps->nconv,&nv,f,&beta,&breakdown);CHKERRQ(ierr);
+      H[nv+(nv-1)*ld] = beta;
     } else if (orthog_ref == IP_ORTHOG_REFINE_NEVER) {
-      ierr = EPSDelayedArnoldi1(eps,H,eps->ncv,eps->V,eps->nconv,&nv,f,&beta,&breakdown);CHKERRQ(ierr);
+      ierr = EPSDelayedArnoldi1(eps,H,ld,eps->V,eps->nconv,&nv,f,&beta,&breakdown);CHKERRQ(ierr);
     } else {
-      ierr = EPSDelayedArnoldi(eps,H,eps->ncv,eps->V,eps->nconv,&nv,f,&beta,&breakdown);CHKERRQ(ierr);
+      ierr = EPSDelayedArnoldi(eps,H,ld,eps->V,eps->nconv,&nv,f,&beta,&breakdown);CHKERRQ(ierr);
+    }
+    ierr = DSRestoreArray(eps->ds,DS_MAT_A,&H);CHKERRQ(ierr);
+    ierr = DSSetState(eps->ds,DS_STATE_INTERMEDIATE);CHKERRQ(ierr);
+
+    /* Compute translation of Krylov decomposition if harmonic extraction used */
+    if (harmonic) {
+      ierr = DSTranslateHarmonic(eps->ds,eps->target,beta,PETSC_FALSE,NULL,&gamma);CHKERRQ(ierr);
     }
 
-    if (eps->extraction==EPS_REFINED || eps->extraction==EPS_REFINED_HARMONIC) {
-      ierr = PetscMemcpy(Hcopy,H,eps->ncv*eps->ncv*sizeof(PetscScalar));CHKERRQ(ierr);
-      for (i=0;i<nv-1;i++) Hcopy[nv+i*eps->ncv] = 0.0; 
-      Hcopy[nv+(nv-1)*eps->ncv] = beta;
+    /* Solve projected problem */
+    ierr = DSSolve(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
+    ierr = DSSort(eps->ds,eps->eigr,eps->eigi,NULL,NULL,NULL);CHKERRQ(ierr);
+    ierr = DSUpdateExtraRow(eps->ds);CHKERRQ(ierr);
+
+    /* Check convergence */
+    ierr = EPSKrylovConvergence(eps,PETSC_FALSE,eps->nconv,nv-eps->nconv,eps->V,nv,beta,gamma,&k);CHKERRQ(ierr);
+    if (refined) {
+      ierr = DSGetArray(eps->ds,DS_MAT_X,&X);CHKERRQ(ierr);
+      ierr = SlepcVecMAXPBY(eps->V[k],0.0,1.0,nv,X+k*ld,eps->V);CHKERRQ(ierr);
+      ierr = DSRestoreArray(eps->ds,DS_MAT_X,&X);CHKERRQ(ierr);
+      ierr = DSGetArray(eps->ds,DS_MAT_Q,&U);CHKERRQ(ierr);
+      ierr = SlepcUpdateVectors(nv,eps->V,eps->nconv,PetscMin(k,nv),U,ld,PETSC_FALSE);CHKERRQ(ierr);
+      ierr = DSRestoreArray(eps->ds,DS_MAT_Q,&U);CHKERRQ(ierr);
+      ierr = IPOrthogonalize(eps->ip,0,NULL,k,NULL,eps->V,eps->V[k],NULL,NULL,NULL);CHKERRQ(ierr);
+    } else {
+      ierr = DSGetArray(eps->ds,DS_MAT_Q,&U);CHKERRQ(ierr);
+      ierr = SlepcUpdateVectors(nv,eps->V,eps->nconv,PetscMin(k+1,nv),U,ld,PETSC_FALSE);CHKERRQ(ierr);
+      ierr = DSRestoreArray(eps->ds,DS_MAT_Q,&U);CHKERRQ(ierr);
     }
-
-    /* Compute translation of Krylov decomposition if harmonic extraction used */ 
-    if (eps->extraction==EPS_HARMONIC || eps->extraction==EPS_REFINED_HARMONIC) {
-      ierr = EPSTranslateHarmonic(nv,H,eps->ncv,eps->target,(PetscScalar)beta,g,work);CHKERRQ(ierr);
-      gnorm = 0.0;
-      for (i=0;i<nv;i++)
-        gnorm = gnorm + PetscRealPart(g[i]*PetscConj(g[i]));
-      corrf = PetscSqrtReal(1.0+gnorm);
-    }
-
-    /* Solve projected problem */ 
-    ierr = EPSProjectedArnoldi(eps,H,eps->ncv,U,nv);CHKERRQ(ierr);
-
-    /* Check convergence */ 
-    ierr = EPSKrylovConvergence(eps,PETSC_FALSE,eps->trackall,eps->nconv,nv-eps->nconv,H,eps->ncv,U,eps->V,nv,beta,corrf,&k,work);CHKERRQ(ierr);
-
-    ierr = EPSUpdateVectors(eps,nv,eps->V,eps->nconv,PetscMin(k+1,nv),U,nv,Hcopy,eps->ncv);CHKERRQ(ierr);
     eps->nconv = k;
 
     ierr = EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest,nv);CHKERRQ(ierr);
@@ -433,25 +331,21 @@ PetscErrorCode EPSSolve_Arnoldi(EPS eps)
     if (eps->its >= eps->max_it) eps->reason = EPS_DIVERGED_ITS;
     if (eps->nconv >= eps->nev) eps->reason = EPS_CONVERGED_TOL;
   }
-  
-  ierr = PetscFree(U);CHKERRQ(ierr);
-  ierr = PetscFree(work);CHKERRQ(ierr);
-  if (eps->extraction==EPS_HARMONIC || eps->extraction==EPS_REFINED_HARMONIC) {
-    ierr = PetscFree(g);CHKERRQ(ierr);
-  }
-  if (eps->extraction==EPS_REFINED || eps->extraction==EPS_REFINED_HARMONIC) {
-    ierr = PetscFree(Hcopy);CHKERRQ(ierr);
-  }
+
+  /* truncate Schur decomposition and change the state to raw so that
+     PSVectors() computes eigenvectors from scratch */
+  ierr = DSSetDimensions(eps->ds,eps->nconv,0,0,0);CHKERRQ(ierr);
+  ierr = DSSetState(eps->ds,DS_STATE_RAW);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetFromOptions_Arnoldi"
 PetscErrorCode EPSSetFromOptions_Arnoldi(EPS eps)
 {
   PetscErrorCode ierr;
   PetscBool      set,val;
-  EPS_ARNOLDI    *arnoldi = (EPS_ARNOLDI *)eps->data;
+  EPS_ARNOLDI    *arnoldi = (EPS_ARNOLDI*)eps->data;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead("EPS Arnoldi Options");CHKERRQ(ierr);
@@ -463,24 +357,22 @@ PetscErrorCode EPSSetFromOptions_Arnoldi(EPS eps)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSArnoldiSetDelayed_Arnoldi"
-PetscErrorCode EPSArnoldiSetDelayed_Arnoldi(EPS eps,PetscBool delayed)
+static PetscErrorCode EPSArnoldiSetDelayed_Arnoldi(EPS eps,PetscBool delayed)
 {
-  EPS_ARNOLDI *arnoldi = (EPS_ARNOLDI *)eps->data;
+  EPS_ARNOLDI *arnoldi = (EPS_ARNOLDI*)eps->data;
 
   PetscFunctionBegin;
   arnoldi->delayed = delayed;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSArnoldiSetDelayed"
 /*@
-   EPSArnoldiSetDelayed - Activates or deactivates delayed reorthogonalization 
-   in the Arnoldi iteration. 
+   EPSArnoldiSetDelayed - Activates or deactivates delayed reorthogonalization
+   in the Arnoldi iteration.
 
    Logically Collective on EPS
 
@@ -490,7 +382,7 @@ EXTERN_C_END
 
    Options Database Key:
 .  -eps_arnoldi_delayed - Activates delayed reorthogonalization in Arnoldi
-   
+
    Note:
    Delayed reorthogonalization is an aggressive optimization for the Arnoldi
    eigensolver than may provide better scalability, but sometimes makes the
@@ -511,24 +403,22 @@ PetscErrorCode EPSArnoldiSetDelayed(EPS eps,PetscBool delayed)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSArnoldiGetDelayed_Arnoldi"
-PetscErrorCode EPSArnoldiGetDelayed_Arnoldi(EPS eps,PetscBool *delayed)
+static PetscErrorCode EPSArnoldiGetDelayed_Arnoldi(EPS eps,PetscBool *delayed)
 {
-  EPS_ARNOLDI *arnoldi = (EPS_ARNOLDI *)eps->data;
+  EPS_ARNOLDI *arnoldi = (EPS_ARNOLDI*)eps->data;
 
   PetscFunctionBegin;
   *delayed = arnoldi->delayed;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSArnoldiGetDelayed"
 /*@C
    EPSArnoldiGetDelayed - Gets the type of reorthogonalization used during the Arnoldi
-   iteration. 
+   iteration.
 
    Not Collective
 
@@ -553,20 +443,7 @@ PetscErrorCode EPSArnoldiGetDelayed(EPS eps,PetscBool *delayed)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "EPSReset_Arnoldi"
-PetscErrorCode EPSReset_Arnoldi(EPS eps)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  ierr = PetscFree(eps->T);CHKERRQ(ierr);
-  ierr = PetscFree(eps->Tl);CHKERRQ(ierr);
-  ierr = EPSReset_Default(eps);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSDestroy_Arnoldi"
 PetscErrorCode EPSDestroy_Arnoldi(EPS eps)
 {
@@ -574,51 +451,46 @@ PetscErrorCode EPSDestroy_Arnoldi(EPS eps)
 
   PetscFunctionBegin;
   ierr = PetscFree(eps->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSArnoldiSetDelayed_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSArnoldiGetDelayed_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSArnoldiSetDelayed_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSArnoldiGetDelayed_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSView_Arnoldi"
 PetscErrorCode EPSView_Arnoldi(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   PetscBool      isascii;
-  EPS_ARNOLDI    *arnoldi = (EPS_ARNOLDI *)eps->data;
+  EPS_ARNOLDI    *arnoldi = (EPS_ARNOLDI*)eps->data;
 
   PetscFunctionBegin;
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
-  if (!isascii) {
-    SETERRQ1(((PetscObject)eps)->comm,1,"Viewer type %s not supported for EPS Arnoldi",((PetscObject)viewer)->type_name);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  if (isascii) {
+    if (arnoldi->delayed) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  Arnoldi: using delayed reorthogonalization\n");CHKERRQ(ierr);
+    }
   }
-  if (arnoldi->delayed) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  Arnoldi: using delayed reorthogonalization\n");CHKERRQ(ierr);
-  }  
   PetscFunctionReturn(0);
 }
 
-extern PetscErrorCode EPSSolve_TS_Arnoldi(EPS);
-
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSCreate_Arnoldi"
-PetscErrorCode EPSCreate_Arnoldi(EPS eps)
+PETSC_EXTERN PetscErrorCode EPSCreate_Arnoldi(EPS eps)
 {
   PetscErrorCode ierr;
-  
+
   PetscFunctionBegin;
   ierr = PetscNewLog(eps,EPS_ARNOLDI,&eps->data);CHKERRQ(ierr);
   eps->ops->setup                = EPSSetUp_Arnoldi;
   eps->ops->setfromoptions       = EPSSetFromOptions_Arnoldi;
   eps->ops->destroy              = EPSDestroy_Arnoldi;
-  eps->ops->reset                = EPSReset_Arnoldi;
+  eps->ops->reset                = EPSReset_Default;
   eps->ops->view                 = EPSView_Arnoldi;
   eps->ops->backtransform        = EPSBackTransform_Default;
   eps->ops->computevectors       = EPSComputeVectors_Schur;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSArnoldiSetDelayed_C","EPSArnoldiSetDelayed_Arnoldi",EPSArnoldiSetDelayed_Arnoldi);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSArnoldiGetDelayed_C","EPSArnoldiGetDelayed_Arnoldi",EPSArnoldiGetDelayed_Arnoldi);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSArnoldiSetDelayed_C",EPSArnoldiSetDelayed_Arnoldi);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSArnoldiGetDelayed_C",EPSArnoldiGetDelayed_Arnoldi);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
