@@ -1,4 +1,4 @@
-/*                       
+/*
 
    SLEPc eigensolver: "power"
 
@@ -15,24 +15,24 @@
 
    References:
 
-       [1] "Single Vector Iteration Methods in SLEPc", SLEPc Technical Report STR-2, 
-           available at http://www.grycap.upv.es/slepc.
+       [1] "Single Vector Iteration Methods in SLEPc", SLEPc Technical Report
+           STR-2, available at http://www.grycap.upv.es/slepc.
 
    Last update: Feb 2009
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -40,7 +40,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/epsimpl.h>                /*I "slepceps.h" I*/
+#include <slepc-private/epsimpl.h>                /*I "slepceps.h" I*/
 #include <slepcblaslapack.h>
 
 PetscErrorCode EPSSolve_Power(EPS);
@@ -50,41 +50,37 @@ typedef struct {
   EPSPowerShiftType shift_type;
 } EPS_POWER;
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetUp_Power"
 PetscErrorCode EPSSetUp_Power(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_POWER      *power = (EPS_POWER *)eps->data;
+  EPS_POWER      *power = (EPS_POWER*)eps->data;
   PetscBool      flg;
   STMatMode      mode;
 
   PetscFunctionBegin;
   if (eps->ncv) {
-    if (eps->ncv<eps->nev) SETERRQ(((PetscObject)eps)->comm,1,"The value of ncv must be at least nev"); 
-  }
-  else eps->ncv = eps->nev;
+    if (eps->ncv<eps->nev) SETERRQ(PetscObjectComm((PetscObject)eps),1,"The value of ncv must be at least nev");
+  } else eps->ncv = eps->nev;
   if (eps->mpd) { ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr); }
   if (!eps->max_it) eps->max_it = PetscMax(2000,100*eps->n);
-  if (!eps->which) { ierr = EPSDefaultSetWhich(eps);CHKERRQ(ierr); }
-  if (eps->which!=EPS_LARGEST_MAGNITUDE && eps->which !=EPS_TARGET_MAGNITUDE)
-    SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->which");
+  if (!eps->which) { ierr = EPSSetWhichEigenpairs_Default(eps);CHKERRQ(ierr); }
+  if (eps->which!=EPS_LARGEST_MAGNITUDE && eps->which !=EPS_TARGET_MAGNITUDE) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->which");
   if (power->shift_type != EPS_POWER_SHIFT_CONSTANT) {
-    ierr = PetscTypeCompareAny((PetscObject)eps->OP,&flg,STSINVERT,STCAYLEY,"");CHKERRQ(ierr);
-    if (!flg) 
-      SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Variable shifts only allowed in shift-and-invert or Cayley ST");
-    ierr = STGetMatMode(eps->OP,&mode);CHKERRQ(ierr); 
-    if (mode == ST_MATMODE_INPLACE)
-      SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"ST matrix mode inplace does not work with variable shifts");
+    ierr = PetscObjectTypeCompareAny((PetscObject)eps->st,&flg,STSINVERT,STCAYLEY,"");CHKERRQ(ierr);
+    if (!flg) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Variable shifts only allowed in shift-and-invert or Cayley ST");
+    ierr = STGetMatMode(eps->st,&mode);CHKERRQ(ierr);
+    if (mode == ST_MATMODE_INPLACE) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"ST matrix mode inplace does not work with variable shifts");
   }
   if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
-  if (eps->balance!=EPS_BALANCE_NONE)
-    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Balancing not supported in this solver");
+  if (eps->balance!=EPS_BALANCE_NONE) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Balancing not supported in this solver");
+  if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs not supported in this solver");
   ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
   if (eps->leftvecs) {
-    ierr = EPSDefaultGetWork(eps,3);CHKERRQ(ierr);
+    ierr = EPSSetWorkVecs(eps,3);CHKERRQ(ierr);
   } else {
-    ierr = EPSDefaultGetWork(eps,2);CHKERRQ(ierr);
+    ierr = EPSSetWorkVecs(eps,2);CHKERRQ(ierr);
   }
 
   /* dispatch solve method */
@@ -93,18 +89,22 @@ PetscErrorCode EPSSetUp_Power(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSolve_Power"
 PetscErrorCode EPSSolve_Power(EPS eps)
 {
+#if defined(SLEPC_MISSING_LAPACK_LAEV2)
+  PetscFunctionBegin;
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"LAEV2 - Lapack routine is unavailable");
+#else
   PetscErrorCode ierr;
-  EPS_POWER      *power = (EPS_POWER *)eps->data;
+  EPS_POWER      *power = (EPS_POWER*)eps->data;
   PetscInt       i;
   Vec            v,y,e;
   Mat            A;
   PetscReal      relerr,norm,rt1,rt2,cs1,anorm;
   PetscScalar    theta,rho,delta,sigma,alpha2,beta1,sn1;
-  PetscBool      breakdown,*select = PETSC_NULL,hasnorm;
+  PetscBool      breakdown,*select = NULL,hasnorm;
 
   PetscFunctionBegin;
   v = eps->V[0];
@@ -113,7 +113,7 @@ PetscErrorCode EPSSolve_Power(EPS eps)
 
   /* prepare for selective orthogonalization of converged vectors */
   if (power->shift_type != EPS_POWER_SHIFT_CONSTANT && eps->nev>1) {
-    ierr = STGetOperators(eps->OP,&A,PETSC_NULL);CHKERRQ(ierr);
+    ierr = STGetOperators(eps->st,0,&A);CHKERRQ(ierr);
     ierr = MatHasOperation(A,MATOP_NORM,&hasnorm);CHKERRQ(ierr);
     if (hasnorm) {
       ierr = MatNorm(A,NORM_INFINITY,&anorm);CHKERRQ(ierr);
@@ -121,15 +121,15 @@ PetscErrorCode EPSSolve_Power(EPS eps)
     }
   }
 
-  ierr = EPSGetStartVector(eps,0,v,PETSC_NULL);CHKERRQ(ierr);
-  ierr = STGetShift(eps->OP,&sigma);CHKERRQ(ierr);    /* original shift */
+  ierr = EPSGetStartVector(eps,0,v,NULL);CHKERRQ(ierr);
+  ierr = STGetShift(eps->st,&sigma);CHKERRQ(ierr);    /* original shift */
   rho = sigma;
 
   while (eps->reason == EPS_CONVERGED_ITERATING) {
     eps->its = eps->its + 1;
 
     /* y = OP v */
-    ierr = STApply(eps->OP,v,y);CHKERRQ(ierr);
+    ierr = STApply(eps->st,v,y);CHKERRQ(ierr);
 
     /* theta = (v,y)_B */
     ierr = IPInnerProduct(eps->ip,v,y,&theta);CHKERRQ(ierr);
@@ -160,41 +160,39 @@ PetscErrorCode EPSSolve_Power(EPS eps)
 
       /* compute new shift */
       if (relerr<eps->tol) {
-        rho = sigma; /* if converged, restore original shift */ 
-        ierr = STSetShift(eps->OP,rho);CHKERRQ(ierr);
+        rho = sigma; /* if converged, restore original shift */
+        ierr = STSetShift(eps->st,rho);CHKERRQ(ierr);
       } else {
         rho = rho + theta/(delta*delta);  /* Rayleigh quotient R(v) */
         if (power->shift_type == EPS_POWER_SHIFT_WILKINSON) {
-#if defined(SLEPC_MISSING_LAPACK_LAEV2)
-          SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"LAEV2 - Lapack routine is unavailable.");
-#else 
           /* beta1 is the norm of the residual associated to R(v) */
           ierr = VecAXPY(v,-theta/(delta*delta),y);CHKERRQ(ierr);
           ierr = VecScale(v,1.0/delta);CHKERRQ(ierr);
           ierr = IPNorm(eps->ip,v,&norm);CHKERRQ(ierr);
           beta1 = norm;
-    
+
           /* alpha2 = (e'*A*e)/(beta1*beta1), where e is the residual */
-          ierr = STGetOperators(eps->OP,&A,PETSC_NULL);CHKERRQ(ierr);
+          ierr = STGetOperators(eps->st,0,&A);CHKERRQ(ierr);
           ierr = MatMult(A,v,e);CHKERRQ(ierr);
           ierr = VecDot(v,e,&alpha2);CHKERRQ(ierr);
           alpha2 = alpha2 / (beta1 * beta1);
 
           /* choose the eigenvalue of [rho beta1; beta1 alpha2] closest to rho */
-          LAPACKlaev2_(&rho,&beta1,&alpha2,&rt1,&rt2,&cs1,&sn1);
+          ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+          PetscStackCallBLAS("LAPACKlaev2",LAPACKlaev2_(&rho,&beta1,&alpha2,&rt1,&rt2,&cs1,&sn1));
+          ierr = PetscFPTrapPop();CHKERRQ(ierr);
           if (PetscAbsScalar(rt1-rho) < PetscAbsScalar(rt2-rho)) rho = rt1;
           else rho = rt2;
-#endif 
         }
         /* update operator according to new shift */
-        PetscPushErrorHandler(PetscIgnoreErrorHandler,PETSC_NULL);
-        ierr = STSetShift(eps->OP,rho);
+        PetscPushErrorHandler(PetscIgnoreErrorHandler,NULL);
+        ierr = STSetShift(eps->st,rho);CHKERRQ(ierr);
         PetscPopErrorHandler();
         if (ierr) {
           eps->eigr[eps->nconv] = rho;
           relerr = PETSC_MACHINE_EPSILON;
           rho = sigma;
-          ierr = STSetShift(eps->OP,rho);CHKERRQ(ierr);
+          ierr = STSetShift(eps->st,rho);CHKERRQ(ierr);
         }
       }
     }
@@ -205,12 +203,12 @@ PetscErrorCode EPSSolve_Power(EPS eps)
     /* purge previously converged eigenvectors */
     if (select) {
       for (i=0;i<eps->nconv;i++) {
-        if(PetscAbsScalar(rho-eps->eigr[i])>eps->its*anorm/1000) select[i] = PETSC_TRUE;
+        if (PetscAbsScalar(rho-eps->eigr[i])>eps->its*anorm/1000) select[i] = PETSC_TRUE;
         else select[i] = PETSC_FALSE;
       }
-      ierr = IPOrthogonalize(eps->ip,eps->nds,eps->DS,eps->nconv,select,eps->V,y,PETSC_NULL,&norm,PETSC_NULL);CHKERRQ(ierr);
+      ierr = IPOrthogonalize(eps->ip,eps->nds,eps->defl,eps->nconv,select,eps->V,y,NULL,&norm,NULL);CHKERRQ(ierr);
     } else {
-      ierr = IPOrthogonalize(eps->ip,eps->nds,eps->DS,eps->nconv,PETSC_NULL,eps->V,y,PETSC_NULL,&norm,PETSC_NULL);CHKERRQ(ierr);
+      ierr = IPOrthogonalize(eps->ip,eps->nds,eps->defl,eps->nconv,NULL,eps->V,y,NULL,&norm,NULL);CHKERRQ(ierr);
     }
 
     /* v = y/||y||_B */
@@ -234,14 +232,19 @@ PetscErrorCode EPSSolve_Power(EPS eps)
   }
   ierr = PetscFree(select);CHKERRQ(ierr);
   PetscFunctionReturn(0);
+#endif
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSolve_TS_Power"
 PetscErrorCode EPSSolve_TS_Power(EPS eps)
 {
+#if defined(SLEPC_MISSING_LAPACK_LAEV2)
+  PetscFunctionBegin;
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"LAEV2 - Lapack routine is unavailable");
+#else
   PetscErrorCode ierr;
-  EPS_POWER      *power = (EPS_POWER *)eps->data;
+  EPS_POWER      *power = (EPS_POWER*)eps->data;
   Vec            v,w,y,z,e;
   Mat            A;
   PetscReal      relerr,norm,rt1,rt2,cs1;
@@ -254,17 +257,17 @@ PetscErrorCode EPSSolve_TS_Power(EPS eps)
   w = eps->W[0];
   z = eps->work[2];
 
-  ierr = EPSGetStartVector(eps,0,v,PETSC_NULL);CHKERRQ(ierr);
-  ierr = EPSGetStartVectorLeft(eps,0,w,PETSC_NULL);CHKERRQ(ierr);
-  ierr = STGetShift(eps->OP,&sigma);CHKERRQ(ierr);    /* original shift */
+  ierr = EPSGetStartVector(eps,0,v,NULL);CHKERRQ(ierr);
+  ierr = EPSGetStartVectorLeft(eps,0,w,NULL);CHKERRQ(ierr);
+  ierr = STGetShift(eps->st,&sigma);CHKERRQ(ierr);    /* original shift */
   rho = sigma;
 
   while (eps->its<eps->max_it) {
     eps->its++;
-    
+
     /* y = OP v, z = OP' w */
-    ierr = STApply(eps->OP,v,y);CHKERRQ(ierr);
-    ierr = STApplyTranspose(eps->OP,w,z);CHKERRQ(ierr);
+    ierr = STApply(eps->st,v,y);CHKERRQ(ierr);
+    ierr = STApplyTranspose(eps->st,w,z);CHKERRQ(ierr);
 
     /* theta = (v,z)_B */
     ierr = IPInnerProduct(eps->ip,v,z,&theta);CHKERRQ(ierr);
@@ -290,7 +293,7 @@ PetscErrorCode EPSSolve_TS_Power(EPS eps)
 
       /* delta = sqrt(y,z)_B */
       ierr = IPInnerProduct(eps->ip,y,z,&alpha);CHKERRQ(ierr);
-      if (alpha==0.0) SETERRQ(((PetscObject)eps)->comm,1,"Breakdown in two-sided Power/RQI");
+      if (alpha==0.0) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Breakdown in two-sided Power/RQI");
       delta = PetscSqrtScalar(alpha);
 
       /* compute relative error */
@@ -304,42 +307,40 @@ PetscErrorCode EPSSolve_TS_Power(EPS eps)
 
       /* compute new shift */
       if (eps->errest[eps->nconv]<eps->tol && eps->errest_left[eps->nconv]<eps->tol) {
-        rho = sigma; /* if converged, restore original shift */ 
-        ierr = STSetShift(eps->OP,rho);CHKERRQ(ierr);
+        rho = sigma; /* if converged, restore original shift */
+        ierr = STSetShift(eps->st,rho);CHKERRQ(ierr);
       } else {
         rho = rho + theta/(delta*delta);  /* Rayleigh quotient R(v,w) */
         if (power->shift_type == EPS_POWER_SHIFT_WILKINSON) {
-#if defined(SLEPC_MISSING_LAPACK_LAEV2)
-          SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"LAEV2 - Lapack routine is unavailable.");
-#else 
           /* beta1 is the norm of the residual associated to R(v,w) */
           ierr = VecAXPY(v,-theta/(delta*delta),y);CHKERRQ(ierr);
           ierr = VecScale(v,1.0/delta);CHKERRQ(ierr);
           ierr = IPNorm(eps->ip,v,&norm);CHKERRQ(ierr);
           beta1 = norm;
-    
+
           /* alpha2 = (e'*A*e)/(beta1*beta1), where e is the residual */
-          ierr = STGetOperators(eps->OP,&A,PETSC_NULL);CHKERRQ(ierr);
+          ierr = STGetOperators(eps->st,0,&A);CHKERRQ(ierr);
           ierr = MatMult(A,v,e);CHKERRQ(ierr);
           ierr = VecDot(v,e,&alpha2);CHKERRQ(ierr);
           alpha2 = alpha2 / (beta1 * beta1);
 
           /* choose the eigenvalue of [rho beta1; beta1 alpha2] closest to rho */
-          LAPACKlaev2_(&rho,&beta1,&alpha2,&rt1,&rt2,&cs1,&sn1);
+          ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+          PetscStackCallBLAS("LAPACKlaev2",LAPACKlaev2_(&rho,&beta1,&alpha2,&rt1,&rt2,&cs1,&sn1));
+          ierr = PetscFPTrapPop();CHKERRQ(ierr);
           if (PetscAbsScalar(rt1-rho) < PetscAbsScalar(rt2-rho)) rho = rt1;
           else rho = rt2;
-#endif 
         }
         /* update operator according to new shift */
-        PetscPushErrorHandler(PetscIgnoreErrorHandler,PETSC_NULL);
-        ierr = STSetShift(eps->OP,rho);
+        PetscPushErrorHandler(PetscIgnoreErrorHandler,NULL);
+        ierr = STSetShift(eps->st,rho);CHKERRQ(ierr);
         PetscPopErrorHandler();
         if (ierr) {
           eps->eigr[eps->nconv] = rho;
           eps->errest[eps->nconv] = PETSC_MACHINE_EPSILON;
           eps->errest_left[eps->nconv] = PETSC_MACHINE_EPSILON;
           rho = sigma;
-          ierr = STSetShift(eps->OP,rho);CHKERRQ(ierr);
+          ierr = STSetShift(eps->st,rho);CHKERRQ(ierr);
         }
       }
     }
@@ -348,14 +349,14 @@ PetscErrorCode EPSSolve_TS_Power(EPS eps)
     ierr = EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest_left,eps->nconv+1);CHKERRQ(ierr);
 
     /* purge previously converged eigenvectors */
-    ierr = IPBiOrthogonalize(eps->ip,eps->nconv,eps->V,eps->W,z,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
-    ierr = IPBiOrthogonalize(eps->ip,eps->nconv,eps->W,eps->V,y,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+    ierr = IPBiOrthogonalize(eps->ip,eps->nconv,eps->V,eps->W,z,NULL,NULL);CHKERRQ(ierr);
+    ierr = IPBiOrthogonalize(eps->ip,eps->nconv,eps->W,eps->V,y,NULL,NULL);CHKERRQ(ierr);
 
     /* normalize so that (y,z)_B=1  */
     ierr = VecCopy(y,v);CHKERRQ(ierr);
     ierr = VecCopy(z,w);CHKERRQ(ierr);
     ierr = IPInnerProduct(eps->ip,y,z,&alpha);CHKERRQ(ierr);
-    if (alpha==0.0) SETERRQ(((PetscObject)eps)->comm,1,"Breakdown in two-sided Power/RQI");
+    if (alpha==0.0) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Breakdown in two-sided Power/RQI");
     delta = PetscSqrtScalar(PetscAbsScalar(alpha));
     beta = 1.0/PetscConj(alpha/delta);
     delta = 1.0/delta;
@@ -367,22 +368,23 @@ PetscErrorCode EPSSolve_TS_Power(EPS eps)
       eps->nconv = eps->nconv + 1;
       if (eps->nconv==eps->nev) break;
       v = eps->V[eps->nconv];
-      ierr = EPSGetStartVector(eps,eps->nconv,v,PETSC_NULL);CHKERRQ(ierr);
+      ierr = EPSGetStartVector(eps,eps->nconv,v,NULL);CHKERRQ(ierr);
       w = eps->W[eps->nconv];
-      ierr = EPSGetStartVectorLeft(eps,eps->nconv,w,PETSC_NULL);CHKERRQ(ierr);
+      ierr = EPSGetStartVectorLeft(eps,eps->nconv,w,NULL);CHKERRQ(ierr);
     }
   }
   if (eps->nconv == eps->nev) eps->reason = EPS_CONVERGED_TOL;
   else eps->reason = EPS_DIVERGED_ITS;
   PetscFunctionReturn(0);
+#endif
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSBackTransform_Power"
 PetscErrorCode EPSBackTransform_Power(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_POWER      *power = (EPS_POWER *)eps->data;
+  EPS_POWER      *power = (EPS_POWER*)eps->data;
 
   PetscFunctionBegin;
   if (power->shift_type == EPS_POWER_SHIFT_CONSTANT) {
@@ -391,32 +393,33 @@ PetscErrorCode EPSBackTransform_Power(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetFromOptions_Power"
 PetscErrorCode EPSSetFromOptions_Power(EPS eps)
 {
   PetscErrorCode    ierr;
-  EPS_POWER         *power = (EPS_POWER *)eps->data;
+  EPS_POWER         *power = (EPS_POWER*)eps->data;
   PetscBool         flg;
   EPSPowerShiftType shift;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead("EPS Power Options");CHKERRQ(ierr);
   ierr = PetscOptionsEnum("-eps_power_shift_type","Shift type","EPSPowerSetShiftType",EPSPowerShiftTypes,(PetscEnum)power->shift_type,(PetscEnum*)&shift,&flg);CHKERRQ(ierr);
-  if (flg) { ierr = EPSPowerSetShiftType(eps,shift);CHKERRQ(ierr); }
+  if (flg) {
+    ierr = EPSPowerSetShiftType(eps,shift);CHKERRQ(ierr);
+  }
   if (power->shift_type != EPS_POWER_SHIFT_CONSTANT) {
-    ierr = STSetType(eps->OP,STSINVERT);CHKERRQ(ierr);
+    ierr = STSetType(eps->st,STSINVERT);CHKERRQ(ierr);
   }
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSPowerSetShiftType_Power"
-PetscErrorCode EPSPowerSetShiftType_Power(EPS eps,EPSPowerShiftType shift)
+static PetscErrorCode EPSPowerSetShiftType_Power(EPS eps,EPSPowerShiftType shift)
 {
-  EPS_POWER *power = (EPS_POWER *)eps->data;
+  EPS_POWER *power = (EPS_POWER*)eps->data;
 
   PetscFunctionBegin;
   switch (shift) {
@@ -426,13 +429,12 @@ PetscErrorCode EPSPowerSetShiftType_Power(EPS eps,EPSPowerShiftType shift)
       power->shift_type = shift;
       break;
     default:
-      SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid shift type");
+      SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Invalid shift type");
   }
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSPowerSetShiftType"
 /*@
    EPSPowerSetShiftType - Sets the type of shifts used during the power
@@ -446,7 +448,7 @@ EXTERN_C_END
 -  shift - the type of shift
 
    Options Database Key:
-.  -eps_power_shift_type - Sets the shift type (either 'constant' or 
+.  -eps_power_shift_type - Sets the shift type (either 'constant' or
                            'rayleigh' or 'wilkinson')
 
    Notes:
@@ -457,7 +459,7 @@ EXTERN_C_END
    A variable shift can be specified (EPS_POWER_SHIFT_RAYLEIGH or
    EPS_POWER_SHIFT_WILKINSON). In this case, the iteration behaves rather like
    a cubic converging method as RQI. See the users manual for details.
-   
+
    Level: advanced
 
 .seealso: EPSPowerGetShiftType(), STSetShift(), EPSPowerShiftType
@@ -473,24 +475,22 @@ PetscErrorCode EPSPowerSetShiftType(EPS eps,EPSPowerShiftType shift)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSPowerGetShiftType_Power"
-PetscErrorCode EPSPowerGetShiftType_Power(EPS eps,EPSPowerShiftType *shift)
+static PetscErrorCode EPSPowerGetShiftType_Power(EPS eps,EPSPowerShiftType *shift)
 {
-  EPS_POWER  *power = (EPS_POWER *)eps->data;
+  EPS_POWER  *power = (EPS_POWER*)eps->data;
 
   PetscFunctionBegin;
   *shift = power->shift_type;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSPowerGetShiftType"
 /*@C
    EPSPowerGetShiftType - Gets the type of shifts used during the power
-   iteration. 
+   iteration.
 
    Not Collective
 
@@ -515,7 +515,7 @@ PetscErrorCode EPSPowerGetShiftType(EPS eps,EPSPowerShiftType *shift)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSDestroy_Power"
 PetscErrorCode EPSDestroy_Power(EPS eps)
 {
@@ -523,32 +523,30 @@ PetscErrorCode EPSDestroy_Power(EPS eps)
 
   PetscFunctionBegin;
   ierr = PetscFree(eps->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPowerSetShiftType_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPowerGetShiftType_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSPowerSetShiftType_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSPowerGetShiftType_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSView_Power"
 PetscErrorCode EPSView_Power(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  EPS_POWER      *power = (EPS_POWER *)eps->data;
+  EPS_POWER      *power = (EPS_POWER*)eps->data;
   PetscBool      isascii;
 
   PetscFunctionBegin;
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
-  if (!isascii) {
-    SETERRQ1(((PetscObject)eps)->comm,1,"Viewer type %s not supported for EPS Power",((PetscObject)viewer)->type_name);
-  }  
-  ierr = PetscViewerASCIIPrintf(viewer,"  Power: %s shifts\n",EPSPowerShiftTypes[power->shift_type]);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  if (isascii) {
+    ierr = PetscViewerASCIIPrintf(viewer,"  Power: %s shifts\n",EPSPowerShiftTypes[power->shift_type]);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSCreate_Power"
-PetscErrorCode EPSCreate_Power(EPS eps)
+PETSC_EXTERN PetscErrorCode EPSCreate_Power(EPS eps)
 {
   PetscErrorCode ierr;
 
@@ -561,9 +559,8 @@ PetscErrorCode EPSCreate_Power(EPS eps)
   eps->ops->view                 = EPSView_Power;
   eps->ops->backtransform        = EPSBackTransform_Power;
   eps->ops->computevectors       = EPSComputeVectors_Default;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPowerSetShiftType_C","EPSPowerSetShiftType_Power",EPSPowerSetShiftType_Power);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPowerGetShiftType_C","EPSPowerGetShiftType_Power",EPSPowerGetShiftType_Power);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSPowerSetShiftType_C",EPSPowerSetShiftType_Power);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSPowerGetShiftType_C",EPSPowerGetShiftType_Power);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
