@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,13 +21,13 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/epsimpl.h>      /*I "slepceps.h" I*/
+#include <slepc-private/epsimpl.h>      /*I "slepceps.h" I*/
 
-PetscFList       EPSList = 0;
-PetscBool        EPSRegisterAllCalled = PETSC_FALSE;
-PetscClassId     EPS_CLASSID = 0;
-PetscLogEvent    EPS_SetUp = 0,EPS_Solve = 0,EPS_Dense = 0;
-static PetscBool EPSPackageInitialized = PETSC_FALSE;
+PetscFunctionList EPSList = 0;
+PetscBool         EPSRegisterAllCalled = PETSC_FALSE;
+PetscClassId      EPS_CLASSID = 0;
+PetscLogEvent     EPS_SetUp = 0,EPS_Solve = 0;
+static PetscBool  EPSPackageInitialized = PETSC_FALSE;
 
 const char *EPSPowerShiftTypes[] = {"CONSTANT","RAYLEIGH","WILKINSON","EPSPowerShiftType","EPS_POWER_SHIFT_",0};
 const char *EPSLanczosReorthogTypes[] = {"LOCAL","FULL","SELECTIVE","PERIODIC","PARTIAL","DELAYED","EPSLanczosReorthogType","EPS_LANCZOS_REORTHOG_",0};
@@ -36,18 +36,20 @@ const char *EPSPRIMMEMethods[] = {"DYNAMIC","DEFAULT_MIN_TIME","DEFAULT_MIN_MATV
 #undef __FUNCT__
 #define __FUNCT__ "EPSFinalizePackage"
 /*@C
-  EPSFinalizePackage - This function destroys everything in the Slepc interface to the EPS package. It is
-  called from SlepcFinalize().
+  EPSFinalizePackage - This function destroys everything in the SLEPc interface
+  to the EPS package. It is called from SlepcFinalize().
 
   Level: developer
 
 .seealso: SlepcFinalize()
 @*/
-PetscErrorCode EPSFinalizePackage(void) 
+PetscErrorCode EPSFinalizePackage(void)
 {
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
+  ierr = PetscFunctionListDestroy(&EPSList);CHKERRQ(ierr);
   EPSPackageInitialized = PETSC_FALSE;
-  EPSList               = 0;
   EPSRegisterAllCalled  = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
@@ -55,18 +57,16 @@ PetscErrorCode EPSFinalizePackage(void)
 #undef __FUNCT__
 #define __FUNCT__ "EPSInitializePackage"
 /*@C
-  EPSInitializePackage - This function initializes everything in the EPS package. It is called
-  from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to EPSCreate()
-  when using static libraries.
-
-  Input Parameter:
-  path - The dynamic library path, or PETSC_NULL
+  EPSInitializePackage - This function initializes everything in the EPS package.
+  It is called from PetscDLLibraryRegister() when using dynamic libraries, and
+  on the first call to EPSCreate() when using static libraries.
 
   Level: developer
 
 .seealso: SlepcInitialize()
 @*/
-PetscErrorCode EPSInitializePackage(const char *path) {
+PetscErrorCode EPSInitializePackage()
+{
   char           logList[256];
   char           *className;
   PetscBool      opt;
@@ -76,15 +76,14 @@ PetscErrorCode EPSInitializePackage(const char *path) {
   if (EPSPackageInitialized) PetscFunctionReturn(0);
   EPSPackageInitialized = PETSC_TRUE;
   /* Register Classes */
-  ierr = PetscClassIdRegister("Eigenproblem Solver",&EPS_CLASSID);CHKERRQ(ierr);
+  ierr = PetscClassIdRegister("Eigenvalue Problem Solver",&EPS_CLASSID);CHKERRQ(ierr);
   /* Register Constructors */
-  ierr = EPSRegisterAll(path);CHKERRQ(ierr);
+  ierr = EPSRegisterAll();CHKERRQ(ierr);
   /* Register Events */
   ierr = PetscLogEventRegister("EPSSetUp",EPS_CLASSID,&EPS_SetUp);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("EPSSolve",EPS_CLASSID,&EPS_Solve);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("EPSDense",EPS_CLASSID,&EPS_Dense);CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
     ierr = PetscStrstr(logList,"eps",&className);CHKERRQ(ierr);
     if (className) {
@@ -92,7 +91,7 @@ PetscErrorCode EPSInitializePackage(const char *path) {
     }
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL,"-log_summary_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,"-log_summary_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
     ierr = PetscStrstr(logList,"eps",&className);CHKERRQ(ierr);
     if (className) {
@@ -122,8 +121,8 @@ PetscErrorCode EPSInitializePackage(const char *path) {
 +     PETSC_VIEWER_STDOUT_SELF - standard output (default)
 -     PETSC_VIEWER_STDOUT_WORLD - synchronized standard
          output where only the first processor opens
-         the file.  All other processors send their 
-         data to the first processor to print. 
+         the file.  All other processors send their
+         data to the first processor to print.
 
    The user can open an alternative visualization context with
    PetscViewerASCIIOpen() - output to a specified file.
@@ -136,11 +135,12 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   const char     *type,*extr,*bal;
-  PetscBool      isascii;
+  char           str[50];
+  PetscBool      isascii,ispower,isexternal;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(((PetscObject)eps)->comm);
+  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)eps));
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(eps,1,viewer,2);
 
@@ -149,7 +149,7 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
 #else
 #define HERM "symmetric"
 #endif
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)eps,viewer,"EPS Object");CHKERRQ(ierr);
     if (eps->ops->view) {
@@ -164,7 +164,8 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
         case EPS_NHEP:  type = "non-" HERM " eigenvalue problem"; break;
         case EPS_GNHEP: type = "generalized non-" HERM " eigenvalue problem"; break;
         case EPS_PGNHEP: type = "generalized non-" HERM " eigenvalue problem with " HERM " positive definite B"; break;
-        default: SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->problem_type");
+        case EPS_GHIEP: type = "generalized " HERM "-indefinite eigenvalue problem"; break;
+        default: SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->problem_type");
       }
     } else type = "not yet set";
     ierr = PetscViewerASCIIPrintf(viewer,"  problem type: %s\n",type);CHKERRQ(ierr);
@@ -177,7 +178,7 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
         case EPS_HARMONIC_LARGEST: extr = "largest harmonic Ritz"; break;
         case EPS_REFINED:          extr = "refined Ritz"; break;
         case EPS_REFINED_HARMONIC: extr = "refined harmonic Ritz"; break;
-        default: SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->extraction");
+        default: SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->extraction");
       }
       ierr = PetscViewerASCIIPrintf(viewer,"  extraction type: %s\n",extr);CHKERRQ(ierr);
     }
@@ -186,7 +187,7 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
         case EPS_BALANCE_ONESIDE:   bal = "one-sided Krylov"; break;
         case EPS_BALANCE_TWOSIDE:   bal = "two-sided Krylov"; break;
         case EPS_BALANCE_USER:      bal = "user-defined matrix"; break;
-        default: SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->balance");
+        default: SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->balance");
       }
       ierr = PetscViewerASCIIPrintf(viewer,"  balancing enabled: %s",bal);CHKERRQ(ierr);
       if (eps->balance==EPS_BALANCE_ONESIDE || eps->balance==EPS_BALANCE_TWOSIDE) {
@@ -198,6 +199,7 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
       ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
     }
     ierr = PetscViewerASCIIPrintf(viewer,"  selected portion of the spectrum: ");CHKERRQ(ierr);
+    ierr = SlepcSNPrintfScalar(str,50,eps->target,PETSC_FALSE);CHKERRQ(ierr);
     if (!eps->which) {
       ierr = PetscViewerASCIIPrintf(viewer,"not yet set\n");CHKERRQ(ierr);
     } else switch (eps->which) {
@@ -205,24 +207,14 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
         ierr = PetscViewerASCIIPrintf(viewer,"user defined\n");CHKERRQ(ierr);
         break;
       case EPS_TARGET_MAGNITUDE:
-#if !defined(PETSC_USE_COMPLEX)
-        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %G (in magnitude)\n",eps->target);CHKERRQ(ierr);
-#else
-        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %G+%G i (in magnitude)\n",PetscRealPart(eps->target),PetscImaginaryPart(eps->target));CHKERRQ(ierr);
-#endif
+        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %s (in magnitude)\n",str);CHKERRQ(ierr);
         break;
       case EPS_TARGET_REAL:
-#if !defined(PETSC_USE_COMPLEX)
-        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %G (along the real axis)\n",eps->target);CHKERRQ(ierr);
-#else
-        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %G+%G i (along the real axis)\n",PetscRealPart(eps->target),PetscImaginaryPart(eps->target));CHKERRQ(ierr);
-#endif
+        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %s (along the real axis)\n",str);CHKERRQ(ierr);
         break;
-#if defined(PETSC_USE_COMPLEX)
       case EPS_TARGET_IMAGINARY:
-        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %G+%G i (along the imaginary axis)\n",PetscRealPart(eps->target),PetscImaginaryPart(eps->target));CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %s (along the imaginary axis)\n",str);CHKERRQ(ierr);
         break;
-#endif
       case EPS_LARGEST_MAGNITUDE:
         ierr = PetscViewerASCIIPrintf(viewer,"largest eigenvalues in magnitude\n");CHKERRQ(ierr);
         break;
@@ -244,8 +236,8 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
       case EPS_ALL:
         ierr = PetscViewerASCIIPrintf(viewer,"all eigenvalues in interval [%G,%G]\n",eps->inta,eps->intb);CHKERRQ(ierr);
         break;
-      default: SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->which");
-    }    
+      default: SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->which");
+    }
     if (eps->leftvecs) {
       ierr = PetscViewerASCIIPrintf(viewer,"  computing left eigenvectors also\n");CHKERRQ(ierr);
     }
@@ -258,23 +250,23 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
     ierr = PetscViewerASCIIPrintf(viewer,"  number of eigenvalues (nev): %D\n",eps->nev);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  number of column vectors (ncv): %D\n",eps->ncv);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  maximum dimension of projected problem (mpd): %D\n",eps->mpd);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of iterations: %D\n",eps->max_it);
+    ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of iterations: %D\n",eps->max_it);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  tolerance: %G\n",eps->tol);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  convergence test: ");CHKERRQ(ierr);
-    switch(eps->conv) {
-    case EPS_CONV_ABS: 
+    switch (eps->conv) {
+    case EPS_CONV_ABS:
       ierr = PetscViewerASCIIPrintf(viewer,"absolute\n");CHKERRQ(ierr);break;
-    case EPS_CONV_EIG: 
+    case EPS_CONV_EIG:
       ierr = PetscViewerASCIIPrintf(viewer,"relative to the eigenvalue\n");CHKERRQ(ierr);break;
-    case EPS_CONV_NORM: 
+    case EPS_CONV_NORM:
       ierr = PetscViewerASCIIPrintf(viewer,"relative to the eigenvalue and matrix norms\n");CHKERRQ(ierr);break;
     default:
       ierr = PetscViewerASCIIPrintf(viewer,"user-defined\n");CHKERRQ(ierr);break;
     }
-    if (eps->nini!=0) {
+    if (eps->nini) {
       ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial space: %D\n",PetscAbs(eps->nini));CHKERRQ(ierr);
     }
-    if (eps->ninil!=0) {
+    if (eps->ninil) {
       ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial left space: %D\n",PetscAbs(eps->ninil));CHKERRQ(ierr);
     }
     if (eps->nds>0) {
@@ -290,10 +282,20 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
       ierr = (*eps->ops->view)(eps,viewer);CHKERRQ(ierr);
     }
   }
-  if (!eps->ip) { ierr = EPSGetIP(eps,&eps->ip);CHKERRQ(ierr); }
-  ierr = IPView(eps->ip,viewer);CHKERRQ(ierr);
-  if (!eps->OP) { ierr = EPSGetST(eps,&eps->OP);CHKERRQ(ierr); }
-  ierr = STView(eps->OP,viewer);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompareAny((PetscObject)eps,&isexternal,EPSARPACK,EPSBLZPACK,EPSTRLAN,EPSBLOPEX,EPSPRIMME,"");CHKERRQ(ierr);
+  if (!isexternal) {
+    if (!eps->ip) { ierr = EPSGetIP(eps,&eps->ip);CHKERRQ(ierr); }
+    ierr = IPView(eps->ip,viewer);CHKERRQ(ierr);
+    ierr = PetscObjectTypeCompare((PetscObject)eps,EPSPOWER,&ispower);CHKERRQ(ierr);
+    if (!ispower) {
+      if (!eps->ds) { ierr = EPSGetDS(eps,&eps->ds);CHKERRQ(ierr); }
+      ierr = PetscViewerPushFormat(viewer,PETSC_VIEWER_ASCII_INFO);CHKERRQ(ierr);
+      ierr = DSView(eps->ds,viewer);CHKERRQ(ierr);
+      ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
+    }
+  }
+  if (!eps->st) { ierr = EPSGetST(eps,&eps->st);CHKERRQ(ierr); }
+  ierr = STView(eps->st,viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -308,7 +310,7 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
 +  eps - the eigensolver context
 -  viewer - optional visualization context
 
-   Options Database:
+   Options Database Key:
 .  -eps_terse - print only minimal information
 
    Note:
@@ -329,16 +331,14 @@ PetscErrorCode EPSPrintSolution(EPS eps,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(((PetscObject)eps)->comm);
+  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)eps));
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(eps,1,viewer,2);
-  if (!eps->eigr || !eps->eigi || !eps->V) { 
-    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_WRONGSTATE,"EPSSolve must be called first"); 
-  }
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  if (!eps->eigr || !eps->eigi || !eps->V) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"EPSSolve must be called first");
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (!isascii) PetscFunctionReturn(0);
 
-  ierr = PetscOptionsHasName(PETSC_NULL,"-eps_terse",&terse);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,"-eps_terse",&terse);CHKERRQ(ierr);
   if (terse) {
     if (eps->nconv<eps->nev) {
       ierr = PetscViewerASCIIPrintf(viewer," Problem: less than %D eigenvalues converged\n\n",eps->nev);CHKERRQ(ierr);
@@ -353,20 +353,20 @@ PetscErrorCode EPSPrintSolution(EPS eps,PetscViewer viewer)
         for (i=0;i<=(eps->nev-1)/8;i++) {
           ierr = PetscViewerASCIIPrintf(viewer,"\n     ");CHKERRQ(ierr);
           for (j=0;j<PetscMin(8,eps->nev-8*i);j++) {
-            ierr = EPSGetEigenpair(eps,8*i+j,&kr,&ki,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+            ierr = EPSGetEigenpair(eps,8*i+j,&kr,&ki,NULL,NULL);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
             re = PetscRealPart(kr);
             im = PetscImaginaryPart(kr);
 #else
             re = kr;
             im = ki;
-#endif 
+#endif
             if (PetscAbs(re)/PetscAbs(im)<PETSC_SMALL) re = 0.0;
             if (PetscAbs(im)/PetscAbs(re)<PETSC_SMALL) im = 0.0;
             if (im!=0.0) {
               ierr = PetscViewerASCIIPrintf(viewer,"%.5F%+.5Fi",re,im);CHKERRQ(ierr);
             } else {
-              ierr = PetscViewerASCIIPrintf(viewer,"%.5F",re);CHKERRQ(ierr); 
+              ierr = PetscViewerASCIIPrintf(viewer,"%.5F",re);CHKERRQ(ierr);
             }
             if (8*i+j+1<eps->nev) { ierr = PetscViewerASCIIPrintf(viewer,", ");CHKERRQ(ierr); }
           }
@@ -383,7 +383,7 @@ PetscErrorCode EPSPrintSolution(EPS eps,PetscViewer viewer)
            "           k          ||Ax-k%sx||/||kx||\n"
            "   ----------------- ------------------\n",eps->isgeneralized?"B":"");CHKERRQ(ierr);
       for (i=0;i<eps->nconv;i++) {
-        ierr = EPSGetEigenpair(eps,i,&kr,&ki,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+        ierr = EPSGetEigenpair(eps,i,&kr,&ki,NULL,NULL);CHKERRQ(ierr);
         ierr = EPSComputeRelativeError(eps,i,&error);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
         re = PetscRealPart(kr);
@@ -391,11 +391,11 @@ PetscErrorCode EPSPrintSolution(EPS eps,PetscViewer viewer)
 #else
         re = kr;
         im = ki;
-#endif 
+#endif
         if (im!=0.0) {
           ierr = PetscViewerASCIIPrintf(viewer," % 9F%+9F i %12G\n",re,im,error);CHKERRQ(ierr);
         } else {
-          ierr = PetscViewerASCIIPrintf(viewer,"   % 12F       %12G\n",re,error);CHKERRQ(ierr); 
+          ierr = PetscViewerASCIIPrintf(viewer,"   % 12F       %12G\n",re,error);CHKERRQ(ierr);
         }
       }
       ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
@@ -432,8 +432,11 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   PetscFunctionBegin;
   PetscValidPointer(outeps,2);
   *outeps = 0;
+#if !defined(PETSC_USE_DYNAMIC_LIBRARIES)
+  ierr = EPSInitializePackage();CHKERRQ(ierr);
+#endif
 
-  ierr = PetscHeaderCreate(eps,_p_EPS,struct _EPSOps,EPS_CLASSID,-1,"EPS","Eigenvalue Problem Solver","EPS",comm,EPSDestroy,EPSView);CHKERRQ(ierr);
+  ierr = SlepcHeaderCreate(eps,_p_EPS,struct _EPSOps,EPS_CLASSID,"EPS","Eigenvalue Problem Solver","EPS",comm,EPSDestroy,EPSView);CHKERRQ(ierr);
 
   eps->max_it          = 0;
   eps->nev             = 1;
@@ -445,11 +448,13 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   eps->nds             = 0;
   eps->tol             = PETSC_DEFAULT;
   eps->conv            = EPS_CONV_EIG;
-  eps->conv_func       = EPSEigRelativeConverged;
-  eps->conv_ctx        = PETSC_NULL;
+  eps->converged       = EPSConvergedEigRelative;
+  eps->convergedctx    = NULL;
   eps->which           = (EPSWhich)0;
-  eps->which_func      = PETSC_NULL;
-  eps->which_ctx       = PETSC_NULL;
+  eps->comparison      = NULL;
+  eps->comparisonctx   = NULL;
+  eps->arbitrary       = NULL;
+  eps->arbitraryctx    = NULL;
   eps->leftvecs        = PETSC_FALSE;
   eps->trueres         = PETSC_FALSE;
   eps->trackall        = PETSC_FALSE;
@@ -468,9 +473,8 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
 
   eps->V               = 0;
   eps->W               = 0;
-  eps->T               = 0;
   eps->D               = 0;
-  eps->DS              = 0;
+  eps->defl            = 0;
   eps->IS              = 0;
   eps->ISL             = 0;
   eps->t               = 0;
@@ -479,13 +483,14 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   eps->eigi            = 0;
   eps->errest          = 0;
   eps->errest_left     = 0;
-  eps->OP              = 0;
+  eps->st              = 0;
   eps->ip              = 0;
+  eps->ds              = 0;
   eps->rand            = 0;
   eps->data            = 0;
   eps->nconv           = 0;
   eps->its             = 0;
-  eps->perm            = PETSC_NULL;
+  eps->perm            = NULL;
 
   eps->nwork           = 0;
   eps->work            = 0;
@@ -502,11 +507,11 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   *outeps = eps;
   PetscFunctionReturn(0);
 }
- 
+
 #undef __FUNCT__
 #define __FUNCT__ "EPSSetType"
 /*@C
-   EPSSetType - Selects the particular solver to be used in the EPS object. 
+   EPSSetType - Selects the particular solver to be used in the EPS object.
 
    Logically Collective on EPS
 
@@ -515,10 +520,10 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
 -  type - a known method
 
    Options Database Key:
-.  -eps_type <method> - Sets the method; use -help for a list 
-    of available methods 
-    
-   Notes:  
+.  -eps_type <method> - Sets the method; use -help for a list
+    of available methods
+
+   Notes:
    See "slepc/include/slepceps.h" for available methods. The default
    is EPSKRYLOVSCHUR.
 
@@ -528,13 +533,13 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
    maximum flexibility in evaluating the different available methods.
    The EPSSetType() routine is provided for those situations where it
    is necessary to set the iterative solver independently of the command
-   line or options database. 
+   line or options database.
 
    Level: intermediate
 
 .seealso: STSetType(), EPSType
 @*/
-PetscErrorCode EPSSetType(EPS eps,const EPSType type)
+PetscErrorCode EPSSetType(EPS eps,EPSType type)
 {
   PetscErrorCode ierr,(*r)(EPS);
   PetscBool      match;
@@ -543,11 +548,11 @@ PetscErrorCode EPSSetType(EPS eps,const EPSType type)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidCharPointer(type,2);
 
-  ierr = PetscTypeCompare((PetscObject)eps,type,&match);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)eps,type,&match);CHKERRQ(ierr);
   if (match) PetscFunctionReturn(0);
 
-  ierr = PetscFListFind(EPSList,((PetscObject)eps)->comm,type,PETSC_TRUE,(void (**)(void)) &r);CHKERRQ(ierr);
-  if (!r) SETERRQ1(((PetscObject)eps)->comm,PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown EPS type given: %s",type);
+  ierr = PetscFunctionListFind(EPSList,type,&r);CHKERRQ(ierr);
+  if (!r) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown EPS type given: %s",type);
 
   if (eps->ops->destroy) { ierr = (*eps->ops->destroy)(eps);CHKERRQ(ierr); }
   ierr = PetscMemzero(eps->ops,sizeof(struct _EPSOps));CHKERRQ(ierr);
@@ -566,16 +571,16 @@ PetscErrorCode EPSSetType(EPS eps,const EPSType type)
    Not Collective
 
    Input Parameter:
-.  eps - the eigensolver context 
+.  eps - the eigensolver context
 
    Output Parameter:
-.  name - name of EPS method 
+.  name - name of EPS method
 
    Level: intermediate
 
 .seealso: EPSSetType()
 @*/
-PetscErrorCode EPSGetType(EPS eps,const EPSType *type)
+PetscErrorCode EPSGetType(EPS eps,EPSType *type)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
@@ -587,40 +592,37 @@ PetscErrorCode EPSGetType(EPS eps,const EPSType *type)
 #undef __FUNCT__
 #define __FUNCT__ "EPSRegister"
 /*@C
-  EPSRegister - See EPSRegisterDynamic()
-
-  Level: advanced
-@*/
-PetscErrorCode EPSRegister(const char *sname,const char *path,const char *name,PetscErrorCode (*function)(EPS))
-{
-  PetscErrorCode ierr;
-  char           fullname[PETSC_MAX_PATH_LEN];
-
-  PetscFunctionBegin;
-  ierr = PetscFListConcat(path,name,fullname);CHKERRQ(ierr);
-  ierr = PetscFListAdd(&EPSList,sname,fullname,(void (*)(void))function);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "EPSRegisterDestroy"
-/*@
-   EPSRegisterDestroy - Frees the list of EPS methods that were
-   registered by EPSRegisterDynamic().
+   EPSRegister - Adds a method to the eigenproblem solver package.
 
    Not Collective
 
+   Input Parameters:
++  name - name of a new user-defined solver
+-  function - routine to create the solver context
+
+   Notes:
+   EPSRegister() may be called multiple times to add several user-defined solvers.
+
+   Sample usage:
+.vb
+   EPSRegister("my_solver",MySolverCreate);
+.ve
+
+   Then, your solver can be chosen with the procedural interface via
+$     EPSSetType(eps,"my_solver")
+   or at runtime via the option
+$     -eps_type my_solver
+
    Level: advanced
 
-.seealso: EPSRegisterDynamic(), EPSRegisterAll()
+.seealso: EPSRegisterAll()
 @*/
-PetscErrorCode EPSRegisterDestroy(void)
+PetscErrorCode EPSRegister(const char *name,PetscErrorCode (*function)(EPS))
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscFListDestroy(&EPSList);CHKERRQ(ierr);
-  EPSRegisterAllCalled = PETSC_FALSE;
+  ierr = PetscFunctionListAdd(&EPSList,name,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -646,8 +648,9 @@ PetscErrorCode EPSReset(EPS eps)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   if (eps->ops->reset) { ierr = (eps->ops->reset)(eps);CHKERRQ(ierr); }
-  if (eps->OP) { ierr = STReset(eps->OP);CHKERRQ(ierr); }
+  if (eps->st) { ierr = STReset(eps->st);CHKERRQ(ierr); }
   if (eps->ip) { ierr = IPReset(eps->ip);CHKERRQ(ierr); }
+  if (eps->ds) { ierr = DSReset(eps->ds);CHKERRQ(ierr); }
   ierr = VecDestroy(&eps->t);CHKERRQ(ierr);
   ierr = VecDestroy(&eps->D);CHKERRQ(ierr);
   eps->setupcalled = 0;
@@ -677,11 +680,14 @@ PetscErrorCode EPSDestroy(EPS *eps)
   PetscValidHeaderSpecific(*eps,EPS_CLASSID,1);
   if (--((PetscObject)(*eps))->refct > 0) { *eps = 0; PetscFunctionReturn(0); }
   ierr = EPSReset(*eps);CHKERRQ(ierr);
-  ierr = PetscObjectDepublish(*eps);CHKERRQ(ierr);
   if ((*eps)->ops->destroy) { ierr = (*(*eps)->ops->destroy)(*eps);CHKERRQ(ierr); }
-  ierr = STDestroy(&(*eps)->OP);CHKERRQ(ierr);
+  ierr = STDestroy(&(*eps)->st);CHKERRQ(ierr);
   ierr = IPDestroy(&(*eps)->ip);CHKERRQ(ierr);
+  ierr = DSDestroy(&(*eps)->ds);CHKERRQ(ierr);
   ierr = PetscRandomDestroy(&(*eps)->rand);CHKERRQ(ierr);
+  /* just in case the initial vectors have not been used */
+  ierr = SlepcBasisDestroy_Private(&(*eps)->nini,&(*eps)->IS);CHKERRQ(ierr);
+  ierr = SlepcBasisDestroy_Private(&(*eps)->ninil,&(*eps)->ISL);CHKERRQ(ierr);
   ierr = EPSRemoveDeflationSpace(*eps);CHKERRQ(ierr);
   ierr = EPSMonitorCancel(*eps);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(eps);CHKERRQ(ierr);
@@ -702,7 +708,7 @@ PetscErrorCode EPSDestroy(EPS *eps)
    Notes:
    The target is a scalar value used to determine the portion of the spectrum
    of interest. It is used in combination with EPSSetWhichEigenpairs().
-   
+
    Level: beginner
 
 .seealso: EPSGetTarget(), EPSSetWhichEigenpairs()
@@ -715,8 +721,8 @@ PetscErrorCode EPSSetTarget(EPS eps,PetscScalar target)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveScalar(eps,target,2);
   eps->target = target;
-  if (!eps->OP) { ierr = EPSGetST(eps,&eps->OP);CHKERRQ(ierr); }
-  ierr = STSetDefaultShift(eps->OP,target);CHKERRQ(ierr);
+  if (!eps->st) { ierr = EPSGetST(eps,&eps->st);CHKERRQ(ierr); }
+  ierr = STSetDefaultShift(eps->st,target);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -772,9 +778,9 @@ PetscErrorCode EPSGetTarget(EPS eps,PetscScalar* target)
 
    In the command-line option, two values must be provided. For an open interval,
    one can give an infinite, e.g., -eps_interval 1.0,inf or -eps_interval -inf,1.0.
-   An open interval in the programmatic interface can be specified with 
+   An open interval in the programmatic interface can be specified with
    PETSC_MAX_REAL and -PETSC_MAX_REAL.
-   
+
    Level: intermediate
 
 .seealso: EPSGetInterval(), EPSSetWhichEigenpairs()
@@ -785,7 +791,7 @@ PetscErrorCode EPSSetInterval(EPS eps,PetscReal inta,PetscReal intb)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveReal(eps,inta,2);
   PetscValidLogicalCollectiveReal(eps,intb,3);
-  if (inta>=intb) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_WRONG,"Badly defined interval, must be inta<intb");
+  if (inta>=intb) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"Badly defined interval, must be inta<intb");
   eps->inta = inta;
   eps->intb = intb;
   PetscFunctionReturn(0);
@@ -826,7 +832,7 @@ PetscErrorCode EPSGetInterval(EPS eps,PetscReal* inta,PetscReal* intb)
 #undef __FUNCT__
 #define __FUNCT__ "EPSSetST"
 /*@
-   EPSSetST - Associates a spectral transformation object to the eigensolver. 
+   EPSSetST - Associates a spectral transformation object to the eigensolver.
 
    Collective on EPS
 
@@ -851,9 +857,9 @@ PetscErrorCode EPSSetST(EPS eps,ST st)
   PetscValidHeaderSpecific(st,ST_CLASSID,2);
   PetscCheckSameComm(eps,1,st,2);
   ierr = PetscObjectReference((PetscObject)st);CHKERRQ(ierr);
-  ierr = STDestroy(&eps->OP);CHKERRQ(ierr);
-  eps->OP = st;
-  ierr = PetscLogObjectParent(eps,eps->OP);CHKERRQ(ierr);
+  ierr = STDestroy(&eps->st);CHKERRQ(ierr);
+  eps->st = st;
+  ierr = PetscLogObjectParent(eps,eps->st);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -882,18 +888,18 @@ PetscErrorCode EPSGetST(EPS eps,ST *st)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidPointer(st,2);
-  if (!eps->OP) {
-    ierr = STCreate(((PetscObject)eps)->comm,&eps->OP);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent(eps,eps->OP);CHKERRQ(ierr);
+  if (!eps->st) {
+    ierr = STCreate(PetscObjectComm((PetscObject)eps),&eps->st);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(eps,eps->st);CHKERRQ(ierr);
   }
-  *st = eps->OP;
+  *st = eps->st;
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__
 #define __FUNCT__ "EPSSetIP"
 /*@
-   EPSSetIP - Associates an inner product object to the eigensolver. 
+   EPSSetIP - Associates an inner product object to the eigensolver.
 
    Collective on EPS
 
@@ -949,17 +955,83 @@ PetscErrorCode EPSGetIP(EPS eps,IP *ip)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidPointer(ip,2);
   if (!eps->ip) {
-    ierr = IPCreate(((PetscObject)eps)->comm,&eps->ip);CHKERRQ(ierr);
+    ierr = IPCreate(PetscObjectComm((PetscObject)eps),&eps->ip);CHKERRQ(ierr);
     ierr = PetscLogObjectParent(eps,eps->ip);CHKERRQ(ierr);
   }
   *ip = eps->ip;
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__
+#define __FUNCT__ "EPSSetDS"
+/*@
+   EPSSetDS - Associates a direct solver object to the eigensolver.
+
+   Collective on EPS
+
+   Input Parameters:
++  eps - eigensolver context obtained from EPSCreate()
+-  ds  - the direct solver object
+
+   Note:
+   Use EPSGetDS() to retrieve the direct solver context (for example,
+   to free it at the end of the computations).
+
+   Level: advanced
+
+.seealso: EPSGetDS()
+@*/
+PetscErrorCode EPSSetDS(EPS eps,DS ds)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidHeaderSpecific(ds,DS_CLASSID,2);
+  PetscCheckSameComm(eps,1,ds,2);
+  ierr = PetscObjectReference((PetscObject)ds);CHKERRQ(ierr);
+  ierr = DSDestroy(&eps->ds);CHKERRQ(ierr);
+  eps->ds = ds;
+  ierr = PetscLogObjectParent(eps,eps->ds);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "EPSGetDS"
+/*@C
+   EPSGetDS - Obtain the direct solver object associated to the eigensolver object.
+
+   Not Collective
+
+   Input Parameters:
+.  eps - eigensolver context obtained from EPSCreate()
+
+   Output Parameter:
+.  ds - direct solver context
+
+   Level: advanced
+
+.seealso: EPSSetDS()
+@*/
+PetscErrorCode EPSGetDS(EPS eps,DS *ds)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidPointer(ds,2);
+  if (!eps->ds) {
+    ierr = DSCreate(PetscObjectComm((PetscObject)eps),&eps->ds);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(eps,eps->ds);CHKERRQ(ierr);
+  }
+  *ds = eps->ds;
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__
 #define __FUNCT__ "EPSIsGeneralized"
 /*@
-   EPSIsGeneralized - Ask if the EPS object corresponds to a generalized 
+   EPSIsGeneralized - Ask if the EPS object corresponds to a generalized
    eigenvalue problem.
 
    Not collective
@@ -972,12 +1044,13 @@ PetscErrorCode EPSGetIP(EPS eps,IP *ip)
 
    Level: intermediate
 
-.seealso: EPSIsHermitian()
+.seealso: EPSIsHermitian(), EPSIsPositive()
 @*/
 PetscErrorCode EPSIsGeneralized(EPS eps,PetscBool* is)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidPointer(is,2);
   *is = eps->isgeneralized;
   PetscFunctionReturn(0);
 }
@@ -985,7 +1058,7 @@ PetscErrorCode EPSIsGeneralized(EPS eps,PetscBool* is)
 #undef __FUNCT__
 #define __FUNCT__ "EPSIsHermitian"
 /*@
-   EPSIsHermitian - Ask if the EPS object corresponds to a Hermitian 
+   EPSIsHermitian - Ask if the EPS object corresponds to a Hermitian
    eigenvalue problem.
 
    Not collective
@@ -998,12 +1071,40 @@ PetscErrorCode EPSIsGeneralized(EPS eps,PetscBool* is)
 
    Level: intermediate
 
-.seealso: EPSIsGeneralized()
+.seealso: EPSIsGeneralized(), EPSIsPositive()
 @*/
 PetscErrorCode EPSIsHermitian(EPS eps,PetscBool* is)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidPointer(is,2);
   *is = eps->ishermitian;
   PetscFunctionReturn(0);
 }
+
+#undef __FUNCT__
+#define __FUNCT__ "EPSIsPositive"
+/*@
+   EPSIsPositive - Ask if the EPS object corresponds to an eigenvalue
+   problem type that requires a positive (semi-) definite matrix B.
+
+   Not collective
+
+   Input Parameter:
+.  eps - the eigenproblem solver context
+
+   Output Parameter:
+.  is - the answer
+
+   Level: intermediate
+
+.seealso: EPSIsGeneralized(), EPSIsHermitian()
+@*/
+PetscErrorCode EPSIsPositive(EPS eps,PetscBool* is)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidPointer(is,2);
+  *is = eps->ispositive;
+  PetscFunctionReturn(0);
+}
