@@ -1,19 +1,19 @@
 /*
-     This file contains some simple default routines for common operations.  
+     This file contains some simple default routines for common operations.
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,33 +21,34 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/epsimpl.h>   /*I "slepceps.h" I*/
+#include <slepc-private/epsimpl.h>   /*I "slepceps.h" I*/
 #include <slepcblaslapack.h>
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSReset_Default"
 PetscErrorCode EPSReset_Default(EPS eps)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = EPSDefaultFreeWork(eps);CHKERRQ(ierr);
+  ierr = VecDestroyVecs(eps->nwork,&eps->work);CHKERRQ(ierr);
+  eps->nwork = 0;
   ierr = EPSFreeSolution(eps);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSBackTransform_Default"
 PetscErrorCode EPSBackTransform_Default(EPS eps)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = STBackTransform(eps->OP,eps->nconv,eps->eigr,eps->eigi);CHKERRQ(ierr);
+  ierr = STBackTransform(eps->st,eps->nconv,eps->eigr,eps->eigi);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSComputeVectors_Default"
 /*
   EPSComputeVectors_Default - Compute eigenvectors from the vectors
@@ -61,7 +62,7 @@ PetscErrorCode EPSComputeVectors_Default(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSComputeVectors_Hermitian"
 /*
   EPSComputeVectors_Hermitian - Copies the Lanczos vectors as eigenvectors
@@ -80,7 +81,7 @@ PetscErrorCode EPSComputeVectors_Hermitian(EPS eps)
     ierr = VecDuplicate(eps->V[0],&w);CHKERRQ(ierr);
     for (i=0;i<eps->nconv;i++) {
       ierr = VecCopy(eps->V[i],w);CHKERRQ(ierr);
-      ierr = STApply(eps->OP,w,eps->V[i]);CHKERRQ(ierr);
+      ierr = STApply(eps->st,w,eps->V[i]);CHKERRQ(ierr);
       ierr = IPNorm(eps->ip,eps->V[i],&norm);CHKERRQ(ierr);
       ierr = VecScale(eps->V[i],1.0/norm);CHKERRQ(ierr);
     }
@@ -90,100 +91,119 @@ PetscErrorCode EPSComputeVectors_Hermitian(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
+#define __FUNCT__ "EPSComputeVectors_Indefinite"
+/*
+  EPSComputeVectors_Indefinite - similar to the Schur version but
+  for indefinite problems
+ */
+PetscErrorCode EPSComputeVectors_Indefinite(EPS eps)
+{
+  PetscErrorCode ierr;
+  PetscInt       n,ld,i;
+  PetscScalar    *Z;
+  Vec            v;
+#if !defined(PETSC_USE_COMPLEX)
+  PetscScalar    tmp;
+  PetscReal      norm,normi;
+#endif
+
+  PetscFunctionBegin;
+  ierr = DSGetLeadingDimension(eps->ds,&ld);CHKERRQ(ierr);
+  ierr = DSGetDimensions(eps->ds,&n,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
+  ierr = DSVectors(eps->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
+  ierr = DSGetArray(eps->ds,DS_MAT_X,&Z);CHKERRQ(ierr);
+  ierr = SlepcUpdateVectors(n,eps->V,0,n,Z,ld,PETSC_FALSE);CHKERRQ(ierr);
+  ierr = DSRestoreArray(eps->ds,DS_MAT_X,&Z);CHKERRQ(ierr);
+  /* purification */
+  ierr = VecDuplicate(eps->V[0],&v);CHKERRQ(ierr);
+  for (i=0;i<eps->nconv;i++) {
+    ierr = VecCopy(eps->V[i],v);CHKERRQ(ierr);
+    ierr = STApply(eps->st,v,eps->V[i]);CHKERRQ(ierr);
+  }
+  ierr = VecDestroy(&v);CHKERRQ(ierr);
+  /* normalization */
+  for (i=0;i<n;i++) {
+#if !defined(PETSC_USE_COMPLEX)
+    if (eps->eigi[i] != 0.0) {
+      ierr = VecNorm(eps->V[i],NORM_2,&norm);CHKERRQ(ierr);
+      ierr = VecNorm(eps->V[i+1],NORM_2,&normi);CHKERRQ(ierr);
+      tmp = 1.0 / SlepcAbsEigenvalue(norm,normi);
+      ierr = VecScale(eps->V[i],tmp);CHKERRQ(ierr);
+      ierr = VecScale(eps->V[i+1],tmp);CHKERRQ(ierr);
+      i++;
+    } else
+#endif
+    {
+      ierr = VecNormalize(eps->V[i],NULL);CHKERRQ(ierr);
+    }
+  }
+  eps->evecsavailable = PETSC_TRUE;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
 #define __FUNCT__ "EPSComputeVectors_Schur"
 /*
   EPSComputeVectors_Schur - Compute eigenvectors from the vectors
-  provided by the eigensolver. This version is intended for solvers 
+  provided by the eigensolver. This version is intended for solvers
   that provide Schur vectors. Given the partial Schur decomposition
   OP*V=V*T, the following steps are performed:
       1) compute eigenvectors of T: T*Z=Z*D
       2) compute eigenvectors of OP: X=V*Z
-  If left eigenvectors are required then also do Z'*Tl=D*Z', Y=W*Z
+  If left eigenvectors are required then also do Z'*T=D*Z', Y=W*Z
  */
 PetscErrorCode EPSComputeVectors_Schur(EPS eps)
 {
-#if defined(SLEPC_MISSING_LAPACK_TREVC)
-  SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"TREVC - Lapack routine is unavailable.");
-#else
   PetscErrorCode ierr;
-  PetscInt       i;
-  PetscBLASInt   ncv,nconv,mout,info,one = 1; 
-  PetscScalar    *Z,*work,tmp;
-#if defined(PETSC_USE_COMPLEX)
-  PetscReal      *rwork;
-#else 
-  PetscReal      normi;
+  PetscInt       n,i,ld;
+  PetscScalar    *Z;
+#if !defined(PETSC_USE_COMPLEX)
+  PetscScalar    tmp;
+  PetscReal      norm,normi;
 #endif
-  PetscReal      norm;
   Vec            w;
-  
+
   PetscFunctionBegin;
-  ncv = PetscBLASIntCast(eps->ncv);
-  nconv = PetscBLASIntCast(eps->nconv);
   if (eps->ishermitian) {
-    ierr = EPSComputeVectors_Hermitian(eps);CHKERRQ(ierr);
+    if (eps->isgeneralized && !eps->ispositive) {
+      ierr =  EPSComputeVectors_Indefinite(eps);CHKERRQ(ierr);
+    } else {
+      ierr = EPSComputeVectors_Hermitian(eps);CHKERRQ(ierr);
+    }
     PetscFunctionReturn(0);
   }
-
-  ierr = PetscMalloc(nconv*nconv*sizeof(PetscScalar),&Z);CHKERRQ(ierr);
-  ierr = PetscMalloc(3*nconv*sizeof(PetscScalar),&work);CHKERRQ(ierr);
-#if defined(PETSC_USE_COMPLEX)
-  ierr = PetscMalloc(nconv*sizeof(PetscReal),&rwork);CHKERRQ(ierr);
-#endif
+  ierr = DSGetLeadingDimension(eps->ds,&ld);CHKERRQ(ierr);
+  ierr = DSGetDimensions(eps->ds,&n,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
 
   /* right eigenvectors */
-#if !defined(PETSC_USE_COMPLEX)
-  LAPACKtrevc_("R","A",PETSC_NULL,&nconv,eps->T,&ncv,PETSC_NULL,&nconv,Z,&nconv,&nconv,&mout,work,&info);
-#else
-  LAPACKtrevc_("R","A",PETSC_NULL,&nconv,eps->T,&ncv,PETSC_NULL,&nconv,Z,&nconv,&nconv,&mout,work,rwork,&info);
-#endif
-  if (info) SETERRQ1(((PetscObject)eps)->comm,PETSC_ERR_LIB,"Error in Lapack xTREVC %i",info);
+  ierr = DSVectors(eps->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
 
-  /* normalize eigenvectors (when not using purification nor balancing)*/
-  if (!(eps->ispositive || (eps->balance!=EPS_BALANCE_NONE && eps->D))) {
-    for (i=0;i<eps->nconv;i++) {
-#if !defined(PETSC_USE_COMPLEX)
-      if (eps->eigi[i] != 0.0) {
-        norm = BLASnrm2_(&nconv,Z+i*nconv,&one);
-        normi = BLASnrm2_(&nconv,Z+(i+1)*nconv,&one);
-        tmp = 1.0 / SlepcAbsEigenvalue(norm,normi);
-        BLASscal_(&nconv,&tmp,Z+i*nconv,&one);
-        BLASscal_(&nconv,&tmp,Z+(i+1)*nconv,&one);
-        i++;     
-      } else
-#endif
-      {
-        norm = BLASnrm2_(&nconv,Z+i*nconv,&one);
-        tmp = 1.0 / norm;
-        BLASscal_(&nconv,&tmp,Z+i*nconv,&one);
-      }
-    }
-  }
-  
-  /* AV = V * Z */
-  ierr = SlepcUpdateVectors(eps->nconv,eps->V,0,eps->nconv,Z,eps->nconv,PETSC_FALSE);CHKERRQ(ierr);
+  /* V = V * Z */
+  ierr = DSGetArray(eps->ds,DS_MAT_X,&Z);CHKERRQ(ierr);
+  ierr = SlepcUpdateVectors(n,eps->V,0,n,Z,ld,PETSC_FALSE);CHKERRQ(ierr);
+  ierr = DSRestoreArray(eps->ds,DS_MAT_X,&Z);CHKERRQ(ierr);
 
   /* Purify eigenvectors */
   if (eps->ispositive) {
     ierr = VecDuplicate(eps->V[0],&w);CHKERRQ(ierr);
-    for (i=0;i<eps->nconv;i++) {
-      ierr = VecCopy(eps->V[i],w);CHKERRQ(ierr); 
-      ierr = STApply(eps->OP,w,eps->V[i]);CHKERRQ(ierr);
+    for (i=0;i<n;i++) {
+      ierr = VecCopy(eps->V[i],w);CHKERRQ(ierr);
+      ierr = STApply(eps->st,w,eps->V[i]);CHKERRQ(ierr);
     }
     ierr = VecDestroy(&w);CHKERRQ(ierr);
   }
 
   /* Fix eigenvectors if balancing was used */
   if (eps->balance!=EPS_BALANCE_NONE && eps->D) {
-    for (i=0;i<eps->nconv;i++) {
+    for (i=0;i<n;i++) {
       ierr = VecPointwiseDivide(eps->V[i],eps->V[i],eps->D);CHKERRQ(ierr);
     }
   }
 
   /* normalize eigenvectors (when using purification or balancing) */
   if (eps->ispositive || (eps->balance!=EPS_BALANCE_NONE && eps->D)) {
-    for (i=0;i<eps->nconv;i++) {
+    for (i=0;i<n;i++) {
 #if !defined(PETSC_USE_COMPLEX)
       if (eps->eigi[i] != 0.0) {
         ierr = VecNorm(eps->V[i],NORM_2,&norm);CHKERRQ(ierr);
@@ -191,44 +211,45 @@ PetscErrorCode EPSComputeVectors_Schur(EPS eps)
         tmp = 1.0 / SlepcAbsEigenvalue(norm,normi);
         ierr = VecScale(eps->V[i],tmp);CHKERRQ(ierr);
         ierr = VecScale(eps->V[i+1],tmp);CHKERRQ(ierr);
-        i++;     
+        i++;
       } else
 #endif
       {
-        ierr = VecNormalize(eps->V[i],PETSC_NULL);CHKERRQ(ierr);
+        ierr = VecNormalize(eps->V[i],NULL);CHKERRQ(ierr);
       }
     }
   }
-   
+
   /* left eigenvectors */
   if (eps->leftvecs) {
-#if !defined(PETSC_USE_COMPLEX)
-    LAPACKtrevc_("R","A",PETSC_NULL,&nconv,eps->Tl,&ncv,PETSC_NULL,&nconv,Z,&nconv,&nconv,&mout,work,&info);
-#else
-    LAPACKtrevc_("R","A",PETSC_NULL,&nconv,eps->Tl,&ncv,PETSC_NULL,&nconv,Z,&nconv,&nconv,&mout,work,rwork,&info);
-#endif
-    if (info) SETERRQ1(((PetscObject)eps)->comm,PETSC_ERR_LIB,"Error in Lapack xTREVC %i",info);
-
-    /* AW = W * Z */
-    ierr = SlepcUpdateVectors(eps->nconv,eps->W,0,eps->nconv,Z,eps->nconv,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = DSVectors(eps->ds,DS_MAT_Y,NULL,NULL);CHKERRQ(ierr);
+    /* W = W * Z */
+    ierr = DSGetArray(eps->ds,DS_MAT_Y,&Z);CHKERRQ(ierr);
+    ierr = SlepcUpdateVectors(n,eps->W,0,n,Z,ld,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = DSRestoreArray(eps->ds,DS_MAT_Y,&Z);CHKERRQ(ierr);
   }
-   
-  ierr = PetscFree(Z);CHKERRQ(ierr);
-  ierr = PetscFree(work);CHKERRQ(ierr);
-#if defined(PETSC_USE_COMPLEX)
-  ierr = PetscFree(rwork);CHKERRQ(ierr);
-#endif
   eps->evecsavailable = PETSC_TRUE;
   PetscFunctionReturn(0);
-#endif 
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "EPSDefaultGetWork"
-/*
-  EPSDefaultGetWork - Gets a number of work vectors.
- */
-PetscErrorCode EPSDefaultGetWork(EPS eps,PetscInt nw)
+#undef __FUNCT__
+#define __FUNCT__ "EPSSetWorkVecs"
+/*@
+   EPSSetWorkVecs - Sets a number of work vectors into a EPS object
+
+   Collective on EPS
+
+   Input Parameters:
++  eps - eigensolver context
+-  nw  - number of work vectors to allocate
+
+   Developers Note:
+   This is PETSC_EXTERN because it may be required by user plugin EPS
+   implementations.
+
+   Level: developer
+@*/
+PetscErrorCode EPSSetWorkVecs(EPS eps,PetscInt nw)
 {
   PetscErrorCode ierr;
 
@@ -237,78 +258,63 @@ PetscErrorCode EPSDefaultGetWork(EPS eps,PetscInt nw)
     ierr = VecDestroyVecs(eps->nwork,&eps->work);CHKERRQ(ierr);
     eps->nwork = nw;
     ierr = VecDuplicateVecs(eps->t,nw,&eps->work);CHKERRQ(ierr);
-    ierr = PetscLogObjectParents(eps,nw,eps->work);
+    ierr = PetscLogObjectParents(eps,nw,eps->work);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "EPSDefaultFreeWork"
+#undef __FUNCT__
+#define __FUNCT__ "EPSSetWhichEigenpairs_Default"
 /*
-  EPSDefaultFreeWork - Free work vectors.
+  EPSSetWhichEigenpairs_Default - Sets the default value for which,
+  depending on the ST.
  */
-PetscErrorCode EPSDefaultFreeWork(EPS eps)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  ierr = VecDestroyVecs(eps->nwork,&eps->work);CHKERRQ(ierr);
-  eps->nwork = 0;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
-#define __FUNCT__ "EPSDefaultSetWhich"
-/*
-  EPSDefaultSetWhich - Sets the default value for which, depending on the ST.
- */
-PetscErrorCode EPSDefaultSetWhich(EPS eps)
+PetscErrorCode EPSSetWhichEigenpairs_Default(EPS eps)
 {
   PetscBool      target;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscTypeCompareAny((PetscObject)eps->OP,&target,STSINVERT,STCAYLEY,STFOLD,"");CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompareAny((PetscObject)eps->st,&target,STSINVERT,STCAYLEY,STFOLD,"");CHKERRQ(ierr);
   if (target) eps->which = EPS_TARGET_MAGNITUDE;
   else eps->which = EPS_LARGEST_MAGNITUDE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "EPSEigRelativeConverged"
+#undef __FUNCT__
+#define __FUNCT__ "EPSConvergedEigRelative"
 /*
-  EPSDefaultConverged - Checks convergence relative to the eigenvalue.
+  EPSConvergedEigRelative - Checks convergence relative to the eigenvalue.
 */
-PetscErrorCode EPSEigRelativeConverged(EPS eps,PetscScalar eigr,PetscScalar eigi,PetscReal res,PetscReal *errest,void *ctx)
+PetscErrorCode EPSConvergedEigRelative(EPS eps,PetscScalar eigr,PetscScalar eigi,PetscReal res,PetscReal *errest,void *ctx)
 {
   PetscReal w;
 
   PetscFunctionBegin;
   w = SlepcAbsEigenvalue(eigr,eigi);
-  *errest = res;
-  if (w > res) *errest = res / w;
+  *errest = res/w;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "EPSAbsoluteConverged"
+#undef __FUNCT__
+#define __FUNCT__ "EPSConvergedAbsolute"
 /*
-  EPSAbsoluteConverged - Checks convergence absolutely.
+  EPSConvergedAbsolute - Checks convergence absolutely.
 */
-PetscErrorCode EPSAbsoluteConverged(EPS eps,PetscScalar eigr,PetscScalar eigi,PetscReal res,PetscReal *errest,void *ctx)
+PetscErrorCode EPSConvergedAbsolute(EPS eps,PetscScalar eigr,PetscScalar eigi,PetscReal res,PetscReal *errest,void *ctx)
 {
   PetscFunctionBegin;
   *errest = res;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "EPSNormRelativeConverged"
+#undef __FUNCT__
+#define __FUNCT__ "EPSConvergedNormRelative"
 /*
-  EPSNormRelativeConverged - Checks convergence relative to the eigenvalue and 
+  EPSConvergedNormRelative - Checks convergence relative to the eigenvalue and
   the matrix norms.
 */
-PetscErrorCode EPSNormRelativeConverged(EPS eps,PetscScalar eigr,PetscScalar eigi,PetscReal res,PetscReal *errest,void *ctx)
+PetscErrorCode EPSConvergedNormRelative(EPS eps,PetscScalar eigr,PetscScalar eigi,PetscReal res,PetscReal *errest,void *ctx)
 {
   PetscReal w;
 
@@ -318,44 +324,37 @@ PetscErrorCode EPSNormRelativeConverged(EPS eps,PetscScalar eigr,PetscScalar eig
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "EPSComputeTrueResidual"
+#undef __FUNCT__
+#define __FUNCT__ "EPSComputeRitzVector"
 /*
-  EPSComputeTrueResidual - Computes the true residual norm of a given Ritz pair:
-    ||r|| = ||A*x - lambda*B*x||
-  where lambda is the Ritz value and x is the Ritz vector.
+  EPSComputeRitzVector - Computes the current Ritz vector.
 
-  Real lambda:
-    lambda = eigr
-    x = V*Z  (V is an array of nv vectors, Z has length nv)
+  Simple case (complex scalars or real scalars with Zi=NULL):
+    x = V*Zr  (V is an array of nv vectors, Zr has length nv)
 
-  Complex lambda:
-    lambda = eigr+i*eigi
-    x = V*Z[0*nv]+i*V*Z[1*nv]  (Z has length 2*nv)
+  Split case:
+    x = V*Zr  y = V*Zi  (Zr and Zi have length nv)
 */
-PetscErrorCode EPSComputeTrueResidual(EPS eps,PetscScalar eigr,PetscScalar eigi,PetscScalar *Z,Vec *V,PetscInt nv,PetscReal *resnorm)
+PetscErrorCode EPSComputeRitzVector(EPS eps,PetscScalar *Zr,PetscScalar *Zi,Vec *V,PetscInt nv,Vec x,Vec y)
 {
   PetscErrorCode ierr;
-  Vec            x,y,z=0;
   PetscReal      norm;
-  
-  PetscFunctionBegin;
-  /* allocate workspace */
-  ierr = VecDuplicate(V[0],&x);CHKERRQ(ierr);
-  ierr = VecDuplicate(V[0],&y);CHKERRQ(ierr);
-  if (!eps->ishermitian && eps->ispositive) { ierr = VecDuplicate(V[0],&z);CHKERRQ(ierr); }
+#if !defined(PETSC_USE_COMPLEX)
+  Vec            z;
+#endif
 
+  PetscFunctionBegin;
   /* compute eigenvector */
-  ierr = SlepcVecMAXPBY(x,0.0,1.0,nv,Z,V);CHKERRQ(ierr);
+  ierr = SlepcVecMAXPBY(x,0.0,1.0,nv,Zr,V);CHKERRQ(ierr);
 
   /* purify eigenvector in positive generalized problems */
   if (eps->ispositive) {
-    ierr = STApply(eps->OP,x,y);CHKERRQ(ierr);
+    ierr = STApply(eps->st,x,y);CHKERRQ(ierr);
     if (eps->ishermitian) {
       ierr = IPNorm(eps->ip,y,&norm);CHKERRQ(ierr);
     } else {
-      ierr = VecNorm(y,NORM_2,&norm);CHKERRQ(ierr);          
-    } 
+      ierr = VecNorm(y,NORM_2,&norm);CHKERRQ(ierr);
+    }
     ierr = VecScale(y,1.0/norm);CHKERRQ(ierr);
     ierr = VecCopy(y,x);CHKERRQ(ierr);
   }
@@ -366,31 +365,27 @@ PetscErrorCode EPSComputeTrueResidual(EPS eps,PetscScalar eigr,PetscScalar eigi,
   }
 #if !defined(PETSC_USE_COMPLEX)
   /* compute imaginary part of eigenvector */
-  if (!eps->ishermitian && eigi != 0.0) {
-    ierr = SlepcVecMAXPBY(y,0.0,1.0,nv,Z+nv,V);CHKERRQ(ierr);
+  if (Zi) {
+    ierr = SlepcVecMAXPBY(y,0.0,1.0,nv,Zi,V);CHKERRQ(ierr);
     if (eps->ispositive) {
-      ierr = STApply(eps->OP,y,z);CHKERRQ(ierr);
-      ierr = VecNorm(z,NORM_2,&norm);CHKERRQ(ierr);          
+      ierr = VecDuplicate(V[0],&z);CHKERRQ(ierr);
+      ierr = STApply(eps->st,y,z);CHKERRQ(ierr);
+      ierr = VecNorm(z,NORM_2,&norm);CHKERRQ(ierr);
       ierr = VecScale(z,1.0/norm);CHKERRQ(ierr);
       ierr = VecCopy(z,y);CHKERRQ(ierr);
+      ierr = VecDestroy(&z);CHKERRQ(ierr);
     }
     if (eps->balance!=EPS_BALANCE_NONE && eps->D) {
       ierr = VecPointwiseDivide(y,y,eps->D);CHKERRQ(ierr);
       ierr = VecNormalize(y,&norm);CHKERRQ(ierr);
     }
-  }
+  } else
 #endif
-  /* compute relative error and update convergence flag */
-  ierr = EPSComputeResidualNorm_Private(eps,eigr,eigi,x,y,resnorm);
-
-  /* free workspace */
-  ierr = VecDestroy(&x);CHKERRQ(ierr);
-  ierr = VecDestroy(&y);CHKERRQ(ierr);
-  ierr = VecDestroy(&z);CHKERRQ(ierr);
+  { ierr = VecSet(y,0.0);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSBuildBalance_Krylov"
 /*
   EPSBuildBalance_Krylov - uses a Krylov subspace method to compute the
@@ -424,20 +419,20 @@ PetscErrorCode EPSBuildBalance_Krylov(EPS eps)
 
     /* Compute p=DA(D\z) */
     ierr = VecPointwiseDivide(r,z,eps->D);CHKERRQ(ierr);
-    ierr = STApply(eps->OP,r,p);CHKERRQ(ierr);
+    ierr = STApply(eps->st,r,p);CHKERRQ(ierr);
     ierr = VecPointwiseMult(p,p,eps->D);CHKERRQ(ierr);
     if (j==0) {
       /* Estimate the matrix inf-norm */
       ierr = VecAbs(p);CHKERRQ(ierr);
-      ierr = VecMax(p,PETSC_NULL,&norma);CHKERRQ(ierr);
+      ierr = VecMax(p,NULL,&norma);CHKERRQ(ierr);
     }
     if (eps->balance == EPS_BALANCE_TWOSIDE) {
       /* Compute r=D\(A'Dz) */
       ierr = VecPointwiseMult(z,z,eps->D);CHKERRQ(ierr);
-      ierr = STApplyTranspose(eps->OP,z,r);CHKERRQ(ierr);
+      ierr = STApplyTranspose(eps->st,z,r);CHKERRQ(ierr);
       ierr = VecPointwiseDivide(r,r,eps->D);CHKERRQ(ierr);
     }
-    
+
     /* Adjust values of D */
     ierr = VecGetArrayRead(r,&pr);CHKERRQ(ierr);
     ierr = VecGetArrayRead(p,&pp);CHKERRQ(ierr);
