@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,10 +21,10 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/epsimpl.h>       /*I "slepceps.h" I*/
-#include <private/ipimpl.h>
+#include <slepc-private/epsimpl.h>       /*I "slepceps.h" I*/
+#include <slepc-private/ipimpl.h>
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetUp"
 /*@
    EPSSetUp - Sets up all the internal data structures necessary for the
@@ -38,7 +38,7 @@
 
    Notes:
    This function need not be called explicitly in most cases, since EPSSolve()
-   calls it. It can be useful when one wants to measure the set-up time 
+   calls it. It can be useful when one wants to measure the set-up time
    separately from the solve time.
 
    Level: advanced
@@ -48,88 +48,97 @@
 PetscErrorCode EPSSetUp(EPS eps)
 {
   PetscErrorCode ierr;
-  Mat            A,B; 
-  PetscInt       i,k;
+  Mat            A,B;
+  PetscInt       i,k,nmat;
   PetscBool      flg,lindep;
   Vec            *newDS;
   PetscReal      norm;
 #if defined(PETSC_USE_COMPLEX)
   PetscScalar    sigma;
 #endif
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   if (eps->setupcalled) PetscFunctionReturn(0);
   ierr = PetscLogEventBegin(EPS_SetUp,eps,0,0,0);CHKERRQ(ierr);
 
+  /* reset the convergence flag from the previous solves */
+  eps->reason = EPS_CONVERGED_ITERATING;
+
   /* Set default solver type (EPSSetFromOptions was not called) */
   if (!((PetscObject)eps)->type_name) {
     ierr = EPSSetType(eps,EPSKRYLOVSCHUR);CHKERRQ(ierr);
   }
-  if (!eps->OP) { ierr = EPSGetST(eps,&eps->OP);CHKERRQ(ierr); }
-  if (!((PetscObject)eps->OP)->type_name) {
-    ierr = PetscTypeCompareAny((PetscObject)eps,&flg,EPSGD,EPSJD,"");CHKERRQ(ierr);
-    ierr = STSetType(eps->OP,flg?STPRECOND:STSHIFT);CHKERRQ(ierr);
+  if (!eps->st) { ierr = EPSGetST(eps,&eps->st);CHKERRQ(ierr); }
+  if (!((PetscObject)eps->st)->type_name) {
+    ierr = PetscObjectTypeCompareAny((PetscObject)eps,&flg,EPSGD,EPSJD,EPSRQCG,EPSBLOPEX,EPSPRIMME,"");CHKERRQ(ierr);
+    ierr = STSetType(eps->st,flg?STPRECOND:STSHIFT);CHKERRQ(ierr);
   }
   if (!eps->ip) { ierr = EPSGetIP(eps,&eps->ip);CHKERRQ(ierr); }
   if (!((PetscObject)eps->ip)->type_name) {
-    ierr = IPSetDefaultType_Private(eps->ip);CHKERRQ(ierr);
+    ierr = IPSetType_Default(eps->ip);CHKERRQ(ierr);
   }
+  if (!eps->ds) { ierr = EPSGetDS(eps,&eps->ds);CHKERRQ(ierr); }
+  ierr = DSReset(eps->ds);CHKERRQ(ierr);
   if (!((PetscObject)eps->rand)->type_name) {
     ierr = PetscRandomSetFromOptions(eps->rand);CHKERRQ(ierr);
   }
-  
+
   /* Set problem dimensions */
-  ierr = STGetOperators(eps->OP,&A,&B);CHKERRQ(ierr);
-  if (!A) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_WRONGSTATE,"EPSSetOperators must be called first"); 
-  ierr = MatGetSize(A,&eps->n,PETSC_NULL);CHKERRQ(ierr);
-  ierr = MatGetLocalSize(A,&eps->nloc,PETSC_NULL);CHKERRQ(ierr);
+  ierr = STGetNumMatrices(eps->st,&nmat);CHKERRQ(ierr);
+  if (!nmat) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"EPSSetOperators must be called first");
+  ierr = STGetOperators(eps->st,0,&A);CHKERRQ(ierr);
+  ierr = MatGetSize(A,&eps->n,NULL);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(A,&eps->nloc,NULL);CHKERRQ(ierr);
   ierr = VecDestroy(&eps->t);CHKERRQ(ierr);
-  ierr = SlepcMatGetVecsTemplate(A,&eps->t,PETSC_NULL);CHKERRQ(ierr);
+  ierr = SlepcMatGetVecsTemplate(A,&eps->t,NULL);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent(eps,eps->t);CHKERRQ(ierr);
 
   /* Set default problem type */
   if (!eps->problem_type) {
-    if (B==PETSC_NULL) {
+    if (nmat==1) {
       ierr = EPSSetProblemType(eps,EPS_NHEP);CHKERRQ(ierr);
     } else {
       ierr = EPSSetProblemType(eps,EPS_GNHEP);CHKERRQ(ierr);
     }
-  } else if (!B && eps->isgeneralized) {
+  } else if (nmat==1 && eps->isgeneralized) {
     ierr = PetscInfo(eps,"Eigenproblem set as generalized but no matrix B was provided; reverting to a standard eigenproblem\n");CHKERRQ(ierr);
     eps->isgeneralized = PETSC_FALSE;
     eps->problem_type = eps->ishermitian? EPS_HEP: EPS_NHEP;
-  } else if (B && !eps->isgeneralized) {
-    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_INCOMP,"Inconsistent EPS state"); 
-  }
+  } else if (nmat>1 && !eps->isgeneralized) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_INCOMP,"Inconsistent EPS state");
 #if defined(PETSC_USE_COMPLEX)
-  ierr = STGetShift(eps->OP,&sigma);CHKERRQ(ierr);
-  if (eps->ishermitian && PetscImaginaryPart(sigma) != 0.0)
-    SETERRQ(((PetscObject)eps)->comm,1,"Hermitian problems are not compatible with complex shifts");
+  ierr = STGetShift(eps->st,&sigma);CHKERRQ(ierr);
+  if (eps->ishermitian && PetscImaginaryPart(sigma) != 0.0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Hermitian problems are not compatible with complex shifts");
 #endif
-  if (eps->ishermitian && eps->leftvecs)
-    SETERRQ(((PetscObject)eps)->comm,1,"Requesting left eigenvectors not allowed in Hermitian problems");
-  
-  if (eps->ispositive) {
-    ierr = STGetBilinearForm(eps->OP,&B);CHKERRQ(ierr);
+  if (eps->ishermitian && eps->leftvecs) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Requesting left eigenvectors not allowed in Hermitian problems");
+
+  if (eps->ispositive || (eps->isgeneralized && eps->ishermitian)) {
+    ierr = STGetBilinearForm(eps->st,&B);CHKERRQ(ierr);
     ierr = IPSetMatrix(eps->ip,B);CHKERRQ(ierr);
     ierr = MatDestroy(&B);CHKERRQ(ierr);
+    if (!eps->ispositive) {
+      ierr = IPSetType(eps->ip,IPINDEFINITE);CHKERRQ(ierr);
+    }
   } else {
-    ierr = IPSetMatrix(eps->ip,PETSC_NULL);CHKERRQ(ierr);
+    ierr = IPSetMatrix(eps->ip,NULL);CHKERRQ(ierr);
   }
-  
+
   if (eps->nev > eps->n) eps->nev = eps->n;
   if (eps->ncv > eps->n) eps->ncv = eps->n;
 
   /* initialization of matrix norms */
   if (eps->nrma == PETSC_DETERMINE) {
     ierr = MatHasOperation(A,MATOP_NORM,&flg);CHKERRQ(ierr);
-    if (flg) { ierr = MatNorm(A,NORM_INFINITY,&eps->nrma);CHKERRQ(ierr); }
-    else eps->nrma = 1.0;
+    if (flg) {
+      ierr = MatNorm(A,NORM_INFINITY,&eps->nrma);CHKERRQ(ierr);
+    } else eps->nrma = 1.0;
   }
   if (eps->nrmb == PETSC_DETERMINE) {
+    if (nmat>1) { ierr = STGetOperators(eps->st,1,&B);CHKERRQ(ierr); }
     ierr = MatHasOperation(B,MATOP_NORM,&flg);CHKERRQ(ierr);
-    if (flg) { ierr = MatNorm(B,NORM_INFINITY,&eps->nrmb);CHKERRQ(ierr); }
-    else eps->nrmb = 1.0;
+    if (flg) {
+      ierr = MatNorm(B,NORM_INFINITY,&eps->nrmb);CHKERRQ(ierr);
+    } else eps->nrmb = 1.0;
   }
 
   if (!eps->balance) eps->balance = EPS_BALANCE_NONE;
@@ -137,98 +146,122 @@ PetscErrorCode EPSSetUp(EPS eps)
   /* call specific solver setup */
   ierr = (*eps->ops->setup)(eps);CHKERRQ(ierr);
 
+  /* check extraction */
+  ierr = PetscObjectTypeCompareAny((PetscObject)eps->st,&flg,STPRECOND,STSHIFT,"");CHKERRQ(ierr);
+  if (!flg && eps->extraction && eps->extraction!=EPS_RITZ) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Cannot use a spectral transformation combined with harmonic extraction");
+
   /* set tolerance if not yet set */
   if (eps->tol==PETSC_DEFAULT) eps->tol = SLEPC_DEFAULT_TOL;
 
+  /* set eigenvalue comparison */
+  switch (eps->which) {
+    case EPS_LARGEST_MAGNITUDE:
+      eps->comparison    = SlepcCompareLargestMagnitude;
+      eps->comparisonctx = NULL;
+      break;
+    case EPS_SMALLEST_MAGNITUDE:
+      eps->comparison    = SlepcCompareSmallestMagnitude;
+      eps->comparisonctx = NULL;
+      break;
+    case EPS_LARGEST_REAL:
+      eps->comparison    = SlepcCompareLargestReal;
+      eps->comparisonctx = NULL;
+      break;
+    case EPS_SMALLEST_REAL:
+      eps->comparison    = SlepcCompareSmallestReal;
+      eps->comparisonctx = NULL;
+      break;
+    case EPS_LARGEST_IMAGINARY:
+      eps->comparison    = SlepcCompareLargestImaginary;
+      eps->comparisonctx = NULL;
+      break;
+    case EPS_SMALLEST_IMAGINARY:
+      eps->comparison    = SlepcCompareSmallestImaginary;
+      eps->comparisonctx = NULL;
+      break;
+    case EPS_TARGET_MAGNITUDE:
+      eps->comparison    = SlepcCompareTargetMagnitude;
+      eps->comparisonctx = &eps->target;
+      break;
+    case EPS_TARGET_REAL:
+      eps->comparison    = SlepcCompareTargetReal;
+      eps->comparisonctx = &eps->target;
+      break;
+    case EPS_TARGET_IMAGINARY:
+      eps->comparison    = SlepcCompareTargetImaginary;
+      eps->comparisonctx = &eps->target;
+      break;
+    case EPS_ALL:
+      eps->comparison    = SlepcCompareSmallestReal;
+      eps->comparisonctx = NULL;
+      break;
+    case EPS_WHICH_USER:
+      break;
+  }
+
   /* Build balancing matrix if required */
   if (!eps->ishermitian && (eps->balance==EPS_BALANCE_ONESIDE || eps->balance==EPS_BALANCE_TWOSIDE)) {
     if (!eps->D) {
       ierr = VecDuplicate(eps->V[0],&eps->D);CHKERRQ(ierr);
+      ierr = PetscLogObjectParent(eps,eps->D);CHKERRQ(ierr);
     } else {
       ierr = VecSet(eps->D,1.0);CHKERRQ(ierr);
     }
     ierr = EPSBuildBalance_Krylov(eps);CHKERRQ(ierr);
-    ierr = STSetBalanceMatrix(eps->OP,eps->D);CHKERRQ(ierr);
+    ierr = STSetBalanceMatrix(eps->st,eps->D);CHKERRQ(ierr);
   }
 
   /* Setup ST */
-  ierr = STSetUp(eps->OP);CHKERRQ(ierr); 
-  
-  ierr = PetscTypeCompare((PetscObject)eps->OP,STCAYLEY,&flg);CHKERRQ(ierr);
-  if (flg && eps->problem_type == EPS_PGNHEP)
-    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Cayley spectral transformation is not compatible with PGNHEP");
+  ierr = STSetUp(eps->st);CHKERRQ(ierr);
 
-  ierr = PetscTypeCompare((PetscObject)eps->OP,STFOLD,&flg);CHKERRQ(ierr);
-  if (flg && !eps->ishermitian)
-    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Fold spectral transformation requires a Hermitian problem");
+  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STCAYLEY,&flg);CHKERRQ(ierr);
+  if (flg && eps->problem_type == EPS_PGNHEP) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Cayley spectral transformation is not compatible with PGNHEP");
+
+  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STFOLD,&flg);CHKERRQ(ierr);
+  if (flg && !eps->ishermitian) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Fold spectral transformation requires a Hermitian problem");
 
   if (eps->nds>0) {
     if (!eps->ds_ortho) {
-      /* allocate memory and copy deflation basis vectors into DS */
+      /* allocate memory and copy deflation basis vectors into defl */
       ierr = VecDuplicateVecs(eps->t,eps->nds,&newDS);CHKERRQ(ierr);
       for (i=0;i<eps->nds;i++) {
-        ierr = VecCopy(eps->DS[i],newDS[i]);CHKERRQ(ierr);
-        ierr = VecDestroy(&eps->DS[i]);CHKERRQ(ierr);
+        ierr = VecCopy(eps->defl[i],newDS[i]);CHKERRQ(ierr);
+        ierr = VecDestroy(&eps->defl[i]);CHKERRQ(ierr);
       }
-      ierr = PetscFree(eps->DS);CHKERRQ(ierr);
-      eps->DS = newDS;
-      /* orthonormalize vectors in DS */
+      ierr = PetscFree(eps->defl);CHKERRQ(ierr);
+      eps->defl = newDS;
+      ierr = PetscLogObjectParents(eps,eps->nds,eps->defl);CHKERRQ(ierr);
+      /* orthonormalize vectors in defl */
       k = 0;
       for (i=0;i<eps->nds;i++) {
-        ierr = IPOrthogonalize(eps->ip,0,PETSC_NULL,k,PETSC_NULL,eps->DS,eps->DS[k],PETSC_NULL,&norm,&lindep);CHKERRQ(ierr); 
+        ierr = IPOrthogonalize(eps->ip,0,NULL,k,NULL,eps->defl,eps->defl[k],NULL,&norm,&lindep);CHKERRQ(ierr);
         if (norm==0.0 || lindep) {
           ierr = PetscInfo(eps,"Linearly dependent deflation vector found, removing...\n");CHKERRQ(ierr);
         } else {
-          ierr = VecScale(eps->DS[k],1.0/norm);CHKERRQ(ierr);
+          ierr = VecScale(eps->defl[k],1.0/norm);CHKERRQ(ierr);
           k++;
         }
       }
-      for (i=k;i<eps->nds;i++) { ierr = VecDestroy(&eps->DS[i]);CHKERRQ(ierr); }
+      for (i=k;i<eps->nds;i++) { ierr = VecDestroy(&eps->defl[i]);CHKERRQ(ierr); }
       eps->nds = k;
       eps->ds_ortho = PETSC_TRUE;
     }
   }
-  ierr = STCheckNullSpace(eps->OP,eps->nds,eps->DS);CHKERRQ(ierr);
+  ierr = STCheckNullSpace(eps->st,eps->nds,eps->defl);CHKERRQ(ierr);
 
   /* process initial vectors */
   if (eps->nini<0) {
     eps->nini = -eps->nini;
-    if (eps->nini>eps->ncv) SETERRQ(((PetscObject)eps)->comm,1,"The number of initial vectors is larger than ncv");
-    k = 0;
-    for (i=0;i<eps->nini;i++) {
-      ierr = VecCopy(eps->IS[i],eps->V[k]);CHKERRQ(ierr);
-      ierr = VecDestroy(&eps->IS[i]);CHKERRQ(ierr);
-      ierr = IPOrthogonalize(eps->ip,eps->nds,eps->DS,k,PETSC_NULL,eps->V,eps->V[k],PETSC_NULL,&norm,&lindep);CHKERRQ(ierr); 
-      if (norm==0.0 || lindep) {
-        ierr = PetscInfo(eps,"Linearly dependent initial vector found, removing...\n");CHKERRQ(ierr);
-      } else {
-        ierr = VecScale(eps->V[k],1.0/norm);CHKERRQ(ierr);
-        k++;
-      }
-    }
-    eps->nini = k;
-    ierr = PetscFree(eps->IS);CHKERRQ(ierr);
+    if (eps->nini>eps->ncv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"The number of initial vectors is larger than ncv");
+    ierr = IPOrthonormalizeBasis_Private(eps->ip,&eps->nini,&eps->IS,eps->V);CHKERRQ(ierr);
   }
   if (eps->ninil<0) {
     if (!eps->leftvecs) {
       ierr = PetscInfo(eps,"Ignoring initial left vectors\n");CHKERRQ(ierr);
     } else {
       eps->ninil = -eps->ninil;
-      if (eps->ninil>eps->ncv) SETERRQ(((PetscObject)eps)->comm,1,"The number of initial left vectors is larger than ncv");
-      k = 0;
-      for (i=0;i<eps->ninil;i++) {
-        ierr = VecCopy(eps->ISL[i],eps->W[k]);CHKERRQ(ierr);
-        ierr = VecDestroy(&eps->ISL[i]);CHKERRQ(ierr);
-        ierr = IPOrthogonalize(eps->ip,0,PETSC_NULL,k,PETSC_NULL,eps->W,eps->W[k],PETSC_NULL,&norm,&lindep);CHKERRQ(ierr); 
-        if (norm==0.0 || lindep) {
-          ierr = PetscInfo(eps,"Linearly dependent initial left vector found, removing...\n");CHKERRQ(ierr);
-        } else {
-          ierr = VecScale(eps->W[k],1.0/norm);CHKERRQ(ierr);
-          k++;
-        }
-      }
-      eps->ninil = k;
-      ierr = PetscFree(eps->ISL);CHKERRQ(ierr);
+      if (eps->ninil>eps->ncv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"The number of initial left vectors is larger than ncv");
+      ierr = IPOrthonormalizeBasis_Private(eps->ip,&eps->ninil,&eps->ISL,eps->W);CHKERRQ(ierr);
     }
   }
 
@@ -237,7 +270,7 @@ PetscErrorCode EPSSetUp(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetOperators"
 /*@
    EPSSetOperators - Sets the matrices associated with the eigenvalue problem.
@@ -249,8 +282,8 @@ PetscErrorCode EPSSetUp(EPS eps)
 .  A  - the matrix associated with the eigensystem
 -  B  - the second matrix in the case of generalized eigenproblems
 
-   Notes: 
-   To specify a standard eigenproblem, use PETSC_NULL for parameter B.
+   Notes:
+   To specify a standard eigenproblem, use NULL for parameter B.
 
    It must be called after EPSSetUp(). If it is called again after EPSSetUp() then
    the EPS object is reset.
@@ -262,7 +295,8 @@ PetscErrorCode EPSSetUp(EPS eps)
 PetscErrorCode EPSSetOperators(EPS eps,Mat A,Mat B)
 {
   PetscErrorCode ierr;
-  PetscInt       m,n,m0;
+  PetscInt       m,n,m0,nmat;
+  Mat            mat[2];
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
@@ -273,22 +307,27 @@ PetscErrorCode EPSSetOperators(EPS eps,Mat A,Mat B)
 
   /* Check for square matrices */
   ierr = MatGetSize(A,&m,&n);CHKERRQ(ierr);
-  if (m!=n) SETERRQ(((PetscObject)eps)->comm,1,"A is a non-square matrix");
-  if (B) { 
+  if (m!=n) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"A is a non-square matrix");
+  if (B) {
     ierr = MatGetSize(B,&m0,&n);CHKERRQ(ierr);
-    if (m0!=n) SETERRQ(((PetscObject)eps)->comm,1,"B is a non-square matrix");
-    if (m!=m0) SETERRQ(((PetscObject)eps)->comm,1,"Dimensions of A and B do not match");
+    if (m0!=n) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"B is a non-square matrix");
+    if (m!=m0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_INCOMP,"Dimensions of A and B do not match");
   }
 
   if (eps->setupcalled) { ierr = EPSReset(eps);CHKERRQ(ierr); }
-  if (!eps->OP) { ierr = EPSGetST(eps,&eps->OP);CHKERRQ(ierr); }
-  ierr = STSetOperators(eps->OP,A,B);CHKERRQ(ierr);
+  if (!eps->st) { ierr = EPSGetST(eps,&eps->st);CHKERRQ(ierr); }
+  mat[0] = A;
+  if (B) {
+    mat[1] = B;
+    nmat = 2;
+  } else nmat = 1;
+  ierr = STSetOperators(eps->st,nmat,mat);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__
 #define __FUNCT__ "EPSGetOperators"
-/*@
+/*@C
    EPSGetOperators - Gets the matrices associated with the eigensystem.
 
    Collective on EPS and Mat
@@ -308,17 +347,23 @@ PetscErrorCode EPSGetOperators(EPS eps,Mat *A,Mat *B)
 {
   PetscErrorCode ierr;
   ST             st;
+  PetscInt       k;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  if (A) PetscValidPointer(A,2);
-  if (B) PetscValidPointer(B,3);
   ierr = EPSGetST(eps,&st);CHKERRQ(ierr);
-  ierr = STGetOperators(st,A,B);CHKERRQ(ierr);
+  if (A) { ierr = STGetOperators(st,0,A);CHKERRQ(ierr); }
+  if (B) {
+    ierr = STGetNumMatrices(st,&k);CHKERRQ(ierr);
+    if (k==1) B = NULL;
+    else {
+      ierr = STGetOperators(st,1,B);CHKERRQ(ierr);
+    }
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetDeflationSpace"
 /*@
    EPSSetDeflationSpace - Specify a basis of vectors that constitute
@@ -329,12 +374,12 @@ PetscErrorCode EPSGetOperators(EPS eps,Mat *A,Mat *B)
    Input Parameter:
 +  eps   - the eigenproblem solver context
 .  n     - number of vectors
--  ds    - set of basis vectors of the deflation space
+-  v     - set of basis vectors of the deflation space
 
    Notes:
    When a deflation space is given, the eigensolver seeks the eigensolution
    in the restriction of the problem to the orthogonal complement of this
-   space. This can be used for instance in the case that an invariant 
+   space. This can be used for instance in the case that an invariant
    subspace is known beforehand (such as the nullspace of the matrix).
 
    Basis vectors set by a previous call to EPSSetDeflationSpace() are
@@ -350,25 +395,26 @@ PetscErrorCode EPSGetOperators(EPS eps,Mat *A,Mat *B)
 
 .seealso: EPSRemoveDeflationSpace()
 @*/
-PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec *ds)
+PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec *v)
 {
   PetscErrorCode ierr;
   PetscInt       i;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveInt(eps,n,2);
-  if (n<0) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument n out of range"); 
+  if (n<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument n out of range");
 
   /* free previous vectors */
   ierr = EPSRemoveDeflationSpace(eps);CHKERRQ(ierr);
 
   /* get references of passed vectors */
   if (n>0) {
-    ierr = PetscMalloc(n*sizeof(Vec),&eps->DS);CHKERRQ(ierr);
+    ierr = PetscMalloc(n*sizeof(Vec),&eps->defl);CHKERRQ(ierr);
+    ierr = PetscLogObjectMemory(eps,n*sizeof(Vec));CHKERRQ(ierr);
     for (i=0;i<n;i++) {
-      ierr = PetscObjectReference((PetscObject)ds[i]);CHKERRQ(ierr);
-      eps->DS[i] = ds[i];
+      ierr = PetscObjectReference((PetscObject)v[i]);CHKERRQ(ierr);
+      eps->defl[i] = v[i];
     }
     eps->setupcalled = 0;
     eps->ds_ortho = PETSC_FALSE;
@@ -378,7 +424,7 @@ PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec *ds)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSRemoveDeflationSpace"
 /*@
    EPSRemoveDeflationSpace - Removes the deflation space.
@@ -395,17 +441,17 @@ PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec *ds)
 PetscErrorCode EPSRemoveDeflationSpace(EPS eps)
 {
   PetscErrorCode ierr;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  ierr = VecDestroyVecs(eps->nds,&eps->DS);CHKERRQ(ierr);
+  ierr = VecDestroyVecs(eps->nds,&eps->defl);CHKERRQ(ierr);
   eps->nds = 0;
   eps->setupcalled = 0;
   eps->ds_ortho = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetInitialSpace"
 /*@
    EPSSetInitialSpace - Specify a basis of vectors that constitute the initial
@@ -438,36 +484,17 @@ PetscErrorCode EPSRemoveDeflationSpace(EPS eps)
 PetscErrorCode EPSSetInitialSpace(EPS eps,PetscInt n,Vec *is)
 {
   PetscErrorCode ierr;
-  PetscInt       i;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveInt(eps,n,2);
-  if (n<0) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative"); 
-
-  /* free previous non-processed vectors */
-  if (eps->nini<0) {
-    for (i=0;i<-eps->nini;i++) {
-      ierr = VecDestroy(&eps->IS[i]);CHKERRQ(ierr);
-    }
-    ierr = PetscFree(eps->IS);CHKERRQ(ierr);
-  }
-
-  /* get references of passed vectors */
-  if (n>0) {
-    ierr = PetscMalloc(n*sizeof(Vec),&eps->IS);CHKERRQ(ierr);
-    for (i=0;i<n;i++) {
-      ierr = PetscObjectReference((PetscObject)is[i]);CHKERRQ(ierr);
-      eps->IS[i] = is[i];
-    }
-    eps->setupcalled = 0;
-  }
-
-  eps->nini = -n;
+  if (n<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
+  ierr = SlepcBasisReference_Private(n,is,&eps->nini,&eps->IS);CHKERRQ(ierr);
+  if (n>0) eps->setupcalled = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSSetInitialSpaceLeft"
 /*@
    EPSSetInitialSpaceLeft - Specify a basis of vectors that constitute the initial
@@ -501,32 +528,13 @@ PetscErrorCode EPSSetInitialSpace(EPS eps,PetscInt n,Vec *is)
 PetscErrorCode EPSSetInitialSpaceLeft(EPS eps,PetscInt n,Vec *is)
 {
   PetscErrorCode ierr;
-  PetscInt       i;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveInt(eps,n,2);
-  if (n<0) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative"); 
-
-  /* free previous non-processed vectors */
-  if (eps->ninil<0) {
-    for (i=0;i<-eps->ninil;i++) {
-      ierr = VecDestroy(&eps->ISL[i]);CHKERRQ(ierr);
-    }
-    ierr = PetscFree(eps->ISL);CHKERRQ(ierr);
-  }
-
-  /* get references of passed vectors */
-  if (n>0) {
-    ierr = PetscMalloc(n*sizeof(Vec),&eps->ISL);CHKERRQ(ierr);
-    for (i=0;i<n;i++) {
-      ierr = PetscObjectReference((PetscObject)is[i]);CHKERRQ(ierr);
-      eps->ISL[i] = is[i];
-    }
-    eps->setupcalled = 0;
-  }
-
-  eps->ninil = -n;
+  if (n<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
+  ierr = SlepcBasisReference_Private(n,is,&eps->ninil,&eps->ISL);CHKERRQ(ierr);
+  if (n>0) eps->setupcalled = 0;
   PetscFunctionReturn(0);
 }
 
