@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,48 +21,47 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/ipimpl.h>      /*I "slepcip.h" I*/
+#include <slepc-private/ipimpl.h>      /*I "slepcip.h" I*/
 
-PetscFList       IPList = 0;
-PetscBool        IPRegisterAllCalled = PETSC_FALSE;
-PetscClassId     IP_CLASSID = 0;
-PetscLogEvent    IP_InnerProduct = 0,IP_Orthogonalize = 0,IP_ApplyMatrix = 0;
-static PetscBool IPPackageInitialized = PETSC_FALSE;
+PetscFunctionList IPList = 0;
+PetscBool         IPRegisterAllCalled = PETSC_FALSE;
+PetscClassId      IP_CLASSID = 0;
+PetscLogEvent     IP_InnerProduct = 0,IP_Orthogonalize = 0,IP_ApplyMatrix = 0;
+static PetscBool  IPPackageInitialized = PETSC_FALSE;
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPFinalizePackage"
 /*@C
-   IPFinalizePackage - This function destroys everything in the Slepc interface 
+   IPFinalizePackage - This function destroys everything in the Slepc interface
    to the IP package. It is called from SlepcFinalize().
 
    Level: developer
 
 .seealso: SlepcFinalize()
 @*/
-PetscErrorCode IPFinalizePackage(void) 
+PetscErrorCode IPFinalizePackage(void)
 {
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
+  ierr = PetscFunctionListDestroy(&IPList);CHKERRQ(ierr);
   IPPackageInitialized = PETSC_FALSE;
-  IPList               = 0;
   IPRegisterAllCalled  = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPInitializePackage"
 /*@C
   IPInitializePackage - This function initializes everything in the IP package. It is called
   from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to IPCreate()
   when using static libraries.
 
-  Input Parameter:
-  path - The dynamic library path, or PETSC_NULL
-
   Level: developer
 
 .seealso: SlepcInitialize()
 @*/
-PetscErrorCode IPInitializePackage(const char *path) 
+PetscErrorCode IPInitializePackage(void)
 {
   char             logList[256];
   char             *className;
@@ -75,13 +74,13 @@ PetscErrorCode IPInitializePackage(const char *path)
   /* Register Classes */
   ierr = PetscClassIdRegister("Inner product",&IP_CLASSID);CHKERRQ(ierr);
   /* Register Constructors */
-  ierr = IPRegisterAll(path);CHKERRQ(ierr);
+  ierr = IPRegisterAll();CHKERRQ(ierr);
   /* Register Events */
   ierr = PetscLogEventRegister("IPOrthogonalize",IP_CLASSID,&IP_Orthogonalize);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("IPInnerProduct",IP_CLASSID,&IP_InnerProduct);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("IPApplyMatrix",IP_CLASSID,&IP_ApplyMatrix);CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
     ierr = PetscStrstr(logList,"ip",&className);CHKERRQ(ierr);
     if (className) {
@@ -89,7 +88,7 @@ PetscErrorCode IPInitializePackage(const char *path)
     }
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL,"-log_summary_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,"-log_summary_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
     ierr = PetscStrstr(logList,"ip",&className);CHKERRQ(ierr);
     if (className) {
@@ -100,7 +99,7 @@ PetscErrorCode IPInitializePackage(const char *path)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPCreate"
 /*@C
    IPCreate - Creates an IP context.
@@ -115,7 +114,7 @@ PetscErrorCode IPInitializePackage(const char *path)
 
    Level: beginner
 
-   Note: 
+   Note:
    IP objects are not intended for normal users but only for
    advanced user that for instance implement their own solvers.
 
@@ -128,29 +127,38 @@ PetscErrorCode IPCreate(MPI_Comm comm,IP *newip)
 
   PetscFunctionBegin;
   PetscValidPointer(newip,2);
-  ierr = PetscHeaderCreate(ip,_p_IP,struct _IPOps,IP_CLASSID,-1,"IP","Inner Product","IP",comm,IPDestroy,IPView);CHKERRQ(ierr);
-  *newip            = ip;
+  *newip = 0;
+#if !defined(PETSC_USE_DYNAMIC_LIBRARIES)
+  ierr = IPInitializePackage();CHKERRQ(ierr);
+#endif
+
+  ierr = SlepcHeaderCreate(ip,_p_IP,struct _IPOps,IP_CLASSID,"IP","Inner Product","IP",comm,IPDestroy,IPView);CHKERRQ(ierr);
+
   ip->orthog_type   = IP_ORTHOG_CGS;
   ip->orthog_ref    = IP_ORTHOG_REFINE_IFNEEDED;
   ip->orthog_eta    = 0.7071;
   ip->innerproducts = 0;
-  ip->matrix        = PETSC_NULL;
-  ip->Bx            = PETSC_NULL;
+  ip->matrix        = NULL;
+  ip->Bx            = NULL;
   ip->xid           = 0;
   ip->xstate        = 0;
+  ip->work          = NULL;
+  ip->lwork         = 0;
+
+  *newip = ip;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPSetOptionsPrefix"
 /*@C
-   IPSetOptionsPrefix - Sets the prefix used for searching for all 
+   IPSetOptionsPrefix - Sets the prefix used for searching for all
    IP options in the database.
 
    Logically Collective on IP
 
    Input Parameters:
-+  ip - the innerproduct context
++  ip - the inner product context
 -  prefix - the prefix string to prepend to all IP option requests
 
    Notes:
@@ -172,16 +180,16 @@ PetscErrorCode IPSetOptionsPrefix(IP ip,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPAppendOptionsPrefix"
 /*@C
-   IPAppendOptionsPrefix - Appends to the prefix used for searching for all 
+   IPAppendOptionsPrefix - Appends to the prefix used for searching for all
    IP options in the database.
 
    Logically Collective on IP
 
    Input Parameters:
-+  ip - the innerproduct context
++  ip - the inner product context
 -  prefix - the prefix string to prepend to all IP option requests
 
    Notes:
@@ -205,13 +213,13 @@ PetscErrorCode IPAppendOptionsPrefix(IP ip,const char *prefix)
 #undef __FUNCT__
 #define __FUNCT__ "IPGetOptionsPrefix"
 /*@C
-   IPGetOptionsPrefix - Gets the prefix used for searching for all 
+   IPGetOptionsPrefix - Gets the prefix used for searching for all
    IP options in the database.
 
    Not Collective
 
    Input Parameters:
-.  ip - the innerproduct context
+.  ip - the inner product context
 
    Output Parameters:
 .  prefix - pointer to the prefix string used is returned
@@ -234,7 +242,7 @@ PetscErrorCode IPGetOptionsPrefix(IP ip,const char *prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPSetType"
 /*@C
    IPSetType - Selects the type for the IP object.
@@ -242,11 +250,11 @@ PetscErrorCode IPGetOptionsPrefix(IP ip,const char *prefix[])
    Logically Collective on IP
 
    Input Parameter:
-+  ip   - the inner product context.
++  ip   - the inner product context
 -  type - a known type
 
    Notes:
-   Two types are available: IPBILINEAR and IPSESQUILINEAR.
+   Three types are available: IPBILINEAR, IPSESQUILINEAR, and IPINDEFINITE.
 
    For complex scalars, the default is a sesquilinear form (x,y)=x^H*M*y and it is
    also possible to choose a bilinear form (x,y)=x^T*M*y (without complex conjugation).
@@ -254,12 +262,14 @@ PetscErrorCode IPGetOptionsPrefix(IP ip,const char *prefix[])
 
    In the case of real scalars, only the bilinear form (x,y)=x^T*M*y is available.
 
+   The indefinite inner product is reserved for the case of an indefinite
+   matrix M. This is used for instance in symmetric-indefinite eigenproblems.
+
    Level: advanced
 
 .seealso: IPGetType()
-
 @*/
-PetscErrorCode IPSetType(IP ip,const IPType type)
+PetscErrorCode IPSetType(IP ip,IPType type)
 {
   PetscErrorCode ierr,(*r)(IP);
   PetscBool      match;
@@ -268,11 +278,11 @@ PetscErrorCode IPSetType(IP ip,const IPType type)
   PetscValidHeaderSpecific(ip,IP_CLASSID,1);
   PetscValidCharPointer(type,2);
 
-  ierr = PetscTypeCompare((PetscObject)ip,type,&match);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)ip,type,&match);CHKERRQ(ierr);
   if (match) PetscFunctionReturn(0);
 
-  ierr =  PetscFListFind(IPList,((PetscObject)ip)->comm,type,PETSC_TRUE,(void (**)(void))&r);CHKERRQ(ierr);
-  if (!r) SETERRQ1(((PetscObject)ip)->comm,PETSC_ERR_ARG_UNKNOWN_TYPE,"Unable to find requested IP type %s",type);
+  ierr =  PetscFunctionListFind(IPList,type,&r);CHKERRQ(ierr);
+  if (!r) SETERRQ1(PetscObjectComm((PetscObject)ip),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unable to find requested IP type %s",type);
 
   ierr = PetscMemzero(ip->ops,sizeof(struct _IPOps));CHKERRQ(ierr);
 
@@ -281,7 +291,7 @@ PetscErrorCode IPSetType(IP ip,const IPType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPGetType"
 /*@C
    IPGetType - Gets the IP type name (as a string) from the IP context.
@@ -297,9 +307,8 @@ PetscErrorCode IPSetType(IP ip,const IPType type)
    Level: advanced
 
 .seealso: IPSetType()
-
 @*/
-PetscErrorCode IPGetType(IP ip,const IPType *type)
+PetscErrorCode IPGetType(IP ip,IPType *type)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ip,IP_CLASSID,1);
@@ -308,13 +317,13 @@ PetscErrorCode IPGetType(IP ip,const IPType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "IPSetDefaultType_Private"
+#undef __FUNCT__
+#define __FUNCT__ "IPSetType_Default"
 /*
   Sets the default IP type, depending on whether complex arithmetic
   is used or not.
 */
-PetscErrorCode IPSetDefaultType_Private(IP ip)
+PetscErrorCode IPSetType_Default(IP ip)
 {
   PetscErrorCode ierr;
 
@@ -328,7 +337,7 @@ PetscErrorCode IPSetDefaultType_Private(IP ip)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPSetFromOptions"
 /*@
    IPSetFromOptions - Sets IP options from the options database.
@@ -336,9 +345,9 @@ PetscErrorCode IPSetDefaultType_Private(IP ip)
    Collective on IP
 
    Input Parameters:
-.  ip - the innerproduct context
+.  ip - the inner product context
 
-   Notes:  
+   Notes:
    To see all options, run your program with the -help option.
 
    Level: beginner
@@ -353,25 +362,25 @@ PetscErrorCode IPSetFromOptions(IP ip)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ip,IP_CLASSID,1);
-  if (!IPRegisterAllCalled) { ierr = IPRegisterAll(PETSC_NULL);CHKERRQ(ierr); }
+  if (!IPRegisterAllCalled) { ierr = IPRegisterAll();CHKERRQ(ierr); }
   /* Set default type (we do not allow changing it with -ip_type) */
   if (!((PetscObject)ip)->type_name) {
-    ierr = IPSetDefaultType_Private(ip);CHKERRQ(ierr);
+    ierr = IPSetType_Default(ip);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsBegin(((PetscObject)ip)->comm,((PetscObject)ip)->prefix,"Inner Product (IP) Options","IP");CHKERRQ(ierr);
+  ierr = PetscObjectOptionsBegin((PetscObject)ip);CHKERRQ(ierr);
     i = ip->orthog_type;
-    ierr = PetscOptionsEList("-ip_orthog_type","Orthogonalization method","IPSetOrthogonalization",orth_list,2,orth_list[i],&i,PETSC_NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsEList("-ip_orthog_type","Orthogonalization method","IPSetOrthogonalization",orth_list,2,orth_list[i],&i,NULL);CHKERRQ(ierr);
     j = ip->orthog_ref;
-    ierr = PetscOptionsEList("-ip_orthog_refine","Iterative refinement mode during orthogonalization","IPSetOrthogonalization",ref_list,3,ref_list[j],&j,PETSC_NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsEList("-ip_orthog_refine","Iterative refinement mode during orthogonalization","IPSetOrthogonalization",ref_list,3,ref_list[j],&j,NULL);CHKERRQ(ierr);
     r = ip->orthog_eta;
-    ierr = PetscOptionsReal("-ip_orthog_eta","Parameter of iterative refinement during orthogonalization","IPSetOrthogonalization",r,&r,PETSC_NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsReal("-ip_orthog_eta","Parameter of iterative refinement during orthogonalization","IPSetOrthogonalization",r,&r,NULL);CHKERRQ(ierr);
     ierr = IPSetOrthogonalization(ip,(IPOrthogType)i,(IPOrthogRefineType)j,r);CHKERRQ(ierr);
     ierr = PetscObjectProcessOptionsHandlers((PetscObject)ip);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPSetOrthogonalization"
 /*@
    IPSetOrthogonalization - Specifies the type of orthogonalization technique
@@ -380,7 +389,7 @@ PetscErrorCode IPSetFromOptions(IP ip)
    Logically Collective on IP
 
    Input Parameters:
-+  ip     - the innerproduct context
++  ip     - the inner product context
 .  type   - the type of orthogonalization technique
 .  refine - type of refinement
 -  eta    - parameter for selective refinement
@@ -388,14 +397,14 @@ PetscErrorCode IPSetFromOptions(IP ip)
    Options Database Keys:
 +  -orthog_type <type> - Where <type> is cgs for Classical Gram-Schmidt orthogonalization
                          (default) or mgs for Modified Gram-Schmidt orthogonalization
-.  -orthog_refine <type> - Where <type> is one of never, ifneeded (default) or always 
+.  -orthog_refine <type> - Where <type> is one of never, ifneeded (default) or always
 -  -orthog_eta <eta> -  For setting the value of eta
-    
-   Notes:  
-   The default settings work well for most problems. 
+
+   Notes:
+   The default settings work well for most problems.
 
    The parameter eta should be a real value between 0 and 1 (or PETSC_DEFAULT).
-   The value of eta is used only when the refinement type is "ifneeded". 
+   The value of eta is used only when the refinement type is "ifneeded".
 
    When using several processors, MGS is likely to result in bad scalability.
 
@@ -417,7 +426,7 @@ PetscErrorCode IPSetOrthogonalization(IP ip,IPOrthogType type,IPOrthogRefineType
       ip->orthog_type = type;
       break;
     default:
-      SETERRQ(((PetscObject)ip)->comm,PETSC_ERR_ARG_WRONG,"Unknown orthogonalization type");
+      SETERRQ(PetscObjectComm((PetscObject)ip),PETSC_ERR_ARG_WRONG,"Unknown orthogonalization type");
   }
   switch (refine) {
     case IP_ORTHOG_REFINE_NEVER:
@@ -426,27 +435,27 @@ PetscErrorCode IPSetOrthogonalization(IP ip,IPOrthogType type,IPOrthogRefineType
       ip->orthog_ref = refine;
       break;
     default:
-      SETERRQ(((PetscObject)ip)->comm,PETSC_ERR_ARG_WRONG,"Unknown refinement type");
+      SETERRQ(PetscObjectComm((PetscObject)ip),PETSC_ERR_ARG_WRONG,"Unknown refinement type");
   }
   if (eta == PETSC_DEFAULT) {
     ip->orthog_eta = 0.7071;
   } else {
-    if (eta <= 0.0 || eta > 1.0) SETERRQ(((PetscObject)ip)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid eta value");    
+    if (eta <= 0.0 || eta > 1.0) SETERRQ(PetscObjectComm((PetscObject)ip),PETSC_ERR_ARG_OUTOFRANGE,"Invalid eta value");
     ip->orthog_eta = eta;
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPGetOrthogonalization"
 /*@C
-   IPGetOrthogonalization - Gets the orthogonalization settings from the 
+   IPGetOrthogonalization - Gets the orthogonalization settings from the
    IP object.
 
    Not Collective
 
    Input Parameter:
-.  ip - inner product context 
+.  ip - inner product context
 
    Output Parameter:
 +  type   - type of orthogonalization technique
@@ -468,7 +477,7 @@ PetscErrorCode IPGetOrthogonalization(IP ip,IPOrthogType *type,IPOrthogRefineTyp
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPView"
 /*@C
    IPView - Prints the IP data structure.
@@ -476,7 +485,7 @@ PetscErrorCode IPGetOrthogonalization(IP ip,IPOrthogType *type,IPOrthogRefineTyp
    Collective on IP
 
    Input Parameters:
-+  ip - the innerproduct context
++  ip - the inner product context
 -  viewer - optional visualization context
 
    Note:
@@ -484,8 +493,8 @@ PetscErrorCode IPGetOrthogonalization(IP ip,IPOrthogType *type,IPOrthogRefineTyp
 +     PETSC_VIEWER_STDOUT_SELF - standard output (default)
 -     PETSC_VIEWER_STDOUT_WORLD - synchronized standard
          output where only the first processor opens
-         the file.  All other processors send their 
-         data to the first processor to print. 
+         the file.  All other processors send their
+         data to the first processor to print.
 
    The user can open an alternative visualization context with
    PetscViewerASCIIOpen() - output to a specified file.
@@ -501,10 +510,10 @@ PetscErrorCode IPView(IP ip,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ip,IP_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(((PetscObject)ip)->comm);
+  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)ip));
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(ip,1,viewer,2);
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)ip,viewer,"IP Object");CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  orthogonalization method: ");CHKERRQ(ierr);
@@ -515,7 +524,7 @@ PetscErrorCode IPView(IP ip,PetscViewer viewer)
       case IP_ORTHOG_CGS:
         ierr = PetscViewerASCIIPrintf(viewer,"classical Gram-Schmidt\n");CHKERRQ(ierr);
         break;
-      default: SETERRQ(((PetscObject)ip)->comm,1,"Wrong value of ip->orth_type");
+      default: SETERRQ(PetscObjectComm((PetscObject)ip),1,"Wrong value of ip->orth_type");
     }
     ierr = PetscViewerASCIIPrintf(viewer,"  orthogonalization refinement: ");CHKERRQ(ierr);
     switch (ip->orthog_ref) {
@@ -528,7 +537,7 @@ PetscErrorCode IPView(IP ip,PetscViewer viewer)
       case IP_ORTHOG_REFINE_ALWAYS:
         ierr = PetscViewerASCIIPrintf(viewer,"always\n");CHKERRQ(ierr);
         break;
-      default: SETERRQ(((PetscObject)ip)->comm,1,"Wrong value of ip->orth_ref");
+      default: SETERRQ(PetscObjectComm((PetscObject)ip),1,"Wrong value of ip->orth_ref");
     }
     if (ip->matrix) {
       ierr = PetscViewerPushFormat(viewer,PETSC_VIEWER_ASCII_INFO);CHKERRQ(ierr);
@@ -537,15 +546,13 @@ PetscErrorCode IPView(IP ip,PetscViewer viewer)
       ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
       ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
     }
-  } else {
-    SETERRQ1(((PetscObject)ip)->comm,1,"Viewer type %s not supported for IP",((PetscObject)viewer)->type_name);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPReset"
-/*@C
+/*@
    IPReset - Resets the IP context to the initial state.
 
    Collective on IP
@@ -567,11 +574,13 @@ PetscErrorCode IPReset(IP ip)
   ierr = VecDestroy(&ip->Bx);CHKERRQ(ierr);
   ip->xid    = 0;
   ip->xstate = 0;
+  ierr = PetscFree(ip->work);CHKERRQ(ierr);
+  ip->lwork  = 0;
   ierr = IPResetOperationCounters(ip);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPDestroy"
 /*@C
    IPDestroy - Destroys IP context that was created with IPCreate().
@@ -598,10 +607,10 @@ PetscErrorCode IPDestroy(IP *ip)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPGetOperationCounters"
 /*@
-   IPGetOperationCounters - Gets the total number of inner product operations 
+   IPGetOperationCounters - Gets the total number of inner product operations
    made by the IP object.
 
    Not Collective
@@ -611,7 +620,7 @@ PetscErrorCode IPDestroy(IP *ip)
 
    Output Parameter:
 .  dots - number of inner product operations
-   
+
    Level: intermediate
 
 .seealso: IPResetOperationCounters()
@@ -625,10 +634,10 @@ PetscErrorCode IPGetOperationCounters(IP ip,PetscInt *dots)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPResetOperationCounters"
 /*@
-   IPResetOperationCounters - Resets the counters for inner product operations 
+   IPResetOperationCounters - Resets the counters for inner product operations
    made by of the IP object.
 
    Logically Collective on IP
@@ -648,75 +657,59 @@ PetscErrorCode IPResetOperationCounters(IP ip)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPRegister"
 /*@C
-   IPRegister - See IPRegisterDynamic()
+   IPRegister - Adds an inner product to the IP package.
 
-   Level: advanced
-@*/
-PetscErrorCode IPRegister(const char *sname,const char *path,const char *name,PetscErrorCode (*function)(IP))
-{
-  PetscErrorCode ierr;
-  char           fullname[PETSC_MAX_PATH_LEN];
+   Not collective
 
-  PetscFunctionBegin;
-  ierr = PetscFListConcat(path,name,fullname);CHKERRQ(ierr);
-  ierr = PetscFListAdd(&IPList,sname,fullname,(void (*)(void))function);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
+   Input Parameters:
++  name - name of a new user-defined IP
+-  function - routine to create context
 
-#undef __FUNCT__  
-#define __FUNCT__ "IPRegisterDestroy"
-/*@
-   IPRegisterDestroy - Frees the list of IP methods that were
-   registered by IPRegisterDynamic().
-
-   Not Collective
+   Notes:
+   IPRegister() may be called multiple times to add several user-defined inner products.
 
    Level: advanced
 
-.seealso: IPRegisterDynamic(), IPRegisterAll()
+.seealso: IPRegisterAll()
 @*/
-PetscErrorCode IPRegisterDestroy(void)
+PetscErrorCode IPRegister(const char *name,PetscErrorCode (*function)(IP))
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscFListDestroy(&IPList);CHKERRQ(ierr);
-  IPRegisterAllCalled = PETSC_FALSE;
+  ierr = PetscFunctionListAdd(&IPList,name,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-extern PetscErrorCode IPCreate_Bilinear(IP);
+PETSC_EXTERN PetscErrorCode IPCreate_Bilinear(IP);
 #if defined(PETSC_USE_COMPLEX)
-extern PetscErrorCode IPCreate_Sesquilinear(IP);
+PETSC_EXTERN PetscErrorCode IPCreate_Sesquilin(IP);
 #endif
-EXTERN_C_END
+PETSC_EXTERN PetscErrorCode IPCreate_Indefinite(IP);
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPRegisterAll"
 /*@C
    IPRegisterAll - Registers all of the inner products in the IP package.
 
    Not Collective
 
-   Input Parameter:
-.  path - the library where the routines are to be found (optional)
-
    Level: advanced
 @*/
-PetscErrorCode IPRegisterAll(const char *path)
+PetscErrorCode IPRegisterAll(void)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   IPRegisterAllCalled = PETSC_TRUE;
-  ierr = IPRegisterDynamic(IPBILINEAR,path,"IPCreate_Bilinear",IPCreate_Bilinear);CHKERRQ(ierr);
+  ierr = IPRegister(IPBILINEAR,IPCreate_Bilinear);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
-  ierr = IPRegisterDynamic(IPSESQUILINEAR,path,"IPCreate_Sesquilinear",IPCreate_Sesquilinear);CHKERRQ(ierr);
+  ierr = IPRegister(IPSESQUILINEAR,IPCreate_Sesquilin);CHKERRQ(ierr);
 #endif
+  ierr = IPRegister(IPINDEFINITE,IPCreate_Indefinite);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
