@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,26 +21,26 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/ipimpl.h>      /*I "slepcip.h" I*/
+#include <slepc-private/ipimpl.h>      /*I "slepcip.h" I*/
 
 /* The following definitions are intended to avoid using the "T" versions
    of dot products in the case of real scalars */
 #if defined(PETSC_USE_COMPLEX)
 #define VecXDotBegin  VecTDotBegin
-#define VecXDotEnd    VecTDotEnd  
+#define VecXDotEnd    VecTDotEnd
 #define VecMXDotBegin VecMTDotBegin
-#define VecMXDotEnd   VecMTDotEnd  
+#define VecMXDotEnd   VecMTDotEnd
 #else
 #define VecXDotBegin  VecDotBegin
-#define VecXDotEnd    VecDotEnd  
+#define VecXDotEnd    VecDotEnd
 #define VecMXDotBegin VecMDotBegin
-#define VecMXDotEnd   VecMDotEnd  
+#define VecMXDotEnd   VecMDotEnd
 #endif
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPNorm"
 /*@
-   IPNorm - Computes the norm of a vector as the square root of the inner 
+   IPNorm - Computes the norm of a vector as the square root of the inner
    product (x,x) as defined by IPInnerProduct().
 
    Collective on IP and Vec
@@ -54,10 +54,13 @@
 
    Notes:
    This function will usually compute the 2-norm of a vector, ||x||_2. But
-   this behaviour may be different if using a non-standard inner product changed 
-   via IPSetMatrix(). For example, if using the B-inner product for 
-   positive definite B, (x,y)_B=y^H Bx, then the computed norm is ||x||_B = 
-   sqrt( x^H Bx ).
+   this behaviour may be different if using a non-standard inner product changed
+   via IPSetMatrix(). For example, if using the B-inner product for
+   positive definite B, (x,y)_B=y^H Bx, then the computed norm is ||x||_B =
+   sqrt(x^H Bx).
+
+   In an indefinite inner product, matrix B is indefinite and the norm is
+   defined as s*sqrt(abs(x^H Bx)), where s = sign(x^H Bx).
 
    Level: developer
 
@@ -76,7 +79,7 @@ PetscErrorCode IPNorm(IP ip,Vec x,PetscReal *norm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPNormBegin_Bilinear"
 PetscErrorCode IPNormBegin_Bilinear(IP ip,Vec x,PetscReal *norm)
 {
@@ -88,9 +91,9 @@ PetscErrorCode IPNormBegin_Bilinear(IP ip,Vec x,PetscReal *norm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "IPNormBegin_Sesquilinear"
-PetscErrorCode IPNormBegin_Sesquilinear(IP ip,Vec x,PetscReal *norm)
+#undef __FUNCT__
+#define __FUNCT__ "IPNormBegin_Sesquilin"
+PetscErrorCode IPNormBegin_Sesquilin(IP ip,Vec x,PetscReal *norm)
 {
   PetscErrorCode ierr;
   PetscScalar    p;
@@ -104,7 +107,23 @@ PetscErrorCode IPNormBegin_Sesquilinear(IP ip,Vec x,PetscReal *norm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
+#define __FUNCT__ "IPNormBegin_Indefinite"
+PetscErrorCode IPNormBegin_Indefinite(IP ip,Vec x,PetscReal *norm)
+{
+  PetscErrorCode ierr;
+  PetscScalar    p;
+
+  PetscFunctionBegin;
+  if (!ip->matrix) {
+    ierr = VecNormBegin(x,NORM_2,norm);CHKERRQ(ierr);
+  } else {
+    ierr = IPInnerProductBegin(ip,x,x,&p);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
 #define __FUNCT__ "IPNormBegin"
 /*@
    IPNormBegin - Starts a split phase norm computation.
@@ -121,7 +140,7 @@ PetscErrorCode IPNormBegin_Sesquilinear(IP ip,Vec x,PetscReal *norm)
    Notes:
    Each call to IPNormBegin() should be paired with a call to IPNormEnd().
 
-.seealso: IPNormEnd(), IPNorm(), IPInnerProduct(), IPMInnerProduct(), 
+.seealso: IPNormEnd(), IPNorm(), IPInnerProduct(), IPMInnerProduct(),
           IPInnerProductBegin(), IPInnerProductEnd()
 @*/
 PetscErrorCode IPNormBegin(IP ip,Vec x,PetscReal *norm)
@@ -136,7 +155,7 @@ PetscErrorCode IPNormBegin(IP ip,Vec x,PetscReal *norm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPNormEnd_Bilinear"
 PetscErrorCode IPNormEnd_Bilinear(IP ip,Vec x,PetscReal *norm)
 {
@@ -146,21 +165,21 @@ PetscErrorCode IPNormEnd_Bilinear(IP ip,Vec x,PetscReal *norm)
   PetscFunctionBegin;
   ierr = IPInnerProductEnd(ip,x,x,&p);CHKERRQ(ierr);
   if (PetscAbsScalar(p)<PETSC_MACHINE_EPSILON)
-    PetscInfo(ip,"Zero norm, either the vector is zero or a semi-inner product is being used\n");
+    ierr = PetscInfo(ip,"Zero norm, either the vector is zero or a semi-inner product is being used\n");CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
-  if (PetscRealPart(p)<0.0 || PetscAbsReal(PetscImaginaryPart(p))>PETSC_MACHINE_EPSILON) 
-     SETERRQ(((PetscObject)ip)->comm,1,"IPNorm: The inner product is not well defined");
+  if (PetscRealPart(p)<0.0 || PetscAbsReal(PetscImaginaryPart(p))>PETSC_MACHINE_EPSILON)
+    SETERRQ(PetscObjectComm((PetscObject)ip),1,"IPNorm: The inner product is not well defined");
   *norm = PetscSqrtScalar(PetscRealPart(p));
 #else
-  if (p<0.0) SETERRQ(((PetscObject)ip)->comm,1,"IPNorm: The inner product is not well defined");
+  if (p<0.0) SETERRQ(PetscObjectComm((PetscObject)ip),1,"IPNorm: The inner product is not well defined");
   *norm = PetscSqrtScalar(p);
 #endif
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "IPNormEnd_Sesquilinear"
-PetscErrorCode IPNormEnd_Sesquilinear(IP ip,Vec x,PetscReal *norm)
+#undef __FUNCT__
+#define __FUNCT__ "IPNormEnd_Sesquilin"
+PetscErrorCode IPNormEnd_Sesquilin(IP ip,Vec x,PetscReal *norm)
 {
   PetscErrorCode ierr;
   PetscScalar    p;
@@ -171,15 +190,37 @@ PetscErrorCode IPNormEnd_Sesquilinear(IP ip,Vec x,PetscReal *norm)
   } else {
     ierr = IPInnerProductEnd(ip,x,x,&p);CHKERRQ(ierr);
     if (PetscAbsScalar(p)<PETSC_MACHINE_EPSILON)
-      PetscInfo(ip,"Zero norm, either the vector is zero or a semi-inner product is being used\n");
-    if (PetscRealPart(p)<0.0 || PetscAbsReal(PetscImaginaryPart(p))/PetscAbsScalar(p)>PETSC_MACHINE_EPSILON) 
-       SETERRQ(((PetscObject)ip)->comm,1,"IPNorm: The inner product is not well defined");
+      ierr = PetscInfo(ip,"Zero norm, either the vector is zero or a semi-inner product is being used\n");CHKERRQ(ierr);
+    if (PetscRealPart(p)<0.0 || PetscAbsReal(PetscImaginaryPart(p))/PetscAbsScalar(p)>PETSC_MACHINE_EPSILON)
+      SETERRQ(PetscObjectComm((PetscObject)ip),1,"IPNorm: The inner product is not well defined");
     *norm = PetscSqrtScalar(PetscRealPart(p));
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
+#define __FUNCT__ "IPNormEnd_Indefinite"
+PetscErrorCode IPNormEnd_Indefinite(IP ip,Vec x,PetscReal *norm)
+{
+  PetscErrorCode ierr;
+  PetscScalar    p;
+
+  PetscFunctionBegin;
+  if (!ip->matrix) {
+    ierr = VecNormEnd(x,NORM_2,norm);CHKERRQ(ierr);
+  } else {
+    ierr = IPInnerProductEnd(ip,x,x,&p);CHKERRQ(ierr);
+    if (PetscAbsScalar(p)<PETSC_MACHINE_EPSILON)
+      ierr = PetscInfo(ip,"Zero norm, either the vector is zero or a semi-inner product is being used\n");CHKERRQ(ierr);
+    if (PetscAbsReal(PetscImaginaryPart(p))/PetscAbsScalar(p)>PETSC_SQRT_MACHINE_EPSILON)
+      SETERRQ(PetscObjectComm((PetscObject)ip),1,"IPNorm: The inner product is not well defined");
+    if (PetscRealPart(p)<0.0) *norm = -PetscSqrtScalar(-PetscRealPart(p));
+    else *norm = PetscSqrtScalar(PetscRealPart(p));
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
 #define __FUNCT__ "IPNormEnd"
 /*@
    IPNormEnd - Ends a split phase norm computation.
@@ -198,7 +239,7 @@ PetscErrorCode IPNormEnd_Sesquilinear(IP ip,Vec x,PetscReal *norm)
    Notes:
    Each call to IPNormBegin() should be paired with a call to IPNormEnd().
 
-.seealso: IPNormBegin(), IPNorm(), IPInnerProduct(), IPMInnerProduct(), 
+.seealso: IPNormBegin(), IPNorm(), IPInnerProduct(), IPMInnerProduct(),
           IPInnerProductBegin(), IPInnerProductEnd()
 @*/
 PetscErrorCode IPNormEnd(IP ip,Vec x,PetscReal *norm)
@@ -213,7 +254,7 @@ PetscErrorCode IPNormEnd(IP ip,Vec x,PetscReal *norm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPInnerProduct"
 /*@
    IPInnerProduct - Computes the inner product of two vectors.
@@ -230,7 +271,7 @@ PetscErrorCode IPNormEnd(IP ip,Vec x,PetscReal *norm)
 
    Notes:
    This function will usually compute the standard dot product of vectors
-   x and y, (x,y)=y^H x. However this behaviour may be different if changed 
+   x and y, (x,y)=y^H x. However this behaviour may be different if changed
    via IPSetMatrix(). This allows use of other inner products such as
    the indefinite product y^T x for complex symmetric problems or the
    B-inner product for positive definite B, (x,y)_B=y^H Bx.
@@ -256,7 +297,7 @@ PetscErrorCode IPInnerProduct(IP ip,Vec x,Vec y,PetscScalar *p)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPInnerProductBegin_Bilinear"
 PetscErrorCode IPInnerProductBegin_Bilinear(IP ip,Vec x,Vec y,PetscScalar *p)
 {
@@ -272,9 +313,9 @@ PetscErrorCode IPInnerProductBegin_Bilinear(IP ip,Vec x,Vec y,PetscScalar *p)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "IPInnerProductBegin_Sesquilinear"
-PetscErrorCode IPInnerProductBegin_Sesquilinear(IP ip,Vec x,Vec y,PetscScalar *p)
+#undef __FUNCT__
+#define __FUNCT__ "IPInnerProductBegin_Sesquilin"
+PetscErrorCode IPInnerProductBegin_Sesquilin(IP ip,Vec x,Vec y,PetscScalar *p)
 {
   PetscErrorCode ierr;
 
@@ -288,7 +329,7 @@ PetscErrorCode IPInnerProductBegin_Sesquilinear(IP ip,Vec x,Vec y,PetscScalar *p
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPInnerProductBegin"
 /*@
    IPInnerProductBegin - Starts a split phase inner product computation.
@@ -306,8 +347,8 @@ PetscErrorCode IPInnerProductBegin_Sesquilinear(IP ip,Vec x,Vec y,PetscScalar *p
    Notes:
    Each call to IPInnerProductBegin() should be paired with a call to IPInnerProductEnd().
 
-.seealso: IPInnerProductEnd(), IPInnerProduct(), IPNorm(), IPNormBegin(), 
-          IPNormEnd(), IPMInnerProduct() 
+.seealso: IPInnerProductEnd(), IPInnerProduct(), IPNorm(), IPNormBegin(),
+          IPNormEnd(), IPMInnerProduct()
 @*/
 PetscErrorCode IPInnerProductBegin(IP ip,Vec x,Vec y,PetscScalar *p)
 {
@@ -325,7 +366,7 @@ PetscErrorCode IPInnerProductBegin(IP ip,Vec x,Vec y,PetscScalar *p)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPInnerProductEnd_Bilinear"
 PetscErrorCode IPInnerProductEnd_Bilinear(IP ip,Vec x,Vec y,PetscScalar *p)
 {
@@ -340,9 +381,9 @@ PetscErrorCode IPInnerProductEnd_Bilinear(IP ip,Vec x,Vec y,PetscScalar *p)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "IPInnerProductEnd_Sesquilinear"
-PetscErrorCode IPInnerProductEnd_Sesquilinear(IP ip,Vec x,Vec y,PetscScalar *p)
+#undef __FUNCT__
+#define __FUNCT__ "IPInnerProductEnd_Sesquilin"
+PetscErrorCode IPInnerProductEnd_Sesquilin(IP ip,Vec x,Vec y,PetscScalar *p)
 {
   PetscErrorCode ierr;
 
@@ -355,7 +396,7 @@ PetscErrorCode IPInnerProductEnd_Sesquilinear(IP ip,Vec x,Vec y,PetscScalar *p)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPInnerProductEnd"
 /*@
    IPInnerProductEnd - Ends a split phase inner product computation.
@@ -375,8 +416,8 @@ PetscErrorCode IPInnerProductEnd_Sesquilinear(IP ip,Vec x,Vec y,PetscScalar *p)
    Notes:
    Each call to IPInnerProductBegin() should be paired with a call to IPInnerProductEnd().
 
-.seealso: IPInnerProductBegin(), IPInnerProduct(), IPNorm(), IPNormBegin(), 
-          IPNormEnd(), IPMInnerProduct() 
+.seealso: IPInnerProductBegin(), IPInnerProduct(), IPNorm(), IPNormBegin(),
+          IPNormEnd(), IPMInnerProduct()
 @*/
 PetscErrorCode IPInnerProductEnd(IP ip,Vec x,Vec y,PetscScalar *p)
 {
@@ -393,7 +434,7 @@ PetscErrorCode IPInnerProductEnd(IP ip,Vec x,Vec y,PetscScalar *p)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPMInnerProduct"
 /*@
    IPMInnerProduct - Computes the inner products a vector x with a set of
@@ -411,9 +452,9 @@ PetscErrorCode IPInnerProductEnd(IP ip,Vec x,Vec y,PetscScalar *p)
 .  p - result of the inner products
 
    Notes:
-   This function will usually compute the standard dot product of x and y_i, 
+   This function will usually compute the standard dot product of x and y_i,
    (x,y_i)=y_i^H x, for each column of Y. However this behaviour may be different
-   if changed via IPSetMatrix(). This allows use of other inner products 
+   if changed via IPSetMatrix(). This allows use of other inner products
    such as the indefinite product y_i^T x for complex symmetric problems or the
    B-inner product for positive definite B, (x,y_i)_B=y_i^H Bx.
 
@@ -439,7 +480,7 @@ PetscErrorCode IPMInnerProduct(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPMInnerProductBegin_Bilinear"
 PetscErrorCode IPMInnerProductBegin_Bilinear(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
 {
@@ -455,9 +496,9 @@ PetscErrorCode IPMInnerProductBegin_Bilinear(IP ip,Vec x,PetscInt n,const Vec y[
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "IPMInnerProductBegin_Sesquilinear"
-PetscErrorCode IPMInnerProductBegin_Sesquilinear(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
+#undef __FUNCT__
+#define __FUNCT__ "IPMInnerProductBegin_Sesquilin"
+PetscErrorCode IPMInnerProductBegin_Sesquilin(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
 {
   PetscErrorCode ierr;
 
@@ -471,7 +512,7 @@ PetscErrorCode IPMInnerProductBegin_Sesquilinear(IP ip,Vec x,PetscInt n,const Ve
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPMInnerProductBegin"
 /*@
    IPMInnerProductBegin - Starts a split phase multiple inner product computation.
@@ -490,8 +531,8 @@ PetscErrorCode IPMInnerProductBegin_Sesquilinear(IP ip,Vec x,PetscInt n,const Ve
    Notes:
    Each call to IPMInnerProductBegin() should be paired with a call to IPMInnerProductEnd().
 
-.seealso: IPMInnerProductEnd(), IPMInnerProduct(), IPNorm(), IPNormBegin(), 
-          IPNormEnd(), IPInnerProduct() 
+.seealso: IPMInnerProductEnd(), IPMInnerProduct(), IPNorm(), IPNormBegin(),
+          IPNormEnd(), IPInnerProduct()
 @*/
 PetscErrorCode IPMInnerProductBegin(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
 {
@@ -511,7 +552,7 @@ PetscErrorCode IPMInnerProductBegin(IP ip,Vec x,PetscInt n,const Vec y[],PetscSc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPMInnerProductEnd_Bilinear"
 PetscErrorCode IPMInnerProductEnd_Bilinear(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
 {
@@ -526,9 +567,9 @@ PetscErrorCode IPMInnerProductEnd_Bilinear(IP ip,Vec x,PetscInt n,const Vec y[],
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "IPMInnerProductEnd_Sesquilinear"
-PetscErrorCode IPMInnerProductEnd_Sesquilinear(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
+#undef __FUNCT__
+#define __FUNCT__ "IPMInnerProductEnd_Sesquilin"
+PetscErrorCode IPMInnerProductEnd_Sesquilin(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
 {
   PetscErrorCode ierr;
 
@@ -541,7 +582,7 @@ PetscErrorCode IPMInnerProductEnd_Sesquilinear(IP ip,Vec x,PetscInt n,const Vec
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPMInnerProductEnd"
 /*@
    IPMInnerProductEnd - Ends a split phase multiple inner product computation.
@@ -562,8 +603,8 @@ PetscErrorCode IPMInnerProductEnd_Sesquilinear(IP ip,Vec x,PetscInt n,const Vec
    Notes:
    Each call to IPMInnerProductBegin() should be paired with a call to IPMInnerProductEnd().
 
-.seealso: IPMInnerProductBegin(), IPMInnerProduct(), IPNorm(), IPNormBegin(), 
-          IPNormEnd(), IPInnerProduct() 
+.seealso: IPMInnerProductBegin(), IPMInnerProduct(), IPNorm(), IPNormBegin(),
+          IPNormEnd(), IPInnerProduct()
 @*/
 PetscErrorCode IPMInnerProductEnd(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
 {
@@ -582,10 +623,9 @@ PetscErrorCode IPMInnerProductEnd(IP ip,Vec x,PetscInt n,const Vec y[],PetscScal
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "IPCreate_Bilinear"
-PetscErrorCode IPCreate_Bilinear(IP ip)
+PETSC_EXTERN PetscErrorCode IPCreate_Bilinear(IP ip)
 {
   PetscFunctionBegin;
   ip->ops->normbegin          = IPNormBegin_Bilinear;
@@ -598,19 +638,39 @@ PetscErrorCode IPCreate_Bilinear(IP ip)
 }
 
 #if defined(PETSC_USE_COMPLEX)
-#undef __FUNCT__  
-#define __FUNCT__ "IPCreate_Sesquilinear"
-PetscErrorCode IPCreate_Sesquilinear(IP ip)
+#undef __FUNCT__
+#define __FUNCT__ "IPCreate_Sesquilin"
+PETSC_EXTERN PetscErrorCode IPCreate_Sesquilin(IP ip)
 {
   PetscFunctionBegin;
-  ip->ops->normbegin          = IPNormBegin_Sesquilinear;
-  ip->ops->normend            = IPNormEnd_Sesquilinear;
-  ip->ops->innerproductbegin  = IPInnerProductBegin_Sesquilinear;
-  ip->ops->innerproductend    = IPInnerProductEnd_Sesquilinear;
-  ip->ops->minnerproductbegin = IPMInnerProductBegin_Sesquilinear;
-  ip->ops->minnerproductend   = IPMInnerProductEnd_Sesquilinear;
+  ip->ops->normbegin          = IPNormBegin_Sesquilin;
+  ip->ops->normend            = IPNormEnd_Sesquilin;
+  ip->ops->innerproductbegin  = IPInnerProductBegin_Sesquilin;
+  ip->ops->innerproductend    = IPInnerProductEnd_Sesquilin;
+  ip->ops->minnerproductbegin = IPMInnerProductBegin_Sesquilin;
+  ip->ops->minnerproductend   = IPMInnerProductEnd_Sesquilin;
   PetscFunctionReturn(0);
 }
 #endif
-EXTERN_C_END
+
+#undef __FUNCT__
+#define __FUNCT__ "IPCreate_Indefinite"
+PETSC_EXTERN PetscErrorCode IPCreate_Indefinite(IP ip)
+{
+  PetscFunctionBegin;
+  ip->ops->normbegin          = IPNormBegin_Indefinite;
+  ip->ops->normend            = IPNormEnd_Indefinite;
+#if defined(PETSC_USE_COMPLEX)
+  ip->ops->innerproductbegin  = IPInnerProductBegin_Sesquilin;
+  ip->ops->innerproductend    = IPInnerProductEnd_Sesquilin;
+  ip->ops->minnerproductbegin = IPMInnerProductBegin_Sesquilin;
+  ip->ops->minnerproductend   = IPMInnerProductEnd_Sesquilin;
+#else
+  ip->ops->innerproductbegin  = IPInnerProductBegin_Bilinear;
+  ip->ops->innerproductend    = IPInnerProductEnd_Bilinear;
+  ip->ops->minnerproductbegin = IPMInnerProductBegin_Bilinear;
+  ip->ops->minnerproductend   = IPMInnerProductEnd_Bilinear;
+#endif
+  PetscFunctionReturn(0);
+}
 
