@@ -1,20 +1,20 @@
-/*                       
+/*
 
    Straightforward linearization for quadratic eigenproblems.
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -22,11 +22,11 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/qepimpl.h>         /*I "slepcqep.h" I*/
-#include <private/epsimpl.h>         /*I "slepceps.h" I*/
+#include <slepc-private/qepimpl.h>         /*I "slepcqep.h" I*/
+#include <slepc-private/epsimpl.h>         /*I "slepceps.h" I*/
 #include "linearp.h"
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetUp_Linear"
 PetscErrorCode QEPSetUp_Linear(QEP qep)
 {
@@ -35,6 +35,7 @@ PetscErrorCode QEPSetUp_Linear(QEP qep)
   PetscInt       i=0;
   EPSWhich       which;
   PetscBool      trackall;
+  PetscScalar    sigma;
   /* function tables */
   PetscErrorCode (*fcreate[][2])(MPI_Comm,QEP_LINEAR*,Mat*) = {
     { MatCreateExplicit_Linear_N1A, MatCreateExplicit_Linear_N1B },   /* N1 */
@@ -62,6 +63,7 @@ PetscErrorCode QEPSetUp_Linear(QEP qep)
   };
 
   PetscFunctionBegin;
+  if (!ctx->cform) ctx->cform = 1;
   if (!qep->which) qep->which = QEP_LARGEST_MAGNITUDE;
   ctx->M = qep->M;
   ctx->C = qep->C;
@@ -70,38 +72,49 @@ PetscErrorCode QEPSetUp_Linear(QEP qep)
 
   ierr = MatDestroy(&ctx->A);CHKERRQ(ierr);
   ierr = MatDestroy(&ctx->B);CHKERRQ(ierr);
-  ierr = VecDestroy(&ctx->x1);CHKERRQ(ierr); 
-  ierr = VecDestroy(&ctx->x2);CHKERRQ(ierr); 
-  ierr = VecDestroy(&ctx->y1);CHKERRQ(ierr); 
-  ierr = VecDestroy(&ctx->y2);CHKERRQ(ierr); 
+  ierr = VecDestroy(&ctx->x1);CHKERRQ(ierr);
+  ierr = VecDestroy(&ctx->x2);CHKERRQ(ierr);
+  ierr = VecDestroy(&ctx->y1);CHKERRQ(ierr);
+  ierr = VecDestroy(&ctx->y2);CHKERRQ(ierr);
 
   switch (qep->problem_type) {
     case QEP_GENERAL:    i = 0; break;
     case QEP_HERMITIAN:  i = 2; break;
     case QEP_GYROSCOPIC: i = 4; break;
-    default: SETERRQ(((PetscObject)qep)->comm,1,"Wrong value of qep->problem_type");
+    default: SETERRQ(PetscObjectComm((PetscObject)qep),1,"Wrong value of qep->problem_type");
   }
   i += ctx->cform-1;
 
   if (ctx->explicitmatrix) {
-    ctx->x1 = ctx->x2 = ctx->y1 = ctx->y2 = PETSC_NULL;
-    ierr = (*fcreate[i][0])(((PetscObject)qep)->comm,ctx,&ctx->A);CHKERRQ(ierr);
-    ierr = (*fcreate[i][1])(((PetscObject)qep)->comm,ctx,&ctx->B);CHKERRQ(ierr);
+    ctx->x1 = ctx->x2 = ctx->y1 = ctx->y2 = NULL;
+    ierr = (*fcreate[i][0])(PetscObjectComm((PetscObject)qep),ctx,&ctx->A);CHKERRQ(ierr);
+    ierr = (*fcreate[i][1])(PetscObjectComm((PetscObject)qep),ctx,&ctx->B);CHKERRQ(ierr);
   } else {
-    ierr = VecCreateMPIWithArray(((PetscObject)qep)->comm,qep->nloc,qep->n,PETSC_NULL,&ctx->x1);CHKERRQ(ierr);
-    ierr = VecCreateMPIWithArray(((PetscObject)qep)->comm,qep->nloc,qep->n,PETSC_NULL,&ctx->x2);CHKERRQ(ierr);
-    ierr = VecCreateMPIWithArray(((PetscObject)qep)->comm,qep->nloc,qep->n,PETSC_NULL,&ctx->y1);CHKERRQ(ierr);
-    ierr = VecCreateMPIWithArray(((PetscObject)qep)->comm,qep->nloc,qep->n,PETSC_NULL,&ctx->y2);CHKERRQ(ierr);
-    ierr = MatCreateShell(((PetscObject)qep)->comm,2*qep->nloc,2*qep->nloc,2*qep->n,2*qep->n,ctx,&ctx->A);CHKERRQ(ierr);
+    ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)qep),1,qep->nloc,qep->n,NULL,&ctx->x1);CHKERRQ(ierr);
+    ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)qep),1,qep->nloc,qep->n,NULL,&ctx->x2);CHKERRQ(ierr);
+    ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)qep),1,qep->nloc,qep->n,NULL,&ctx->y1);CHKERRQ(ierr);
+    ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)qep),1,qep->nloc,qep->n,NULL,&ctx->y2);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(qep,ctx->x1);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(qep,ctx->x2);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(qep,ctx->y1);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(qep,ctx->y2);CHKERRQ(ierr);
+    ierr = MatCreateShell(PetscObjectComm((PetscObject)qep),2*qep->nloc,2*qep->nloc,2*qep->n,2*qep->n,ctx,&ctx->A);CHKERRQ(ierr);
     ierr = MatShellSetOperation(ctx->A,MATOP_MULT,(void(*)(void))fmult[i][0]);CHKERRQ(ierr);
     ierr = MatShellSetOperation(ctx->A,MATOP_GET_DIAGONAL,(void(*)(void))fgetdiagonal[i][0]);CHKERRQ(ierr);
-    ierr = MatCreateShell(((PetscObject)qep)->comm,2*qep->nloc,2*qep->nloc,2*qep->n,2*qep->n,ctx,&ctx->B);CHKERRQ(ierr);
+    ierr = MatCreateShell(PetscObjectComm((PetscObject)qep),2*qep->nloc,2*qep->nloc,2*qep->n,2*qep->n,ctx,&ctx->B);CHKERRQ(ierr);
     ierr = MatShellSetOperation(ctx->B,MATOP_MULT,(void(*)(void))fmult[i][1]);CHKERRQ(ierr);
     ierr = MatShellSetOperation(ctx->B,MATOP_GET_DIAGONAL,(void(*)(void))fgetdiagonal[i][1]);CHKERRQ(ierr);
   }
+  ierr = PetscLogObjectParent(qep,ctx->A);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent(qep,ctx->B);CHKERRQ(ierr);
 
+  if (!ctx->eps) { ierr = QEPLinearGetEPS(qep,&ctx->eps);CHKERRQ(ierr); }
   ierr = EPSSetOperators(ctx->eps,ctx->A,ctx->B);CHKERRQ(ierr);
-  ierr = EPSSetProblemType(ctx->eps,EPS_GNHEP);CHKERRQ(ierr);
+  if (qep->problem_type==QEP_HERMITIAN) {
+    ierr = EPSSetProblemType(ctx->eps,EPS_GHIEP);CHKERRQ(ierr);
+  } else {
+    ierr = EPSSetProblemType(ctx->eps,EPS_GNHEP);CHKERRQ(ierr);
+  }
   switch (qep->which) {
       case QEP_LARGEST_MAGNITUDE:  which = EPS_LARGEST_MAGNITUDE; break;
       case QEP_SMALLEST_MAGNITUDE: which = EPS_SMALLEST_MAGNITUDE; break;
@@ -109,7 +122,7 @@ PetscErrorCode QEPSetUp_Linear(QEP qep)
       case QEP_SMALLEST_REAL:      which = EPS_SMALLEST_REAL; break;
       case QEP_LARGEST_IMAGINARY:  which = EPS_LARGEST_IMAGINARY; break;
       case QEP_SMALLEST_IMAGINARY: which = EPS_SMALLEST_IMAGINARY; break;
-      default: SETERRQ(((PetscObject)qep)->comm,1,"Wrong value of which");
+      default: SETERRQ(PetscObjectComm((PetscObject)qep),1,"Wrong value of which");
   }
   ierr = EPSSetWhichEigenpairs(ctx->eps,which);CHKERRQ(ierr);
   ierr = EPSSetLeftVectorsWanted(ctx->eps,qep->leftvecs);CHKERRQ(ierr);
@@ -122,19 +135,24 @@ PetscErrorCode QEPSetUp_Linear(QEP qep)
     ierr = EPSSetFromOptions(ctx->eps);CHKERRQ(ierr);
     ctx->setfromoptionscalled = PETSC_FALSE;
   }
+  /* temporary change of target */
+  if (qep->sfactor!=1.0) {
+    ierr = EPSGetTarget(ctx->eps,&sigma);CHKERRQ(ierr);
+    ierr = EPSSetTarget(ctx->eps,sigma/qep->sfactor);CHKERRQ(ierr);
+  }
   ierr = EPSSetUp(ctx->eps);CHKERRQ(ierr);
-  ierr = EPSGetDimensions(ctx->eps,PETSC_NULL,&qep->ncv,&qep->mpd);CHKERRQ(ierr);
-  ierr = EPSGetTolerances(ctx->eps,PETSC_NULL,&qep->max_it);CHKERRQ(ierr);
-  if (qep->nini>0 || qep->ninil>0) PetscInfo(qep,"Ignoring initial vectors\n");
+  ierr = EPSGetDimensions(ctx->eps,NULL,&qep->ncv,&qep->mpd);CHKERRQ(ierr);
+  ierr = EPSGetTolerances(ctx->eps,NULL,&qep->max_it);CHKERRQ(ierr);
+  if (qep->nini>0 || qep->ninil>0) { ierr = PetscInfo(qep,"Ignoring initial vectors\n");CHKERRQ(ierr); }
   ierr = QEPAllocateSolution(qep);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPLinearSelect_Norm"
 /*
    QEPLinearSelect_Norm - Auxiliary routine that copies the solution of the
-   linear eigenproblem to the QEP object. The eigenvector of the generalized 
+   linear eigenproblem to the QEP object. The eigenvector of the generalized
    problem is supposed to be
                                z = [  x  ]
                                    [ l*x ]
@@ -142,7 +160,7 @@ PetscErrorCode QEPSetUp_Linear(QEP qep)
    computed residual norm.
    Finally, x is normalized so that ||x||_2 = 1.
 */
-PetscErrorCode QEPLinearSelect_Norm(QEP qep,EPS eps)
+static PetscErrorCode QEPLinearSelect_Norm(QEP qep,EPS eps)
 {
   PetscErrorCode ierr;
   PetscInt       i;
@@ -153,13 +171,13 @@ PetscErrorCode QEPLinearSelect_Norm(QEP qep,EPS eps)
 #if !defined(PETSC_USE_COMPLEX)
   PetscScalar    *py;
 #endif
-  
+
   PetscFunctionBegin;
-  ierr = EPSGetOperators(eps,&A,PETSC_NULL);CHKERRQ(ierr);
-  ierr = MatGetVecs(A,&xr,PETSC_NULL);CHKERRQ(ierr);
+  ierr = EPSGetOperators(eps,&A,NULL);CHKERRQ(ierr);
+  ierr = MatGetVecs(A,&xr,NULL);CHKERRQ(ierr);
   ierr = VecDuplicate(xr,&xi);CHKERRQ(ierr);
-  ierr = VecCreateMPIWithArray(((PetscObject)qep)->comm,qep->nloc,qep->n,PETSC_NULL,&wr);CHKERRQ(ierr);
-  ierr = VecCreateMPIWithArray(((PetscObject)qep)->comm,qep->nloc,qep->n,PETSC_NULL,&wi);CHKERRQ(ierr);
+  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)qep),1,qep->nloc,qep->n,NULL,&wr);CHKERRQ(ierr);
+  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)qep),1,qep->nloc,qep->n,NULL,&wi);CHKERRQ(ierr);
   for (i=0;i<qep->nconv;i++) {
     ierr = EPSGetEigenpair(eps,i,&qep->eigr[i],&qep->eigi[i],xr,xi);CHKERRQ(ierr);
     qep->eigr[i] *= qep->sfactor;
@@ -170,7 +188,7 @@ PetscErrorCode QEPLinearSelect_Norm(QEP qep,EPS eps)
       ierr = VecGetArray(xi,&py);CHKERRQ(ierr);
       ierr = VecPlaceArray(wr,px);CHKERRQ(ierr);
       ierr = VecPlaceArray(wi,py);CHKERRQ(ierr);
-      ierr = SlepcVecNormalize(wr,wi,PETSC_TRUE,PETSC_NULL);CHKERRQ(ierr);
+      ierr = SlepcVecNormalize(wr,wi,PETSC_TRUE,NULL);CHKERRQ(ierr);
       ierr = QEPComputeResidualNorm_Private(qep,qep->eigr[i],qep->eigi[i],wr,wi,&rn1);CHKERRQ(ierr);
       ierr = VecCopy(wr,qep->V[i]);CHKERRQ(ierr);
       ierr = VecCopy(wi,qep->V[i+1]);CHKERRQ(ierr);
@@ -178,7 +196,7 @@ PetscErrorCode QEPLinearSelect_Norm(QEP qep,EPS eps)
       ierr = VecResetArray(wi);CHKERRQ(ierr);
       ierr = VecPlaceArray(wr,px+qep->nloc);CHKERRQ(ierr);
       ierr = VecPlaceArray(wi,py+qep->nloc);CHKERRQ(ierr);
-      ierr = SlepcVecNormalize(wr,wi,PETSC_TRUE,PETSC_NULL);CHKERRQ(ierr);
+      ierr = SlepcVecNormalize(wr,wi,PETSC_TRUE,NULL);CHKERRQ(ierr);
       ierr = QEPComputeResidualNorm_Private(qep,qep->eigr[i],qep->eigi[i],wr,wi,&rn2);CHKERRQ(ierr);
       if (rn1>rn2) {
         ierr = VecCopy(wr,qep->V[i]);CHKERRQ(ierr);
@@ -188,19 +206,18 @@ PetscErrorCode QEPLinearSelect_Norm(QEP qep,EPS eps)
       ierr = VecResetArray(wi);CHKERRQ(ierr);
       ierr = VecRestoreArray(xr,&px);CHKERRQ(ierr);
       ierr = VecRestoreArray(xi,&py);CHKERRQ(ierr);
-    }
-    else if (qep->eigi[i]==0.0)   /* real eigenvalue */
+    } else if (qep->eigi[i]==0.0)   /* real eigenvalue */
 #endif
     {
       ierr = VecGetArray(xr,&px);CHKERRQ(ierr);
       ierr = VecPlaceArray(wr,px);CHKERRQ(ierr);
-      ierr = SlepcVecNormalize(wr,PETSC_NULL,PETSC_FALSE,PETSC_NULL);CHKERRQ(ierr);
-      ierr = QEPComputeResidualNorm_Private(qep,qep->eigr[i],qep->eigi[i],wr,PETSC_NULL,&rn1);CHKERRQ(ierr);
+      ierr = SlepcVecNormalize(wr,NULL,PETSC_FALSE,NULL);CHKERRQ(ierr);
+      ierr = QEPComputeResidualNorm_Private(qep,qep->eigr[i],qep->eigi[i],wr,NULL,&rn1);CHKERRQ(ierr);
       ierr = VecCopy(wr,qep->V[i]);CHKERRQ(ierr);
       ierr = VecResetArray(wr);CHKERRQ(ierr);
       ierr = VecPlaceArray(wr,px+qep->nloc);CHKERRQ(ierr);
-      ierr = SlepcVecNormalize(wr,PETSC_NULL,PETSC_FALSE,PETSC_NULL);CHKERRQ(ierr);
-      ierr = QEPComputeResidualNorm_Private(qep,qep->eigr[i],qep->eigi[i],wr,PETSC_NULL,&rn2);CHKERRQ(ierr);
+      ierr = SlepcVecNormalize(wr,NULL,PETSC_FALSE,NULL);CHKERRQ(ierr);
+      ierr = QEPComputeResidualNorm_Private(qep,qep->eigr[i],qep->eigi[i],wr,NULL,&rn2);CHKERRQ(ierr);
       if (rn1>rn2) {
         ierr = VecCopy(wr,qep->V[i]);CHKERRQ(ierr);
       }
@@ -215,30 +232,30 @@ PetscErrorCode QEPLinearSelect_Norm(QEP qep,EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPLinearSelect_Simple"
 /*
    QEPLinearSelect_Simple - Auxiliary routine that copies the solution of the
-   linear eigenproblem to the QEP object. The eigenvector of the generalized 
+   linear eigenproblem to the QEP object. The eigenvector of the generalized
    problem is supposed to be
                                z = [  x  ]
                                    [ l*x ]
    If |l|<1.0, the eigenvector is taken from z(1:n), otherwise from z(n+1:2*n).
    Finally, x is normalized so that ||x||_2 = 1.
 */
-PetscErrorCode QEPLinearSelect_Simple(QEP qep,EPS eps)
+static PetscErrorCode QEPLinearSelect_Simple(QEP qep,EPS eps)
 {
   PetscErrorCode ierr;
   PetscInt       i,offset;
   PetscScalar    *px;
   Vec            xr,xi,w;
   Mat            A;
-  
+
   PetscFunctionBegin;
-  ierr = EPSGetOperators(eps,&A,PETSC_NULL);CHKERRQ(ierr);
-  ierr = MatGetVecs(A,&xr,PETSC_NULL);CHKERRQ(ierr);
+  ierr = EPSGetOperators(eps,&A,NULL);CHKERRQ(ierr);
+  ierr = MatGetVecs(A,&xr,NULL);CHKERRQ(ierr);
   ierr = VecDuplicate(xr,&xi);CHKERRQ(ierr);
-  ierr = VecCreateMPIWithArray(((PetscObject)qep)->comm,qep->nloc,qep->n,PETSC_NULL,&w);CHKERRQ(ierr);
+  ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)qep),1,qep->nloc,qep->n,NULL,&w);CHKERRQ(ierr);
   for (i=0;i<qep->nconv;i++) {
     ierr = EPSGetEigenpair(eps,i,&qep->eigr[i],&qep->eigi[i],xr,xi);CHKERRQ(ierr);
     qep->eigr[i] *= qep->sfactor;
@@ -257,9 +274,8 @@ PetscErrorCode QEPLinearSelect_Simple(QEP qep,EPS eps)
       ierr = VecCopy(w,qep->V[i+1]);CHKERRQ(ierr);
       ierr = VecResetArray(w);CHKERRQ(ierr);
       ierr = VecRestoreArray(xi,&px);CHKERRQ(ierr);
-      ierr = SlepcVecNormalize(qep->V[i],qep->V[i+1],PETSC_TRUE,PETSC_NULL);CHKERRQ(ierr);
-    }
-    else if (qep->eigi[i]==0.0)   /* real eigenvalue */
+      ierr = SlepcVecNormalize(qep->V[i],qep->V[i+1],PETSC_TRUE,NULL);CHKERRQ(ierr);
+    } else if (qep->eigi[i]==0.0)   /* real eigenvalue */
 #endif
     {
       ierr = VecGetArray(xr,&px);CHKERRQ(ierr);
@@ -267,7 +283,7 @@ PetscErrorCode QEPLinearSelect_Simple(QEP qep,EPS eps)
       ierr = VecCopy(w,qep->V[i]);CHKERRQ(ierr);
       ierr = VecResetArray(w);CHKERRQ(ierr);
       ierr = VecRestoreArray(xr,&px);CHKERRQ(ierr);
-      ierr = SlepcVecNormalize(qep->V[i],PETSC_NULL,PETSC_FALSE,PETSC_NULL);CHKERRQ(ierr);
+      ierr = SlepcVecNormalize(qep->V[i],NULL,PETSC_FALSE,NULL);CHKERRQ(ierr);
     }
   }
   ierr = VecDestroy(&w);CHKERRQ(ierr);
@@ -276,23 +292,28 @@ PetscErrorCode QEPLinearSelect_Simple(QEP qep,EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSolve_Linear"
 PetscErrorCode QEPSolve_Linear(QEP qep)
 {
   PetscErrorCode ierr;
   QEP_LINEAR     *ctx = (QEP_LINEAR*)qep->data;
   PetscBool      flg=PETSC_FALSE;
+  PetscScalar    sigma;
 
   PetscFunctionBegin;
   ierr = EPSSolve(ctx->eps);CHKERRQ(ierr);
   ierr = EPSGetConverged(ctx->eps,&qep->nconv);CHKERRQ(ierr);
   ierr = EPSGetIterationNumber(ctx->eps,&qep->its);CHKERRQ(ierr);
   ierr = EPSGetConvergedReason(ctx->eps,(EPSConvergedReason*)&qep->reason);CHKERRQ(ierr);
-  ierr = EPSGetOperationCounters(ctx->eps,&qep->matvecs,PETSC_NULL,&qep->linits);CHKERRQ(ierr);
+  ierr = EPSGetOperationCounters(ctx->eps,&qep->matvecs,NULL,&qep->linits);CHKERRQ(ierr);
+  /* restore target */
+  ierr = EPSGetTarget(ctx->eps,&sigma);CHKERRQ(ierr);
+  ierr = EPSSetTarget(ctx->eps,sigma*qep->sfactor);CHKERRQ(ierr);
+
   qep->matvecs *= 2;  /* convention: count one matvec for each non-trivial block in A */
-  ierr = PetscOptionsGetBool(((PetscObject)qep)->prefix,"-qep_linear_select_simple",&flg,PETSC_NULL);CHKERRQ(ierr); 
-  if (flg) { 
+  ierr = PetscOptionsGetBool(((PetscObject)qep)->prefix,"-qep_linear_select_simple",&flg,NULL);CHKERRQ(ierr);
+  if (flg) {
     ierr = QEPLinearSelect_Simple(qep,ctx->eps);CHKERRQ(ierr);
   } else {
     ierr = QEPLinearSelect_Norm(qep,ctx->eps);CHKERRQ(ierr);
@@ -300,7 +321,7 @@ PetscErrorCode QEPSolve_Linear(QEP qep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "EPSMonitor_Linear"
 static PetscErrorCode EPSMonitor_Linear(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *ctx)
 {
@@ -316,12 +337,12 @@ static PetscErrorCode EPSMonitor_Linear(EPS eps,PetscInt its,PetscInt nconv,Pets
     qep->errest[i] = errest[i];
     if (0.0 < errest[i] && errest[i] < qep->tol) nconv++;
   }
-  ierr = STBackTransform(eps->OP,nest,qep->eigr,qep->eigi);CHKERRQ(ierr);
+  ierr = STBackTransform(eps->st,nest,qep->eigr,qep->eigi);CHKERRQ(ierr);
   ierr = QEPMonitor(qep,its,nconv,qep->eigr,qep->eigi,qep->errest,nest);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetFromOptions_Linear"
 PetscErrorCode QEPSetFromOptions_Linear(QEP qep)
 {
@@ -343,7 +364,8 @@ PetscErrorCode QEPSetFromOptions_Linear(QEP qep)
     ierr = QEPLinearSetExplicitMatrix(qep,val);CHKERRQ(ierr);
   }
   if (!ctx->explicitmatrix) {
-    /* use as default an ST with shell matrix and Jacobi */ 
+    /* use as default an ST with shell matrix and Jacobi */
+    if (!ctx->eps) { ierr = QEPLinearGetEPS(qep,&ctx->eps);CHKERRQ(ierr); }
     ierr = EPSGetST(ctx->eps,&st);CHKERRQ(ierr);
     ierr = STSetMatMode(st,ST_MATMODE_SHELL);CHKERRQ(ierr);
   }
@@ -351,23 +373,21 @@ PetscErrorCode QEPSetFromOptions_Linear(QEP qep)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPLinearSetCompanionForm_Linear"
-PetscErrorCode QEPLinearSetCompanionForm_Linear(QEP qep,PetscInt cform)
+static PetscErrorCode QEPLinearSetCompanionForm_Linear(QEP qep,PetscInt cform)
 {
   QEP_LINEAR *ctx = (QEP_LINEAR*)qep->data;
 
   PetscFunctionBegin;
-  if (cform==PETSC_IGNORE) PetscFunctionReturn(0);
+  if (!cform) PetscFunctionReturn(0);
   if (cform==PETSC_DECIDE || cform==PETSC_DEFAULT) ctx->cform = 1;
   else {
-    if (cform!=1 && cform!=2) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid value of argument 'cform'");
+    if (cform!=1 && cform!=2) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_OUTOFRANGE,"Invalid value of argument 'cform'");
     ctx->cform = cform;
   }
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
 #undef __FUNCT__
 #define __FUNCT__ "QEPLinearSetCompanionForm"
@@ -399,10 +419,9 @@ PetscErrorCode QEPLinearSetCompanionForm(QEP qep,PetscInt cform)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPLinearGetCompanionForm_Linear"
-PetscErrorCode QEPLinearGetCompanionForm_Linear(QEP qep,PetscInt *cform)
+static PetscErrorCode QEPLinearGetCompanionForm_Linear(QEP qep,PetscInt *cform)
 {
   QEP_LINEAR *ctx = (QEP_LINEAR*)qep->data;
 
@@ -410,7 +429,6 @@ PetscErrorCode QEPLinearGetCompanionForm_Linear(QEP qep,PetscInt *cform)
   *cform = ctx->cform;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
 #undef __FUNCT__
 #define __FUNCT__ "QEPLinearGetCompanionForm"
@@ -441,10 +459,9 @@ PetscErrorCode QEPLinearGetCompanionForm(QEP qep,PetscInt *cform)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPLinearSetExplicitMatrix_Linear"
-PetscErrorCode QEPLinearSetExplicitMatrix_Linear(QEP qep,PetscBool explicitmatrix)
+static PetscErrorCode QEPLinearSetExplicitMatrix_Linear(QEP qep,PetscBool explicitmatrix)
 {
   QEP_LINEAR *ctx = (QEP_LINEAR*)qep->data;
 
@@ -452,7 +469,6 @@ PetscErrorCode QEPLinearSetExplicitMatrix_Linear(QEP qep,PetscBool explicitmatri
   ctx->explicitmatrix = explicitmatrix;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
 #undef __FUNCT__
 #define __FUNCT__ "QEPLinearSetExplicitMatrix"
@@ -484,10 +500,9 @@ PetscErrorCode QEPLinearSetExplicitMatrix(QEP qep,PetscBool explicitmatrix)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPLinearGetExplicitMatrix_Linear"
-PetscErrorCode QEPLinearGetExplicitMatrix_Linear(QEP qep,PetscBool *explicitmatrix)
+static PetscErrorCode QEPLinearGetExplicitMatrix_Linear(QEP qep,PetscBool *explicitmatrix)
 {
   QEP_LINEAR *ctx = (QEP_LINEAR*)qep->data;
 
@@ -495,7 +510,6 @@ PetscErrorCode QEPLinearGetExplicitMatrix_Linear(QEP qep,PetscBool *explicitmatr
   *explicitmatrix = ctx->explicitmatrix;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
 #undef __FUNCT__
 #define __FUNCT__ "QEPLinearGetExplicitMatrix"
@@ -526,29 +540,27 @@ PetscErrorCode QEPLinearGetExplicitMatrix(QEP qep,PetscBool *explicitmatrix)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPLinearSetEPS_Linear"
-PetscErrorCode QEPLinearSetEPS_Linear(QEP qep,EPS eps)
+static PetscErrorCode QEPLinearSetEPS_Linear(QEP qep,EPS eps)
 {
   PetscErrorCode ierr;
   QEP_LINEAR     *ctx = (QEP_LINEAR*)qep->data;
 
   PetscFunctionBegin;
   ierr = PetscObjectReference((PetscObject)eps);CHKERRQ(ierr);
-  ierr = EPSDestroy(&ctx->eps);CHKERRQ(ierr);  
+  ierr = EPSDestroy(&ctx->eps);CHKERRQ(ierr);
   ctx->eps = eps;
   ierr = PetscLogObjectParent(qep,ctx->eps);CHKERRQ(ierr);
   qep->setupcalled = 0;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPLinearSetEPS"
 /*@
    QEPLinearSetEPS - Associate an eigensolver object (EPS) to the
-   quadratic eigenvalue solver. 
+   quadratic eigenvalue solver.
 
    Collective on QEP
 
@@ -572,20 +584,30 @@ PetscErrorCode QEPLinearSetEPS(QEP qep,EPS eps)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPLinearGetEPS_Linear"
-PetscErrorCode QEPLinearGetEPS_Linear(QEP qep,EPS *eps)
+static PetscErrorCode QEPLinearGetEPS_Linear(QEP qep,EPS *eps)
 {
-  QEP_LINEAR *ctx = (QEP_LINEAR*)qep->data;
+  PetscErrorCode ierr;
+  QEP_LINEAR     *ctx = (QEP_LINEAR*)qep->data;
 
   PetscFunctionBegin;
+  if (!ctx->eps) {
+    ierr = EPSCreate(PetscObjectComm((PetscObject)qep),&ctx->eps);CHKERRQ(ierr);
+    ierr = EPSSetOptionsPrefix(ctx->eps,((PetscObject)qep)->prefix);CHKERRQ(ierr);
+    ierr = EPSAppendOptionsPrefix(ctx->eps,"qep_");CHKERRQ(ierr);
+    ierr = STSetOptionsPrefix(ctx->eps->st,((PetscObject)ctx->eps)->prefix);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)ctx->eps,(PetscObject)qep,1);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(qep,ctx->eps);CHKERRQ(ierr);
+    if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
+    ierr = EPSSetIP(ctx->eps,qep->ip);CHKERRQ(ierr);
+    ierr = EPSMonitorSet(ctx->eps,EPSMonitor_Linear,qep,NULL);CHKERRQ(ierr);
+  }
   *eps = ctx->eps;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPLinearGetEPS"
 /*@
    QEPLinearGetEPS - Retrieve the eigensolver object (EPS) associated
@@ -614,7 +636,7 @@ PetscErrorCode QEPLinearGetEPS(QEP qep,EPS *eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPView_Linear"
 PetscErrorCode QEPView_Linear(QEP qep,PetscViewer viewer)
 {
@@ -622,6 +644,7 @@ PetscErrorCode QEPView_Linear(QEP qep,PetscViewer viewer)
   QEP_LINEAR     *ctx = (QEP_LINEAR*)qep->data;
 
   PetscFunctionBegin;
+  if (!ctx->eps) { ierr = QEPLinearGetEPS(qep,&ctx->eps);CHKERRQ(ierr); }
   ierr = PetscViewerASCIIPrintf(viewer,"  Linear: %s matrices\n",ctx->explicitmatrix? "explicit": "implicit");CHKERRQ(ierr);
   ierr = PetscViewerASCIIPrintf(viewer,"  Linear: %s companion form\n",ctx->cform==1? "1st": "2nd");CHKERRQ(ierr);
   ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
@@ -630,7 +653,7 @@ PetscErrorCode QEPView_Linear(QEP qep,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPReset_Linear"
 PetscErrorCode QEPReset_Linear(QEP qep)
 {
@@ -638,19 +661,18 @@ PetscErrorCode QEPReset_Linear(QEP qep)
   QEP_LINEAR     *ctx = (QEP_LINEAR*)qep->data;
 
   PetscFunctionBegin;
-  ierr = EPSReset(ctx->eps);CHKERRQ(ierr);
+  if (!ctx->eps) { ierr = EPSReset(ctx->eps);CHKERRQ(ierr); }
   ierr = MatDestroy(&ctx->A);CHKERRQ(ierr);
   ierr = MatDestroy(&ctx->B);CHKERRQ(ierr);
   ierr = VecDestroy(&ctx->x1);CHKERRQ(ierr);
   ierr = VecDestroy(&ctx->x2);CHKERRQ(ierr);
   ierr = VecDestroy(&ctx->y1);CHKERRQ(ierr);
   ierr = VecDestroy(&ctx->y2);CHKERRQ(ierr);
-  ierr = QEPDefaultFreeWork(qep);CHKERRQ(ierr);
-  ierr = QEPFreeSolution(qep);CHKERRQ(ierr);
+  ierr = QEPReset_Default(qep);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPDestroy_Linear"
 PetscErrorCode QEPDestroy_Linear(QEP qep)
 {
@@ -660,58 +682,37 @@ PetscErrorCode QEPDestroy_Linear(QEP qep)
   PetscFunctionBegin;
   ierr = EPSDestroy(&ctx->eps);CHKERRQ(ierr);
   ierr = PetscFree(qep->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)qep,"QEPLinearSetCompanionForm_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)qep,"QEPLinearGetCompanionForm_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)qep,"QEPLinearSetEPS_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)qep,"QEPLinearGetEPS_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)qep,"QEPLinearSetExplicitMatrix_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)qep,"QEPLinearGetExplicitMatrix_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)qep,"QEPLinearSetCompanionForm_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)qep,"QEPLinearGetCompanionForm_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)qep,"QEPLinearSetEPS_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)qep,"QEPLinearGetEPS_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)qep,"QEPLinearSetExplicitMatrix_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)qep,"QEPLinearGetExplicitMatrix_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPCreate_Linear"
-PetscErrorCode QEPCreate_Linear(QEP qep)
+PETSC_EXTERN PetscErrorCode QEPCreate_Linear(QEP qep)
 {
   PetscErrorCode ierr;
   QEP_LINEAR     *ctx;
 
   PetscFunctionBegin;
   ierr = PetscNewLog(qep,QEP_LINEAR,&ctx);CHKERRQ(ierr);
-  qep->data                      = (void *)ctx;
+  qep->data                      = (void*)ctx;
   qep->ops->solve                = QEPSolve_Linear;
   qep->ops->setup                = QEPSetUp_Linear;
   qep->ops->setfromoptions       = QEPSetFromOptions_Linear;
   qep->ops->destroy              = QEPDestroy_Linear;
   qep->ops->reset                = QEPReset_Linear;
   qep->ops->view                 = QEPView_Linear;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)qep,"QEPLinearSetCompanionForm_C","QEPLinearSetCompanionForm_Linear",QEPLinearSetCompanionForm_Linear);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)qep,"QEPLinearGetCompanionForm_C","QEPLinearGetCompanionForm_Linear",QEPLinearGetCompanionForm_Linear);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)qep,"QEPLinearSetEPS_C","QEPLinearSetEPS_Linear",QEPLinearSetEPS_Linear);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)qep,"QEPLinearGetEPS_C","QEPLinearGetEPS_Linear",QEPLinearGetEPS_Linear);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)qep,"QEPLinearSetExplicitMatrix_C","QEPLinearSetExplicitMatrix_Linear",QEPLinearSetExplicitMatrix_Linear);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)qep,"QEPLinearGetExplicitMatrix_C","QEPLinearGetExplicitMatrix_Linear",QEPLinearGetExplicitMatrix_Linear);CHKERRQ(ierr);
-
-  ierr = EPSCreate(((PetscObject)qep)->comm,&ctx->eps);CHKERRQ(ierr);
-  ierr = EPSSetOptionsPrefix(ctx->eps,((PetscObject)qep)->prefix);CHKERRQ(ierr);
-  ierr = EPSAppendOptionsPrefix(ctx->eps,"qep_");CHKERRQ(ierr);
-  ierr = STSetOptionsPrefix(ctx->eps->OP,((PetscObject)ctx->eps)->prefix);CHKERRQ(ierr);
-  ierr = PetscObjectIncrementTabLevel((PetscObject)ctx->eps,(PetscObject)qep,1);CHKERRQ(ierr);  
-  ierr = PetscLogObjectParent(qep,ctx->eps);CHKERRQ(ierr);
-  if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
-  ierr = EPSSetIP(ctx->eps,qep->ip);CHKERRQ(ierr);
-  ierr = EPSMonitorSet(ctx->eps,EPSMonitor_Linear,qep,PETSC_NULL);CHKERRQ(ierr);
-  ctx->explicitmatrix = PETSC_FALSE;
-  ctx->cform = 1;
-  ctx->A = PETSC_NULL;
-  ctx->B = PETSC_NULL;
-  ctx->x1 = PETSC_NULL;
-  ctx->x2 = PETSC_NULL;
-  ctx->y1 = PETSC_NULL;
-  ctx->y2 = PETSC_NULL;
-  ctx->setfromoptionscalled = PETSC_FALSE;
+  ierr = PetscObjectComposeFunction((PetscObject)qep,"QEPLinearSetCompanionForm_C",QEPLinearSetCompanionForm_Linear);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)qep,"QEPLinearGetCompanionForm_C",QEPLinearGetCompanionForm_Linear);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)qep,"QEPLinearSetEPS_C",QEPLinearSetEPS_Linear);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)qep,"QEPLinearGetEPS_C",QEPLinearGetEPS_Linear);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)qep,"QEPLinearSetExplicitMatrix_C",QEPLinearSetExplicitMatrix_Linear);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)qep,"QEPLinearGetExplicitMatrix_C",QEPLinearGetExplicitMatrix_Linear);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
