@@ -1,20 +1,20 @@
-/*                       
+/*
 
    Linearization for general QEP, companion form 1.
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -22,7 +22,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/qepimpl.h>         /*I "slepcqep.h" I*/
+#include <slepc-private/qepimpl.h>         /*I "slepcqep.h" I*/
 #include "linearp.h"
 
 /*
@@ -33,7 +33,7 @@
                                 [ -K  -C ]         [ 0  M ]         [ l*x ]
  */
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatMult_Linear_N1A"
 PetscErrorCode MatMult_Linear_N1A(Mat A,Vec x,Vec y)
 {
@@ -42,10 +42,10 @@ PetscErrorCode MatMult_Linear_N1A(Mat A,Vec x,Vec y)
   const PetscScalar *px;
   PetscScalar       *py;
   PetscInt          m;
-  
+
   PetscFunctionBegin;
   ierr = MatShellGetContext(A,(void**)&ctx);CHKERRQ(ierr);
-  ierr = MatGetLocalSize(ctx->M,&m,PETSC_NULL);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(ctx->M,&m,NULL);CHKERRQ(ierr);
   ierr = VecGetArrayRead(x,&px);CHKERRQ(ierr);
   ierr = VecGetArray(y,&py);CHKERRQ(ierr);
   ierr = VecPlaceArray(ctx->x1,px);CHKERRQ(ierr);
@@ -68,7 +68,7 @@ PetscErrorCode MatMult_Linear_N1A(Mat A,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatMult_Linear_N1B"
 PetscErrorCode MatMult_Linear_N1B(Mat B,Vec x,Vec y)
 {
@@ -77,10 +77,10 @@ PetscErrorCode MatMult_Linear_N1B(Mat B,Vec x,Vec y)
   const PetscScalar *px;
   PetscScalar       *py;
   PetscInt          m;
-  
+
   PetscFunctionBegin;
   ierr = MatShellGetContext(B,(void**)&ctx);CHKERRQ(ierr);
-  ierr = MatGetLocalSize(ctx->M,&m,PETSC_NULL);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(ctx->M,&m,NULL);CHKERRQ(ierr);
   ierr = VecGetArrayRead(x,&px);CHKERRQ(ierr);
   ierr = VecGetArray(y,&py);CHKERRQ(ierr);
   ierr = VecPlaceArray(ctx->x1,px);CHKERRQ(ierr);
@@ -101,7 +101,7 @@ PetscErrorCode MatMult_Linear_N1B(Mat B,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatGetDiagonal_Linear_N1A"
 PetscErrorCode MatGetDiagonal_Linear_N1A(Mat A,Vec diag)
 {
@@ -109,10 +109,10 @@ PetscErrorCode MatGetDiagonal_Linear_N1A(Mat A,Vec diag)
   QEP_LINEAR     *ctx;
   PetscScalar    *pd;
   PetscInt       m;
-  
+
   PetscFunctionBegin;
   ierr = MatShellGetContext(A,(void**)&ctx);CHKERRQ(ierr);
-  ierr = MatGetLocalSize(ctx->M,&m,PETSC_NULL);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(ctx->M,&m,NULL);CHKERRQ(ierr);
   ierr = VecGetArray(diag,&pd);CHKERRQ(ierr);
   ierr = VecPlaceArray(ctx->x1,pd);CHKERRQ(ierr);
   ierr = VecPlaceArray(ctx->x2,pd+m);CHKERRQ(ierr);
@@ -125,7 +125,7 @@ PetscErrorCode MatGetDiagonal_Linear_N1A(Mat A,Vec diag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatGetDiagonal_Linear_N1B"
 PetscErrorCode MatGetDiagonal_Linear_N1B(Mat B,Vec diag)
 {
@@ -133,10 +133,10 @@ PetscErrorCode MatGetDiagonal_Linear_N1B(Mat B,Vec diag)
   QEP_LINEAR     *ctx;
   PetscScalar    *pd;
   PetscInt       m;
-  
+
   PetscFunctionBegin;
   ierr = MatShellGetContext(B,(void**)&ctx);CHKERRQ(ierr);
-  ierr = MatGetLocalSize(ctx->M,&m,PETSC_NULL);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(ctx->M,&m,NULL);CHKERRQ(ierr);
   ierr = VecGetArray(diag,&pd);CHKERRQ(ierr);
   ierr = VecPlaceArray(ctx->x1,pd);CHKERRQ(ierr);
   ierr = VecPlaceArray(ctx->x2,pd+m);CHKERRQ(ierr);
@@ -149,20 +149,21 @@ PetscErrorCode MatGetDiagonal_Linear_N1B(Mat B,Vec diag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatCreateExplicit_Linear_N1A"
 PetscErrorCode MatCreateExplicit_Linear_N1A(MPI_Comm comm,QEP_LINEAR *ctx,Mat *A)
 {
   PetscErrorCode ierr;
   PetscInt       M,N,m,n;
   Mat            Id;
-  
+
   PetscFunctionBegin;
   ierr = MatGetSize(ctx->M,&M,&N);CHKERRQ(ierr);
   ierr = MatGetLocalSize(ctx->M,&m,&n);CHKERRQ(ierr);
-  ierr = MatCreate(((PetscObject)ctx->M)->comm,&Id);CHKERRQ(ierr);
+  ierr = MatCreate(PetscObjectComm((PetscObject)ctx->M),&Id);CHKERRQ(ierr);
   ierr = MatSetSizes(Id,m,n,M,N);CHKERRQ(ierr);
   ierr = MatSetFromOptions(Id);CHKERRQ(ierr);
+  ierr = MatSetUp(Id);CHKERRQ(ierr);
   ierr = MatAssemblyBegin(Id,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(Id,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatShift(Id,1.0);CHKERRQ(ierr);
@@ -171,20 +172,21 @@ PetscErrorCode MatCreateExplicit_Linear_N1A(MPI_Comm comm,QEP_LINEAR *ctx,Mat *A
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatCreateExplicit_Linear_N1B"
 PetscErrorCode MatCreateExplicit_Linear_N1B(MPI_Comm comm,QEP_LINEAR *ctx,Mat *B)
 {
   PetscErrorCode ierr;
   PetscInt       M,N,m,n;
   Mat            Id;
-  
+
   PetscFunctionBegin;
   ierr = MatGetSize(ctx->M,&M,&N);CHKERRQ(ierr);
   ierr = MatGetLocalSize(ctx->M,&m,&n);CHKERRQ(ierr);
-  ierr = MatCreate(((PetscObject)ctx->M)->comm,&Id);CHKERRQ(ierr);
+  ierr = MatCreate(PetscObjectComm((PetscObject)ctx->M),&Id);CHKERRQ(ierr);
   ierr = MatSetSizes(Id,m,n,M,N);CHKERRQ(ierr);
   ierr = MatSetFromOptions(Id);CHKERRQ(ierr);
+  ierr = MatSetUp(Id);CHKERRQ(ierr);
   ierr = MatAssemblyBegin(Id,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(Id,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatShift(Id,1.0);CHKERRQ(ierr);
