@@ -1,20 +1,20 @@
-/*                       
+/*
 
    Linearization for Hermitian QEP, companion form 1.
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -22,7 +22,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/qepimpl.h>         /*I "slepcqep.h" I*/
+#include <slepc-private/qepimpl.h>         /*I "slepcqep.h" I*/
 #include "linearp.h"
 
 /*
@@ -33,7 +33,7 @@
                                 [ -K  -C ]         [ 0  M ]         [ l*x ]
  */
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatMult_Linear_S1A"
 PetscErrorCode MatMult_Linear_S1A(Mat A,Vec x,Vec y)
 {
@@ -42,10 +42,10 @@ PetscErrorCode MatMult_Linear_S1A(Mat A,Vec x,Vec y)
   const PetscScalar *px;
   PetscScalar       *py;
   PetscInt          m;
-  
+
   PetscFunctionBegin;
   ierr = MatShellGetContext(A,(void**)&ctx);CHKERRQ(ierr);
-  ierr = MatGetLocalSize(ctx->M,&m,PETSC_NULL);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(ctx->M,&m,NULL);CHKERRQ(ierr);
   ierr = VecGetArrayRead(x,&px);CHKERRQ(ierr);
   ierr = VecGetArray(y,&py);CHKERRQ(ierr);
   ierr = VecPlaceArray(ctx->x1,px);CHKERRQ(ierr);
@@ -69,7 +69,7 @@ PetscErrorCode MatMult_Linear_S1A(Mat A,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatMult_Linear_S1B"
 PetscErrorCode MatMult_Linear_S1B(Mat B,Vec x,Vec y)
 {
@@ -78,10 +78,10 @@ PetscErrorCode MatMult_Linear_S1B(Mat B,Vec x,Vec y)
   const PetscScalar *px;
   PetscScalar       *py;
   PetscInt          m;
-  
+
   PetscFunctionBegin;
   ierr = MatShellGetContext(B,(void**)&ctx);CHKERRQ(ierr);
-  ierr = MatGetLocalSize(ctx->M,&m,PETSC_NULL);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(ctx->M,&m,NULL);CHKERRQ(ierr);
   ierr = VecGetArrayRead(x,&px);CHKERRQ(ierr);
   ierr = VecGetArray(y,&py);CHKERRQ(ierr);
   ierr = VecPlaceArray(ctx->x1,px);CHKERRQ(ierr);
@@ -103,7 +103,7 @@ PetscErrorCode MatMult_Linear_S1B(Mat B,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatGetDiagonal_Linear_S1A"
 PetscErrorCode MatGetDiagonal_Linear_S1A(Mat A,Vec diag)
 {
@@ -111,10 +111,10 @@ PetscErrorCode MatGetDiagonal_Linear_S1A(Mat A,Vec diag)
   QEP_LINEAR     *ctx;
   PetscScalar    *pd;
   PetscInt       m;
-  
+
   PetscFunctionBegin;
   ierr = MatShellGetContext(A,(void**)&ctx);CHKERRQ(ierr);
-  ierr = MatGetLocalSize(ctx->M,&m,PETSC_NULL);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(ctx->M,&m,NULL);CHKERRQ(ierr);
   ierr = VecGetArray(diag,&pd);CHKERRQ(ierr);
   ierr = VecPlaceArray(ctx->x1,pd);CHKERRQ(ierr);
   ierr = VecPlaceArray(ctx->x2,pd+m);CHKERRQ(ierr);
@@ -127,7 +127,7 @@ PetscErrorCode MatGetDiagonal_Linear_S1A(Mat A,Vec diag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatGetDiagonal_Linear_S1B"
 PetscErrorCode MatGetDiagonal_Linear_S1B(Mat B,Vec diag)
 {
@@ -135,10 +135,10 @@ PetscErrorCode MatGetDiagonal_Linear_S1B(Mat B,Vec diag)
   QEP_LINEAR     *ctx;
   PetscScalar    *pd;
   PetscInt       m;
-  
+
   PetscFunctionBegin;
   ierr = MatShellGetContext(B,(void**)&ctx);CHKERRQ(ierr);
-  ierr = MatGetLocalSize(ctx->M,&m,PETSC_NULL);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(ctx->M,&m,NULL);CHKERRQ(ierr);
   ierr = VecGetArray(diag,&pd);CHKERRQ(ierr);
   ierr = VecPlaceArray(ctx->x1,pd);CHKERRQ(ierr);
   ierr = VecPlaceArray(ctx->x2,pd+m);CHKERRQ(ierr);
@@ -152,23 +152,23 @@ PetscErrorCode MatGetDiagonal_Linear_S1B(Mat B,Vec diag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatCreateExplicit_Linear_S1A"
 PetscErrorCode MatCreateExplicit_Linear_S1A(MPI_Comm comm,QEP_LINEAR *ctx,Mat *A)
 {
   PetscErrorCode ierr;
-  
+
   PetscFunctionBegin;
   ierr = SlepcMatTile(0.0,ctx->K,-1.0,ctx->K,-1.0,ctx->K,-ctx->sfactor,ctx->C,A);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatCreateExplicit_Linear_S1B"
 PetscErrorCode MatCreateExplicit_Linear_S1B(MPI_Comm comm,QEP_LINEAR *ctx,Mat *B)
 {
   PetscErrorCode ierr;
-  
+
   PetscFunctionBegin;
   ierr = SlepcMatTile(-1.0,ctx->K,0.0,ctx->M,0.0,ctx->M,ctx->sfactor*ctx->sfactor,ctx->M,B);CHKERRQ(ierr);
   PetscFunctionReturn(0);
