@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,9 +21,10 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/qepimpl.h>      /*I "slepcqep.h" I*/
+#include <slepc-private/qepimpl.h>      /*I "slepcqep.h" I*/
+#include <petscdraw.h>
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPMonitor"
 /*
    Runs the user provided monitor routines, if any.
@@ -40,21 +41,20 @@ PetscErrorCode QEPMonitor(QEP qep,PetscInt it,PetscInt nconv,PetscScalar *eigr,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPMonitorSet"
 /*@C
-   QEPMonitorSet - Sets an ADDITIONAL function to be called at every 
+   QEPMonitorSet - Sets an ADDITIONAL function to be called at every
    iteration to monitor the error estimates for each requested eigenpair.
-      
+
    Logically Collective on QEP
 
    Input Parameters:
 +  qep     - eigensolver context obtained from QEPCreate()
-.  monitor - pointer to function (if this is PETSC_NULL, it turns off monitoring)
+.  monitor - pointer to function (if this is NULL, it turns off monitoring)
 .  mctx    - [optional] context for private data for the
-             monitor routine (use PETSC_NULL if no context is desired)
--  monitordestroy - [optional] routine that frees monitor context
-          (may be PETSC_NULL)
+             monitor routine (use NULL if no context is desired)
+-  monitordestroy - [optional] routine that frees monitor context (may be NULL)
 
    Calling Sequence of monitor:
 $     monitor (QEP qep, int its, int nconv, PetscScalar *eigr, PetscScalar *eigi, PetscReal* errest, int nest, void *mctx)
@@ -69,21 +69,21 @@ $     monitor (QEP qep, int its, int nconv, PetscScalar *eigr, PetscScalar *eigi
 -  mctx   - optional monitoring context, as set by QEPMonitorSet()
 
    Options Database Keys:
-+    -qep_monitor          - print only the first error estimate
-.    -qep_monitor_all      - print error estimates at each iteration
-.    -qep_monitor_conv     - print the eigenvalue approximations only when
++    -qep_monitor        - print only the first error estimate
+.    -qep_monitor_all    - print error estimates at each iteration
+.    -qep_monitor_conv   - print the eigenvalue approximations only when
       convergence has been reached
-.    -qep_monitor_draw     - sets line graph monitor for the first unconverged
+.    -qep_monitor_lg     - sets line graph monitor for the first unconverged
       approximate eigenvalue
-.    -qep_monitor_draw_all - sets line graph monitor for all unconverged
-      approximate eigenvalue
--    -qep_monitor_cancel   - cancels all monitors that have been hardwired into
+.    -qep_monitor_lg_all - sets line graph monitor for all unconverged
+      approximate eigenvalues
+-    -qep_monitor_cancel - cancels all monitors that have been hardwired into
       a code by calls to QEPMonitorSet(), but does not cancel those set via
       the options database.
 
-   Notes:  
+   Notes:
    Several different monitoring routines may be set by calling
-   QEPMonitorSet() multiple times; all will be called in the 
+   QEPMonitorSet() multiple times; all will be called in the
    order in which they were set.
 
    Level: intermediate
@@ -94,16 +94,14 @@ PetscErrorCode QEPMonitorSet(QEP qep,PetscErrorCode (*monitor)(QEP,PetscInt,Pets
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
-  if (qep->numbermonitors >= MAXQEPMONITORS) {
-    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Too many QEP monitors set");
-  }
+  if (qep->numbermonitors >= MAXQEPMONITORS) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_OUTOFRANGE,"Too many QEP monitors set");
   qep->monitor[qep->numbermonitors]           = monitor;
   qep->monitorcontext[qep->numbermonitors]    = (void*)mctx;
   qep->monitordestroy[qep->numbermonitors++]  = monitordestroy;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPMonitorCancel"
 /*@
    QEPMonitorCancel - Clears all monitors for a QEP object.
@@ -114,7 +112,7 @@ PetscErrorCode QEPMonitorSet(QEP qep,PetscErrorCode (*monitor)(QEP,PetscInt,Pets
 .  qep - eigensolver context obtained from QEPCreate()
 
    Options Database Key:
-.    -qep_monitor_cancel - Cancels all monitors that have been hardwired 
+.    -qep_monitor_cancel - Cancels all monitors that have been hardwired
       into a code by calls to QEPMonitorSet(),
       but does not cancel those set via the options database.
 
@@ -138,10 +136,10 @@ PetscErrorCode QEPMonitorCancel(QEP qep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPGetMonitorContext"
 /*@C
-   QEPGetMonitorContext - Gets the monitor context, as set by 
+   QEPGetMonitorContext - Gets the monitor context, as set by
    QEPMonitorSet() for the FIRST monitor only.
 
    Not Collective
@@ -160,14 +158,14 @@ PetscErrorCode QEPGetMonitorContext(QEP qep,void **ctx)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
-  *ctx =      (qep->monitorcontext[0]);
+  *ctx = qep->monitorcontext[0];
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPMonitorAll"
 /*@C
-   QEPMonitorAll - Print the current approximate values and 
+   QEPMonitorAll - Print the current approximate values and
    error estimates at each iteration of the quadratic eigensolver.
 
    Collective on QEP
@@ -180,7 +178,7 @@ PetscErrorCode QEPGetMonitorContext(QEP qep,void **ctx)
 .  eigi   - imaginary part of the eigenvalues
 .  errest - error estimates
 .  nest   - number of error estimates to display
--  monctx - monitor context (contains viewer, can be PETSC_NULL)
+-  monctx - monitor context (contains viewer, can be NULL)
 
    Level: intermediate
 
@@ -190,7 +188,7 @@ PetscErrorCode QEPMonitorAll(QEP qep,PetscInt its,PetscInt nconv,PetscScalar *ei
 {
   PetscErrorCode ierr;
   PetscInt       i;
-  PetscViewer    viewer = monctx? (PetscViewer)monctx: PETSC_VIEWER_STDOUT_(((PetscObject)qep)->comm);
+  PetscViewer    viewer = monctx? (PetscViewer)monctx: PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)qep));
 
   PetscFunctionBegin;
   if (its) {
@@ -211,10 +209,10 @@ PetscErrorCode QEPMonitorAll(QEP qep,PetscInt its,PetscInt nconv,PetscScalar *ei
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPMonitorFirst"
 /*@C
-   QEPMonitorFirst - Print the first unconverged approximate value and 
+   QEPMonitorFirst - Print the first unconverged approximate value and
    error estimate at each iteration of the quadratic eigensolver.
 
    Collective on QEP
@@ -227,7 +225,7 @@ PetscErrorCode QEPMonitorAll(QEP qep,PetscInt its,PetscInt nconv,PetscScalar *ei
 .  eigi   - imaginary part of the eigenvalues
 .  errest - error estimates
 .  nest   - number of error estimates to display
--  monctx - monitor context (contains viewer, can be PETSC_NULL)
+-  monctx - monitor context (contains viewer, can be NULL)
 
    Level: intermediate
 
@@ -236,7 +234,7 @@ PetscErrorCode QEPMonitorAll(QEP qep,PetscInt its,PetscInt nconv,PetscScalar *ei
 PetscErrorCode QEPMonitorFirst(QEP qep,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *monctx)
 {
   PetscErrorCode ierr;
-  PetscViewer    viewer = monctx? (PetscViewer)monctx: PETSC_VIEWER_STDOUT_(((PetscObject)qep)->comm);
+  PetscViewer    viewer = monctx? (PetscViewer)monctx: PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)qep));
 
   PetscFunctionBegin;
   if (its && nconv<nest) {
@@ -254,10 +252,10 @@ PetscErrorCode QEPMonitorFirst(QEP qep,PetscInt its,PetscInt nconv,PetscScalar *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPMonitorConverged"
 /*@C
-   QEPMonitorConverged - Print the approximate values and 
+   QEPMonitorConverged - Print the approximate values and
    error estimates as they converge.
 
    Collective on QEP
@@ -270,7 +268,7 @@ PetscErrorCode QEPMonitorFirst(QEP qep,PetscInt its,PetscInt nconv,PetscScalar *
 .  eigi   - imaginary part of the eigenvalues
 .  errest - error estimates
 .  nest   - number of error estimates to display
--  monctx - monitor context 
+-  monctx - monitor context
 
    Level: intermediate
 
@@ -288,11 +286,11 @@ PetscErrorCode QEPMonitorConverged(QEP qep,PetscInt its,PetscInt nconv,PetscScal
   SlepcConvMonitor ctx = (SlepcConvMonitor)monctx;
 
   PetscFunctionBegin;
-  if (!monctx) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_WRONG,"Must provide a context for QEPMonitorConverged");
+  if (!monctx) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_WRONG,"Must provide a context for QEPMonitorConverged");
   if (!its) {
     ctx->oldnconv = 0;
   } else {
-    viewer = ctx->viewer? ctx->viewer: PETSC_VIEWER_STDOUT_(((PetscObject)qep)->comm);
+    viewer = ctx->viewer? ctx->viewer: PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)qep));
     for (i=ctx->oldnconv;i<nconv;i++) {
       ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)qep)->tablevel);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPrintf(viewer,"%3D QEP converged value (error) #%D",its,i);CHKERRQ(ierr);
@@ -310,18 +308,18 @@ PetscErrorCode QEPMonitorConverged(QEP qep,PetscInt its,PetscInt nconv,PetscScal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPMonitorLG"
 PetscErrorCode QEPMonitorLG(QEP qep,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *monctx)
 {
-  PetscViewer    viewer = (PetscViewer) monctx;
+  PetscViewer    viewer = (PetscViewer)monctx;
   PetscDraw      draw;
   PetscDrawLG    lg;
   PetscErrorCode ierr;
   PetscReal      x,y;
 
   PetscFunctionBegin;
-  if (!viewer) { viewer = PETSC_VIEWER_DRAW_(((PetscObject)qep)->comm); }
+  if (!viewer) viewer = PETSC_VIEWER_DRAW_(PetscObjectComm((PetscObject)qep));
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
   ierr = PetscViewerDrawGetDrawLG(viewer,0,&lg);CHKERRQ(ierr);
   if (!its) {
@@ -332,19 +330,19 @@ PetscErrorCode QEPMonitorLG(QEP qep,PetscInt its,PetscInt nconv,PetscScalar *eig
     ierr = PetscDrawLGSetLimits(lg,0,1.0,log10(qep->tol)-2,0.0);CHKERRQ(ierr);
   }
 
-  x = (PetscReal) its;
+  x = (PetscReal)its;
   if (errest[nconv] > 0.0) y = log10(errest[nconv]); else y = 0.0;
   ierr = PetscDrawLGAddPoint(lg,&x,&y);CHKERRQ(ierr);
 
   ierr = PetscDrawLGDraw(lg);CHKERRQ(ierr);
   PetscFunctionReturn(0);
-} 
+}
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPMonitorLGAll"
 PetscErrorCode QEPMonitorLGAll(QEP qep,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *monctx)
 {
-  PetscViewer    viewer = (PetscViewer) monctx;
+  PetscViewer    viewer = (PetscViewer)monctx;
   PetscDraw      draw;
   PetscDrawLG    lg;
   PetscErrorCode ierr;
@@ -352,7 +350,7 @@ PetscErrorCode QEPMonitorLGAll(QEP qep,PetscInt its,PetscInt nconv,PetscScalar *
   PetscInt       i,n = PetscMin(qep->nev,255);
 
   PetscFunctionBegin;
-  if (!viewer) { viewer = PETSC_VIEWER_DRAW_(((PetscObject)qep)->comm); }
+  if (!viewer) viewer = PETSC_VIEWER_DRAW_(PetscObjectComm((PetscObject)qep));
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
   ierr = PetscViewerDrawGetDrawLG(viewer,0,&lg);CHKERRQ(ierr);
   if (!its) {
@@ -366,7 +364,7 @@ PetscErrorCode QEPMonitorLGAll(QEP qep,PetscInt its,PetscInt nconv,PetscScalar *
   ierr = PetscMalloc(sizeof(PetscReal)*n,&x);CHKERRQ(ierr);
   ierr = PetscMalloc(sizeof(PetscReal)*n,&y);CHKERRQ(ierr);
   for (i=0;i<n;i++) {
-    x[i] = (PetscReal) its;
+    x[i] = (PetscReal)its;
     if (i < nest && errest[i] > 0.0) y[i] = log10(errest[i]);
     else y[i] = 0.0;
   }
@@ -374,7 +372,7 @@ PetscErrorCode QEPMonitorLGAll(QEP qep,PetscInt its,PetscInt nconv,PetscScalar *
 
   ierr = PetscDrawLGDraw(lg);CHKERRQ(ierr);
   ierr = PetscFree(x);CHKERRQ(ierr);
-  ierr = PetscFree(y);CHKERRQ(ierr);  
+  ierr = PetscFree(y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
-} 
+}
 
