@@ -1,20 +1,20 @@
 /*
-      QEP routines related to options that can be set via the command-line 
+      QEP routines related to options that can be set via the command-line
       or procedurally.
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -22,21 +22,21 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/qepimpl.h>       /*I "slepcqep.h" I*/
+#include <slepc-private/qepimpl.h>       /*I "slepcqep.h" I*/
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetFromOptions"
 /*@
    QEPSetFromOptions - Sets QEP options from the options database.
-   This routine must be called before QEPSetUp() if the user is to be 
-   allowed to set the solver type. 
+   This routine must be called before QEPSetUp() if the user is to be
+   allowed to set the solver type.
 
    Collective on QEP
 
    Input Parameters:
 .  qep - the quadratic eigensolver context
 
-   Notes:  
+   Notes:
    To see all options, run your program with the -help option.
 
    Level: beginner
@@ -47,15 +47,15 @@ PetscErrorCode QEPSetFromOptions(QEP qep)
   char             type[256],monfilename[PETSC_MAX_PATH_LEN];
   PetscBool        flg,val;
   PetscReal        r;
+  PetscScalar      s;
   PetscInt         i,j,k;
   PetscViewer      monviewer;
   SlepcConvMonitor ctx;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
-  if (!QEPRegisterAllCalled) { ierr = QEPRegisterAll(PETSC_NULL);CHKERRQ(ierr); }
-  if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
-  ierr = PetscOptionsBegin(((PetscObject)qep)->comm,((PetscObject)qep)->prefix,"Quadratic Eigenvalue Problem (QEP) Solver Options","QEP");CHKERRQ(ierr);
+  if (!QEPRegisterAllCalled) { ierr = QEPRegisterAll();CHKERRQ(ierr); }
+  ierr = PetscObjectOptionsBegin((PetscObject)qep);CHKERRQ(ierr);
     ierr = PetscOptionsList("-qep_type","Quadratic Eigenvalue Problem method","QEPSetType",QEPList,(char*)(((PetscObject)qep)->type_name?((PetscObject)qep)->type_name:QEPLINEAR),type,256,&flg);CHKERRQ(ierr);
     if (flg) {
       ierr = QEPSetType(qep,type);CHKERRQ(ierr);
@@ -64,85 +64,97 @@ PetscErrorCode QEPSetFromOptions(QEP qep)
     }
 
     ierr = PetscOptionsBoolGroupBegin("-qep_general","general quadratic eigenvalue problem","QEPSetProblemType",&flg);CHKERRQ(ierr);
-    if (flg) {ierr = QEPSetProblemType(qep,QEP_GENERAL);CHKERRQ(ierr);}
+    if (flg) { ierr = QEPSetProblemType(qep,QEP_GENERAL);CHKERRQ(ierr); }
     ierr = PetscOptionsBoolGroup("-qep_hermitian","hermitian quadratic eigenvalue problem","QEPSetProblemType",&flg);CHKERRQ(ierr);
-    if (flg) {ierr = QEPSetProblemType(qep,QEP_HERMITIAN);CHKERRQ(ierr);}
+    if (flg) { ierr = QEPSetProblemType(qep,QEP_HERMITIAN);CHKERRQ(ierr); }
     ierr = PetscOptionsBoolGroupEnd("-qep_gyroscopic","gyroscopic quadratic eigenvalue problem","QEPSetProblemType",&flg);CHKERRQ(ierr);
-    if (flg) {ierr = QEPSetProblemType(qep,QEP_GYROSCOPIC);CHKERRQ(ierr);}
+    if (flg) { ierr = QEPSetProblemType(qep,QEP_GYROSCOPIC);CHKERRQ(ierr); }
 
-    r = PETSC_IGNORE;
-    ierr = PetscOptionsReal("-qep_scale","Scale factor","QEPSetScaleFactor",qep->sfactor,&r,PETSC_NULL);CHKERRQ(ierr);
+    r = 0;
+    ierr = PetscOptionsReal("-qep_scale","Scale factor","QEPSetScaleFactor",qep->sfactor,&r,NULL);CHKERRQ(ierr);
     ierr = QEPSetScaleFactor(qep,r);CHKERRQ(ierr);
 
-    r = i = PETSC_IGNORE;
-    ierr = PetscOptionsInt("-qep_max_it","Maximum number of iterations","QEPSetTolerances",qep->max_it,&i,PETSC_NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsReal("-qep_tol","Tolerance","QEPSetTolerances",qep->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:qep->tol,&r,PETSC_NULL);CHKERRQ(ierr);
+    r = i = 0;
+    ierr = PetscOptionsInt("-qep_max_it","Maximum number of iterations","QEPSetTolerances",qep->max_it,&i,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsReal("-qep_tol","Tolerance","QEPSetTolerances",qep->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:qep->tol,&r,NULL);CHKERRQ(ierr);
     ierr = QEPSetTolerances(qep,r,i);CHKERRQ(ierr);
     ierr = PetscOptionsBoolGroupBegin("-qep_convergence_default","Default (relative error) convergence test","QEPSetConvergenceTest",&flg);CHKERRQ(ierr);
-    if (flg) {ierr = QEPSetConvergenceTest(qep,QEPDefaultConverged,PETSC_NULL);CHKERRQ(ierr);}
+    if (flg) { ierr = QEPSetConvergenceTest(qep,QEPConvergedDefault,NULL);CHKERRQ(ierr); }
     ierr = PetscOptionsBoolGroupEnd("-qep_convergence_absolute","Absolute error convergence test","QEPSetConvergenceTest",&flg);CHKERRQ(ierr);
-    if (flg) {ierr = QEPSetConvergenceTest(qep,QEPAbsoluteConverged,PETSC_NULL);CHKERRQ(ierr);}
+    if (flg) { ierr = QEPSetConvergenceTest(qep,QEPConvergedAbsolute,NULL);CHKERRQ(ierr); }
 
-    i = j = k = PETSC_IGNORE;
-    ierr = PetscOptionsInt("-qep_nev","Number of eigenvalues to compute","QEPSetDimensions",qep->nev,&i,PETSC_NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsInt("-qep_ncv","Number of basis vectors","QEPSetDimensions",qep->ncv,&j,PETSC_NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsInt("-qep_mpd","Maximum dimension of projected problem","QEPSetDimensions",qep->mpd,&k,PETSC_NULL);CHKERRQ(ierr);
+    i = j = k = 0;
+    ierr = PetscOptionsInt("-qep_nev","Number of eigenvalues to compute","QEPSetDimensions",qep->nev,&i,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-qep_ncv","Number of basis vectors","QEPSetDimensions",qep->ncv,&j,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-qep_mpd","Maximum dimension of projected problem","QEPSetDimensions",qep->mpd,&k,NULL);CHKERRQ(ierr);
     ierr = QEPSetDimensions(qep,i,j,k);CHKERRQ(ierr);
-    
+
+    ierr = PetscOptionsScalar("-qep_target","Value of the target","QEPSetTarget",qep->target,&s,&flg);CHKERRQ(ierr);
+    if (flg) {
+      ierr = QEPSetWhichEigenpairs(qep,QEP_TARGET_MAGNITUDE);CHKERRQ(ierr);
+      ierr = QEPSetTarget(qep,s);CHKERRQ(ierr);
+    }
+
     /* -----------------------------------------------------------------------*/
     /*
       Cancels all monitors hardwired into code before call to QEPSetFromOptions()
     */
     flg  = PETSC_FALSE;
-    ierr = PetscOptionsBool("-qep_monitor_cancel","Remove any hardwired monitor routines","QEPMonitorCancel",flg,&flg,PETSC_NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsBool("-qep_monitor_cancel","Remove any hardwired monitor routines","QEPMonitorCancel",flg,&flg,NULL);CHKERRQ(ierr);
     if (flg) {
       ierr = QEPMonitorCancel(qep);CHKERRQ(ierr);
     }
     /*
       Prints approximate eigenvalues and error estimates at each iteration
     */
-    ierr = PetscOptionsString("-qep_monitor","Monitor first unconverged approximate eigenvalue and error estimate","QEPMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
+    ierr = PetscOptionsString("-qep_monitor","Monitor first unconverged approximate eigenvalue and error estimate","QEPMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
     if (flg) {
-      ierr = PetscViewerASCIIOpen(((PetscObject)qep)->comm,monfilename,&monviewer);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIOpen(PetscObjectComm((PetscObject)qep),monfilename,&monviewer);CHKERRQ(ierr);
       ierr = QEPMonitorSet(qep,QEPMonitorFirst,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
     }
-    ierr = PetscOptionsString("-qep_monitor_conv","Monitor approximate eigenvalues and error estimates as they converge","QEPMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
+    ierr = PetscOptionsString("-qep_monitor_conv","Monitor approximate eigenvalues and error estimates as they converge","QEPMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
     if (flg) {
       ierr = PetscNew(struct _n_SlepcConvMonitor,&ctx);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIOpen(((PetscObject)qep)->comm,monfilename,&ctx->viewer);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIOpen(PetscObjectComm((PetscObject)qep),monfilename,&ctx->viewer);CHKERRQ(ierr);
       ierr = QEPMonitorSet(qep,QEPMonitorConverged,ctx,(PetscErrorCode (*)(void**))SlepcConvMonitorDestroy);CHKERRQ(ierr);
     }
-    ierr = PetscOptionsString("-qep_monitor_all","Monitor approximate eigenvalues and error estimates","QEPMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
+    ierr = PetscOptionsString("-qep_monitor_all","Monitor approximate eigenvalues and error estimates","QEPMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
     if (flg) {
-      ierr = PetscViewerASCIIOpen(((PetscObject)qep)->comm,monfilename,&monviewer);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIOpen(PetscObjectComm((PetscObject)qep),monfilename,&monviewer);CHKERRQ(ierr);
       ierr = QEPMonitorSet(qep,QEPMonitorAll,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
       ierr = QEPSetTrackAll(qep,PETSC_TRUE);CHKERRQ(ierr);
     }
     flg = PETSC_FALSE;
-    ierr = PetscOptionsBool("-qep_monitor_draw","Monitor first unconverged approximate error estimate graphically","QEPMonitorSet",flg,&flg,PETSC_NULL);CHKERRQ(ierr); 
+    ierr = PetscOptionsBool("-qep_monitor_lg","Monitor first unconverged approximate error estimate graphically","QEPMonitorSet",flg,&flg,NULL);CHKERRQ(ierr);
     if (flg) {
-      ierr = QEPMonitorSet(qep,QEPMonitorLG,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+      ierr = QEPMonitorSet(qep,QEPMonitorLG,NULL,NULL);CHKERRQ(ierr);
     }
     flg = PETSC_FALSE;
-    ierr = PetscOptionsBool("-qep_monitor_draw_all","Monitor error estimates graphically","QEPMonitorSet",flg,&flg,PETSC_NULL);CHKERRQ(ierr); 
+    ierr = PetscOptionsBool("-qep_monitor_lg_all","Monitor error estimates graphically","QEPMonitorSet",flg,&flg,NULL);CHKERRQ(ierr);
     if (flg) {
-      ierr = QEPMonitorSet(qep,QEPMonitorLGAll,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+      ierr = QEPMonitorSet(qep,QEPMonitorLGAll,NULL,NULL);CHKERRQ(ierr);
       ierr = QEPSetTrackAll(qep,PETSC_TRUE);CHKERRQ(ierr);
     }
   /* -----------------------------------------------------------------------*/
 
     ierr = PetscOptionsBoolGroupBegin("-qep_largest_magnitude","compute largest eigenvalues in magnitude","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) {ierr = QEPSetWhichEigenpairs(qep,QEP_LARGEST_MAGNITUDE);CHKERRQ(ierr);}
+    if (flg) { ierr = QEPSetWhichEigenpairs(qep,QEP_LARGEST_MAGNITUDE);CHKERRQ(ierr); }
     ierr = PetscOptionsBoolGroup("-qep_smallest_magnitude","compute smallest eigenvalues in magnitude","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) {ierr = QEPSetWhichEigenpairs(qep,QEP_SMALLEST_MAGNITUDE);CHKERRQ(ierr);}
+    if (flg) { ierr = QEPSetWhichEigenpairs(qep,QEP_SMALLEST_MAGNITUDE);CHKERRQ(ierr); }
     ierr = PetscOptionsBoolGroup("-qep_largest_real","compute largest real parts","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) {ierr = QEPSetWhichEigenpairs(qep,QEP_LARGEST_REAL);CHKERRQ(ierr);}
+    if (flg) { ierr = QEPSetWhichEigenpairs(qep,QEP_LARGEST_REAL);CHKERRQ(ierr); }
     ierr = PetscOptionsBoolGroup("-qep_smallest_real","compute smallest real parts","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) {ierr = QEPSetWhichEigenpairs(qep,QEP_SMALLEST_REAL);CHKERRQ(ierr);}
+    if (flg) { ierr = QEPSetWhichEigenpairs(qep,QEP_SMALLEST_REAL);CHKERRQ(ierr); }
     ierr = PetscOptionsBoolGroup("-qep_largest_imaginary","compute largest imaginary parts","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) {ierr = QEPSetWhichEigenpairs(qep,QEP_LARGEST_IMAGINARY);CHKERRQ(ierr);}
-    ierr = PetscOptionsBoolGroupEnd("-qep_smallest_imaginary","compute smallest imaginary parts","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
-    if (flg) {ierr = QEPSetWhichEigenpairs(qep,QEP_SMALLEST_IMAGINARY);CHKERRQ(ierr);}
+    if (flg) { ierr = QEPSetWhichEigenpairs(qep,QEP_LARGEST_IMAGINARY);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-qep_smallest_imaginary","compute smallest imaginary parts","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = QEPSetWhichEigenpairs(qep,QEP_SMALLEST_IMAGINARY);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-qep_target_magnitude","compute nearest eigenvalues to target","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = QEPSetWhichEigenpairs(qep,QEP_TARGET_MAGNITUDE);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-qep_target_real","compute eigenvalues with real parts close to target","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = QEPSetWhichEigenpairs(qep,QEP_TARGET_REAL);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroupEnd("-qep_target_imaginary","compute eigenvalues with imaginary parts close to target","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = QEPSetWhichEigenpairs(qep,QEP_TARGET_IMAGINARY);CHKERRQ(ierr); }
 
     ierr = PetscOptionsBool("-qep_left_vectors","Compute left eigenvectors also","QEPSetLeftVectorsWanted",qep->leftvecs,&val,&flg);CHKERRQ(ierr);
     if (flg) {
@@ -150,37 +162,41 @@ PetscErrorCode QEPSetFromOptions(QEP qep)
     }
 
     ierr = PetscOptionsName("-qep_view","Print detailed information on solver used","QEPView",0);CHKERRQ(ierr);
-    ierr = PetscOptionsName("-qep_view_binary","Save the matrices associated to the eigenproblem","QEPSetFromOptions",0);CHKERRQ(ierr);
     ierr = PetscOptionsName("-qep_plot_eigs","Make a plot of the computed eigenvalues","QEPSolve",0);CHKERRQ(ierr);
-   
+
     if (qep->ops->setfromoptions) {
       ierr = (*qep->ops->setfromoptions)(qep);CHKERRQ(ierr);
     }
     ierr = PetscObjectProcessOptionsHandlers((PetscObject)qep);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
 
+  if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
   ierr = IPSetFromOptions(qep->ip);CHKERRQ(ierr);
+  if (!qep->ds) { ierr = QEPGetDS(qep,&qep->ds);CHKERRQ(ierr); }
+  ierr = DSSetFromOptions(qep->ds);CHKERRQ(ierr);
+  if (!qep->st) { ierr = QEPGetST(qep,&qep->st);CHKERRQ(ierr); }
+  ierr = STSetFromOptions(qep->st);CHKERRQ(ierr);
   ierr = PetscRandomSetFromOptions(qep->rand);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPGetTolerances"
 /*@
    QEPGetTolerances - Gets the tolerance and maximum iteration count used
-   by the QEP convergence tests. 
+   by the QEP convergence tests.
 
    Not Collective
 
    Input Parameter:
 .  qep - the quadratic eigensolver context
-  
+
    Output Parameters:
 +  tol - the convergence tolerance
 -  maxits - maximum number of iterations
 
    Notes:
-   The user can specify PETSC_NULL for any parameter that is not needed.
+   The user can specify NULL for any parameter that is not needed.
 
    Level: intermediate
 
@@ -195,11 +211,11 @@ PetscErrorCode QEPGetTolerances(QEP qep,PetscReal *tol,PetscInt *maxits)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetTolerances"
 /*@
    QEPSetTolerances - Sets the tolerance and maximum iteration count used
-   by the QEP convergence tests. 
+   by the QEP convergence tests.
 
    Logically Collective on QEP
 
@@ -213,9 +229,9 @@ PetscErrorCode QEPGetTolerances(QEP qep,PetscReal *tol,PetscInt *maxits)
 -  -qep_max_it <maxits> - Sets the maximum number of iterations allowed
 
    Notes:
-   Use PETSC_IGNORE for an argument that need not be changed.
+   Pass 0 for an argument that need not be changed.
 
-   Use PETSC_DECIDE for maxits to assign a reasonably good value, which is 
+   Use PETSC_DECIDE for maxits to assign a reasonably good value, which is
    dependent on the solution method.
 
    Level: intermediate
@@ -228,27 +244,27 @@ PetscErrorCode QEPSetTolerances(QEP qep,PetscReal tol,PetscInt maxits)
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidLogicalCollectiveReal(qep,tol,2);
   PetscValidLogicalCollectiveInt(qep,maxits,3);
-  if (tol != PETSC_IGNORE) {
+  if (tol) {
     if (tol == PETSC_DEFAULT) {
       qep->tol = PETSC_DEFAULT;
     } else {
-      if (tol < 0.0) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
+      if (tol < 0.0) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
       qep->tol = tol;
     }
   }
-  if (maxits != PETSC_IGNORE) {
+  if (maxits) {
     if (maxits == PETSC_DEFAULT || maxits == PETSC_DECIDE) {
       qep->max_it = 0;
       qep->setupcalled = 0;
     } else {
-      if (maxits < 0) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of maxits. Must be > 0");
+      if (maxits < 0) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of maxits. Must be > 0");
       qep->max_it = maxits;
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPGetDimensions"
 /*@
    QEPGetDimensions - Gets the number of eigenvalues to compute
@@ -258,14 +274,14 @@ PetscErrorCode QEPSetTolerances(QEP qep,PetscReal tol,PetscInt maxits)
 
    Input Parameter:
 .  qep - the quadratic eigensolver context
-  
+
    Output Parameters:
 +  nev - number of eigenvalues to compute
 .  ncv - the maximum dimension of the subspace to be used by the solver
 -  mpd - the maximum dimension allowed for the projected problem
 
    Notes:
-   The user can specify PETSC_NULL for any parameter that is not needed.
+   The user can specify NULL for any parameter that is not needed.
 
    Level: intermediate
 
@@ -281,7 +297,7 @@ PetscErrorCode QEPGetDimensions(QEP qep,PetscInt *nev,PetscInt *ncv,PetscInt *mp
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetDimensions"
 /*@
    QEPSetDimensions - Sets the number of eigenvalues to compute
@@ -301,7 +317,7 @@ PetscErrorCode QEPGetDimensions(QEP qep,PetscInt *nev,PetscInt *ncv,PetscInt *mp
 -  -qep_mpd <mpd> - Sets the maximum projected dimension
 
    Notes:
-   Use PETSC_IGNORE to retain the previous value of any parameter.
+   Pass 0 to retain the previous value of any parameter.
 
    Use PETSC_DECIDE for ncv and mpd to assign a reasonably good value, which is
    dependent on the solution method.
@@ -326,35 +342,35 @@ PetscErrorCode QEPSetDimensions(QEP qep,PetscInt nev,PetscInt ncv,PetscInt mpd)
   PetscValidLogicalCollectiveInt(qep,nev,2);
   PetscValidLogicalCollectiveInt(qep,ncv,3);
   PetscValidLogicalCollectiveInt(qep,mpd,4);
-  if (nev != PETSC_IGNORE) {
-    if (nev<1) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of nev. Must be > 0");
+  if (nev) {
+    if (nev<1) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of nev. Must be > 0");
     qep->nev = nev;
     qep->setupcalled = 0;
   }
-  if (ncv != PETSC_IGNORE) {
+  if (ncv) {
     if (ncv == PETSC_DECIDE || ncv == PETSC_DEFAULT) {
       qep->ncv = 0;
     } else {
-      if (ncv<1) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of ncv. Must be > 0");
+      if (ncv<1) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of ncv. Must be > 0");
       qep->ncv = ncv;
     }
     qep->setupcalled = 0;
   }
-  if (mpd != PETSC_IGNORE) {
+  if (mpd) {
     if (mpd == PETSC_DECIDE || mpd == PETSC_DEFAULT) {
       qep->mpd = 0;
     } else {
-      if (mpd<1) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of mpd. Must be > 0");
+      if (mpd<1) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of mpd. Must be > 0");
       qep->mpd = mpd;
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetWhichEigenpairs"
 /*@
-    QEPSetWhichEigenpairs - Specifies which portion of the spectrum is 
+    QEPSetWhichEigenpairs - Specifies which portion of the spectrum is
     to be sought.
 
     Logically Collective on QEP
@@ -365,13 +381,16 @@ PetscErrorCode QEPSetDimensions(QEP qep,PetscInt nev,PetscInt ncv,PetscInt mpd)
 
     Possible values:
     The parameter 'which' can have one of these values
-    
+
 +     QEP_LARGEST_MAGNITUDE - largest eigenvalues in magnitude (default)
 .     QEP_SMALLEST_MAGNITUDE - smallest eigenvalues in magnitude
 .     QEP_LARGEST_REAL - largest real parts
 .     QEP_SMALLEST_REAL - smallest real parts
 .     QEP_LARGEST_IMAGINARY - largest imaginary parts
--     QEP_SMALLEST_IMAGINARY - smallest imaginary parts
+.     QEP_SMALLEST_IMAGINARY - smallest imaginary parts
+.     QEP_TARGET_MAGNITUDE - eigenvalues closest to the target (in magnitude)
+.     QEP_TARGET_REAL - eigenvalues with real part closest to target
+-     QEP_TARGET_IMAGINARY - eigenvalues with imaginary part closest to target
 
     Options Database Keys:
 +   -qep_largest_magnitude - Sets largest eigenvalues in magnitude
@@ -379,14 +398,17 @@ PetscErrorCode QEPSetDimensions(QEP qep,PetscInt nev,PetscInt ncv,PetscInt mpd)
 .   -qep_largest_real - Sets largest real parts
 .   -qep_smallest_real - Sets smallest real parts
 .   -qep_largest_imaginary - Sets largest imaginary parts
--   -qep_smallest_imaginary - Sets smallest imaginary parts
+.   -qep_smallest_imaginary - Sets smallest imaginary parts
+.   -qep_target_magnitude - Sets eigenvalues closest to target
+.   -qep_target_real - Sets real parts closest to target
+-   -qep_target_imaginary - Sets imaginary parts closest to target
 
     Notes:
     Not all eigensolvers implemented in QEP account for all the possible values
     stated above. If SLEPc is compiled for real numbers QEP_LARGEST_IMAGINARY
-    and QEP_SMALLEST_IMAGINARY use the absolute value of the imaginary part 
+    and QEP_SMALLEST_IMAGINARY use the absolute value of the imaginary part
     for eigenvalue selection.
-    
+
     Level: intermediate
 
 .seealso: QEPGetWhichEigenpairs(), QEPWhich
@@ -396,7 +418,7 @@ PetscErrorCode QEPSetWhichEigenpairs(QEP qep,QEPWhich which)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidLogicalCollectiveEnum(qep,which,2);
-  if (which!=PETSC_IGNORE) {
+  if (which) {
     if (which==PETSC_DECIDE || which==PETSC_DEFAULT) qep->which = (QEPWhich)0;
     else switch (which) {
       case QEP_LARGEST_MAGNITUDE:
@@ -405,22 +427,27 @@ PetscErrorCode QEPSetWhichEigenpairs(QEP qep,QEPWhich which)
       case QEP_SMALLEST_REAL:
       case QEP_LARGEST_IMAGINARY:
       case QEP_SMALLEST_IMAGINARY:
+      case QEP_TARGET_MAGNITUDE:
+      case QEP_TARGET_REAL:
+#if defined(PETSC_USE_COMPLEX)
+      case QEP_TARGET_IMAGINARY:
+#endif
         if (qep->which != which) {
           qep->setupcalled = 0;
           qep->which = which;
         }
         break;
       default:
-        SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'which' value"); 
+        SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'which' value");
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPGetWhichEigenpairs"
 /*@C
-    QEPGetWhichEigenpairs - Returns which portion of the spectrum is to be 
+    QEPGetWhichEigenpairs - Returns which portion of the spectrum is to be
     sought.
 
     Not Collective
@@ -438,7 +465,7 @@ PetscErrorCode QEPSetWhichEigenpairs(QEP qep,QEPWhich which)
 
 .seealso: QEPSetWhichEigenpairs(), QEPWhich
 @*/
-PetscErrorCode QEPGetWhichEigenpairs(QEP qep,QEPWhich *which) 
+PetscErrorCode QEPGetWhichEigenpairs(QEP qep,QEPWhich *which)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
@@ -447,7 +474,7 @@ PetscErrorCode QEPGetWhichEigenpairs(QEP qep,QEPWhich *which)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetLeftVectorsWanted"
 /*@
     QEPSetLeftVectorsWanted - Specifies which eigenvectors are required.
@@ -482,10 +509,10 @@ PetscErrorCode QEPSetLeftVectorsWanted(QEP qep,PetscBool leftvecs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPGetLeftVectorsWanted"
 /*@C
-    QEPGetLeftVectorsWanted - Returns the flag indicating whether left 
+    QEPGetLeftVectorsWanted - Returns the flag indicating whether left
     eigenvectors are required or not.
 
     Not Collective
@@ -500,7 +527,7 @@ PetscErrorCode QEPSetLeftVectorsWanted(QEP qep,PetscBool leftvecs)
 
 .seealso: QEPSetLeftVectorsWanted()
 @*/
-PetscErrorCode QEPGetLeftVectorsWanted(QEP qep,PetscBool *leftvecs) 
+PetscErrorCode QEPGetLeftVectorsWanted(QEP qep,PetscBool *leftvecs)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
@@ -509,7 +536,7 @@ PetscErrorCode QEPGetLeftVectorsWanted(QEP qep,PetscBool *leftvecs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPGetScaleFactor"
 /*@
    QEPGetScaleFactor - Gets the factor used for scaling the quadratic eigenproblem.
@@ -518,7 +545,7 @@ PetscErrorCode QEPGetLeftVectorsWanted(QEP qep,PetscBool *leftvecs)
 
    Input Parameter:
 .  qep - the quadratic eigensolver context
-  
+
    Output Parameters:
 .  alpha - the scaling factor
 
@@ -539,7 +566,7 @@ PetscErrorCode QEPGetScaleFactor(QEP qep,PetscReal *alpha)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetScaleFactor"
 /*@
    QEPSetScaleFactor - Sets the scaling factor to be used for scaling the
@@ -569,18 +596,20 @@ PetscErrorCode QEPSetScaleFactor(QEP qep,PetscReal alpha)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidLogicalCollectiveReal(qep,alpha,2);
-  if (alpha != PETSC_IGNORE) {
+  if (alpha) {
     if (alpha == PETSC_DEFAULT || alpha == PETSC_DECIDE) {
       qep->sfactor = 0.0;
+      qep->sfactor_set = PETSC_FALSE;
     } else {
-      if (alpha < 0.0) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of alpha. Must be > 0");
+      if (alpha < 0.0) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of alpha. Must be > 0");
       qep->sfactor = alpha;
+      qep->sfactor_set = PETSC_TRUE;
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetProblemType"
 /*@
    QEPSetProblemType - Specifies the type of the quadratic eigenvalue problem.
@@ -589,23 +618,23 @@ PetscErrorCode QEPSetScaleFactor(QEP qep,PetscReal alpha)
 
    Input Parameters:
 +  qep      - the quadratic eigensolver context
--  type     - a known type of quadratic eigenvalue problem 
+-  type     - a known type of quadratic eigenvalue problem
 
    Options Database Keys:
 +  -qep_general - general problem with no particular structure
 .  -qep_hermitian - problem whose coefficient matrices are Hermitian
 -  -qep_gyroscopic - problem with Hamiltonian structure
-    
-   Notes:  
+
+   Notes:
    Allowed values for the problem type are: general (QEP_GENERAL), Hermitian
    (QEP_HERMITIAN), and gyroscopic (QEP_GYROSCOPIC).
 
    This function is used to instruct SLEPc to exploit certain structure in
    the quadratic eigenproblem. By default, no particular structure is assumed.
 
-   If the problem matrices are Hermitian (symmetric in the real case) or 
+   If the problem matrices are Hermitian (symmetric in the real case) or
    Hermitian/skew-Hermitian then the solver can exploit this fact to perform
-   less operations or provide better stability. 
+   less operations or provide better stability.
 
    Level: intermediate
 
@@ -617,12 +646,12 @@ PetscErrorCode QEPSetProblemType(QEP qep,QEPProblemType type)
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidLogicalCollectiveEnum(qep,type,2);
   if (type!=QEP_GENERAL && type!=QEP_HERMITIAN && type!=QEP_GYROSCOPIC)
-    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_WRONG,"Unknown eigenvalue problem type");
+    SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_WRONG,"Unknown eigenvalue problem type");
   qep->problem_type = type;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPGetProblemType"
 /*@C
    QEPGetProblemType - Gets the problem type from the QEP object.
@@ -630,10 +659,10 @@ PetscErrorCode QEPSetProblemType(QEP qep,QEPProblemType type)
    Not Collective
 
    Input Parameter:
-.  qep - the quadratic eigensolver context 
+.  qep - the quadratic eigensolver context
 
    Output Parameter:
-.  type - name of QEP problem type 
+.  type - name of QEP problem type
 
    Level: intermediate
 
@@ -648,10 +677,10 @@ PetscErrorCode QEPGetProblemType(QEP qep,QEPProblemType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetConvergenceTest"
 /*@C
-    QEPSetConvergenceTest - Sets a function to compute the error estimate used in 
+    QEPSetConvergenceTest - Sets a function to compute the error estimate used in
     the convergence test.
 
     Logically Collective on QEP
@@ -679,16 +708,16 @@ $   func(QEP qep,PetscScalar eigr,PetscScalar eigi,PetscReal res,PetscReal* erre
 
 .seealso: QEPSetTolerances()
 @*/
-extern PetscErrorCode QEPSetConvergenceTest(QEP qep,PetscErrorCode (*func)(QEP,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*),void* ctx)
+PetscErrorCode QEPSetConvergenceTest(QEP qep,PetscErrorCode (*func)(QEP,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*),void* ctx)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
-  qep->conv_func = func;
-  qep->conv_ctx = ctx;
+  qep->converged    = func;
+  qep->convergedctx = ctx;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetTrackAll"
 /*@
    QEPSetTrackAll - Specifies if the solver must compute the residual of all
@@ -705,8 +734,8 @@ extern PetscErrorCode QEPSetConvergenceTest(QEP qep,PetscErrorCode (*func)(QEP,P
    the residual for each eigenpair approximation. Computing the residual is
    usually an expensive operation and solvers commonly compute the associated
    residual to the first unconverged eigenpair.
-   The options '-qep_monitor_all' and '-qep_monitor_draw_all' automatically
-   activates this option.
+   The options '-qep_monitor_all' and '-qep_monitor_lg_all' automatically
+   activate this option.
 
    Level: intermediate
 
@@ -721,7 +750,7 @@ PetscErrorCode QEPSetTrackAll(QEP qep,PetscBool trackall)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPGetTrackAll"
 /*@
    QEPGetTrackAll - Returns the flag indicating whether all residual norms must
@@ -739,7 +768,7 @@ PetscErrorCode QEPSetTrackAll(QEP qep,PetscBool trackall)
 
 .seealso: QEPSetTrackAll()
 @*/
-PetscErrorCode QEPGetTrackAll(QEP qep,PetscBool *trackall) 
+PetscErrorCode QEPGetTrackAll(QEP qep,PetscBool *trackall)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
@@ -748,10 +777,10 @@ PetscErrorCode QEPGetTrackAll(QEP qep,PetscBool *trackall)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetOptionsPrefix"
 /*@C
-   QEPSetOptionsPrefix - Sets the prefix used for searching for all 
+   QEPSetOptionsPrefix - Sets the prefix used for searching for all
    QEP options in the database.
 
    Logically Collective on QEP
@@ -784,14 +813,16 @@ PetscErrorCode QEPSetOptionsPrefix(QEP qep,const char *prefix)
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
   ierr = IPSetOptionsPrefix(qep->ip,prefix);CHKERRQ(ierr);
+  if (!qep->ds) { ierr = QEPGetDS(qep,&qep->ds);CHKERRQ(ierr); }
+  ierr = DSSetOptionsPrefix(qep->ds,prefix);CHKERRQ(ierr);
   ierr = PetscObjectSetOptionsPrefix((PetscObject)qep,prefix);CHKERRQ(ierr);
-  PetscFunctionReturn(0);  
+  PetscFunctionReturn(0);
 }
- 
-#undef __FUNCT__  
+
+#undef __FUNCT__
 #define __FUNCT__ "QEPAppendOptionsPrefix"
 /*@C
-   QEPAppendOptionsPrefix - Appends to the prefix used for searching for all 
+   QEPAppendOptionsPrefix - Appends to the prefix used for searching for all
    QEP options in the database.
 
    Logically Collective on QEP
@@ -818,8 +849,10 @@ PetscErrorCode QEPAppendOptionsPrefix(QEP qep,const char *prefix)
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
   ierr = IPSetOptionsPrefix(qep->ip,prefix);CHKERRQ(ierr);
+  if (!qep->ds) { ierr = QEPGetDS(qep,&qep->ds);CHKERRQ(ierr); }
+  ierr = DSSetOptionsPrefix(qep->ds,prefix);CHKERRQ(ierr);
   ierr = PetscObjectAppendOptionsPrefix((PetscObject)qep,prefix);CHKERRQ(ierr);
-  ierr = PetscTypeCompare((PetscObject)qep,QEPLINEAR,&flg);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)qep,QEPLINEAR,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = QEPLinearGetEPS(qep,&eps);CHKERRQ(ierr);
     ierr = EPSSetOptionsPrefix(eps,((PetscObject)qep)->prefix);CHKERRQ(ierr);
@@ -828,10 +861,10 @@ PetscErrorCode QEPAppendOptionsPrefix(QEP qep,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPGetOptionsPrefix"
 /*@C
-   QEPGetOptionsPrefix - Gets the prefix used for searching for all 
+   QEPGetOptionsPrefix - Gets the prefix used for searching for all
    QEP options in the database.
 
    Not Collective
