@@ -1,17 +1,17 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -19,14 +19,13 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/qepimpl.h>      /*I "slepcqep.h" I*/
+#include <slepc-private/qepimpl.h>      /*I "slepcqep.h" I*/
 
-EXTERN_C_BEGIN
-extern PetscErrorCode QEPCreate_Linear(QEP);
-extern PetscErrorCode QEPCreate_QArnoldi(QEP);
-EXTERN_C_END
-  
-#undef __FUNCT__  
+PETSC_EXTERN PetscErrorCode QEPCreate_Linear(QEP);
+PETSC_EXTERN PetscErrorCode QEPCreate_QArnoldi(QEP);
+PETSC_EXTERN PetscErrorCode QEPCreate_QLanczos(QEP);
+
+#undef __FUNCT__
 #define __FUNCT__ "QEPRegisterAll"
 /*@C
    QEPRegisterAll - Registers all the solvers in the QEP package.
@@ -35,15 +34,16 @@ EXTERN_C_END
 
    Level: advanced
 
-.seealso:  QEPRegisterDynamic()
+.seealso:  QEPRegister()
 @*/
-PetscErrorCode QEPRegisterAll(const char *path)
+PetscErrorCode QEPRegisterAll(void)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   QEPRegisterAllCalled = PETSC_TRUE;
-  ierr = QEPRegisterDynamic(QEPLINEAR,path,"QEPCreate_Linear",QEPCreate_Linear);CHKERRQ(ierr);
-  ierr = QEPRegisterDynamic(QEPQARNOLDI,path,"QEPCreate_QArnoldi",QEPCreate_QArnoldi);CHKERRQ(ierr);
+  ierr = QEPRegister(QEPLINEAR,QEPCreate_Linear);CHKERRQ(ierr);
+  ierr = QEPRegister(QEPQARNOLDI,QEPCreate_QArnoldi);CHKERRQ(ierr);
+  ierr = QEPRegister(QEPQLANCZOS,QEPCreate_QLanczos);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
