@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,10 +21,10 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/qepimpl.h>       /*I "slepcqep.h" I*/
-#include <private/ipimpl.h>
+#include <slepc-private/qepimpl.h>       /*I "slepcqep.h" I*/
+#include <slepc-private/ipimpl.h>
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetUp"
 /*@
    QEPSetUp - Sets up all the internal data structures necessary for the
@@ -37,7 +37,7 @@
 
    Notes:
    This function need not be called explicitly in most cases, since QEPSolve()
-   calls it. It can be useful when one wants to measure the set-up time 
+   calls it. It can be useful when one wants to measure the set-up time
    separately from the solve time.
 
    Level: advanced
@@ -47,36 +47,54 @@
 PetscErrorCode QEPSetUp(QEP qep)
 {
   PetscErrorCode ierr;
-  PetscInt       i,k;
-  PetscBool      khas,mhas,lindep;
-  PetscReal      knorm,mnorm,norm;
-  
+  PetscBool      khas,mhas,islinear,flg;
+  PetscReal      knorm,mnorm;
+  Mat            mat[3];
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   if (qep->setupcalled) PetscFunctionReturn(0);
   ierr = PetscLogEventBegin(QEP_SetUp,qep,0,0,0);CHKERRQ(ierr);
 
+  /* reset the convergence flag from the previous solves */
+  qep->reason = QEP_CONVERGED_ITERATING;
+
   /* Set default solver type (QEPSetFromOptions was not called) */
   if (!((PetscObject)qep)->type_name) {
     ierr = QEPSetType(qep,QEPLINEAR);CHKERRQ(ierr);
   }
+  ierr = PetscObjectTypeCompare((PetscObject)qep,QEPLINEAR,&islinear);CHKERRQ(ierr);
+  if (!islinear) {
+    if (!qep->st) { ierr = QEPGetST(qep,&qep->st);CHKERRQ(ierr); }
+    if (!((PetscObject)qep->st)->type_name) {
+      ierr = STSetType(qep->st,STSHIFT);CHKERRQ(ierr);
+    }
+  }
   if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
   if (!((PetscObject)qep->ip)->type_name) {
-    ierr = IPSetDefaultType_Private(qep->ip);CHKERRQ(ierr);
+    ierr = IPSetType_Default(qep->ip);CHKERRQ(ierr);
   }
+  if (!qep->ds) { ierr = QEPGetDS(qep,&qep->ds);CHKERRQ(ierr); }
+  ierr = DSReset(qep->ds);CHKERRQ(ierr);
   if (!((PetscObject)qep->rand)->type_name) {
     ierr = PetscRandomSetFromOptions(qep->rand);CHKERRQ(ierr);
   }
 
-  /* Check matrices */
-  if (!qep->M || !qep->C || !qep->K)
-    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_WRONGSTATE,"QEPSetOperators must be called first"); 
-  
+  /* Check matrices, transfer them to ST */
+  if (!qep->M || !qep->C || !qep->K) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_WRONGSTATE,"QEPSetOperators must be called first");
+  if (!islinear) {
+    mat[0] = qep->K;
+    mat[1] = qep->C;
+    mat[2] = qep->M;
+    ierr = STSetOperators(qep->st,3,mat);CHKERRQ(ierr);
+  }
+
   /* Set problem dimensions */
-  ierr = MatGetSize(qep->M,&qep->n,PETSC_NULL);CHKERRQ(ierr);
-  ierr = MatGetLocalSize(qep->M,&qep->nloc,PETSC_NULL);CHKERRQ(ierr);
+  ierr = MatGetSize(qep->M,&qep->n,NULL);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(qep->M,&qep->nloc,NULL);CHKERRQ(ierr);
   ierr = VecDestroy(&qep->t);CHKERRQ(ierr);
-  ierr = SlepcMatGetVecsTemplate(qep->M,&qep->t,PETSC_NULL);CHKERRQ(ierr);
+  ierr = SlepcMatGetVecsTemplate(qep->M,&qep->t,NULL);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent(qep,qep->t);CHKERRQ(ierr);
 
   /* Set default problem type */
   if (!qep->problem_type) {
@@ -91,8 +109,7 @@ PetscErrorCode QEPSetUp(QEP qep)
       ierr = MatNorm(qep->K,NORM_INFINITY,&knorm);CHKERRQ(ierr);
       ierr = MatNorm(qep->M,NORM_INFINITY,&mnorm);CHKERRQ(ierr);
       qep->sfactor = PetscSqrtReal(knorm/mnorm);
-    }
-    else qep->sfactor = 1.0;
+    } else qep->sfactor = 1.0;
   }
 
   /* Call specific solver setup */
@@ -101,47 +118,68 @@ PetscErrorCode QEPSetUp(QEP qep)
   /* set tolerance if not yet set */
   if (qep->tol==PETSC_DEFAULT) qep->tol = SLEPC_DEFAULT_TOL;
 
-  if (qep->ncv > 2*qep->n)
-    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"ncv must be twice the problem size at most");
-  if (qep->nev > qep->ncv)
-    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"nev bigger than ncv");
+  /* set eigenvalue comparison */
+  switch (qep->which) {
+    case QEP_LARGEST_MAGNITUDE:
+      qep->comparison    = SlepcCompareLargestMagnitude;
+      qep->comparisonctx = NULL;
+      break;
+    case QEP_SMALLEST_MAGNITUDE:
+      qep->comparison    = SlepcCompareSmallestMagnitude;
+      qep->comparisonctx = NULL;
+      break;
+    case QEP_LARGEST_REAL:
+      qep->comparison    = SlepcCompareLargestReal;
+      qep->comparisonctx = NULL;
+      break;
+    case QEP_SMALLEST_REAL:
+      qep->comparison    = SlepcCompareSmallestReal;
+      qep->comparisonctx = NULL;
+      break;
+    case QEP_LARGEST_IMAGINARY:
+      qep->comparison    = SlepcCompareLargestImaginary;
+      qep->comparisonctx = NULL;
+      break;
+    case QEP_SMALLEST_IMAGINARY:
+      qep->comparison    = SlepcCompareSmallestImaginary;
+      qep->comparisonctx = NULL;
+      break;
+    case QEP_TARGET_MAGNITUDE:
+      qep->comparison    = SlepcCompareTargetMagnitude;
+      qep->comparisonctx = &qep->target;
+      break;
+    case QEP_TARGET_REAL:
+      qep->comparison    = SlepcCompareTargetReal;
+      qep->comparisonctx = &qep->target;
+      break;
+    case QEP_TARGET_IMAGINARY:
+      qep->comparison    = SlepcCompareTargetImaginary;
+      qep->comparisonctx = &qep->target;
+      break;
+  }
+
+  if (qep->ncv > 2*qep->n) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_OUTOFRANGE,"ncv must be twice the problem size at most");
+  if (qep->nev > qep->ncv) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_OUTOFRANGE,"nev bigger than ncv");
+
+  /* Setup ST */
+  if (!islinear) {
+    ierr = PetscObjectTypeCompareAny((PetscObject)qep->st,&flg,STSHIFT,STSINVERT,"");CHKERRQ(ierr);
+    if (!flg) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_SUP,"Only STSHIFT and STSINVERT spectral transformations can be used in QEP");
+    ierr = STSetUp(qep->st);CHKERRQ(ierr);
+  }
 
   /* process initial vectors */
   if (qep->nini<0) {
     qep->nini = -qep->nini;
-    if (qep->nini>qep->ncv) SETERRQ(((PetscObject)qep)->comm,1,"The number of initial vectors is larger than ncv");
-    k = 0;
-    for (i=0;i<qep->nini;i++) {
-      ierr = VecCopy(qep->IS[i],qep->V[k]);CHKERRQ(ierr);
-      ierr = VecDestroy(&qep->IS[i]);CHKERRQ(ierr);
-      ierr = IPOrthogonalize(qep->ip,0,PETSC_NULL,k,PETSC_NULL,qep->V,qep->V[k],PETSC_NULL,&norm,&lindep);CHKERRQ(ierr); 
-      if (norm==0.0 || lindep) PetscInfo(qep,"Linearly dependent initial vector found, removing...\n");
-      else {
-        ierr = VecScale(qep->V[k],1.0/norm);CHKERRQ(ierr);
-        k++;
-      }
-    }
-    qep->nini = k;
-    ierr = PetscFree(qep->IS);CHKERRQ(ierr);
+    if (qep->nini>qep->ncv) SETERRQ(PetscObjectComm((PetscObject)qep),1,"The number of initial vectors is larger than ncv");
+    ierr = IPOrthonormalizeBasis_Private(qep->ip,&qep->nini,&qep->IS,qep->V);CHKERRQ(ierr);
   }
   if (qep->ninil<0) {
-    if (!qep->leftvecs) PetscInfo(qep,"Ignoring initial left vectors\n");
+    if (!qep->leftvecs) { ierr = PetscInfo(qep,"Ignoring initial left vectors\n");CHKERRQ(ierr); }
     else {
       qep->ninil = -qep->ninil;
-      if (qep->ninil>qep->ncv) SETERRQ(((PetscObject)qep)->comm,1,"The number of initial left vectors is larger than ncv");
-      k = 0;
-      for (i=0;i<qep->ninil;i++) {
-        ierr = VecCopy(qep->ISL[i],qep->W[k]);CHKERRQ(ierr);
-        ierr = VecDestroy(&qep->ISL[i]);CHKERRQ(ierr);
-        ierr = IPOrthogonalize(qep->ip,0,PETSC_NULL,k,PETSC_NULL,qep->W,qep->W[k],PETSC_NULL,&norm,&lindep);CHKERRQ(ierr); 
-        if (norm==0.0 || lindep) PetscInfo(qep,"Linearly dependent initial left vector found, removing...\n");
-        else {
-          ierr = VecScale(qep->W[k],1.0/norm);CHKERRQ(ierr);
-          k++;
-        }
-      }
-      qep->ninil = k;
-      ierr = PetscFree(qep->ISL);CHKERRQ(ierr);
+      if (qep->ninil>qep->ncv) SETERRQ(PetscObjectComm((PetscObject)qep),1,"The number of initial left vectors is larger than ncv");
+      ierr = IPOrthonormalizeBasis_Private(qep->ip,&qep->ninil,&qep->ISL,qep->W);CHKERRQ(ierr);
     }
   }
   ierr = PetscLogEventEnd(QEP_SetUp,qep,0,0,0);CHKERRQ(ierr);
@@ -149,7 +187,7 @@ PetscErrorCode QEPSetUp(QEP qep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetOperators"
 /*@
    QEPSetOperators - Sets the matrices associated with the quadratic eigenvalue problem.
@@ -162,7 +200,7 @@ PetscErrorCode QEPSetUp(QEP qep)
 .  C   - the second coefficient matrix
 -  K   - the third coefficient matrix
 
-   Notes: 
+   Notes:
    The quadratic eigenproblem is defined as (l^2*M + l*C + K)*x = 0, where l is
    the eigenvalue and x is the eigenvector.
 
@@ -186,14 +224,14 @@ PetscErrorCode QEPSetOperators(QEP qep,Mat M,Mat C,Mat K)
 
   /* Check for square matrices */
   ierr = MatGetSize(M,&m,&n);CHKERRQ(ierr);
-  if (m!=n) { SETERRQ(((PetscObject)qep)->comm,1,"M is a non-square matrix"); }
+  if (m!=n) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_WRONG,"M is a non-square matrix");
   m0=m;
   ierr = MatGetSize(C,&m,&n);CHKERRQ(ierr);
-  if (m!=n) { SETERRQ(((PetscObject)qep)->comm,1,"C is a non-square matrix"); }
-  if (m!=m0) { SETERRQ(((PetscObject)qep)->comm,1,"Dimensions of M and C do not match"); }
+  if (m!=n) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_WRONG,"C is a non-square matrix");
+  if (m!=m0) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_INCOMP,"Dimensions of M and C do not match");
   ierr = MatGetSize(K,&m,&n);CHKERRQ(ierr);
-  if (m!=n) { SETERRQ(((PetscObject)qep)->comm,1,"K is a non-square matrix"); }
-  if (m!=m0) { SETERRQ(((PetscObject)qep)->comm,1,"Dimensions of M and K do not match"); }
+  if (m!=n) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_WRONG,"K is a non-square matrix");
+  if (m!=m0) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_INCOMP,"Dimensions of M and K do not match");
 
   /* Store a copy of the matrices */
   if (qep->setupcalled) { ierr = QEPReset(qep);CHKERRQ(ierr); }
@@ -238,7 +276,7 @@ PetscErrorCode QEPGetOperators(QEP qep,Mat *M,Mat *C,Mat *K)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetInitialSpace"
 /*@
    QEPSetInitialSpace - Specify a basis of vectors that constitute the initial
@@ -271,34 +309,17 @@ PetscErrorCode QEPGetOperators(QEP qep,Mat *M,Mat *C,Mat *K)
 PetscErrorCode QEPSetInitialSpace(QEP qep,PetscInt n,Vec *is)
 {
   PetscErrorCode ierr;
-  PetscInt       i;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidLogicalCollectiveInt(qep,n,2);
-  if (n<0) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative"); 
-
-  /* free previous non-processed vectors */
-  if (qep->nini<0) {
-    for (i=0;i<-qep->nini;i++) {
-      ierr = VecDestroy(&qep->IS[i]);CHKERRQ(ierr);
-    }
-    ierr = PetscFree(qep->IS);CHKERRQ(ierr);
-  }
-
-  /* get references of passed vectors */
-  ierr = PetscMalloc(n*sizeof(Vec),&qep->IS);CHKERRQ(ierr);
-  for (i=0;i<n;i++) {
-    ierr = PetscObjectReference((PetscObject)is[i]);CHKERRQ(ierr);
-    qep->IS[i] = is[i];
-  }
-
-  qep->nini = -n;
-  qep->setupcalled = 0;
+  if (n<0) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
+  ierr = SlepcBasisReference_Private(n,is,&qep->nini,&qep->IS);CHKERRQ(ierr);
+  if (n>0) qep->setupcalled = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSetInitialSpaceLeft"
 /*@
    QEPSetInitialSpaceLeft - Specify a basis of vectors that constitute the initial
@@ -332,34 +353,17 @@ PetscErrorCode QEPSetInitialSpace(QEP qep,PetscInt n,Vec *is)
 PetscErrorCode QEPSetInitialSpaceLeft(QEP qep,PetscInt n,Vec *is)
 {
   PetscErrorCode ierr;
-  PetscInt       i;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidLogicalCollectiveInt(qep,n,2);
-  if (n<0) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative"); 
-
-  /* free previous non-processed vectors */
-  if (qep->ninil<0) {
-    for (i=0;i<-qep->ninil;i++) {
-      ierr = VecDestroy(&qep->ISL[i]);CHKERRQ(ierr);
-    }
-    ierr = PetscFree(qep->ISL);CHKERRQ(ierr);
-  }
-
-  /* get references of passed vectors */
-  ierr = PetscMalloc(n*sizeof(Vec),&qep->ISL);CHKERRQ(ierr);
-  for (i=0;i<n;i++) {
-    ierr = PetscObjectReference((PetscObject)is[i]);CHKERRQ(ierr);
-    qep->ISL[i] = is[i];
-  }
-
-  qep->ninil = -n;
-  qep->setupcalled = 0;
+  if (n<0) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
+  ierr = SlepcBasisReference_Private(n,is,&qep->ninil,&qep->ISL);CHKERRQ(ierr);
+  if (n>0) qep->setupcalled = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPAllocateSolution"
 /*
   QEPAllocateSolution - Allocate memory storage for common variables such
@@ -369,36 +373,43 @@ PetscErrorCode QEPSetInitialSpaceLeft(QEP qep,PetscInt n,Vec *is)
 PetscErrorCode QEPAllocateSolution(QEP qep)
 {
   PetscErrorCode ierr;
-  
+  PetscInt       newc,cnt;
+
   PetscFunctionBegin;
   if (qep->allocated_ncv != qep->ncv) {
+    newc = PetscMax(0,qep->ncv-qep->allocated_ncv);
     ierr = QEPFreeSolution(qep);CHKERRQ(ierr);
+    cnt = 0;
     ierr = PetscMalloc(qep->ncv*sizeof(PetscScalar),&qep->eigr);CHKERRQ(ierr);
     ierr = PetscMalloc(qep->ncv*sizeof(PetscScalar),&qep->eigi);CHKERRQ(ierr);
+    cnt += 2*newc*sizeof(PetscScalar);
     ierr = PetscMalloc(qep->ncv*sizeof(PetscReal),&qep->errest);CHKERRQ(ierr);
     ierr = PetscMalloc(qep->ncv*sizeof(PetscInt),&qep->perm);CHKERRQ(ierr);
+    cnt += 2*newc*sizeof(PetscReal);
+    ierr = PetscLogObjectMemory(qep,cnt);CHKERRQ(ierr);
     ierr = VecDuplicateVecs(qep->t,qep->ncv,&qep->V);CHKERRQ(ierr);
+    ierr = PetscLogObjectParents(qep,qep->ncv,qep->V);CHKERRQ(ierr);
     qep->allocated_ncv = qep->ncv;
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPFreeSolution"
 /*
-  QEPFreeSolution - Free memory storage. This routine is related to 
+  QEPFreeSolution - Free memory storage. This routine is related to
   QEPAllocateSolution().
 */
 PetscErrorCode QEPFreeSolution(QEP qep)
 {
   PetscErrorCode ierr;
-  
+
   PetscFunctionBegin;
   if (qep->allocated_ncv > 0) {
     ierr = PetscFree(qep->eigr);CHKERRQ(ierr);
     ierr = PetscFree(qep->eigi);CHKERRQ(ierr);
-    ierr = PetscFree(qep->errest);CHKERRQ(ierr); 
-    ierr = PetscFree(qep->perm);CHKERRQ(ierr); 
+    ierr = PetscFree(qep->errest);CHKERRQ(ierr);
+    ierr = PetscFree(qep->perm);CHKERRQ(ierr);
     ierr = VecDestroyVecs(qep->allocated_ncv,&qep->V);CHKERRQ(ierr);
     qep->allocated_ncv = 0;
   }
