@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,9 +21,31 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/qepimpl.h>       /*I "slepcqep.h" I*/
+#include <slepc-private/qepimpl.h>       /*I "slepcqep.h" I*/
+#include <petscdraw.h>
 
-#undef __FUNCT__  
+typedef struct {
+  PetscErrorCode (*comparison)(PetscScalar,PetscScalar,PetscScalar,PetscScalar,PetscInt*,void*);
+  void *comparisonctx;
+  ST st;
+} QEPSortForSTData;
+
+#undef __FUNCT__
+#define __FUNCT__ "QEPSortForSTFunc"
+static PetscErrorCode QEPSortForSTFunc(PetscScalar ar,PetscScalar ai,
+                                PetscScalar br,PetscScalar bi,PetscInt *r,void *ctx)
+{
+  QEPSortForSTData *data = (QEPSortForSTData*)ctx;
+  PetscErrorCode   ierr;
+
+  PetscFunctionBegin;
+  ierr = STBackTransform(data->st,1,&ar,&ai);CHKERRQ(ierr);
+  ierr = STBackTransform(data->st,1,&br,&bi);CHKERRQ(ierr);
+  ierr = (*data->comparison)(ar,ai,br,bi,r,data->comparisonctx);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
 #define __FUNCT__ "QEPSolve"
 /*@
    QEPSolve - Solves the quadratic eigensystem.
@@ -33,53 +55,68 @@
    Input Parameter:
 .  qep - eigensolver context obtained from QEPCreate()
 
-   Options Database:
-+   -qep_view - print information about the solver used
-.   -qep_view_binary - save the matrices to the default binary file
--   -qep_plot_eigs - plot computed eigenvalues
+   Options Database Keys:
++  -qep_view - print information about the solver used
+.  -qep_view_mat0 binary - save the first matrix (M) to the default binary viewer
+.  -qep_view_mat1 binary - save the second matrix (C) to the default binary viewer
+.  -qep_view_mat2 binary - save the third matrix (K) to the default binary viewer
+-  -qep_plot_eigs - plot computed eigenvalues
 
    Level: beginner
 
-.seealso: QEPCreate(), QEPSetUp(), QEPDestroy(), QEPSetTolerances() 
+.seealso: QEPCreate(), QEPSetUp(), QEPDestroy(), QEPSetTolerances()
 @*/
-PetscErrorCode QEPSolve(QEP qep) 
+PetscErrorCode QEPSolve(QEP qep)
 {
-  PetscErrorCode ierr;
-  PetscInt       i;
-  PetscReal      re,im;
-  PetscBool      flg;
-  PetscViewer    viewer;
-  PetscDraw      draw;
-  PetscDrawSP    drawsp;
-  char           filename[PETSC_MAX_PATH_LEN];
+  PetscErrorCode    ierr;
+  PetscInt          i;
+  PetscReal         re,im;
+  PetscBool         flg,islinear;
+  PetscViewer       viewer;
+  PetscViewerFormat format;
+  PetscDraw         draw;
+  PetscDrawSP       drawsp;
+  QEPSortForSTData  data;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  ierr = PetscLogEventBegin(QEP_Solve,qep,0,0,0);CHKERRQ(ierr);
 
-  flg = PETSC_FALSE;
-  ierr = PetscOptionsGetBool(((PetscObject)qep)->prefix,"-qep_view_binary",&flg,PETSC_NULL);CHKERRQ(ierr); 
-  if (flg) {
-    ierr = MatView(qep->M,PETSC_VIEWER_BINARY_(((PetscObject)qep)->comm));CHKERRQ(ierr);
-    ierr = MatView(qep->C,PETSC_VIEWER_BINARY_(((PetscObject)qep)->comm));CHKERRQ(ierr);
-    ierr = MatView(qep->K,PETSC_VIEWER_BINARY_(((PetscObject)qep)->comm));CHKERRQ(ierr);
-  }
-
-  /* reset the convergence flag from the previous solves */
-  qep->reason = QEP_CONVERGED_ITERATING;
-
-  if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
-  if (!qep->setupcalled){ ierr = QEPSetUp(qep);CHKERRQ(ierr); }
+  /* call setup */
+  ierr = QEPSetUp(qep);CHKERRQ(ierr);
   qep->nconv = 0;
-  qep->its = 0;
-  for (i=0;i<qep->ncv;i++) qep->eigr[i]=qep->eigi[i]=qep->errest[i]=0.0;
+  qep->its   = 0;
+  for (i=0;i<qep->ncv;i++) {
+    qep->eigr[i]   = 0.0;
+    qep->eigi[i]   = 0.0;
+    qep->errest[i] = 0.0;
+  }
   ierr = QEPMonitor(qep,qep->its,qep->nconv,qep->eigr,qep->eigi,qep->errest,qep->ncv);CHKERRQ(ierr);
 
-  ierr = PetscLogEventBegin(QEP_Solve,qep,0,0,0);CHKERRQ(ierr);
-  ierr = (*qep->ops->solve)(qep);CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(QEP_Solve,qep,0,0,0);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)qep,QEPLINEAR,&islinear);CHKERRQ(ierr);
+  if (!islinear) {
+    /* temporarily change eigenvalue comparison function */
+    data.comparison    = qep->comparison;
+    data.comparisonctx = qep->comparisonctx;
+    data.st            = qep->st;
+    qep->comparison    = QEPSortForSTFunc;
+    qep->comparisonctx = &data;
+  }
+  ierr = DSSetEigenvalueComparison(qep->ds,qep->comparison,qep->comparisonctx);CHKERRQ(ierr);
 
-  if (!qep->reason) {
-    SETERRQ(((PetscObject)qep)->comm,1,"Internal error, solver returned without setting converged reason");
+  ierr = (*qep->ops->solve)(qep);CHKERRQ(ierr);
+  if (!islinear) {
+    ierr = STPostSolve(qep->st);CHKERRQ(ierr);
+  }
+
+  if (!qep->reason) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_PLIB,"Internal error, solver returned without setting converged reason");
+
+  if (!islinear) {
+    /* restore comparison function */
+    qep->comparison    = data.comparison;
+    qep->comparisonctx = data.comparisonctx;
+    /* Map eigenvalues back to the original problem */
+    ierr = STBackTransform(qep->st,qep->nconv,qep->eigr,qep->eigi);CHKERRQ(ierr);
   }
 
 #if !defined(PETSC_USE_COMPLEX)
@@ -97,27 +134,30 @@ PetscErrorCode QEPSolve(QEP qep)
 #endif
 
   /* sort eigenvalues according to qep->which parameter */
-  ierr = PetscFree(qep->perm);CHKERRQ(ierr);
-  if (qep->nconv > 0) {
-    ierr = PetscMalloc(sizeof(PetscInt)*qep->nconv,&qep->perm);CHKERRQ(ierr);
-    ierr = QEPSortEigenvalues(qep,qep->nconv,qep->eigr,qep->eigi,qep->perm);CHKERRQ(ierr);
-  }
+  ierr = QEPSortEigenvalues(qep,qep->nconv,qep->eigr,qep->eigi,qep->perm);CHKERRQ(ierr);
 
-  ierr = PetscOptionsGetString(((PetscObject)qep)->prefix,"-qep_view",filename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(QEP_Solve,qep,0,0,0);CHKERRQ(ierr);
+
+  /* various viewers */
+  ierr = MatViewFromOptions(qep->M,((PetscObject)qep)->prefix,"-qep_view_mat0");CHKERRQ(ierr);
+  ierr = MatViewFromOptions(qep->C,((PetscObject)qep)->prefix,"-qep_view_mat1");CHKERRQ(ierr);
+  ierr = MatViewFromOptions(qep->K,((PetscObject)qep)->prefix,"-qep_view_mat2");CHKERRQ(ierr);
+
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)qep),((PetscObject)qep)->prefix,"-qep_view",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg && !PetscPreLoadingOn) {
-    ierr = PetscViewerASCIIOpen(((PetscObject)qep)->comm,filename,&viewer);CHKERRQ(ierr);
-    ierr = QEPView(qep,viewer);CHKERRQ(ierr); 
+    ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
+    ierr = QEPView(qep,viewer);CHKERRQ(ierr);
+    ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   }
 
   flg = PETSC_FALSE;
-  ierr = PetscOptionsGetBool(((PetscObject)qep)->prefix,"-qep_plot_eigs",&flg,PETSC_NULL);CHKERRQ(ierr); 
-  if (flg) { 
-    ierr = PetscViewerDrawOpen(PETSC_COMM_SELF,0,"Computed Eigenvalues",
-                             PETSC_DECIDE,PETSC_DECIDE,300,300,&viewer);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(((PetscObject)qep)->prefix,"-qep_plot_eigs",&flg,NULL);CHKERRQ(ierr);
+  if (flg) {
+    ierr = PetscViewerDrawOpen(PETSC_COMM_SELF,0,"Computed Eigenvalues",PETSC_DECIDE,PETSC_DECIDE,300,300,&viewer);CHKERRQ(ierr);
     ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
     ierr = PetscDrawSPCreate(draw,1,&drawsp);CHKERRQ(ierr);
-    for(i=0;i<qep->nconv;i++) {
+    for (i=0;i<qep->nconv;i++) {
 #if defined(PETSC_USE_COMPLEX)
       re = PetscRealPart(qep->eigr[i]);
       im = PetscImaginaryPart(qep->eigi[i]);
@@ -127,7 +167,7 @@ PetscErrorCode QEPSolve(QEP qep)
 #endif
       ierr = PetscDrawSPAddPoint(drawsp,&re,&im);CHKERRQ(ierr);
     }
-    ierr = PetscDrawSPDraw(drawsp);CHKERRQ(ierr);
+    ierr = PetscDrawSPDraw(drawsp,PETSC_TRUE);CHKERRQ(ierr);
     ierr = PetscDrawSPDestroy(&drawsp);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   }
@@ -137,13 +177,13 @@ PetscErrorCode QEPSolve(QEP qep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPGetIterationNumber"
 /*@
-   QEPGetIterationNumber - Gets the current iteration number. If the 
-   call to QEPSolve() is complete, then it returns the number of iterations 
+   QEPGetIterationNumber - Gets the current iteration number. If the
+   call to QEPSolve() is complete, then it returns the number of iterations
    carried out by the solution method.
- 
+
    Not Collective
 
    Input Parameter:
@@ -155,10 +195,10 @@ PetscErrorCode QEPSolve(QEP qep)
    Level: intermediate
 
    Note:
-   During the i-th iteration this call returns i-1. If QEPSolve() is 
+   During the i-th iteration this call returns i-1. If QEPSolve() is
    complete, then parameter "its" contains either the iteration number at
-   which convergence was successfully reached, or failure was detected.  
-   Call QEPGetConvergedReason() to determine if the solver converged or 
+   which convergence was successfully reached, or failure was detected.
+   Call QEPGetConvergedReason() to determine if the solver converged or
    failed and why.
 
 .seealso: QEPGetConvergedReason(), QEPSetTolerances()
@@ -172,7 +212,7 @@ PetscErrorCode QEPGetIterationNumber(QEP qep,PetscInt *its)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPGetConverged"
 /*@
    QEPGetConverged - Gets the number of converged eigenpairs.
@@ -181,9 +221,9 @@ PetscErrorCode QEPGetIterationNumber(QEP qep,PetscInt *its)
 
    Input Parameter:
 .  qep - the quadratic eigensolver context
-  
+
    Output Parameter:
-.  nconv - number of converged eigenpairs 
+.  nconv - number of converged eigenpairs
 
    Note:
    This function should be called after QEPSolve() has finished.
@@ -201,10 +241,10 @@ PetscErrorCode QEPGetConverged(QEP qep,PetscInt *nconv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPGetConvergedReason"
 /*@C
-   QEPGetConvergedReason - Gets the reason why the QEPSolve() iteration was 
+   QEPGetConvergedReason - Gets the reason why the QEPSolve() iteration was
    stopped.
 
    Not Collective
@@ -236,16 +276,16 @@ PetscErrorCode QEPGetConvergedReason(QEP qep,QEPConvergedReason *reason)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "QEPGetEigenpair" 
+#undef __FUNCT__
+#define __FUNCT__ "QEPGetEigenpair"
 /*@
-   QEPGetEigenpair - Gets the i-th solution of the eigenproblem as computed by 
+   QEPGetEigenpair - Gets the i-th solution of the eigenproblem as computed by
    QEPSolve(). The solution consists in both the eigenvalue and the eigenvector.
 
-   Not Collective, but vectors are shared by all processors that share the QEP
+   Logically Collective on EPS
 
    Input Parameters:
-+  qep - quadratic eigensolver context 
++  qep - quadratic eigensolver context
 -  i   - index of the solution
 
    Output Parameters:
@@ -255,13 +295,13 @@ PetscErrorCode QEPGetConvergedReason(QEP qep,QEPConvergedReason *reason)
 -  Vi   - imaginary part of eigenvector
 
    Notes:
-   If the eigenvalue is real, then eigi and Vi are set to zero. If PETSc is 
-   configured with complex scalars the eigenvalue is stored 
-   directly in eigr (eigi is set to zero) and the eigenvector in Vr (Vi is 
+   If the eigenvalue is real, then eigi and Vi are set to zero. If PETSc is
+   configured with complex scalars the eigenvalue is stored
+   directly in eigr (eigi is set to zero) and the eigenvector in Vr (Vi is
    set to zero).
 
    The index i should be a value between 0 and nconv-1 (see QEPGetConverged()).
-   Eigenpairs are indexed according to the ordering criterion established 
+   Eigenpairs are indexed according to the ordering criterion established
    with QEPSetWhichEigenpairs().
 
    Level: beginner
@@ -275,14 +315,11 @@ PetscErrorCode QEPGetEigenpair(QEP qep,PetscInt i,PetscScalar *eigr,PetscScalar
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  PetscValidLogicalCollectiveInt(qep,i,2);
   if (Vr) { PetscValidHeaderSpecific(Vr,VEC_CLASSID,6); PetscCheckSameComm(qep,1,Vr,6); }
   if (Vi) { PetscValidHeaderSpecific(Vi,VEC_CLASSID,7); PetscCheckSameComm(qep,1,Vi,7); }
-  if (!qep->eigr || !qep->eigi || !qep->V) { 
-    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_WRONGSTATE,"QEPSolve must be called first"); 
-  }
-  if (i<0 || i>=qep->nconv) { 
-    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range"); 
-  }
+  if (!qep->eigr || !qep->eigi || !qep->V) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_WRONGSTATE,"QEPSolve must be called first");
+  if (i<0 || i>=qep->nconv) SETERRQ(PetscObjectComm((PetscObject)qep),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
 
   if (!qep->perm) k = i;
   else k = qep->perm[i];
@@ -295,7 +332,7 @@ PetscErrorCode QEPGetEigenpair(QEP qep,PetscInt i,PetscScalar *eigr,PetscScalar
   if (eigr) *eigr = qep->eigr[k];
   if (eigi) *eigi = qep->eigi[k];
 #endif
-  
+
   /* eigenvector */
 #if defined(PETSC_USE_COMPLEX)
   if (Vr) { ierr = VecCopy(qep->V[k],Vr);CHKERRQ(ierr); }
@@ -306,9 +343,9 @@ PetscErrorCode QEPGetEigenpair(QEP qep,PetscInt i,PetscScalar *eigr,PetscScalar
     if (Vi) { ierr = VecCopy(qep->V[k+1],Vi);CHKERRQ(ierr); }
   } else if (qep->eigi[k]<0) { /* second value of conjugate pair */
     if (Vr) { ierr = VecCopy(qep->V[k-1],Vr);CHKERRQ(ierr); }
-    if (Vi) { 
-      ierr = VecCopy(qep->V[k],Vi);CHKERRQ(ierr); 
-      ierr = VecScale(Vi,-1.0);CHKERRQ(ierr); 
+    if (Vi) {
+      ierr = VecCopy(qep->V[k],Vi);CHKERRQ(ierr);
+      ierr = VecScale(Vi,-1.0);CHKERRQ(ierr);
     }
   } else { /* real eigenvalue */
     if (Vr) { ierr = VecCopy(qep->V[k],Vr);CHKERRQ(ierr); }
@@ -318,16 +355,16 @@ PetscErrorCode QEPGetEigenpair(QEP qep,PetscInt i,PetscScalar *eigr,PetscScalar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "QEPGetErrorEstimate" 
+#undef __FUNCT__
+#define __FUNCT__ "QEPGetErrorEstimate"
 /*@
-   QEPGetErrorEstimate - Returns the error estimate associated to the i-th 
+   QEPGetErrorEstimate - Returns the error estimate associated to the i-th
    computed eigenpair.
 
    Not Collective
 
    Input Parameter:
-+  qep - quadratic eigensolver context 
++  qep - quadratic eigensolver context
 -  i   - index of eigenpair
 
    Output Parameter:
@@ -347,21 +384,17 @@ PetscErrorCode QEPGetErrorEstimate(QEP qep,PetscInt i,PetscReal *errest)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidPointer(errest,3);
-  if (!qep->eigr || !qep->eigi) { 
-    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_WRONGSTATE,"QEPSolve must be called first"); 
-  }
-  if (i<0 || i>=qep->nconv) { 
-    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range"); 
-  }
-  if (qep->perm) i = qep->perm[i];  
+  if (!qep->eigr || !qep->eigi) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"QEPSolve must be called first");
+  if (i<0 || i>=qep->nconv) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
+  if (qep->perm) i = qep->perm[i];
   if (errest) *errest = qep->errest[i];
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPComputeResidualNorm_Private"
 /*
-   QEPComputeResidualNorm_Private - Computes the norm of the residual vector 
+   QEPComputeResidualNorm_Private - Computes the norm of the residual vector
    associated with an eigenpair.
 */
 PetscErrorCode QEPComputeResidualNorm_Private(QEP qep,PetscScalar kr,PetscScalar ki,Vec xr,Vec xi,PetscReal *norm)
@@ -374,14 +407,13 @@ PetscErrorCode QEPComputeResidualNorm_Private(QEP qep,PetscScalar kr,PetscScalar
   PetscReal      ni,nr;
   PetscScalar    a1,a2;
 #endif
-  
+
   PetscFunctionBegin;
   ierr = VecDuplicate(qep->V[0],&u);CHKERRQ(ierr);
   ierr = VecDuplicate(u,&w);CHKERRQ(ierr);
-  
+
 #if !defined(PETSC_USE_COMPLEX)
-  if (ki == 0 || 
-    PetscAbsScalar(ki) < PetscAbsScalar(kr*PETSC_MACHINE_EPSILON)) {
+  if (ki == 0 || PetscAbsScalar(ki) < PetscAbsScalar(kr*PETSC_MACHINE_EPSILON)) {
 #endif
     ierr = MatMult(K,xr,u);CHKERRQ(ierr);                 /* u=K*x */
     if (PetscAbsScalar(kr) > PETSC_MACHINE_EPSILON) {
@@ -390,7 +422,7 @@ PetscErrorCode QEPComputeResidualNorm_Private(QEP qep,PetscScalar kr,PetscScalar
       ierr = MatMult(M,xr,w);CHKERRQ(ierr);               /* w=M*x */
       ierr = VecAXPY(u,kr*kr,w);CHKERRQ(ierr);            /* u=l^2*M*x+l*C*x+K*x */
     }
-    ierr = VecNorm(u,NORM_2,norm);CHKERRQ(ierr);  
+    ierr = VecNorm(u,NORM_2,norm);CHKERRQ(ierr);
 #if !defined(PETSC_USE_COMPLEX)
   } else {
     ierr = VecDuplicate(u,&v);CHKERRQ(ierr);
@@ -430,26 +462,26 @@ PetscErrorCode QEPComputeResidualNorm_Private(QEP qep,PetscScalar kr,PetscScalar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPComputeResidualNorm"
 /*@
-   QEPComputeResidualNorm - Computes the norm of the residual vector associated with 
+   QEPComputeResidualNorm - Computes the norm of the residual vector associated with
    the i-th computed eigenpair.
 
    Collective on QEP
 
    Input Parameter:
-.  qep - the quadratic eigensolver context
-.  i   - the solution index
++  qep - the quadratic eigensolver context
+-  i   - the solution index
 
    Output Parameter:
-.  norm - the residual norm, computed as ||(l^2*M+l*C+K)x||_2 where l is the 
-   eigenvalue and x is the eigenvector. 
+.  norm - the residual norm, computed as ||(l^2*M+l*C+K)x||_2 where l is the
+   eigenvalue and x is the eigenvector.
    If l=0 then the residual norm is computed as ||Kx||_2.
 
    Notes:
    The index i should be a value between 0 and nconv-1 (see QEPGetConverged()).
-   Eigenpairs are indexed according to the ordering criterion established 
+   Eigenpairs are indexed according to the ordering criterion established
    with QEPSetWhichEigenpairs().
 
    Level: beginner
@@ -461,7 +493,7 @@ PetscErrorCode QEPComputeResidualNorm(QEP qep,PetscInt i,PetscReal *norm)
   PetscErrorCode ierr;
   Vec            xr,xi;
   PetscScalar    kr,ki;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidLogicalCollectiveInt(qep,i,2);
@@ -475,10 +507,10 @@ PetscErrorCode QEPComputeResidualNorm(QEP qep,PetscInt i,PetscReal *norm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPComputeRelativeError_Private"
 /*
-   QEPComputeRelativeError_Private - Computes the relative error bound 
+   QEPComputeRelativeError_Private - Computes the relative error bound
    associated with an eigenpair.
 */
 PetscErrorCode QEPComputeRelativeError_Private(QEP qep,PetscScalar kr,PetscScalar ki,Vec xr,Vec xi,PetscReal *error)
@@ -488,12 +520,11 @@ PetscErrorCode QEPComputeRelativeError_Private(QEP qep,PetscScalar kr,PetscScala
 #if !defined(PETSC_USE_COMPLEX)
   PetscReal      ei;
 #endif
-  
+
   PetscFunctionBegin;
   ierr = QEPComputeResidualNorm_Private(qep,kr,ki,xr,xi,&norm);CHKERRQ(ierr);
 #if !defined(PETSC_USE_COMPLEX)
-  if (ki == 0 || 
-    PetscAbsScalar(ki) < PetscAbsScalar(kr*PETSC_MACHINE_EPSILON)) {
+  if (ki == 0 || PetscAbsScalar(ki) < PetscAbsScalar(kr*PETSC_MACHINE_EPSILON)) {
 #endif
     ierr = VecNorm(xr,NORM_2,&er);CHKERRQ(ierr);
     if (PetscAbsScalar(kr) > norm) {
@@ -503,33 +534,33 @@ PetscErrorCode QEPComputeRelativeError_Private(QEP qep,PetscScalar kr,PetscScala
     }
 #if !defined(PETSC_USE_COMPLEX)
   } else {
-    ierr = VecNorm(xr,NORM_2,&er);CHKERRQ(ierr);  
-    ierr = VecNorm(xi,NORM_2,&ei);CHKERRQ(ierr);  
+    ierr = VecNorm(xr,NORM_2,&er);CHKERRQ(ierr);
+    ierr = VecNorm(xi,NORM_2,&ei);CHKERRQ(ierr);
     if (SlepcAbsEigenvalue(kr,ki) > norm) {
       *error = norm/(SlepcAbsEigenvalue(kr,ki)*SlepcAbsEigenvalue(er,ei));
     } else {
       *error = norm/SlepcAbsEigenvalue(er,ei);
     }
   }
-#endif    
+#endif
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPComputeRelativeError"
 /*@
-   QEPComputeRelativeError - Computes the relative error bound associated 
+   QEPComputeRelativeError - Computes the relative error bound associated
    with the i-th computed eigenpair.
 
    Collective on QEP
 
    Input Parameter:
-.  qep - the quadratic eigensolver context
-.  i   - the solution index
++  qep - the quadratic eigensolver context
+-  i   - the solution index
 
    Output Parameter:
-.  error - the relative error bound, computed as ||(l^2*M+l*C+K)x||_2/||lx||_2 where 
-   l is the eigenvalue and x is the eigenvector. 
+.  error - the relative error bound, computed as ||(l^2*M+l*C+K)x||_2/||lx||_2 where
+   l is the eigenvalue and x is the eigenvector.
    If l=0 the relative error is computed as ||Kx||_2/||x||_2.
 
    Level: beginner
@@ -539,26 +570,26 @@ PetscErrorCode QEPComputeRelativeError_Private(QEP qep,PetscScalar kr,PetscScala
 PetscErrorCode QEPComputeRelativeError(QEP qep,PetscInt i,PetscReal *error)
 {
   PetscErrorCode ierr;
-  Vec            xr,xi;  
-  PetscScalar    kr,ki;  
-  
+  Vec            xr,xi;
+  PetscScalar    kr,ki;
+
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);  
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidLogicalCollectiveInt(qep,i,2);
   PetscValidPointer(error,3);
   ierr = VecDuplicate(qep->V[0],&xr);CHKERRQ(ierr);
   ierr = VecDuplicate(qep->V[0],&xi);CHKERRQ(ierr);
   ierr = QEPGetEigenpair(qep,i,&kr,&ki,xr,xi);CHKERRQ(ierr);
-  ierr = QEPComputeRelativeError_Private(qep,kr,ki,xr,xi,error);CHKERRQ(ierr);  
+  ierr = QEPComputeRelativeError_Private(qep,kr,ki,xr,xi,error);CHKERRQ(ierr);
   ierr = VecDestroy(&xr);CHKERRQ(ierr);
   ierr = VecDestroy(&xi);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPSortEigenvalues"
 /*@
-   QEPSortEigenvalues - Sorts a list of eigenvalues according to the criterion 
+   QEPSortEigenvalues - Sorts a list of eigenvalues according to the criterion
    specified via QEPSetWhichEigenpairs().
 
    Not Collective
@@ -573,13 +604,13 @@ PetscErrorCode QEPComputeRelativeError(QEP qep,PetscInt i,PetscReal *error)
 .  perm  - resulting permutation
 
    Note:
-   The result is a list of indices in the original eigenvalue array 
+   The result is a list of indices in the original eigenvalue array
    corresponding to the first nev eigenvalues sorted in the specified
    criterion.
 
    Level: developer
 
-.seealso: QEPSortEigenvaluesReal(), QEPSetWhichEigenpairs()
+.seealso: QEPSetWhichEigenpairs()
 @*/
 PetscErrorCode QEPSortEigenvalues(QEP qep,PetscInt n,PetscScalar *eigr,PetscScalar *eigi,PetscInt *perm)
 {
@@ -588,11 +619,11 @@ PetscErrorCode QEPSortEigenvalues(QEP qep,PetscInt n,PetscScalar *eigr,PetscScal
   PetscInt       i,j,result,tmp;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);  
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidScalarPointer(eigr,3);
   PetscValidScalarPointer(eigi,4);
   PetscValidIntPointer(perm,5);
-  for (i=0; i<n; i++) { perm[i] = i; }
+  for (i=0;i<n;i++) perm[i] = i;
   /* insertion sort */
   for (i=n-1; i>=0; i--) {
     re = eigr[perm[i]];
@@ -607,10 +638,10 @@ PetscErrorCode QEPSortEigenvalues(QEP qep,PetscInt n,PetscScalar *eigr,PetscScal
 #endif
     while (j<n) {
       ierr = QEPCompareEigenvalues(qep,re,im,eigr[perm[j]],eigi[perm[j]],&result);CHKERRQ(ierr);
-      if (result >= 0) break;
+      if (result < 0) break;
 #if !defined(PETSC_USE_COMPLEX)
       /* keep together every complex conjugated eigenpair */
-      if (im == 0) { 
+      if (im == 0) {
         if (eigi[perm[j]] == 0) {
 #endif
           tmp = perm[j-1]; perm[j-1] = perm[j]; perm[j] = tmp;
@@ -636,58 +667,7 @@ PetscErrorCode QEPSortEigenvalues(QEP qep,PetscInt n,PetscScalar *eigr,PetscScal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "QEPSortEigenvaluesReal"
-/*@
-   QEPSortEigenvaluesReal - Sorts a list of eigenvalues according to a certain
-   criterion (version for real eigenvalues only).
-
-   Not Collective
-
-   Input Parameters:
-+  qep   - the quadratic eigensolver context
-.  n     - number of eigenvalue in the list
--  eig   - pointer to the array containing the eigenvalues (real)
-
-   Output Parameter:
-.  perm  - resulting permutation
-
-   Note:
-   The result is a list of indices in the original eigenvalue array 
-   corresponding to the first nev eigenvalues sorted in the specified
-   criterion.
-
-   Level: developer
-
-.seealso: QEPSortEigenvalues(), QEPSetWhichEigenpairs(), QEPCompareEigenvalues()
-@*/
-PetscErrorCode QEPSortEigenvaluesReal(QEP qep,PetscInt n,PetscReal *eig,PetscInt *perm)
-{
-  PetscErrorCode ierr;
-  PetscScalar    re;
-  PetscInt       i,j,result,tmp;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);  
-  PetscValidScalarPointer(eig,3);
-  PetscValidIntPointer(perm,4);
-  for (i=0; i<n; i++) { perm[i] = i; }
-  /* insertion sort */
-  for (i=1; i<n; i++) {
-    re = eig[perm[i]];
-    j = i-1;
-    ierr = QEPCompareEigenvalues(qep,re,0.0,eig[perm[j]],0.0,&result);CHKERRQ(ierr);
-    while (result>0 && j>=0) {
-      tmp = perm[j]; perm[j] = perm[j+1]; perm[j+1] = tmp; j--;
-      if (j>=0) {
-        ierr = QEPCompareEigenvalues(qep,re,0.0,eig[perm[j]],0.0,&result);CHKERRQ(ierr);
-      }
-    }
-  }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPCompareEigenvalues"
 /*@
    QEPCompareEigenvalues - Compares two (possibly complex) eigenvalues according
@@ -719,54 +699,20 @@ PetscErrorCode QEPSortEigenvaluesReal(QEP qep,PetscInt n,PetscReal *eig,PetscInt
 @*/
 PetscErrorCode QEPCompareEigenvalues(QEP qep,PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *result)
 {
-  PetscReal a,b;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);  
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidIntPointer(result,6);
-  switch(qep->which) {
-    case QEP_LARGEST_MAGNITUDE:
-    case QEP_SMALLEST_MAGNITUDE:
-      a = SlepcAbsEigenvalue(ar,ai);
-      b = SlepcAbsEigenvalue(br,bi);
-      break;
-    case QEP_LARGEST_REAL:
-    case QEP_SMALLEST_REAL:
-      a = PetscRealPart(ar);
-      b = PetscRealPart(br);
-      break;
-    case QEP_LARGEST_IMAGINARY:
-    case QEP_SMALLEST_IMAGINARY:
-#if defined(PETSC_USE_COMPLEX)
-      a = PetscImaginaryPart(ar);
-      b = PetscImaginaryPart(br);
-#else
-      a = PetscAbsReal(ai);
-      b = PetscAbsReal(bi);
-#endif
-      break;
-    default: SETERRQ(((PetscObject)qep)->comm,1,"Wrong value of which");
-  }
-  switch(qep->which) {
-    case QEP_LARGEST_MAGNITUDE:
-    case QEP_LARGEST_REAL:
-    case QEP_LARGEST_IMAGINARY:
-      if (a<b) *result = -1;
-      else if (a>b) *result = 1;
-      else *result = 0;
-      break;
-    default:
-      if (a>b) *result = -1;
-      else if (a<b) *result = 1;
-      else *result = 0;
-  }
+  if (!qep->comparison) SETERRQ(PETSC_COMM_SELF,1,"Undefined eigenvalue comparison function");
+  ierr = (*qep->comparison)(ar,ai,br,bi,result,qep->comparisonctx);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "QEPGetOperationCounters"
 /*@
-   QEPGetOperationCounters - Gets the total number of matrix-vector products, dot 
+   QEPGetOperationCounters - Gets the total number of matrix-vector products, dot
    products, and linear solve iterations used by the QEP object during the last
    QEPSolve() call.
 
@@ -789,14 +735,14 @@ PetscErrorCode QEPCompareEigenvalues(QEP qep,PetscScalar ar,PetscScalar ai,Petsc
 PetscErrorCode QEPGetOperationCounters(QEP qep,PetscInt* matvecs,PetscInt* dots,PetscInt* lits)
 {
   PetscErrorCode ierr;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
-  if (matvecs) *matvecs = qep->matvecs; 
+  if (matvecs) *matvecs = qep->matvecs;
   if (dots) {
     if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
     ierr = IPGetOperationCounters(qep->ip,dots);CHKERRQ(ierr);
   }
-  if (lits) *lits = qep->linits; 
+  if (lits) *lits = qep->linits;
   PetscFunctionReturn(0);
 }
