@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,7 +21,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/stimpl.h>          /*I "slepcst.h" I*/
+#include <slepc-private/stimpl.h>          /*I "slepcst.h" I*/
 
 typedef struct {
   PetscScalar nu;
@@ -29,105 +29,77 @@ typedef struct {
   Vec         w2;
 } ST_CAYLEY;
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STApply_Cayley"
 PetscErrorCode STApply_Cayley(ST st,Vec x,Vec y)
 {
   PetscErrorCode ierr;
-  ST_CAYLEY      *ctx = (ST_CAYLEY*)st->data;
-  PetscScalar    nu = ctx->nu;
-  
-  PetscFunctionBegin;
-  if (st->shift_matrix == ST_MATMODE_INPLACE) { nu = nu + st->sigma; };
 
-  if (st->B) {
-    /* generalized eigenproblem: y = (A - sB)^-1 (A + tB)x */
-    ierr = MatMult(st->A,x,st->w);CHKERRQ(ierr);
-    ierr = MatMult(st->B,x,ctx->w2);CHKERRQ(ierr);
-    ierr = VecAXPY(st->w,nu,ctx->w2);CHKERRQ(ierr);    
-    ierr = STAssociatedKSPSolve(st,st->w,y);CHKERRQ(ierr);
-  }
-  else {
-    /* standard eigenproblem: y = (A - sI)^-1 (A + tI)x */
-    ierr = MatMult(st->A,x,st->w);CHKERRQ(ierr);
-    ierr = VecAXPY(st->w,nu,x);CHKERRQ(ierr);
-    ierr = STAssociatedKSPSolve(st,st->w,y);CHKERRQ(ierr);
-  }
+  PetscFunctionBegin;
+  /* standard eigenproblem: y = (A - sI)^-1 (A + tI)x */
+  /* generalized eigenproblem: y = (A - sB)^-1 (A + tB)x */
+  ierr = MatMult(st->T[0],x,st->w);CHKERRQ(ierr);
+  ierr = STMatSolve(st,1,st->w,y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STApplyTranspose_Cayley"
 PetscErrorCode STApplyTranspose_Cayley(ST st,Vec x,Vec y)
 {
   PetscErrorCode ierr;
-  ST_CAYLEY      *ctx = (ST_CAYLEY*)st->data;
-  PetscScalar    nu = ctx->nu;
-  
-  PetscFunctionBegin;
-  if (st->shift_matrix == ST_MATMODE_INPLACE) { nu = nu + st->sigma; };
 
-  if (st->B) {
-    /* generalized eigenproblem: y = (A + tB)^T (A - sB)^-T x */
-    ierr = STAssociatedKSPSolveTranspose(st,x,st->w);CHKERRQ(ierr);
-    ierr = MatMultTranspose(st->A,st->w,y);CHKERRQ(ierr);
-    ierr = MatMultTranspose(st->B,st->w,ctx->w2);CHKERRQ(ierr);
-    ierr = VecAXPY(y,nu,ctx->w2);CHKERRQ(ierr);    
-  }
-  else {
-    /* standard eigenproblem: y =  (A + tI)^T (A - sI)^-T x */
-    ierr = STAssociatedKSPSolveTranspose(st,x,st->w);CHKERRQ(ierr);
-    ierr = MatMultTranspose(st->A,st->w,y);CHKERRQ(ierr);
-    ierr = VecAXPY(y,nu,st->w);CHKERRQ(ierr);
-  }
+  PetscFunctionBegin;
+  /* standard eigenproblem: y =  (A + tI)^T (A - sI)^-T x */
+  /* generalized eigenproblem: y = (A + tB)^T (A - sB)^-T x */
+  ierr = STMatSolveTranspose(st,1,x,st->w);CHKERRQ(ierr);
+  ierr = MatMultTranspose(st->T[0],st->w,y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "STBilinearMatMult_Cayley"
-PetscErrorCode STBilinearMatMult_Cayley(Mat B,Vec x,Vec y)
+#undef __FUNCT__
+#define __FUNCT__ "MatMult_Cayley"
+static PetscErrorCode MatMult_Cayley(Mat B,Vec x,Vec y)
 {
   PetscErrorCode ierr;
   ST             st;
   ST_CAYLEY      *ctx;
   PetscScalar    nu;
-  
+
   PetscFunctionBegin;
   ierr = MatShellGetContext(B,(void**)&st);CHKERRQ(ierr);
   ctx = (ST_CAYLEY*)st->data;
   nu = ctx->nu;
-  
+
   if (st->shift_matrix == ST_MATMODE_INPLACE) { nu = nu + st->sigma; };
 
-  if (st->B) {
+  if (st->nmat>1) {
     /* generalized eigenproblem: y = (A + tB)x */
-    ierr = MatMult(st->A,x,y);CHKERRQ(ierr);
-    ierr = MatMult(st->B,x,ctx->w2);CHKERRQ(ierr);
-    ierr = VecAXPY(y,nu,ctx->w2);CHKERRQ(ierr);    
-  }
-  else {
+    ierr = MatMult(st->A[0],x,y);CHKERRQ(ierr);
+    ierr = MatMult(st->A[1],x,ctx->w2);CHKERRQ(ierr);
+    ierr = VecAXPY(y,nu,ctx->w2);CHKERRQ(ierr);
+  } else {
     /* standard eigenproblem: y = (A + tI)x */
-    ierr = MatMult(st->A,x,y);CHKERRQ(ierr);
+    ierr = MatMult(st->A[0],x,y);CHKERRQ(ierr);
     ierr = VecAXPY(y,nu,x);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STGetBilinearForm_Cayley"
 PetscErrorCode STGetBilinearForm_Cayley(ST st,Mat *B)
 {
   PetscErrorCode ierr;
-  PetscInt       n,m;
 
   PetscFunctionBegin;
-  ierr = MatGetLocalSize(st->B,&n,&m);CHKERRQ(ierr);
-  ierr = MatCreateShell(((PetscObject)st)->comm,n,m,PETSC_DETERMINE,PETSC_DETERMINE,st,B);CHKERRQ(ierr);
-  ierr = MatShellSetOperation(*B,MATOP_MULT,(void(*)(void))STBilinearMatMult_Cayley);CHKERRQ(ierr);
+  ierr = STSetUp(st);CHKERRQ(ierr);
+  *B = st->T[0];
+  ierr = PetscObjectReference((PetscObject)*B);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STBackTransform_Cayley"
 PetscErrorCode STBackTransform_Cayley(ST st,PetscInt n,PetscScalar *eigr,PetscScalar *eigi)
 {
@@ -146,9 +118,9 @@ PetscErrorCode STBackTransform_Cayley(ST st,PetscInt n,PetscScalar *eigr,PetscSc
       i = eigi[j];
       r = st->sigma * (r * r + i * i - r) + ctx->nu * (r - 1);
       i = - st->sigma * i - ctx->nu * i;
-      t = i * i + r * (r - 2.0) + 1.0;    
+      t = i * i + r * (r - 2.0) + 1.0;
       eigr[j] = r / t;
-      eigi[j] = i / t;    
+      eigi[j] = i / t;
     }
   }
 #else
@@ -159,7 +131,7 @@ PetscErrorCode STBackTransform_Cayley(ST st,PetscInt n,PetscScalar *eigr,PetscSc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STPostSolve_Cayley"
 PetscErrorCode STPostSolve_Cayley(ST st)
 {
@@ -167,71 +139,58 @@ PetscErrorCode STPostSolve_Cayley(ST st)
 
   PetscFunctionBegin;
   if (st->shift_matrix == ST_MATMODE_INPLACE) {
-    if (st->B) {
-      ierr = MatAXPY(st->A,st->sigma,st->B,st->str);CHKERRQ(ierr);
+    if (st->nmat>1) {
+      ierr = MatAXPY(st->A[0],st->sigma,st->A[1],st->str);CHKERRQ(ierr);
     } else {
-      ierr = MatShift(st->A,st->sigma);CHKERRQ(ierr);
+      ierr = MatShift(st->A[0],st->sigma);CHKERRQ(ierr);
     }
+    st->Astate[0] = ((PetscObject)st->A[0])->state;
     st->setupcalled = 0;
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetUp_Cayley"
 PetscErrorCode STSetUp_Cayley(ST st)
 {
   PetscErrorCode ierr;
+  PetscInt       n,m;
   ST_CAYLEY      *ctx = (ST_CAYLEY*)st->data;
 
   PetscFunctionBegin;
-  ierr = MatDestroy(&st->mat);CHKERRQ(ierr);
-
   /* if the user did not set the shift, use the target value */
   if (!st->sigma_set) st->sigma = st->defsigma;
 
-  if (!ctx->nu_set) { ctx->nu = st->sigma; }
-  if (ctx->nu == 0.0 &&  st->sigma == 0.0) {
-    SETERRQ(((PetscObject)st)->comm,1,"Values of shift and antishift cannot be zero simultaneously");
+  if (!ctx->nu_set) ctx->nu = st->sigma;
+  if (ctx->nu == 0.0 && st->sigma == 0.0) SETERRQ(PetscObjectComm((PetscObject)st),1,"Values of shift and antishift cannot be zero simultaneously");
+
+  /* T[0] = A+nu*B */
+  if (st->shift_matrix==ST_MATMODE_INPLACE) {
+    ierr = MatGetLocalSize(st->A[0],&n,&m);CHKERRQ(ierr);
+    ierr = MatCreateShell(PetscObjectComm((PetscObject)st),n,m,PETSC_DETERMINE,PETSC_DETERMINE,st,&st->T[0]);CHKERRQ(ierr);
+    ierr = MatShellSetOperation(st->T[0],MATOP_MULT,(void(*)(void))MatMult_Cayley);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(st,st->T[0]);CHKERRQ(ierr);
+  } else {
+    ierr = STMatGAXPY_Private(st,ctx->nu,0.0,1,0,PETSC_TRUE);CHKERRQ(ierr);
   }
 
-  if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
-  switch (st->shift_matrix) {
-  case ST_MATMODE_INPLACE:
-    st->mat = PETSC_NULL;
-    if (st->sigma != 0.0) {
-      if (st->B) { 
-        ierr = MatAXPY(st->A,-st->sigma,st->B,st->str);CHKERRQ(ierr); 
-      } else { 
-        ierr = MatShift(st->A,-st->sigma);CHKERRQ(ierr); 
-      }
-    }
-    ierr = KSPSetOperators(st->ksp,st->A,st->A,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
-    break;
-  case ST_MATMODE_SHELL:
-    ierr = STMatShellCreate(st,&st->mat);CHKERRQ(ierr);
-    ierr = KSPSetOperators(st->ksp,st->mat,st->mat,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
-    break;
-  default:
-    ierr = MatDuplicate(st->A,MAT_COPY_VALUES,&st->mat);CHKERRQ(ierr);
-    if (st->sigma != 0.0) {
-      if (st->B) { 
-        ierr = MatAXPY(st->mat,-st->sigma,st->B,st->str);CHKERRQ(ierr); 
-      } else { 
-        ierr = MatShift(st->mat,-st->sigma);CHKERRQ(ierr); 
-      }
-    }
-    ierr = KSPSetOperators(st->ksp,st->mat,st->mat,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
-  }
-  if (st->B) { 
+  /* T[1] = A-sigma*B */
+  ierr = STMatGAXPY_Private(st,-st->sigma,0.0,1,1,PETSC_TRUE);CHKERRQ(ierr);
+
+  if (st->nmat>1) {
     ierr = VecDestroy(&ctx->w2);CHKERRQ(ierr);
-    ierr = MatGetVecs(st->B,&ctx->w2,PETSC_NULL);CHKERRQ(ierr); 
+    ierr = MatGetVecs(st->A[1],&ctx->w2,NULL);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(st,ctx->w2);CHKERRQ(ierr);
   }
+  if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
+  ierr = KSPSetOperators(st->ksp,st->T[1],st->T[1],DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
   ierr = KSPSetUp(st->ksp);CHKERRQ(ierr);
+  st->kspidx = 1;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetShift_Cayley"
 PetscErrorCode STSetShift_Cayley(ST st,PetscScalar newshift)
 {
@@ -240,68 +199,40 @@ PetscErrorCode STSetShift_Cayley(ST st,PetscScalar newshift)
   MatStructure   flg;
 
   PetscFunctionBegin;
-  if (!ctx->nu_set) { ctx->nu = newshift; }
-  if (ctx->nu == 0.0 &&  newshift == 0.0) {
-    SETERRQ(((PetscObject)st)->comm,1,"Values of shift and antishift cannot be zero simultaneously");
-  }
+  if (newshift==0.0 && (!ctx->nu_set || (ctx->nu_set && ctx->nu==0.0))) SETERRQ(PetscObjectComm((PetscObject)st),1,"Values of shift and antishift cannot be zero simultaneously");
 
   /* Nothing to be done if STSetUp has not been called yet */
   if (!st->setupcalled) PetscFunctionReturn(0);
 
-  /* Check if the new KSP matrix has the same zero structure */
-  if (st->B && st->str == DIFFERENT_NONZERO_PATTERN && (st->sigma == 0.0 || newshift == 0.0)) {
-    flg = DIFFERENT_NONZERO_PATTERN;
-  } else {
-    flg = SAME_NONZERO_PATTERN;
+  if (!ctx->nu_set) {
+    if (st->shift_matrix!=ST_MATMODE_INPLACE) {
+      ierr = STMatGAXPY_Private(st,newshift,ctx->nu,1,0,PETSC_FALSE);CHKERRQ(ierr);
+    }
+    ctx->nu = newshift;
   }
+  ierr = STMatGAXPY_Private(st,-newshift,-st->sigma,1,1,PETSC_FALSE);CHKERRQ(ierr);
 
-  switch (st->shift_matrix) {
-  case ST_MATMODE_INPLACE:
-    /* Undo previous operations */
-    if (st->sigma != 0.0) {
-      if (st->B) {
-        ierr = MatAXPY(st->A,st->sigma,st->B,st->str);CHKERRQ(ierr);
-      } else {
-        ierr = MatShift(st->A,st->sigma);CHKERRQ(ierr);
-      }
-    }
-    /* Apply new shift */
-    if (newshift != 0.0) {
-      if (st->B) {
-        ierr = MatAXPY(st->A,-newshift,st->B,st->str);CHKERRQ(ierr);
-      } else {
-        ierr = MatShift(st->A,-newshift);CHKERRQ(ierr);
-      }
-    }
-    ierr = KSPSetOperators(st->ksp,st->A,st->A,flg);CHKERRQ(ierr);
-    break;
-  case ST_MATMODE_SHELL:
-    ierr = KSPSetOperators(st->ksp,st->mat,st->mat,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
-    break;
-  default:
-    ierr = MatCopy(st->A,st->mat,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
-    if (newshift != 0.0) {   
-      if (st->B) { ierr = MatAXPY(st->mat,-newshift,st->B,st->str);CHKERRQ(ierr); }
-      else { ierr = MatShift(st->mat,-newshift);CHKERRQ(ierr); }
-    }
-    ierr = KSPSetOperators(st->ksp,st->mat,st->mat,flg);CHKERRQ(ierr);    
+  if (st->kspidx==1) {  /* Update KSP operator */
+    /* Check if the new KSP matrix has the same zero structure */
+    if (st->nmat>1 && st->str == DIFFERENT_NONZERO_PATTERN && (st->sigma == 0.0 || newshift == 0.0)) flg = DIFFERENT_NONZERO_PATTERN;
+    else flg = SAME_NONZERO_PATTERN;
+    ierr = KSPSetOperators(st->ksp,st->T[1],st->T[1],flg);CHKERRQ(ierr);
+    ierr = KSPSetUp(st->ksp);CHKERRQ(ierr);
   }
-  st->sigma = newshift;
-  ierr = KSPSetUp(st->ksp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetFromOptions_Cayley"
-PetscErrorCode STSetFromOptions_Cayley(ST st) 
+PetscErrorCode STSetFromOptions_Cayley(ST st)
 {
   PetscErrorCode ierr;
   PetscScalar    nu;
   PetscBool      flg;
   ST_CAYLEY      *ctx = (ST_CAYLEY*)st->data;
   PC             pc;
-  const PCType   pctype;
-  const KSPType  ksptype;
+  PCType         pctype;
+  KSPType        ksptype;
 
   PetscFunctionBegin;
   if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
@@ -329,21 +260,23 @@ PetscErrorCode STSetFromOptions_Cayley(ST st)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STCayleySetAntishift_Cayley"
-PetscErrorCode STCayleySetAntishift_Cayley(ST st,PetscScalar newshift)
+static PetscErrorCode STCayleySetAntishift_Cayley(ST st,PetscScalar newshift)
 {
+  PetscErrorCode ierr;
   ST_CAYLEY *ctx = (ST_CAYLEY*)st->data;
 
   PetscFunctionBegin;
+  if (st->setupcalled && st->shift_matrix!=ST_MATMODE_INPLACE) {
+    ierr = STMatGAXPY_Private(st,newshift,ctx->nu,1,0,PETSC_FALSE);CHKERRQ(ierr);
+  }
   ctx->nu     = newshift;
   ctx->nu_set = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STCayleySetAntishift"
 /*@
    STCayleySetAntishift - Sets the value of the anti-shift for the Cayley
@@ -377,10 +310,9 @@ PetscErrorCode STCayleySetAntishift(ST st,PetscScalar nu)
   ierr = PetscTryMethod(st,"STCayleySetAntishift_C",(ST,PetscScalar),(st,nu));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STCayleyGetAntishift_Cayley"
-PetscErrorCode STCayleyGetAntishift_Cayley(ST st,PetscScalar *nu)
+static PetscErrorCode STCayleyGetAntishift_Cayley(ST st,PetscScalar *nu)
 {
   ST_CAYLEY *ctx = (ST_CAYLEY*)st->data;
 
@@ -388,9 +320,8 @@ PetscErrorCode STCayleyGetAntishift_Cayley(ST st,PetscScalar *nu)
   *nu = ctx->nu;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STCayleyGetAntishift"
 /*@
    STCayleyGetAntishift - Gets the value of the anti-shift used in the Cayley
@@ -419,23 +350,21 @@ PetscErrorCode STCayleyGetAntishift(ST st,PetscScalar *nu)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STView_Cayley"
 PetscErrorCode STView_Cayley(ST st,PetscViewer viewer)
 {
   PetscErrorCode ierr;
+  char           str[50];
   ST_CAYLEY      *ctx = (ST_CAYLEY*)st->data;
 
   PetscFunctionBegin;
-#if !defined(PETSC_USE_COMPLEX)
-  ierr = PetscViewerASCIIPrintf(viewer,"  Cayley: antishift: %G\n",ctx->nu);CHKERRQ(ierr);
-#else
-  ierr = PetscViewerASCIIPrintf(viewer,"  Cayley: antishift: %G+%G i\n",PetscRealPart(ctx->nu),PetscImaginaryPart(ctx->nu));CHKERRQ(ierr);
-#endif
+  ierr = SlepcSNPrintfScalar(str,50,ctx->nu,PETSC_FALSE);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  Cayley: antishift: %s\n",str);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STReset_Cayley"
 PetscErrorCode STReset_Cayley(ST st)
 {
@@ -447,7 +376,7 @@ PetscErrorCode STReset_Cayley(ST st)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STDestroy_Cayley"
 PetscErrorCode STDestroy_Cayley(ST st)
 {
@@ -455,14 +384,14 @@ PetscErrorCode STDestroy_Cayley(ST st)
 
   PetscFunctionBegin;
   ierr = PetscFree(st->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STCayleySetAntishift_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STCayleySetAntishift_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STCayleyGetAntishift_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STCreate_Cayley"
-PetscErrorCode STCreate_Cayley(ST st)
+PETSC_EXTERN PetscErrorCode STCreate_Cayley(ST st)
 {
   PetscErrorCode ierr;
 
@@ -472,7 +401,7 @@ PetscErrorCode STCreate_Cayley(ST st)
   st->ops->getbilinearform = STGetBilinearForm_Cayley;
   st->ops->applytrans      = STApplyTranspose_Cayley;
   st->ops->postsolve       = STPostSolve_Cayley;
-  st->ops->backtr          = STBackTransform_Cayley;
+  st->ops->backtransform   = STBackTransform_Cayley;
   st->ops->setfromoptions  = STSetFromOptions_Cayley;
   st->ops->setup           = STSetUp_Cayley;
   st->ops->setshift        = STSetShift_Cayley;
@@ -480,8 +409,8 @@ PetscErrorCode STCreate_Cayley(ST st)
   st->ops->reset           = STReset_Cayley;
   st->ops->view            = STView_Cayley;
   st->ops->checknullspace  = STCheckNullSpace_Default;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STCayleySetAntishift_C","STCayleySetAntishift_Cayley",STCayleySetAntishift_Cayley);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STCayleySetAntishift_C",STCayleySetAntishift_Cayley);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STCayleyGetAntishift_C",STCayleyGetAntishift_Cayley);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
