@@ -1,20 +1,20 @@
 /*
-    Folding spectral transformation, applies (A + sigma I)^2 as operator, or 
+    Folding spectral transformation, applies (A + sigma I)^2 as operator, or
     inv(B)(A + sigma I)^2 for generalized problems
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -22,39 +22,49 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/stimpl.h>          /*I "slepcst.h" I*/
+#include <slepc-private/stimpl.h>          /*I "slepcst.h" I*/
 
 typedef struct {
   Vec         w2;
 } ST_FOLD;
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STApply_Fold"
 PetscErrorCode STApply_Fold(ST st,Vec x,Vec y)
 {
-  PetscErrorCode ierr;
-  ST_FOLD        *ctx = (ST_FOLD*)st->data;
+  PetscErrorCode     ierr;
+  ST_FOLD            *ctx = (ST_FOLD*)st->data;
+  PetscInt           its;
+  KSPConvergedReason reason;
 
   PetscFunctionBegin;
-  if (st->B) {
+  if (st->nmat>1) {
     /* generalized eigenproblem: y = (B^-1 A + sI)^2 x */
-    ierr = MatMult(st->A,x,st->w);CHKERRQ(ierr);
-    ierr = STAssociatedKSPSolve(st,st->w,ctx->w2);CHKERRQ(ierr);
+    ierr = MatMult(st->A[0],x,st->w);CHKERRQ(ierr);
+    ierr = KSPSolve(st->ksp,st->w,ctx->w2);CHKERRQ(ierr);
+    ierr = KSPGetConvergedReason(st->ksp,&reason);CHKERRQ(ierr);
+    if (reason<0) SETERRQ1(PetscObjectComm((PetscObject)st),PETSC_ERR_NOT_CONVERGED,"KSP did not converge (reason=%s)",KSPConvergedReasons[reason]);
+    ierr = KSPGetIterationNumber(st->ksp,&its);CHKERRQ(ierr);
+    st->lineariterations += its;
     if (st->sigma != 0.0) {
       ierr = VecAXPY(ctx->w2,-st->sigma,x);CHKERRQ(ierr);
     }
-    ierr = MatMult(st->A,ctx->w2,st->w);CHKERRQ(ierr);
-    ierr = STAssociatedKSPSolve(st,st->w,y);CHKERRQ(ierr);
+    ierr = MatMult(st->A[0],ctx->w2,st->w);CHKERRQ(ierr);
+    ierr = KSPSolve(st->ksp,st->w,y);CHKERRQ(ierr);
+    ierr = KSPGetConvergedReason(st->ksp,&reason);CHKERRQ(ierr);
+    if (reason<0) SETERRQ1(PetscObjectComm((PetscObject)st),PETSC_ERR_NOT_CONVERGED,"KSP did not converge (reason=%s)",KSPConvergedReasons[reason]);
+    ierr = KSPGetIterationNumber(st->ksp,&its);CHKERRQ(ierr);
+    st->lineariterations += its;
     if (st->sigma != 0.0) {
       ierr = VecAXPY(y,-st->sigma,ctx->w2);CHKERRQ(ierr);
     }
   } else {
     /* standard eigenproblem: y = (A + sI)^2 x */
-    ierr = MatMult(st->A,x,st->w);CHKERRQ(ierr);
+    ierr = MatMult(st->A[0],x,st->w);CHKERRQ(ierr);
     if (st->sigma != 0.0) {
       ierr = VecAXPY(st->w,-st->sigma,x);CHKERRQ(ierr);
     }
-    ierr = MatMult(st->A,st->w,y);CHKERRQ(ierr);
+    ierr = MatMult(st->A[0],st->w,y);CHKERRQ(ierr);
     if (st->sigma != 0.0) {
       ierr = VecAXPY(y,-st->sigma,st->w);CHKERRQ(ierr);
     }
@@ -62,33 +72,43 @@ PetscErrorCode STApply_Fold(ST st,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STApplyTranspose_Fold"
 PetscErrorCode STApplyTranspose_Fold(ST st,Vec x,Vec y)
 {
-  PetscErrorCode ierr;
-  ST_FOLD        *ctx = (ST_FOLD*)st->data;
+  PetscErrorCode     ierr;
+  ST_FOLD            *ctx = (ST_FOLD*)st->data;
+  PetscInt           its;
+  KSPConvergedReason reason;
 
   PetscFunctionBegin;
-  if (st->B) {
+  if (st->nmat>1) {
     /* generalized eigenproblem: y = (A^T B^-T + sI)^2 x */
-    ierr = STAssociatedKSPSolveTranspose(st,x,st->w);CHKERRQ(ierr);
-    ierr = MatMult(st->A,st->w,ctx->w2);CHKERRQ(ierr);
+    ierr = KSPSolveTranspose(st->ksp,x,st->w);CHKERRQ(ierr);
+    ierr = KSPGetConvergedReason(st->ksp,&reason);CHKERRQ(ierr);
+    if (reason<0) SETERRQ1(PetscObjectComm((PetscObject)st),PETSC_ERR_NOT_CONVERGED,"KSP did not converge (reason=%s)",KSPConvergedReasons[reason]);
+    ierr = KSPGetIterationNumber(st->ksp,&its);CHKERRQ(ierr);
+    st->lineariterations += its;
+    ierr = MatMult(st->A[0],st->w,ctx->w2);CHKERRQ(ierr);
     if (st->sigma != 0.0) {
       ierr = VecAXPY(ctx->w2,-st->sigma,x);CHKERRQ(ierr);
     }
-    ierr = STAssociatedKSPSolveTranspose(st,ctx->w2,st->w);CHKERRQ(ierr);
-    ierr = MatMult(st->A,st->w,y);CHKERRQ(ierr);
+    ierr = KSPSolveTranspose(st->ksp,ctx->w2,st->w);CHKERRQ(ierr);
+    ierr = KSPGetConvergedReason(st->ksp,&reason);CHKERRQ(ierr);
+    if (reason<0) SETERRQ1(PetscObjectComm((PetscObject)st),PETSC_ERR_NOT_CONVERGED,"KSP did not converge (reason=%s)",KSPConvergedReasons[reason]);
+    ierr = KSPGetIterationNumber(st->ksp,&its);CHKERRQ(ierr);
+    st->lineariterations += its;
+    ierr = MatMult(st->A[0],st->w,y);CHKERRQ(ierr);
     if (st->sigma != 0.0) {
       ierr = VecAXPY(y,-st->sigma,ctx->w2);CHKERRQ(ierr);
     }
   } else {
     /* standard eigenproblem: y = (A^T + sI)^2 x */
-    ierr = MatMultTranspose(st->A,x,st->w);CHKERRQ(ierr);
+    ierr = MatMultTranspose(st->A[0],x,st->w);CHKERRQ(ierr);
     if (st->sigma != 0.0) {
       ierr = VecAXPY(st->w,-st->sigma,x);CHKERRQ(ierr);
     }
-    ierr = MatMultTranspose(st->A,st->w,y);CHKERRQ(ierr);
+    ierr = MatMultTranspose(st->A[0],st->w,y);CHKERRQ(ierr);
     if (st->sigma != 0.0) {
       ierr = VecAXPY(y,-st->sigma,st->w);CHKERRQ(ierr);
     }
@@ -96,25 +116,27 @@ PetscErrorCode STApplyTranspose_Fold(ST st,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STBackTransform_Fold"
 PetscErrorCode STBackTransform_Fold(ST st,PetscInt n,PetscScalar *eigr,PetscScalar *eigi)
 {
-  PetscInt j;
+  PetscInt    j;
+#if !defined(PETSC_USE_COMPLEX)
+  PetscScalar r,x,y;
+#endif
 
   PetscFunctionBegin;
   for (j=0;j<n;j++) {
 #if !defined(PETSC_USE_COMPLEX)
-    if (eigi[j] == 0) {
+    if (eigi[j] == 0.0) {
 #endif
       eigr[j] = st->sigma + PetscSqrtScalar(eigr[j]);
 #if !defined(PETSC_USE_COMPLEX)
     } else {
-      PetscScalar r,x,y;
-      r = PetscSqrtScalar(eigr[j] * eigr[j] + eigi[j] * eigi[j]);
+      r = SlepcAbsEigenvalue(eigr[j],eigi[j]);
       x = PetscSqrtScalar((r + eigr[j]) / 2);
       y = PetscSqrtScalar((r - eigr[j]) / 2);
-      if (eigi[j] < 0) y = - y;
+      if (eigi[j] < 0.0) y = - y;
       eigr[j] = st->sigma + x;
       eigi[j] = y;
     }
@@ -123,7 +145,7 @@ PetscErrorCode STBackTransform_Fold(ST st,PetscInt n,PetscScalar *eigr,PetscScal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetUp_Fold"
 PetscErrorCode STSetUp_Fold(ST st)
 {
@@ -134,24 +156,25 @@ PetscErrorCode STSetUp_Fold(ST st)
   /* if the user did not set the shift, use the target value */
   if (!st->sigma_set) st->sigma = st->defsigma;
 
-  if (st->B) {
+  if (st->nmat>1) {
     if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
-    ierr = KSPSetOperators(st->ksp,st->B,st->B,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
+    ierr = KSPSetOperators(st->ksp,st->A[1],st->A[1],DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
     ierr = KSPSetUp(st->ksp);CHKERRQ(ierr);
     ierr = VecDestroy(&ctx->w2);CHKERRQ(ierr);
-    ierr = MatGetVecs(st->B,&ctx->w2,PETSC_NULL);CHKERRQ(ierr); 
-  } 
+    ierr = MatGetVecs(st->A[1],&ctx->w2,NULL);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(st,ctx->w2);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetFromOptions_Fold"
-PetscErrorCode STSetFromOptions_Fold(ST st) 
+PetscErrorCode STSetFromOptions_Fold(ST st)
 {
   PetscErrorCode ierr;
   PC             pc;
-  const PCType   pctype;
-  const KSPType  ksptype;
+  PCType         pctype;
+  KSPType        ksptype;
 
   PetscFunctionBegin;
   if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
@@ -172,7 +195,7 @@ PetscErrorCode STSetFromOptions_Fold(ST st)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STReset_Fold"
 PetscErrorCode STReset_Fold(ST st)
 {
@@ -184,7 +207,7 @@ PetscErrorCode STReset_Fold(ST st)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STDestroy_Fold"
 PetscErrorCode STDestroy_Fold(ST st)
 {
@@ -195,10 +218,9 @@ PetscErrorCode STDestroy_Fold(ST st)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STCreate_Fold"
-PetscErrorCode STCreate_Fold(ST st)
+PETSC_EXTERN PetscErrorCode STCreate_Fold(ST st)
 {
   PetscErrorCode ierr;
 
@@ -207,12 +229,11 @@ PetscErrorCode STCreate_Fold(ST st)
   st->ops->apply           = STApply_Fold;
   st->ops->getbilinearform = STGetBilinearForm_Default;
   st->ops->applytrans      = STApplyTranspose_Fold;
-  st->ops->backtr          = STBackTransform_Fold;
+  st->ops->backtransform   = STBackTransform_Fold;
   st->ops->setup           = STSetUp_Fold;
   st->ops->setfromoptions  = STSetFromOptions_Fold;
   st->ops->destroy         = STDestroy_Fold;
   st->ops->reset           = STReset_Fold;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
