@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,19 +21,19 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/stimpl.h>          /*I "slepcst.h" I*/
+#include <slepc-private/stimpl.h>          /*I "slepcst.h" I*/
 
 typedef struct {
   PetscBool setmat;
 } ST_PRECOND;
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetFromOptions_Precond"
-PetscErrorCode STSetFromOptions_Precond(ST st) 
+PetscErrorCode STSetFromOptions_Precond(ST st)
 {
   PetscErrorCode ierr;
   PC             pc;
-  const PCType   pctype;
+  PCType         pctype;
   Mat            P;
   PetscBool      t0,t1;
   KSP            ksp;
@@ -43,23 +43,21 @@ PetscErrorCode STSetFromOptions_Precond(ST st)
   ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
   ierr = PetscObjectGetType((PetscObject)pc,&pctype);CHKERRQ(ierr);
   ierr = STPrecondGetMatForPC(st,&P);CHKERRQ(ierr);
-  if (!pctype && st->A) {
+  if (!pctype && st->A[0]) {
     if (P || st->shift_matrix == ST_MATMODE_SHELL) {
       ierr = PCSetType(pc,PCJACOBI);CHKERRQ(ierr);
     } else {
-      ierr = MatHasOperation(st->A,MATOP_DUPLICATE,&t0);CHKERRQ(ierr);
-      if (st->B) {
-        ierr = MatHasOperation(st->A,MATOP_AXPY,&t1);CHKERRQ(ierr);
-      } else {
-        t1 = PETSC_TRUE;
-      }
+      ierr = MatHasOperation(st->A[0],MATOP_DUPLICATE,&t0);CHKERRQ(ierr);
+      if (st->nmat>1) {
+        ierr = MatHasOperation(st->A[0],MATOP_AXPY,&t1);CHKERRQ(ierr);
+      } else t1 = PETSC_TRUE;
       ierr = PCSetType(pc,(t0 && t1)?PCJACOBI:PCNONE);CHKERRQ(ierr);
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetUp_Precond"
 PetscErrorCode STSetUp_Precond(ST st)
 {
@@ -72,12 +70,14 @@ PetscErrorCode STSetUp_Precond(ST st)
   /* if the user did not set the shift, use the target value */
   if (!st->sigma_set) st->sigma = st->defsigma;
 
-  /* If pc is none and no matrix has to be set, exit */
+  /* If either pc is none and no matrix has to be set, or pc is shell , exit */
   ierr = STSetFromOptions_Precond(st);CHKERRQ(ierr);
   if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
   ierr = KSPGetPC(st->ksp,&pc);CHKERRQ(ierr);
-  ierr = PetscTypeCompare((PetscObject)pc,PCNONE,&t0);CHKERRQ(ierr);
-  ierr = STPrecondGetKSPHasMat(st,&setmat);CHKERRQ(ierr); 
+  ierr = PetscObjectTypeCompare((PetscObject)pc,PCSHELL,&t0);CHKERRQ(ierr);
+  if (t0) PetscFunctionReturn(0);
+  ierr = PetscObjectTypeCompare((PetscObject)pc,PCNONE,&t0);CHKERRQ(ierr);
+  ierr = STPrecondGetKSPHasMat(st,&setmat);CHKERRQ(ierr);
   if (t0 && !setmat) PetscFunctionReturn(0);
 
   /* Check if a user matrix is set */
@@ -91,27 +91,28 @@ PetscErrorCode STSetUp_Precond(ST st)
   } else {
     builtP = PETSC_TRUE;
 
-    if (!(PetscAbsScalar(st->sigma) < PETSC_MAX_REAL) && st->B) {
-      P = st->B;
+    if (!(PetscAbsScalar(st->sigma) < PETSC_MAX_REAL) && st->nmat>1) {
+      P = st->A[1];
       destroyP = PETSC_FALSE;
     } else if (st->sigma == 0.0) {
-      P = st->A;
+      P = st->A[0];
       destroyP = PETSC_FALSE;
     } else if (PetscAbsScalar(st->sigma) < PETSC_MAX_REAL && st->shift_matrix != ST_MATMODE_SHELL) {
       if (st->shift_matrix == ST_MATMODE_INPLACE) {
-        P = st->A;
+        P = st->A[0];
         destroyP = PETSC_FALSE;
       } else {
-        ierr = MatDuplicate(st->A,MAT_COPY_VALUES,&P);CHKERRQ(ierr);
+        ierr = MatDuplicate(st->A[0],MAT_COPY_VALUES,&P);CHKERRQ(ierr);
         destroyP = PETSC_TRUE;
-      } 
-      if (st->B) {
-        ierr = MatAXPY(P,-st->sigma,st->B,st->str);CHKERRQ(ierr); 
-      } else {
-        ierr = MatShift(P,-st->sigma);CHKERRQ(ierr); 
       }
-    } else 
-      builtP = PETSC_FALSE;
+      if (st->nmat>1) {
+        ierr = MatAXPY(P,-st->sigma,st->A[1],st->str);CHKERRQ(ierr);
+      } else {
+        ierr = MatShift(P,-st->sigma);CHKERRQ(ierr);
+      }
+      /* TODO: in case of ST_MATMODE_INPLACE should keep the Hermitian flag of st->A and restore at the end */
+      ierr = STMatSetHermitian(st,P);CHKERRQ(ierr);
+    } else builtP = PETSC_FALSE;
   }
 
   /* If P was not possible to obtain, set pc to PCNONE */
@@ -120,28 +121,29 @@ PetscErrorCode STSetUp_Precond(ST st)
 
     /* If some matrix has to be set to ksp, a shell matrix is created */
     if (setmat) {
-      ierr = STMatShellCreate(st,&P);CHKERRQ(ierr);
+      ierr = STMatShellCreate(st,-st->sigma,0,NULL,&P);CHKERRQ(ierr);
+      ierr = STMatSetHermitian(st,P);CHKERRQ(ierr);
       destroyP = PETSC_TRUE;
     }
   }
 
-  ierr = KSPSetOperators(st->ksp,setmat?P:PETSC_NULL,P,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
+  ierr = KSPSetOperators(st->ksp,setmat?P:NULL,P,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
 
   if (destroyP) {
     ierr = MatDestroy(&P);CHKERRQ(ierr);
   } else if (st->shift_matrix == ST_MATMODE_INPLACE && builtP) {
     if (st->sigma != 0.0 && PetscAbsScalar(st->sigma) < PETSC_MAX_REAL) {
-      if (st->B) {
-        ierr = MatAXPY(st->A,st->sigma,st->B,st->str);CHKERRQ(ierr); 
-      } else { 
-        ierr = MatShift(st->A,st->sigma);CHKERRQ(ierr); 
+      if (st->nmat>1) {
+        ierr = MatAXPY(st->A[0],st->sigma,st->A[1],st->str);CHKERRQ(ierr);
+      } else {
+        ierr = MatShift(st->A[0],st->sigma);CHKERRQ(ierr);
       }
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetShift_Precond"
 PetscErrorCode STSetShift_Precond(ST st,PetscScalar newshift)
 {
@@ -157,10 +159,9 @@ PetscErrorCode STSetShift_Precond(ST st,PetscScalar newshift)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
 #undef __FUNCT__
 #define __FUNCT__ "STPrecondGetMatForPC_Precond"
-PetscErrorCode STPrecondGetMatForPC_Precond(ST st,Mat *mat)
+static PetscErrorCode STPrecondGetMatForPC_Precond(ST st,Mat *mat)
 {
   PetscErrorCode ierr;
   PC             pc;
@@ -169,15 +170,14 @@ PetscErrorCode STPrecondGetMatForPC_Precond(ST st,Mat *mat)
   PetscFunctionBegin;
   if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
   ierr = KSPGetPC(st->ksp,&pc);CHKERRQ(ierr);
-  ierr = PCGetOperatorsSet(pc,PETSC_NULL,&flag);CHKERRQ(ierr);
+  ierr = PCGetOperatorsSet(pc,NULL,&flag);CHKERRQ(ierr);
   if (flag) {
-    ierr = PCGetOperators(pc,PETSC_NULL,mat,PETSC_NULL);CHKERRQ(ierr);
-  } else *mat = PETSC_NULL;
+    ierr = PCGetOperators(pc,NULL,mat,NULL);CHKERRQ(ierr);
+  } else *mat = NULL;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STPrecondGetMatForPC"
 /*@
    STPrecondGetMatForPC - Returns the matrix previously set by STPrecondSetMatForPC().
@@ -189,7 +189,7 @@ EXTERN_C_END
 
    Output Parameter:
 .  mat - the matrix that will be used in constructing the preconditioner or
-   PETSC_NULL if no matrix was set by STPrecondSetMatForPC().
+   NULL if no matrix was set by STPrecondSetMatForPC().
 
    Level: advanced
 
@@ -206,10 +206,9 @@ PetscErrorCode STPrecondGetMatForPC(ST st,Mat *mat)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
 #undef __FUNCT__
 #define __FUNCT__ "STPrecondSetMatForPC_Precond"
-PetscErrorCode STPrecondSetMatForPC_Precond(ST st,Mat mat)
+static PetscErrorCode STPrecondSetMatForPC_Precond(ST st,Mat mat)
 {
   PC             pc;
   Mat            A;
@@ -221,12 +220,11 @@ PetscErrorCode STPrecondSetMatForPC_Precond(ST st,Mat mat)
   ierr = KSPGetPC(st->ksp,&pc);CHKERRQ(ierr);
   /* Yes, all these lines are needed to safely set mat as the preconditioner
      matrix in pc */
-  ierr = PCGetOperatorsSet(pc,&flag,PETSC_NULL);CHKERRQ(ierr);
+  ierr = PCGetOperatorsSet(pc,&flag,NULL);CHKERRQ(ierr);
   if (flag) {
-    ierr = PCGetOperators(pc,&A,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+    ierr = PCGetOperators(pc,&A,NULL,NULL);CHKERRQ(ierr);
     ierr = PetscObjectReference((PetscObject)A);CHKERRQ(ierr);
-  } else
-    A = PETSC_NULL;
+  } else A = NULL;
   ierr = PetscObjectReference((PetscObject)mat);CHKERRQ(ierr);
   ierr = PCSetOperators(pc,A,mat,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
   ierr = MatDestroy(&A);CHKERRQ(ierr);
@@ -234,9 +232,8 @@ PetscErrorCode STPrecondSetMatForPC_Precond(ST st,Mat mat)
   ierr = STPrecondSetKSPHasMat(st,PETSC_TRUE);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STPrecondSetMatForPC"
 /*@
    STPrecondSetMatForPC - Sets the matrix that must be used to build the preconditioner.
@@ -252,7 +249,7 @@ EXTERN_C_END
    Notes:
    This matrix will be passed to the KSP object (via KSPSetOperators) as
    the matrix to be used when constructing the preconditioner.
-   If no matrix is set or mat is set to PETSC_NULL, A - sigma*B will
+   If no matrix is set or mat is set to NULL, A - sigma*B will
    be used to build the preconditioner, being sigma the value set by STSetShift().
 
 .seealso: STPrecondSetMatForPC(), STSetShift()
@@ -269,10 +266,9 @@ PetscErrorCode STPrecondSetMatForPC(ST st,Mat mat)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
 #undef __FUNCT__
 #define __FUNCT__ "STPrecondSetKSPHasMat_Precond"
-PetscErrorCode STPrecondSetKSPHasMat_Precond(ST st,PetscBool setmat)
+static PetscErrorCode STPrecondSetKSPHasMat_Precond(ST st,PetscBool setmat)
 {
   ST_PRECOND *data = (ST_PRECOND*)st->data;
 
@@ -280,13 +276,12 @@ PetscErrorCode STPrecondSetKSPHasMat_Precond(ST st,PetscBool setmat)
   data->setmat = setmat;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STPrecondSetKSPHasMat"
 /*@
    STPrecondSetKSPHasMat - Sets a flag indicating that during STSetUp the coefficient
-   matrix of the KSP linear system (A) must be set to be the same matrix as the 
+   matrix of the KSP linear system (A) must be set to be the same matrix as the
    preconditioner (P).
 
    Collective on ST
@@ -315,10 +310,9 @@ PetscErrorCode STPrecondSetKSPHasMat(ST st,PetscBool setmat)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
 #undef __FUNCT__
 #define __FUNCT__ "STPrecondGetKSPHasMat_Precond"
-PetscErrorCode STPrecondGetKSPHasMat_Precond(ST st,PetscBool *setmat)
+static PetscErrorCode STPrecondGetKSPHasMat_Precond(ST st,PetscBool *setmat)
 {
   ST_PRECOND *data = (ST_PRECOND*)st->data;
 
@@ -326,13 +320,12 @@ PetscErrorCode STPrecondGetKSPHasMat_Precond(ST st,PetscBool *setmat)
   *setmat = data->setmat;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STPrecondGetKSPHasMat"
 /*@
    STPrecondGetKSPHasMat - Returns the flag indicating if the coefficient
-   matrix of the KSP linear system (A) is set to be the same matrix as the 
+   matrix of the KSP linear system (A) is set to be the same matrix as the
    preconditioner (P).
 
    Not Collective
@@ -358,7 +351,7 @@ PetscErrorCode STPrecondGetKSPHasMat(ST st,PetscBool *setmat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STDestroy_Precond"
 PetscErrorCode STDestroy_Precond(ST st)
 {
@@ -366,17 +359,16 @@ PetscErrorCode STDestroy_Precond(ST st)
 
   PetscFunctionBegin;
   ierr = PetscFree(st->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STPrecondGetMatForPC_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STPrecondSetMatForPC_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STPrecondGetKSPHasMat_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STPrecondSetKSPHasMat_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STPrecondGetMatForPC_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STPrecondSetMatForPC_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STPrecondGetKSPHasMat_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STPrecondSetKSPHasMat_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STCreate_Precond"
-PetscErrorCode STCreate_Precond(ST st)
+PETSC_EXTERN PetscErrorCode STCreate_Precond(ST st)
 {
   PetscErrorCode ierr;
 
@@ -388,13 +380,12 @@ PetscErrorCode STCreate_Precond(ST st)
   st->ops->destroy         = STDestroy_Precond;
   st->ops->setfromoptions  = STSetFromOptions_Precond;
 
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STPrecondGetMatForPC_C","STPrecondGetMatForPC_Precond",STPrecondGetMatForPC_Precond);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STPrecondSetMatForPC_C","STPrecondSetMatForPC_Precond",STPrecondSetMatForPC_Precond);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STPrecondGetKSPHasMat_C","STPrecondGetKSPHasMat_Precond",STPrecondGetKSPHasMat_Precond);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STPrecondSetKSPHasMat_C","STPrecondSetKSPHasMat_Precond",STPrecondSetKSPHasMat_Precond);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STPrecondGetMatForPC_C",STPrecondGetMatForPC_Precond);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STPrecondSetMatForPC_C",STPrecondSetMatForPC_Precond);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STPrecondGetKSPHasMat_C",STPrecondGetKSPHasMat_Precond);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STPrecondSetKSPHasMat_C",STPrecondSetKSPHasMat_Precond);CHKERRQ(ierr);
 
   ierr = STPrecondSetKSPHasMat_Precond(st,PETSC_TRUE);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
