@@ -1,21 +1,21 @@
 /*
-   This provides a simple shell interface for programmers to 
-   create their own spectral transformations without writing much 
+   This provides a simple shell interface for programmers to
+   create their own spectral transformations without writing much
    interface code.
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -23,18 +23,16 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/stimpl.h"        /*I "slepcst.h" I*/
+#include <slepc-private/stimpl.h>        /*I "slepcst.h" I*/
 
-EXTERN_C_BEGIN 
 typedef struct {
   void           *ctx;                       /* user provided context */
   PetscErrorCode (*apply)(ST,Vec,Vec);
   PetscErrorCode (*applytrans)(ST,Vec,Vec);
-  PetscErrorCode (*backtr)(ST,PetscInt n,PetscScalar*,PetscScalar*);
+  PetscErrorCode (*backtransform)(ST,PetscInt n,PetscScalar*,PetscScalar*);
 } ST_Shell;
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STShellGetContext"
 /*@C
    STShellGetContext - Returns the user-provided context associated with a shell ST
@@ -51,7 +49,7 @@ EXTERN_C_END
 
    Notes:
    This routine is intended for use within various shell routines
-    
+
 .seealso: STShellSetContext()
 @*/
 PetscErrorCode STShellGetContext(ST st,void **ctx)
@@ -61,14 +59,14 @@ PetscErrorCode STShellGetContext(ST st,void **ctx)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
-  PetscValidPointer(ctx,2); 
-  ierr = PetscTypeCompare((PetscObject)st,STSHELL,&flg);CHKERRQ(ierr);
-  if (!flg) *ctx = 0; 
+  PetscValidPointer(ctx,2);
+  ierr = PetscObjectTypeCompare((PetscObject)st,STSHELL,&flg);CHKERRQ(ierr);
+  if (!flg) *ctx = 0;
   else      *ctx = ((ST_Shell*)(st->data))->ctx;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STShellSetContext"
 /*@
    STShellSetContext - Sets the context for a shell ST
@@ -94,14 +92,14 @@ PetscErrorCode STShellSetContext(ST st,void *ctx)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
-  ierr = PetscTypeCompare((PetscObject)st,STSHELL,&flg);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)st,STSHELL,&flg);CHKERRQ(ierr);
   if (flg) {
     shell->ctx = ctx;
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STApply_Shell"
 PetscErrorCode STApply_Shell(ST st,Vec x,Vec y)
 {
@@ -109,7 +107,7 @@ PetscErrorCode STApply_Shell(ST st,Vec x,Vec y)
   ST_Shell       *shell = (ST_Shell*)st->data;
 
   PetscFunctionBegin;
-  if (!shell->apply) SETERRQ(((PetscObject)st)->comm,PETSC_ERR_USER,"No apply() routine provided to Shell ST");
+  if (!shell->apply) SETERRQ(PetscObjectComm((PetscObject)st),PETSC_ERR_USER,"No apply() routine provided to Shell ST");
   PetscStackPush("STSHELL apply() user function");
   CHKMEMQ;
   ierr = (*shell->apply)(st,x,y);CHKERRQ(ierr);
@@ -118,7 +116,7 @@ PetscErrorCode STApply_Shell(ST st,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STApplyTranspose_Shell"
 PetscErrorCode STApplyTranspose_Shell(ST st,Vec x,Vec y)
 {
@@ -126,7 +124,7 @@ PetscErrorCode STApplyTranspose_Shell(ST st,Vec x,Vec y)
   ST_Shell       *shell = (ST_Shell*)st->data;
 
   PetscFunctionBegin;
-  if (!shell->applytrans) SETERRQ(((PetscObject)st)->comm,PETSC_ERR_USER,"No applytranspose() routine provided to Shell ST");
+  if (!shell->applytrans) SETERRQ(PetscObjectComm((PetscObject)st),PETSC_ERR_USER,"No applytranspose() routine provided to Shell ST");
   PetscStackPush("STSHELL applytranspose() user function");
   CHKMEMQ;
   ierr = (*shell->applytrans)(st,x,y);CHKERRQ(ierr);
@@ -135,7 +133,7 @@ PetscErrorCode STApplyTranspose_Shell(ST st,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STBackTransform_Shell"
 PetscErrorCode STBackTransform_Shell(ST st,PetscInt n,PetscScalar *eigr,PetscScalar *eigi)
 {
@@ -143,17 +141,17 @@ PetscErrorCode STBackTransform_Shell(ST st,PetscInt n,PetscScalar *eigr,PetscSca
   ST_Shell       *shell = (ST_Shell*)st->data;
 
   PetscFunctionBegin;
-  if (shell->backtr) {
+  if (shell->backtransform) {
     PetscStackPush("STSHELL backtransform() user function");
     CHKMEMQ;
-    ierr = (*shell->backtr)(st,n,eigr,eigi);CHKERRQ(ierr);
+    ierr = (*shell->backtransform)(st,n,eigr,eigi);CHKERRQ(ierr);
     CHKMEMQ;
     PetscStackPop;
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STDestroy_Shell"
 PetscErrorCode STDestroy_Shell(ST st)
 {
@@ -161,16 +159,15 @@ PetscErrorCode STDestroy_Shell(ST st)
 
   PetscFunctionBegin;
   ierr = PetscFree(st->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetApply_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetApplyTranspose_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetBackTransform_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STShellSetApply_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STShellSetApplyTranspose_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STShellSetBackTransform_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STShellSetApply_Shell"
-PetscErrorCode STShellSetApply_Shell(ST st,PetscErrorCode (*apply)(ST,Vec,Vec))
+static PetscErrorCode STShellSetApply_Shell(ST st,PetscErrorCode (*apply)(ST,Vec,Vec))
 {
   ST_Shell *shell = (ST_Shell*)st->data;
 
@@ -178,12 +175,10 @@ PetscErrorCode STShellSetApply_Shell(ST st,PetscErrorCode (*apply)(ST,Vec,Vec))
   shell->apply = apply;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STShellSetApplyTranspose_Shell"
-PetscErrorCode STShellSetApplyTranspose_Shell(ST st,PetscErrorCode (*applytrans)(ST,Vec,Vec))
+static PetscErrorCode STShellSetApplyTranspose_Shell(ST st,PetscErrorCode (*applytrans)(ST,Vec,Vec))
 {
   ST_Shell *shell = (ST_Shell*)st->data;
 
@@ -191,25 +186,22 @@ PetscErrorCode STShellSetApplyTranspose_Shell(ST st,PetscErrorCode (*applytrans)
   shell->applytrans = applytrans;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STShellSetBackTransform_Shell"
-PetscErrorCode STShellSetBackTransform_Shell(ST st,PetscErrorCode (*backtr)(ST,PetscInt,PetscScalar*,PetscScalar*))
+static PetscErrorCode STShellSetBackTransform_Shell(ST st,PetscErrorCode (*backtr)(ST,PetscInt,PetscScalar*,PetscScalar*))
 {
   ST_Shell *shell = (ST_Shell*)st->data;
 
   PetscFunctionBegin;
-  shell->backtr = backtr;
+  shell->backtransform = backtr;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STShellSetApply"
 /*@C
-   STShellSetApply - Sets routine to use as the application of the 
+   STShellSetApply - Sets routine to use as the application of the
    operator to a vector in the user-defined spectral transformation.
 
    Logically Collective on ST
@@ -241,10 +233,10 @@ PetscErrorCode STShellSetApply(ST st,PetscErrorCode (*apply)(ST,Vec,Vec))
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STShellSetApplyTranspose"
 /*@C
-   STShellSetApplyTranspose - Sets routine to use as the application of the 
+   STShellSetApplyTranspose - Sets routine to use as the application of the
    transposed operator to a vector in the user-defined spectral transformation.
 
    Logically Collective on ST
@@ -276,10 +268,10 @@ PetscErrorCode STShellSetApplyTranspose(ST st,PetscErrorCode (*applytrans)(ST,Ve
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STShellSetBackTransform"
 /*@C
-   STShellSetBackTransform - Sets the routine to be called after the 
+   STShellSetBackTransform - Sets the routine to be called after the
    eigensolution process has finished in order to transform back the
    computed eigenvalues.
 
@@ -291,12 +283,12 @@ PetscErrorCode STShellSetApplyTranspose(ST st,PetscErrorCode (*applytrans)(ST,Ve
 
    Calling sequence of backtr:
 .vb
-   PetscErrorCode backtr (ST st,PetscScalar *eigr,PetscScalar *eigi)
+   PetscErrorCode backtr(ST st,PetscScalar *eigr,PetscScalar *eigi)
 .ve
 
 +  st   - the spectral transformation context
 .  eigr - pointer ot the real part of the eigenvalue to transform back
--  eigi - pointer ot the imaginary part 
+-  eigi - pointer ot the imaginary part
 
    Level: developer
 
@@ -312,14 +304,14 @@ PetscErrorCode STShellSetBackTransform(ST st,PetscErrorCode (*backtr)(ST,PetscIn
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetFromOptions_Shell"
-PetscErrorCode STSetFromOptions_Shell(ST st) 
+PetscErrorCode STSetFromOptions_Shell(ST st)
 {
   PetscErrorCode ierr;
   PC             pc;
-  const PCType   pctype;
-  const KSPType  ksptype;
+  PCType         pctype;
+  KSPType        ksptype;
 
   PetscFunctionBegin;
   if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
@@ -359,10 +351,9 @@ $             STShellSetBackTransform(st,backtr);    (optional)
 
 M*/
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STCreate_Shell"
-PetscErrorCode STCreate_Shell(ST st)
+PETSC_EXTERN PetscErrorCode STCreate_Shell(ST st)
 {
   PetscErrorCode ierr;
 
@@ -370,13 +361,12 @@ PetscErrorCode STCreate_Shell(ST st)
   ierr = PetscNewLog(st,ST_Shell,&st->data);CHKERRQ(ierr);
   st->ops->apply          = STApply_Shell;
   st->ops->applytrans     = STApplyTranspose_Shell;
-  st->ops->backtr         = STBackTransform_Shell;
+  st->ops->backtransform  = STBackTransform_Shell;
   st->ops->setfromoptions = STSetFromOptions_Shell;
   st->ops->destroy        = STDestroy_Shell;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetApply_C","STShellSetApply_Shell",STShellSetApply_Shell);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetApplyTranspose_C","STShellSetApplyTranspose_Shell",STShellSetApplyTranspose_Shell);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetBackTransform_C","STShellSetBackTransform_Shell",STShellSetBackTransform_Shell);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STShellSetApply_C",STShellSetApply_Shell);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STShellSetApplyTranspose_C",STShellSetApplyTranspose_Shell);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)st,"STShellSetBackTransform_C",STShellSetBackTransform_Shell);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
