@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,46 +21,45 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/stimpl.h>            /*I "slepcst.h" I*/
+#include <slepc-private/stimpl.h>            /*I "slepcst.h" I*/
 
 PetscClassId     ST_CLASSID = 0;
 PetscLogEvent    ST_SetUp = 0,ST_Apply = 0,ST_ApplyTranspose = 0;
 static PetscBool STPackageInitialized = PETSC_FALSE;
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STFinalizePackage"
 /*@C
-   STFinalizePackage - This function destroys everything in the Slepc interface 
+   STFinalizePackage - This function destroys everything in the Slepc interface
    to the ST package. It is called from SlepcFinalize().
 
    Level: developer
 
 .seealso: SlepcFinalize()
 @*/
-PetscErrorCode STFinalizePackage(void) 
+PetscErrorCode STFinalizePackage(void)
 {
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
+  ierr = PetscFunctionListDestroy(&STList);CHKERRQ(ierr);
   STPackageInitialized = PETSC_FALSE;
-  STList               = 0;
   STRegisterAllCalled  = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STInitializePackage"
 /*@C
    STInitializePackage - This function initializes everything in the ST package. It is called
    from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to STCreate()
    when using static libraries.
 
-   Input Parameter:
-.  path - The dynamic library path, or PETSC_NULL
-
    Level: developer
 
 .seealso: SlepcInitialize()
 @*/
-PetscErrorCode STInitializePackage(const char *path)
+PetscErrorCode STInitializePackage(void)
 {
   char           logList[256];
   char           *className;
@@ -73,13 +72,13 @@ PetscErrorCode STInitializePackage(const char *path)
   /* Register Classes */
   ierr = PetscClassIdRegister("Spectral Transform",&ST_CLASSID);CHKERRQ(ierr);
   /* Register Constructors */
-  ierr = STRegisterAll(path);CHKERRQ(ierr);
+  ierr = STRegisterAll();CHKERRQ(ierr);
   /* Register Events */
   ierr = PetscLogEventRegister("STSetUp",ST_CLASSID,&ST_SetUp);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("STApply",ST_CLASSID,&ST_Apply);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("STApplyTranspose",ST_CLASSID,&ST_ApplyTranspose);CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
     ierr = PetscStrstr(logList,"st",&className);CHKERRQ(ierr);
     if (className) {
@@ -87,7 +86,7 @@ PetscErrorCode STInitializePackage(const char *path)
     }
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL,"-log_summary_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,"-log_summary_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
     ierr = PetscStrstr(logList,"st",&className);CHKERRQ(ierr);
     if (className) {
@@ -98,7 +97,7 @@ PetscErrorCode STInitializePackage(const char *path)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STReset"
 /*@
    STReset - Resets the ST context and removes any allocated objects.
@@ -120,20 +119,16 @@ PetscErrorCode STReset(ST st)
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
   if (st->ops->reset) { ierr = (*st->ops->reset)(st);CHKERRQ(ierr); }
   if (st->ksp) { ierr = KSPReset(st->ksp);CHKERRQ(ierr); }
-  ierr = MatDestroy(&st->A);CHKERRQ(ierr);
-  ierr = MatDestroy(&st->B);CHKERRQ(ierr);
+  ierr = MatDestroyMatrices(PetscMax(2,st->nmat),&st->T);CHKERRQ(ierr);
   ierr = VecDestroy(&st->w);CHKERRQ(ierr);
-  ierr = VecDestroy(&st->D);CHKERRQ(ierr);
   ierr = VecDestroy(&st->wb);CHKERRQ(ierr);
-  if (st->shift_matrix != ST_MATMODE_INPLACE) { 
-    ierr = MatDestroy(&st->mat);CHKERRQ(ierr); 
-  }
   ierr = STResetOperationCounters(st);CHKERRQ(ierr);
+  st->kspidx = -1;
   st->setupcalled = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STDestroy"
 /*@C
    STDestroy - Destroys ST context that was created with STCreate().
@@ -156,14 +151,16 @@ PetscErrorCode STDestroy(ST *st)
   PetscValidHeaderSpecific(*st,ST_CLASSID,1);
   if (--((PetscObject)(*st))->refct > 0) { *st = 0; PetscFunctionReturn(0); }
   ierr = STReset(*st);CHKERRQ(ierr);
-  ierr = PetscObjectDepublish(*st);CHKERRQ(ierr);
+  ierr = MatDestroyMatrices(PetscMax(2,(*st)->nmat),&(*st)->A);CHKERRQ(ierr);
+  ierr = PetscFree((*st)->Astate);CHKERRQ(ierr);
   if ((*st)->ops->destroy) { ierr = (*(*st)->ops->destroy)(*st);CHKERRQ(ierr); }
+  ierr = VecDestroy(&(*st)->D);CHKERRQ(ierr);
   ierr = KSPDestroy(&(*st)->ksp);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(st);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STCreate"
 /*@C
    STCreate - Creates a spectral transformation context.
@@ -171,7 +168,7 @@ PetscErrorCode STDestroy(ST *st)
    Collective on MPI_Comm
 
    Input Parameter:
-.  comm - MPI communicator 
+.  comm - MPI communicator
 
    Output Parameter:
 .  st - location to put the spectral transformation context
@@ -188,94 +185,141 @@ PetscErrorCode STCreate(MPI_Comm comm,ST *newst)
   PetscFunctionBegin;
   PetscValidPointer(newst,2);
   *newst = 0;
-  ierr = PetscHeaderCreate(st,_p_ST,struct _STOps,ST_CLASSID,-1,"ST","Spectral Transformation","ST",comm,STDestroy,STView);CHKERRQ(ierr);
-  st->A                   = 0;
-  st->B                   = 0;
-  st->sigma               = 0.0;
-  st->sigma_set           = PETSC_FALSE;
-  st->defsigma            = 0.0;
-  st->data                = 0;
-  st->setupcalled         = 0;
-  st->w                   = 0;
-  st->D                   = 0;
-  st->wb                  = 0;
-  st->mat                 = 0;
-  st->shift_matrix        = ST_MATMODE_COPY;
-  st->str                 = DIFFERENT_NONZERO_PATTERN;
+#if !defined(PETSC_USE_DYNAMIC_LIBRARIES)
+  ierr = STInitializePackage();CHKERRQ(ierr);
+#endif
+
+  ierr = SlepcHeaderCreate(st,_p_ST,struct _STOps,ST_CLASSID,"ST","Spectral Transformation","ST",comm,STDestroy,STView);CHKERRQ(ierr);
+
+  st->A            = 0;
+  st->Astate       = 0;
+  st->T            = 0;
+  st->nmat         = 0;
+  st->sigma        = 0.0;
+  st->sigma_set    = PETSC_FALSE;
+  st->defsigma     = 0.0;
+  st->data         = 0;
+  st->setupcalled  = 0;
+  st->ksp          = 0;
+  st->kspidx       = -1;
+  st->w            = 0;
+  st->D            = 0;
+  st->wb           = 0;
+  st->shift_matrix = ST_MATMODE_COPY;
+  st->str          = DIFFERENT_NONZERO_PATTERN;
+
   *newst = st;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetOperators"
 /*@
-   STSetOperators - Sets the matrices associated with the eigenvalue problem. 
+   STSetOperators - Sets the matrices associated with the eigenvalue problem.
 
    Collective on ST and Mat
 
    Input Parameters:
 +  st - the spectral transformation context
-.  A  - the matrix associated with the eigensystem
--  B  - the second matrix in the case of generalized eigenproblems
+.  n  - number of matrices in array A
+-  A  - the array of matrices associated with the eigensystem
 
    Notes:
-   To specify a standard eigenproblem, use PETSC_NULL for B.
-
-   It must be called after STSetUp(). If it is called again after STSetUp() then
+   It must be called before STSetUp(). If it is called again after STSetUp() then
    the ST object is reset.
 
    Level: intermediate
 
-.seealso: STGetOperators(), STSetUp(), STReset()
+.seealso: STGetOperators(), STGetNumMatrices(), STSetUp(), STReset()
  @*/
-PetscErrorCode STSetOperators(ST st,Mat A,Mat B)
+PetscErrorCode STSetOperators(ST st,PetscInt n,Mat A[])
 {
+  PetscInt       i;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
-  PetscValidHeaderSpecific(A,MAT_CLASSID,2);
-  if (B) PetscValidHeaderSpecific(B,MAT_CLASSID,3);
-  PetscCheckSameComm(st,1,A,2);
-  if (B) PetscCheckSameComm(st,1,B,3);
+  PetscValidLogicalCollectiveInt(st,n,2);
+  if (n <= 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Must have one or more matrices, you have %D",n);
+  PetscValidPointer(A,3);
+  PetscCheckSameComm(st,1,*A,3);
   if (st->setupcalled) { ierr = STReset(st);CHKERRQ(ierr); }
-  ierr = MatDestroy(&st->A);CHKERRQ(ierr);
-  ierr = PetscObjectReference((PetscObject)A);CHKERRQ(ierr);
-  st->A = A;
-  ierr = MatDestroy(&st->B);CHKERRQ(ierr);
-  if (B) { ierr = PetscObjectReference((PetscObject)B);CHKERRQ(ierr); }
-  st->B = B;
+  ierr = MatDestroyMatrices(PetscMax(2,st->nmat),&st->A);CHKERRQ(ierr);
+  ierr = PetscMalloc(PetscMax(2,n)*sizeof(Mat),&st->A);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory(st,PetscMax(2,n)*sizeof(Mat));CHKERRQ(ierr);
+  ierr = PetscFree(st->Astate);CHKERRQ(ierr);
+  ierr = PetscMalloc(PetscMax(2,n)*sizeof(PetscInt),&st->Astate);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory(st,PetscMax(2,n)*sizeof(PetscInt));CHKERRQ(ierr);
+  for (i=0;i<n;i++) {
+    PetscValidHeaderSpecific(A[i],MAT_CLASSID,3);
+    ierr = PetscObjectReference((PetscObject)A[i]);CHKERRQ(ierr);
+    st->A[i] = A[i];
+    st->Astate[i] = ((PetscObject)A[i])->state;
+  }
+  if (n==1) {
+    st->A[1] = NULL;
+    st->Astate[1] = 0;
+  }
+  st->nmat = n;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STGetOperators"
-/*@C
-   STGetOperators - Gets the matrices associated with the eigensystem.
+/*@
+   STGetOperators - Gets the matrices associated with the original eigensystem.
 
    Not collective, though parallel Mats are returned if the ST is parallel
 
+   Input Parameter:
++  st - the spectral transformation context
+-  k  - the index of the requested matrix (starting in 0)
+
+   Output Parameters:
+.  A - the requested matrix
+
+   Level: intermediate
+
+.seealso: STSetOperators(), STGetNumMatrices()
+@*/
+PetscErrorCode STGetOperators(ST st,PetscInt k,Mat *A)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  PetscValidPointer(A,3);
+  if (k<0 || k>=st->nmat) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"k must be between 0 and %d",st->nmat-1);
+  if (((PetscObject)st->A[k])->state!=st->Astate[k]) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot retrieve original matrices (have been modified)");
+  *A = st->A[k];
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "STGetNumMatrices"
+/*@
+   STGetNumMatrices - Returns the number of matrices stored in the ST.
+
+   Not collective
+
    Input Parameter:
 .  st - the spectral transformation context
 
    Output Parameters:
-.  A - the matrix associated with the eigensystem
--  B - the second matrix in the case of generalized eigenproblems
+.  n - the number of matrices passed in STSetOperators()
 
    Level: intermediate
 
 .seealso: STSetOperators()
 @*/
-PetscErrorCode STGetOperators(ST st,Mat *A,Mat *B)
+PetscErrorCode STGetNumMatrices(ST st,PetscInt *n)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
-  if (A) *A = st->A;
-  if (B) *B = st->B;
+  PetscValidPointer(n,2);
+  *n = st->nmat;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetShift"
 /*@
    STSetShift - Sets the shift associated with the spectral transformation.
@@ -289,7 +333,7 @@ PetscErrorCode STGetOperators(ST st,Mat *A,Mat *B)
    Note:
    In some spectral transformations, changing the shift may have associated
    a lot of work, for example recomputing a factorization.
-   
+
    Level: beginner
 
 @*/
@@ -310,7 +354,7 @@ PetscErrorCode STSetShift(ST st,PetscScalar shift)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STGetShift"
 /*@
    STGetShift - Gets the shift associated with the spectral transformation.
@@ -335,7 +379,7 @@ PetscErrorCode STGetShift(ST st,PetscScalar* shift)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetDefaultShift"
 /*@
    STSetDefaultShift - Sets the value of the shift that should be employed if
@@ -359,7 +403,7 @@ PetscErrorCode STSetDefaultShift(ST st,PetscScalar defaultshift)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetBalanceMatrix"
 /*@
    STSetBalanceMatrix - Sets the diagonal matrix to be used for balancing.
@@ -375,7 +419,7 @@ PetscErrorCode STSetDefaultShift(ST st,PetscScalar defaultshift)
 
    Balancing is usually set via EPSSetBalance, but the advanced user may use
    this function to bypass the usual balancing methods.
-   
+
    Level: developer
 
 .seealso: EPSSetBalance(), STApply(), STGetBalanceMatrix()
@@ -395,7 +439,7 @@ PetscErrorCode STSetBalanceMatrix(ST st,Vec D)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STGetBalanceMatrix"
 /*@
    STGetBalanceMatrix - Gets the balance matrix used by the spectral transformation.
@@ -424,10 +468,10 @@ PetscErrorCode STGetBalanceMatrix(ST st,Vec *D)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetOptionsPrefix"
 /*@C
-   STSetOptionsPrefix - Sets the prefix used for searching for all 
+   STSetOptionsPrefix - Sets the prefix used for searching for all
    ST options in the database.
 
    Logically Collective on ST
@@ -458,10 +502,10 @@ PetscErrorCode STSetOptionsPrefix(ST st,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STAppendOptionsPrefix"
 /*@C
-   STAppendOptionsPrefix - Appends to the prefix used for searching for all 
+   STAppendOptionsPrefix - Appends to the prefix used for searching for all
    ST options in the database.
 
    Logically Collective on ST
@@ -492,10 +536,10 @@ PetscErrorCode STAppendOptionsPrefix(ST st,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STGetOptionsPrefix"
 /*@C
-   STGetOptionsPrefix - Gets the prefix used for searching for all 
+   STGetOptionsPrefix - Gets the prefix used for searching for all
    ST options in the database.
 
    Not Collective
@@ -524,7 +568,7 @@ PetscErrorCode STGetOptionsPrefix(ST st,const char *prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STView"
 /*@C
    STView - Prints the ST data structure.
@@ -540,8 +584,8 @@ PetscErrorCode STGetOptionsPrefix(ST st,const char *prefix[])
 +     PETSC_VIEWER_STDOUT_SELF - standard output (default)
 -     PETSC_VIEWER_STDOUT_WORLD - synchronized standard
          output where only the first processor opens
-         the file.  All other processors send their 
-         data to the first processor to print. 
+         the file.  All other processors send their
+         data to the first processor to print.
 
    The user can open an alternative visualization contexts with
    PetscViewerASCIIOpen() (output to a specified file).
@@ -552,20 +596,20 @@ PetscErrorCode STGetOptionsPrefix(ST st,const char *prefix[])
 @*/
 PetscErrorCode STView(ST st,PetscViewer viewer)
 {
-  PetscErrorCode    ierr;
-  const STType      cstr;
-  const char*       str;
-  PetscBool         isascii,isstring,flg;
-  PC                pc;
+  PetscErrorCode ierr;
+  STType         cstr;
+  const char*    pat;
+  char           str[50];
+  PetscBool      isascii,isstring,flg;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(((PetscObject)st)->comm);
-  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2); 
+  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)st));
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(st,1,viewer,2);
 
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERSTRING,&isstring);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERSTRING,&isstring);CHKERRQ(ierr);
   if (isascii) {
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)st,viewer,"ST Object");CHKERRQ(ierr);
     if (st->ops->view) {
@@ -573,11 +617,9 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
       ierr = (*st->ops->view)(st,viewer);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     }
-#if !defined(PETSC_USE_COMPLEX)
-    ierr = PetscViewerASCIIPrintf(viewer,"  shift: %G\n",st->sigma);CHKERRQ(ierr);
-#else
-    ierr = PetscViewerASCIIPrintf(viewer,"  shift: %G+%G i\n",PetscRealPart(st->sigma),PetscImaginaryPart(st->sigma));CHKERRQ(ierr);
-#endif
+    ierr = SlepcSNPrintfScalar(str,50,st->sigma,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  shift: %s\n",str);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  number of matrices: %D\n",st->nmat);CHKERRQ(ierr);
     switch (st->shift_matrix) {
     case ST_MATMODE_COPY:
       break;
@@ -588,31 +630,23 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
       ierr = PetscViewerASCIIPrintf(viewer,"  using a shell matrix\n");CHKERRQ(ierr);
       break;
     }
-    if (st->B && st->shift_matrix != ST_MATMODE_SHELL) { 
+    if (st->nmat>1 && st->shift_matrix != ST_MATMODE_SHELL) {
       switch (st->str) {
-        case SAME_NONZERO_PATTERN:      str = "same nonzero pattern";break;
-        case DIFFERENT_NONZERO_PATTERN: str = "different nonzero pattern";break;
-        case SUBSET_NONZERO_PATTERN:    str = "subset nonzero pattern";break;
-        default:                        SETERRQ(((PetscObject)st)->comm,1,"Wrong structure flag");
+        case SAME_NONZERO_PATTERN:      pat = "same nonzero pattern";break;
+        case DIFFERENT_NONZERO_PATTERN: pat = "different nonzero pattern";break;
+        case SUBSET_NONZERO_PATTERN:    pat = "subset nonzero pattern";break;
+        default: SETERRQ(PetscObjectComm((PetscObject)st),1,"Wrong structure flag");
       }
-      ierr = PetscViewerASCIIPrintf(viewer,"  matrices A and B have %s\n",str);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  all matrices have %s\n",pat);CHKERRQ(ierr);
     }
   } else if (isstring) {
     ierr = STGetType(st,&cstr);CHKERRQ(ierr);
     ierr = PetscViewerStringSPrintf(viewer," %-7.7s",cstr);CHKERRQ(ierr);
-    if (st->ops->view) {ierr = (*st->ops->view)(st,viewer);CHKERRQ(ierr);}
-  } else {
-    SETERRQ1(((PetscObject)st)->comm,1,"Viewer type %s not supported by ST",((PetscObject)viewer)->type_name);
+    if (st->ops->view) { ierr = (*st->ops->view)(st,viewer);CHKERRQ(ierr); }
   }
-  ierr = PetscTypeCompareAny((PetscObject)st,&flg,STSHIFT,STFOLD,"");CHKERRQ(ierr);
-  if (st->B || !flg) {
+  ierr = PetscObjectTypeCompareAny((PetscObject)st,&flg,STSHIFT,STFOLD,"");CHKERRQ(ierr);
+  if (st->nmat>1 || !flg) {
     if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
-    /* Trick for PCView when an unused PC is showed */
-    ierr = KSPGetPC(st->ksp,&pc);CHKERRQ(ierr);
-    ierr = PetscTypeCompare((PetscObject)pc,PCNONE,&flg);
-    if (flg) {
-      ierr = PCSetOperators(pc,PETSC_NULL,PETSC_NULL,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
-    }
     ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
     ierr = KSPView(st->ksp,viewer);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
@@ -620,42 +654,41 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STRegister"
 /*@C
-   STRegister - See STRegisterDynamic()
+   STRegister - Adds a method to the spectral transformation package.
 
-   Level: advanced
-@*/
-PetscErrorCode STRegister(const char *sname,const char *path,const char *name,PetscErrorCode (*function)(ST))
-{
-  PetscErrorCode ierr;
-  char           fullname[PETSC_MAX_PATH_LEN];
+   Not collective
 
-  PetscFunctionBegin;
-  ierr = PetscFListConcat(path,name,fullname);CHKERRQ(ierr);
-  ierr = PetscFListAdd(&STList,sname,fullname,(void (*)(void))function);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
+   Input Parameters:
++  name - name of a new user-defined transformation
+-  function - routine to create method context
 
-#undef __FUNCT__  
-#define __FUNCT__ "STRegisterDestroy"
-/*@
-   STRegisterDestroy - Frees the list of ST methods that were
-   registered by STRegisterDynamic().
+   Notes:
+   STRegister() may be called multiple times to add several user-defined
+   spectral transformations.
 
-   Not Collective
+   Sample usage:
+.vb
+   STRegister("my_solver",MySolverCreate);
+.ve
+
+   Then, your solver can be chosen with the procedural interface via
+$     STSetType(st,"my_solver")
+   or at runtime via the option
+$     -st_type my_solver
 
    Level: advanced
 
-.seealso: STRegisterDynamic(), STRegisterAll()
+.seealso: STRegisterAll()
 @*/
-PetscErrorCode STRegisterDestroy(void)
+PetscErrorCode STRegister(const char *name,PetscErrorCode (*function)(ST))
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscFListDestroy(&STList);CHKERRQ(ierr);
-  STRegisterAllCalled = PETSC_FALSE;
+  ierr = PetscFunctionListAdd(&STList,name,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
