@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,12 +21,12 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/stimpl.h>      /*I "slepcst.h" I*/
+#include <slepc-private/stimpl.h>      /*I "slepcst.h" I*/
 
-PetscBool  STRegisterAllCalled = PETSC_FALSE;
-PetscFList STList = 0;
+PetscBool         STRegisterAllCalled = PETSC_FALSE;
+PetscFunctionList STList = 0;
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetType"
 /*@C
    STSetType - Builds ST for a particular spectral transformation.
@@ -43,19 +43,19 @@ PetscFList STList = 0;
    Use -help for a list of available transformations
 
    Notes:
-   See "slepc/include/slepcst.h" for available transformations 
+   See "slepc/include/slepcst.h" for available transformations
 
    Normally, it is best to use the EPSSetFromOptions() command and
    then set the ST type from the options database rather than by using
    this routine.  Using the options database provides the user with
-   maximum flexibility in evaluating the many different transformations. 
+   maximum flexibility in evaluating the many different transformations.
 
    Level: intermediate
 
 .seealso: EPSSetType()
 
 @*/
-PetscErrorCode STSetType(ST st,const STType type)
+PetscErrorCode STSetType(ST st,STType type)
 {
   PetscErrorCode ierr,(*r)(ST);
   PetscBool      match;
@@ -64,11 +64,11 @@ PetscErrorCode STSetType(ST st,const STType type)
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
   PetscValidCharPointer(type,2);
 
-  ierr = PetscTypeCompare((PetscObject)st,type,&match);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)st,type,&match);CHKERRQ(ierr);
   if (match) PetscFunctionReturn(0);
 
-  ierr =  PetscFListFind(STList,((PetscObject)st)->comm,type,PETSC_TRUE,(void (**)(void))&r);CHKERRQ(ierr);
-  if (!r) SETERRQ1(((PetscObject)st)->comm,PETSC_ERR_ARG_UNKNOWN_TYPE,"Unable to find requested ST type %s",type);
+  ierr =  PetscFunctionListFind(STList,type,&r);CHKERRQ(ierr);
+  if (!r) SETERRQ1(PetscObjectComm((PetscObject)st),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unable to find requested ST type %s",type);
 
   if (st->ops->destroy) { ierr = (*st->ops->destroy)(st);CHKERRQ(ierr); }
   ierr = PetscMemzero(st->ops,sizeof(struct _STOps));CHKERRQ(ierr);
@@ -79,7 +79,7 @@ PetscErrorCode STSetType(ST st,const STType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STGetType"
 /*@C
    STGetType - Gets the ST type name (as a string) from the ST context.
@@ -90,14 +90,14 @@ PetscErrorCode STSetType(ST st,const STType type)
 .  st - the spectral transformation context
 
    Output Parameter:
-.  name - name of the spectral transformation 
+.  name - name of the spectral transformation
 
    Level: intermediate
 
 .seealso: STSetType()
 
 @*/
-PetscErrorCode STGetType(ST st,const STType *type)
+PetscErrorCode STGetType(ST st,STType *type)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
@@ -106,7 +106,7 @@ PetscErrorCode STGetType(ST st,const STType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetFromOptions"
 /*@
    STSetFromOptions - Sets ST options from the options database.
@@ -132,8 +132,8 @@ PetscErrorCode STSetFromOptions(ST st)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
-  if (!STRegisterAllCalled) { ierr = STRegisterAll(PETSC_NULL);CHKERRQ(ierr); }
-  ierr = PetscOptionsBegin(((PetscObject)st)->comm,((PetscObject)st)->prefix,"Spectral Transformation (ST) Options","ST");CHKERRQ(ierr);
+  if (!STRegisterAllCalled) { ierr = STRegisterAll();CHKERRQ(ierr); }
+  ierr = PetscObjectOptionsBegin((PetscObject)st);CHKERRQ(ierr);
     ierr = PetscOptionsList("-st_type","Spectral Transformation type","STSetType",STList,(char*)(((PetscObject)st)->type_name?((PetscObject)st)->type_name:STSHIFT),type,256,&flg);CHKERRQ(ierr);
     if (flg) {
       ierr = STSetType(st,type);CHKERRQ(ierr);
@@ -146,33 +146,36 @@ PetscErrorCode STSetFromOptions(ST st)
     }
 
     ierr = PetscOptionsScalar("-st_shift","Value of the shift","STSetShift",st->sigma,&s,&flg);CHKERRQ(ierr);
-    if (flg) { ierr = STSetShift(st,s);CHKERRQ(ierr); }
+    if (flg) {
+      ierr = STSetShift(st,s);CHKERRQ(ierr);
+    }
 
     ierr = PetscOptionsEList("-st_matmode","Shift matrix mode","STSetMatMode",mode_list,3,mode_list[st->shift_matrix],&i,&flg);CHKERRQ(ierr);
-    if (flg) { st->shift_matrix = (STMatMode)i; }
+    if (flg) st->shift_matrix = (STMatMode)i;
 
     ierr = PetscOptionsEList("-st_matstructure","Shift nonzero pattern","STSetMatStructure",structure_list,3,structure_list[st->str],&i,&flg);CHKERRQ(ierr);
-    if (flg) { 
+    if (flg) {
       switch (i) {
         case 0: ierr = STSetMatStructure(st,SAME_NONZERO_PATTERN);CHKERRQ(ierr); break;
         case 1: ierr = STSetMatStructure(st,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr); break;
         case 2: ierr = STSetMatStructure(st,SUBSET_NONZERO_PATTERN);CHKERRQ(ierr); break;
       }
     }
-    
+
     if (st->ops->setfromoptions) {
       ierr = (*st->ops->setfromoptions)(st);CHKERRQ(ierr);
     }
     ierr = PetscObjectProcessOptionsHandlers((PetscObject)st);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
-  ierr = KSPSetFromOptions(st->ksp);CHKERRQ(ierr); 
+  if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
+  ierr = KSPSetFromOptions(st->ksp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetMatStructure"
 /*@
-   STSetMatStructure - Sets an internal MatStructure attribute to 
+   STSetMatStructure - Sets an internal MatStructure attribute to
    indicate which is the relation of the sparsity pattern of the two matrices
    A and B constituting the generalized eigenvalue problem.
 
@@ -185,8 +188,8 @@ PetscErrorCode STSetFromOptions(ST st)
 
    Options Database Key:
 .  -st_matstructure <str> - Indicates the structure flag, where <str> is one
-         of 'same' (A and B have the same nonzero pattern), 'different' (A 
-         and B have different nonzero pattern) or 'subset' (B's nonzero 
+         of 'same' (A and B have the same nonzero pattern), 'different' (A
+         and B have different nonzero pattern) or 'subset' (B's nonzero
          pattern is a subset of A's).
 
    Notes:
@@ -195,7 +198,7 @@ PetscErrorCode STSetFromOptions(ST st)
    for efficiency reasons (in particular, for internal MatAXPY() operations).
 
    This function has no effect in the case of standard eigenproblems.
-   
+
    Level: advanced
 
 .seealso: STSetOperators(), MatAXPY()
@@ -212,15 +215,15 @@ PetscErrorCode STSetMatStructure(ST st,MatStructure str)
       st->str = str;
       break;
     default:
-      SETERRQ(((PetscObject)st)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid matrix structure flag");
+      SETERRQ(PetscObjectComm((PetscObject)st),PETSC_ERR_ARG_OUTOFRANGE,"Invalid matrix structure flag");
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STGetMatStructure"
 /*@
-   STGetMatStructure - Gets the internal MatStructure attribute to 
+   STGetMatStructure - Gets the internal MatStructure attribute to
    indicate which is the relation of the sparsity pattern of the two matrices
    A and B constituting the generalized eigenvalue problem.
 
@@ -249,7 +252,7 @@ PetscErrorCode STGetMatStructure(ST st,MatStructure *str)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STSetMatMode"
 /*@
    STSetMatMode - Sets a flag to indicate how the matrix is
@@ -259,7 +262,7 @@ PetscErrorCode STGetMatStructure(ST st,MatStructure *str)
 
    Input Parameters:
 +  st - the spectral transformation context
--  mode - the mode flag, one of ST_MATMODE_COPY, 
+-  mode - the mode flag, one of ST_MATMODE_COPY,
           ST_MATMODE_INPLACE or ST_MATMODE_SHELL
 
    Options Database Key:
@@ -267,23 +270,23 @@ PetscErrorCode STGetMatStructure(ST st,MatStructure *str)
           'copy', 'inplace' or 'shell' (see explanation below).
 
    Notes:
-   By default (ST_MATMODE_COPY), a copy of matrix A is made and then 
-   this copy is shifted explicitly, e.g. A <- (A - s B). 
+   By default (ST_MATMODE_COPY), a copy of matrix A is made and then
+   this copy is shifted explicitly, e.g. A <- (A - s B).
 
-   With ST_MATMODE_INPLACE, the original matrix A is shifted at 
+   With ST_MATMODE_INPLACE, the original matrix A is shifted at
    STSetUp() and unshifted at the end of the computations. With respect to
    the previous one, this mode avoids a copy of matrix A. However, a
-   backdraw is that the recovered matrix might be slightly different 
+   backdraw is that the recovered matrix might be slightly different
    from the original one (due to roundoff).
 
-   With ST_MATMODE_SHELL, the solver works with an implicit shell 
-   matrix that represents the shifted matrix. This mode is the most efficient 
-   in creating the shifted matrix but it places serious limitations to the 
+   With ST_MATMODE_SHELL, the solver works with an implicit shell
+   matrix that represents the shifted matrix. This mode is the most efficient
+   in creating the shifted matrix but it places serious limitations to the
    linear solves performed in each iteration of the eigensolver (typically,
    only interative solvers with Jacobi preconditioning can be used).
-   
+
    In the case of generalized problems, in the two first modes the matrix
-   A - s B has to be computed explicitly. The efficiency of this computation 
+   A - s B has to be computed explicitly. The efficiency of this computation
    can be controlled with STSetMatStructure().
 
    Level: intermediate
@@ -296,13 +299,14 @@ PetscErrorCode STSetMatMode(ST st,STMatMode mode)
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
   PetscValidLogicalCollectiveEnum(st,mode,2);
   st->shift_matrix = mode;
+  st->setupcalled = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "STGetMatMode"
 /*@C
-   STGetMatMode - Gets a flag that indicates how the matrix is being 
+   STGetMatMode - Gets a flag that indicates how the matrix is being
    shifted in the shift-and-invert and Cayley spectral transformations.
 
    Not Collective
