@@ -1,24 +1,39 @@
-/*                       
+/*
 
    SLEPc singular value solver: "trlanczos"
 
-   Method: Golub-Kahan-Lanczos bidiagonalization with thick-restart
+   Method: Thick-restart Lanczos
+
+   Algorithm:
+
+       Golub-Kahan-Lanczos bidiagonalization with thick-restart.
+
+   References:
+
+       [1] G.H. Golub and W. Kahan, "Calculating the singular values
+           and pseudo-inverse of a matrix", SIAM J. Numer. Anal. Ser.
+           B 2:205-224, 1965.
+
+       [2] V. Hernandez, J.E. Roman, and A. Tomas, "A robust and
+           efficient parallel SVD solver based on restarted Lanczos
+           bidiagonalization", Elec. Trans. Numer. Anal. 31:68-85,
+           2008.
 
    Last update: Jun 2007
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -26,15 +41,15 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/svdimpl.h>                /*I "slepcsvd.h" I*/
-#include <private/ipimpl.h>
+#include <slepc-private/svdimpl.h>                /*I "slepcsvd.h" I*/
+#include <slepc-private/ipimpl.h>
 #include <slepcblaslapack.h>
 
 typedef struct {
   PetscBool oneside;
 } SVD_TRLANCZOS;
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDSetUp_TRLanczos"
 PetscErrorCode SVDSetUp_TRLanczos(SVD svd)
 {
@@ -42,29 +57,34 @@ PetscErrorCode SVDSetUp_TRLanczos(SVD svd)
   PetscInt       N;
 
   PetscFunctionBegin;
-  ierr = SVDMatGetSize(svd,PETSC_NULL,&N);CHKERRQ(ierr);
+  ierr = SVDMatGetSize(svd,NULL,&N);CHKERRQ(ierr);
   if (svd->ncv) { /* ncv set */
-    if (svd->ncv<svd->nsv) SETERRQ(((PetscObject)svd)->comm,1,"The value of ncv must be at least nsv"); 
-  }
-  else if (svd->mpd) { /* mpd set */
+    if (svd->ncv<svd->nsv) SETERRQ(PetscObjectComm((PetscObject)svd),1,"The value of ncv must be at least nsv");
+  } else if (svd->mpd) { /* mpd set */
     svd->ncv = PetscMin(N,svd->nsv+svd->mpd);
-  }
-  else { /* neither set: defaults depend on nsv being small or large */
+  } else { /* neither set: defaults depend on nsv being small or large */
     if (svd->nsv<500) svd->ncv = PetscMin(N,PetscMax(2*svd->nsv,10));
-    else { svd->mpd = 500; svd->ncv = PetscMin(N,svd->nsv+svd->mpd); }
+    else {
+      svd->mpd = 500;
+      svd->ncv = PetscMin(N,svd->nsv+svd->mpd);
+    }
   }
   if (!svd->mpd) svd->mpd = svd->ncv;
-  if (svd->ncv>svd->nsv+svd->mpd) SETERRQ(((PetscObject)svd)->comm,1,"The value of ncv must not be larger than nev+mpd"); 
+  if (svd->ncv>svd->nsv+svd->mpd) SETERRQ(PetscObjectComm((PetscObject)svd),1,"The value of ncv must not be larger than nev+mpd");
   if (!svd->max_it) svd->max_it = PetscMax(N/svd->ncv,100);
-  if (svd->ncv!=svd->n) {  
+  if (svd->ncv!=svd->n) {
     ierr = VecDuplicateVecs(svd->tl,svd->ncv,&svd->U);CHKERRQ(ierr);
+    ierr = PetscLogObjectParents(svd,svd->ncv,svd->U);CHKERRQ(ierr);
   }
+  ierr = DSSetType(svd->ds,DSSVD);CHKERRQ(ierr);
+  ierr = DSSetCompact(svd->ds,PETSC_TRUE);CHKERRQ(ierr);
+  ierr = DSAllocate(svd->ds,svd->ncv);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDOneSideTRLanczosMGS"
-static PetscErrorCode SVDOneSideTRLanczosMGS(SVD svd,PetscReal *alpha,PetscReal *beta,PetscScalar* bb,Vec *V,Vec v,Vec* U,PetscInt nconv,PetscInt l,PetscInt n,PetscScalar* work)
+static PetscErrorCode SVDOneSideTRLanczosMGS(SVD svd,PetscReal *alpha,PetscReal *beta,Vec *V,Vec v,Vec* U,PetscInt nconv,PetscInt l,PetscInt n,PetscScalar* work)
 {
   PetscErrorCode ierr;
   PetscReal      a,b;
@@ -73,66 +93,70 @@ static PetscErrorCode SVDOneSideTRLanczosMGS(SVD svd,PetscReal *alpha,PetscReal
   PetscFunctionBegin;
   ierr = SVDMatMult(svd,PETSC_FALSE,V[k],U[k]);CHKERRQ(ierr);
   if (l>0) {
-    ierr = SlepcVecMAXPBY(U[k],1.0,-1.0,l,bb,U+nconv);CHKERRQ(ierr);
+    for (i=0;i<l;i++) work[i]=beta[i+nconv];
+    ierr = SlepcVecMAXPBY(U[k],1.0,-1.0,l,work,U+nconv);CHKERRQ(ierr);
   }
   ierr = IPNorm(svd->ip,U[k],&a);CHKERRQ(ierr);
   ierr = VecScale(U[k],1.0/a);CHKERRQ(ierr);
-  alpha[0] = a;
+  alpha[k] = a;
   for (i=k+1;i<n;i++) {
     ierr = SVDMatMult(svd,PETSC_TRUE,U[i-1],V[i]);CHKERRQ(ierr);
-    ierr = IPOrthogonalize(svd->ip,0,PETSC_NULL,i,PETSC_NULL,V,V[i],work,&b,PETSC_NULL);CHKERRQ(ierr);  
+    ierr = IPOrthogonalize(svd->ip,0,NULL,i,NULL,V,V[i],work,&b,NULL);CHKERRQ(ierr);
     ierr = VecScale(V[i],1.0/b);CHKERRQ(ierr);
-    beta[i-k-1] = b;
-    
+    beta[i-1] = b;
+
     ierr = SVDMatMult(svd,PETSC_FALSE,V[i],U[i]);CHKERRQ(ierr);
     ierr = VecAXPY(U[i],-b,U[i-1]);CHKERRQ(ierr);
     ierr = IPNorm(svd->ip,U[i],&a);CHKERRQ(ierr);
     ierr = VecScale(U[i],1.0/a);CHKERRQ(ierr);
-    alpha[i-k] = a;
+    alpha[i] = a;
   }
   ierr = SVDMatMult(svd,PETSC_TRUE,U[n-1],v);CHKERRQ(ierr);
-  ierr = IPOrthogonalize(svd->ip,0,PETSC_NULL,n,PETSC_NULL,V,v,work,&b,PETSC_NULL);CHKERRQ(ierr);      
-  beta[n-k-1] = b;
+  ierr = IPOrthogonalize(svd->ip,0,NULL,n,NULL,V,v,work,&b,NULL);CHKERRQ(ierr);
+  beta[n-1] = b;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDOneSideTRLanczosCGS"
-static PetscErrorCode SVDOneSideTRLanczosCGS(SVD svd,PetscReal *alpha,PetscReal *beta,PetscScalar* bb,Vec *V,Vec v,Vec* U,PetscInt nconv,PetscInt l,PetscInt n,PetscScalar* work)
+static PetscErrorCode SVDOneSideTRLanczosCGS(SVD svd,PetscReal *alpha,PetscReal *beta,Vec *V,Vec v,Vec* U,PetscInt nconv,PetscInt l,PetscInt n,PetscScalar* work)
 {
-  PetscErrorCode ierr;
-  PetscReal      a,b,sum,onorm;
-  PetscScalar    dot;
-  PetscInt       i,j,k=nconv+l;
+  PetscErrorCode     ierr;
+  PetscReal          a,b,sum,onorm,eta;
+  PetscScalar        dot;
+  PetscInt           i,j,k=nconv+l;
+  IPOrthogRefineType refine;
 
   PetscFunctionBegin;
   ierr = SVDMatMult(svd,PETSC_FALSE,V[k],U[k]);CHKERRQ(ierr);
   if (l>0) {
-    ierr = SlepcVecMAXPBY(U[k],1.0,-1.0,l,bb,U+nconv);CHKERRQ(ierr);
+    for (i=0;i<l;i++) work[i]=beta[i+nconv];
+    ierr = SlepcVecMAXPBY(U[k],1.0,-1.0,l,work,U+nconv);CHKERRQ(ierr);
   }
+  ierr = IPGetOrthogonalization(svd->ip,NULL,&refine,&eta);CHKERRQ(ierr);
   for (i=k+1;i<n;i++) {
     ierr = SVDMatMult(svd,PETSC_TRUE,U[i-1],V[i]);CHKERRQ(ierr);
     ierr = IPNormBegin(svd->ip,U[i-1],&a);CHKERRQ(ierr);
-    if (svd->ip->orthog_ref == IP_ORTHOG_REFINE_IFNEEDED) {
+    if (refine == IP_ORTHOG_REFINE_IFNEEDED) {
       ierr = IPInnerProductBegin(svd->ip,V[i],V[i],&dot);CHKERRQ(ierr);
     }
     ierr = IPMInnerProductBegin(svd->ip,V[i],i,V,work);CHKERRQ(ierr);
     ierr = IPNormEnd(svd->ip,U[i-1],&a);CHKERRQ(ierr);
-    if (svd->ip->orthog_ref == IP_ORTHOG_REFINE_IFNEEDED) {
+    if (refine == IP_ORTHOG_REFINE_IFNEEDED) {
       ierr = IPInnerProductEnd(svd->ip,V[i],V[i],&dot);CHKERRQ(ierr);
     }
     ierr = IPMInnerProductEnd(svd->ip,V[i],i,V,work);CHKERRQ(ierr);
-    
+
     ierr = VecScale(U[i-1],1.0/a);CHKERRQ(ierr);
     for (j=0;j<i;j++) work[j] = work[j] / a;
     ierr = SlepcVecMAXPBY(V[i],1.0/a,-1.0,i,work,V);CHKERRQ(ierr);
 
-    switch (svd->ip->orthog_ref) {
+    switch (refine) {
     case IP_ORTHOG_REFINE_NEVER:
       ierr = IPNorm(svd->ip,V[i],&b);CHKERRQ(ierr);
-      break;      
+      break;
     case IP_ORTHOG_REFINE_ALWAYS:
-      ierr = IPOrthogonalizeCGS1(svd->ip,0,PETSC_NULL,i,PETSC_NULL,V,V[i],work,PETSC_NULL,&b);CHKERRQ(ierr);
+      ierr = IPOrthogonalizeCGS1(svd->ip,0,NULL,i,NULL,V,V[i],work,NULL,&b);CHKERRQ(ierr);
       break;
     case IP_ORTHOG_REFINE_IFNEEDED:
       onorm = PetscSqrtReal(PetscRealPart(dot)) / a;
@@ -145,42 +169,42 @@ static PetscErrorCode SVDOneSideTRLanczosCGS(SVD svd,PetscReal *alpha,PetscReal
       else {
         ierr = IPNorm(svd->ip,V[i],&b);CHKERRQ(ierr);
       }
-      if (b < svd->ip->orthog_eta * onorm) {
-        ierr = IPOrthogonalizeCGS1(svd->ip,0,PETSC_NULL,i,PETSC_NULL,V,V[i],work,PETSC_NULL,&b);CHKERRQ(ierr);
+      if (b < eta*onorm) {
+        ierr = IPOrthogonalizeCGS1(svd->ip,0,NULL,i,NULL,V,V[i],work,NULL,&b);CHKERRQ(ierr);
       }
       break;
     }
-    
+
     ierr = VecScale(V[i],1.0/b);CHKERRQ(ierr);
-  
+
     ierr = SVDMatMult(svd,PETSC_FALSE,V[i],U[i]);CHKERRQ(ierr);
     ierr = VecAXPY(U[i],-b,U[i-1]);CHKERRQ(ierr);
 
-    alpha[i-k-1] = a;
-    beta[i-k-1] = b;
+    alpha[i-1] = a;
+    beta[i-1] = b;
   }
   ierr = SVDMatMult(svd,PETSC_TRUE,U[n-1],v);CHKERRQ(ierr);
   ierr = IPNormBegin(svd->ip,U[n-1],&a);CHKERRQ(ierr);
-  if (svd->ip->orthog_ref == IP_ORTHOG_REFINE_IFNEEDED) {
+  if (refine == IP_ORTHOG_REFINE_IFNEEDED) {
     ierr = IPInnerProductBegin(svd->ip,v,v,&dot);CHKERRQ(ierr);
   }
   ierr = IPMInnerProductBegin(svd->ip,v,n,V,work);CHKERRQ(ierr);
   ierr = IPNormEnd(svd->ip,U[n-1],&a);CHKERRQ(ierr);
-  if (svd->ip->orthog_ref == IP_ORTHOG_REFINE_IFNEEDED) {
+  if (refine == IP_ORTHOG_REFINE_IFNEEDED) {
     ierr = IPInnerProductEnd(svd->ip,v,v,&dot);CHKERRQ(ierr);
   }
   ierr = IPMInnerProductEnd(svd->ip,v,n,V,work);CHKERRQ(ierr);
-    
+
   ierr = VecScale(U[n-1],1.0/a);CHKERRQ(ierr);
   for (j=0;j<n;j++) work[j] = work[j] / a;
   ierr = SlepcVecMAXPBY(v,1.0/a,-1.0,n,work,V);CHKERRQ(ierr);
 
-  switch (svd->ip->orthog_ref) {
+  switch (refine) {
   case IP_ORTHOG_REFINE_NEVER:
     ierr = IPNorm(svd->ip,v,&b);CHKERRQ(ierr);
-    break;      
+    break;
   case IP_ORTHOG_REFINE_ALWAYS:
-    ierr = IPOrthogonalizeCGS1(svd->ip,0,PETSC_NULL,n,PETSC_NULL,V,v,work,PETSC_NULL,&b);CHKERRQ(ierr);
+    ierr = IPOrthogonalizeCGS1(svd->ip,0,NULL,n,NULL,V,v,work,NULL,&b);CHKERRQ(ierr);
     break;
   case IP_ORTHOG_REFINE_IFNEEDED:
     onorm = PetscSqrtReal(PetscRealPart(dot)) / a;
@@ -193,174 +217,138 @@ static PetscErrorCode SVDOneSideTRLanczosCGS(SVD svd,PetscReal *alpha,PetscReal
     else {
       ierr = IPNorm(svd->ip,v,&b);CHKERRQ(ierr);
     }
-    if (b < svd->ip->orthog_eta * onorm) {
-      ierr = IPOrthogonalizeCGS1(svd->ip,0,PETSC_NULL,n,PETSC_NULL,V,v,work,PETSC_NULL,&b);CHKERRQ(ierr);
+    if (b < eta*onorm) {
+      ierr = IPOrthogonalizeCGS1(svd->ip,0,NULL,n,NULL,V,v,work,NULL,&b);CHKERRQ(ierr);
     }
     break;
   }
-      
-  alpha[n-k-1] = a;
-  beta[n-k-1] = b;
+
+  alpha[n-1] = a;
+  beta[n-1] = b;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDSolve_TRLanczos"
 PetscErrorCode SVDSolve_TRLanczos(SVD svd)
 {
   PetscErrorCode ierr;
-  SVD_TRLANCZOS  *lanczos = (SVD_TRLANCZOS *)svd->data;
-  PetscReal      *alpha,*beta,norm;
-  PetscScalar    *b,*Q,*PT,*swork;
-  PetscInt       i,j,k,l,m,n,nv;
+  SVD_TRLANCZOS  *lanczos = (SVD_TRLANCZOS*)svd->data;
+  PetscReal      *alpha,*beta,lastbeta,norm;
+  PetscScalar    *Q,*PT,*swork,*w;
+  PetscInt       i,k,l,nv,ld,off;
   Vec            v;
   PetscBool      conv;
   IPOrthogType   orthog;
-  
+
   PetscFunctionBegin;
   /* allocate working space */
-  ierr = PetscMalloc(sizeof(PetscReal)*svd->n,&alpha);CHKERRQ(ierr);
-  ierr = PetscMalloc(sizeof(PetscReal)*svd->n,&beta);CHKERRQ(ierr);
-  ierr = PetscMalloc(sizeof(PetscScalar)*svd->n,&b);CHKERRQ(ierr);
-  ierr = PetscMalloc(sizeof(PetscScalar)*svd->n*svd->n,&Q);CHKERRQ(ierr);
-  ierr = PetscMalloc(sizeof(PetscScalar)*svd->n*svd->n,&PT);CHKERRQ(ierr);
-#if !defined(PETSC_USE_COMPLEX)
-  if (svd->which == SVD_SMALLEST) { 
-#endif
-    ierr = PetscMalloc(sizeof(PetscScalar)*svd->n*svd->n,&swork);CHKERRQ(ierr);
-#if !defined(PETSC_USE_COMPLEX)
-  } else {
-    ierr = PetscMalloc(sizeof(PetscScalar)*svd->n,&swork);CHKERRQ(ierr);
-  }
-#endif
+  ierr = DSGetLeadingDimension(svd->ds,&ld);CHKERRQ(ierr);
+  ierr = PetscMalloc(sizeof(PetscScalar)*ld,&w);CHKERRQ(ierr);
+  ierr = PetscMalloc(sizeof(PetscScalar)*svd->n,&swork);CHKERRQ(ierr);
   ierr = VecDuplicate(svd->V[0],&v);CHKERRQ(ierr);
-  ierr = IPGetOrthogonalization(svd->ip,&orthog,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
-  
+  ierr = IPGetOrthogonalization(svd->ip,&orthog,NULL,NULL);CHKERRQ(ierr);
+
   /* normalize start vector */
-  if (svd->nini==0) {
+  if (!svd->nini) {
     ierr = SlepcVecSetRandom(svd->V[0],svd->rand);CHKERRQ(ierr);
   }
   ierr = VecNormalize(svd->V[0],&norm);CHKERRQ(ierr);
-  
+
   l = 0;
   while (svd->reason == SVD_CONVERGED_ITERATING) {
     svd->its++;
 
     /* inner loop */
     nv = PetscMin(svd->nconv+svd->mpd,svd->n);
+    ierr = DSGetArrayReal(svd->ds,DS_MAT_T,&alpha);CHKERRQ(ierr);
+    beta = alpha + ld;
     if (lanczos->oneside) {
       if (orthog == IP_ORTHOG_MGS) {
-        ierr = SVDOneSideTRLanczosMGS(svd,alpha,beta,b+svd->nconv,svd->V,v,svd->U,svd->nconv,l,nv,swork);CHKERRQ(ierr);
+        ierr = SVDOneSideTRLanczosMGS(svd,alpha,beta,svd->V,v,svd->U,svd->nconv,l,nv,swork);CHKERRQ(ierr);
       } else {
-        ierr = SVDOneSideTRLanczosCGS(svd,alpha,beta,b+svd->nconv,svd->V,v,svd->U,svd->nconv,l,nv,swork);CHKERRQ(ierr);
+        ierr = SVDOneSideTRLanczosCGS(svd,alpha,beta,svd->V,v,svd->U,svd->nconv,l,nv,swork);CHKERRQ(ierr);
       }
     } else {
       ierr = SVDTwoSideLanczos(svd,alpha,beta,svd->V,v,svd->U,svd->nconv+l,nv,swork);CHKERRQ(ierr);
     }
-    ierr = VecScale(v,1.0/beta[nv-svd->nconv-l-1]);CHKERRQ(ierr);
-   
+    lastbeta = beta[nv-1];
+    ierr = DSRestoreArrayReal(svd->ds,DS_MAT_T,&alpha);CHKERRQ(ierr);
+    ierr = VecScale(v,1.0/lastbeta);CHKERRQ(ierr);
+
     /* compute SVD of general matrix */
-    n = nv - svd->nconv;
-    /* first l columns */
-    for (j=0;j<l;j++) {
-      for (i=0;i<j;i++) Q[j*n+i] = 0.0;    
-      Q[j*n+j] = svd->sigma[svd->nconv+j];
-      for (i=j+1;i<n;i++) Q[j*n+i] = 0.0;
+    ierr = DSSetDimensions(svd->ds,nv,nv,svd->nconv,svd->nconv+l);CHKERRQ(ierr);
+    if (l==0) {
+      ierr = DSSetState(svd->ds,DS_STATE_INTERMEDIATE);CHKERRQ(ierr);
+    } else {
+      ierr = DSSetState(svd->ds,DS_STATE_RAW);CHKERRQ(ierr);
     }
-    /* l+1 column */
-    for (i=0;i<l;i++) Q[l*n+i] = b[i+svd->nconv];
-    Q[l*n+l] = alpha[0];
-    for (i=l+1;i<n;i++) Q[l*n+i] = 0.0;
-    /* rest of matrix */
-    for (j=l+1;j<n;j++) {
-      for (i=0;i<j-1;i++) Q[j*n+i] = 0.0;
-      Q[j*n+j-1] = beta[j-l-1];
-      Q[j*n+j] = alpha[j-l];
-      for (i=j+1;i<n;i++) Q[j*n+i] = 0.0;
-    }
-    ierr = SVDDense(n,n,Q,alpha,PETSC_NULL,PT);CHKERRQ(ierr);
+    ierr = DSSolve(svd->ds,w,NULL);CHKERRQ(ierr);
+    ierr = DSSort(svd->ds,w,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
 
     /* compute error estimates */
     k = 0;
     conv = PETSC_TRUE;
+    ierr = DSGetArray(svd->ds,DS_MAT_U,&Q);CHKERRQ(ierr);
+    ierr = DSGetArrayReal(svd->ds,DS_MAT_T,&alpha);CHKERRQ(ierr);
+    beta = alpha + ld;
     for (i=svd->nconv;i<nv;i++) {
-      if (svd->which == SVD_SMALLEST) j = n-i+svd->nconv-1;
-      else j = i-svd->nconv;
-      svd->sigma[i] = alpha[j];
-      b[i] = Q[j*n+n-1]*beta[n-l-1];
-      svd->errest[i] = PetscAbsScalar(b[i]);
-      if (alpha[j] > svd->tol) svd->errest[i] /= alpha[j];
+      svd->sigma[i] = PetscRealPart(w[i]);
+      beta[i] = PetscRealPart(Q[nv-1+i*ld])*lastbeta;
+      svd->errest[i] = PetscAbsScalar(beta[i]);
+      if (svd->sigma[i] > svd->tol) svd->errest[i] /= svd->sigma[i];
       if (conv) {
         if (svd->errest[i] < svd->tol) k++;
         else conv = PETSC_FALSE;
       }
     }
-    
+    ierr = DSRestoreArrayReal(svd->ds,DS_MAT_T,&alpha);CHKERRQ(ierr);
+
     /* check convergence and update l */
     if (svd->its >= svd->max_it) svd->reason = SVD_DIVERGED_ITS;
     if (svd->nconv+k >= svd->nsv) svd->reason = SVD_CONVERGED_TOL;
     if (svd->reason != SVD_CONVERGED_ITERATING) l = 0;
-    else l = PetscMax((nv - svd->nconv - k) / 2,0);
-    
-    /* compute converged singular vectors and restart vectors*/
-#if !defined(PETSC_USE_COMPLEX)
-    if (svd->which == SVD_SMALLEST) {
-#endif
-    for (i=0;i<k+l;i++) {
-      if (svd->which == SVD_SMALLEST) j = n-i-1;
-      else j = i;
-      for (m=0;m<n;m++) swork[j*n+m] = PT[m*n+j];
-    }
-    ierr = SlepcUpdateVectors(n,svd->V+svd->nconv,0,k+l,swork,n,PETSC_FALSE);CHKERRQ(ierr);
-    for (i=0;i<k+l;i++) {
-      if (svd->which == SVD_SMALLEST) j = n-i-1;
-      else j = i;
-      for (m=0;m<n;m++) swork[j*n+m] = Q[j*n+m];
-    }
-    ierr = SlepcUpdateVectors(n,svd->U+svd->nconv,0,k+l,swork,n,PETSC_FALSE);CHKERRQ(ierr);
-#if !defined(PETSC_USE_COMPLEX)
-    } else {
-      ierr = SlepcUpdateVectors(n,svd->V+svd->nconv,0,k+l,PT,n,PETSC_TRUE);CHKERRQ(ierr);
-      ierr = SlepcUpdateVectors(n,svd->U+svd->nconv,0,k+l,Q,n,PETSC_FALSE);CHKERRQ(ierr);
-    }
-#endif
-    
+    else l = PetscMax((nv-svd->nconv-k)/2,0);
+
+    /* compute converged singular vectors and restart vectors */
+    off = svd->nconv+svd->nconv*ld;
+    ierr = DSGetArray(svd->ds,DS_MAT_VT,&PT);CHKERRQ(ierr);
+    ierr = SlepcUpdateVectors(nv-svd->nconv,svd->V+svd->nconv,0,k+l,PT+off,ld,PETSC_TRUE);CHKERRQ(ierr);
+    ierr = SlepcUpdateVectors(nv-svd->nconv,svd->U+svd->nconv,0,k+l,Q+off,ld,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = DSRestoreArray(svd->ds,DS_MAT_VT,&PT);CHKERRQ(ierr);
+    ierr = DSRestoreArray(svd->ds,DS_MAT_U,&Q);CHKERRQ(ierr);
+
     /* copy the last vector to be the next initial vector */
     if (svd->reason == SVD_CONVERGED_ITERATING) {
       ierr = VecCopy(v,svd->V[svd->nconv+k+l]);CHKERRQ(ierr);
     }
-    
+
     svd->nconv += k;
     ierr = SVDMonitor(svd,svd->its,svd->nconv,svd->sigma,svd->errest,nv);CHKERRQ(ierr);
   }
-  
+
   /* orthonormalize U columns in one side method */
   if (lanczos->oneside) {
     for (i=0;i<svd->nconv;i++) {
-      ierr = IPOrthogonalize(svd->ip,0,PETSC_NULL,i,PETSC_NULL,svd->U,svd->U[i],PETSC_NULL,&norm,PETSC_NULL);CHKERRQ(ierr);
+      ierr = IPOrthogonalize(svd->ip,0,NULL,i,NULL,svd->U,svd->U[i],NULL,&norm,NULL);CHKERRQ(ierr);
       ierr = VecScale(svd->U[i],1.0/norm);CHKERRQ(ierr);
     }
   }
-  
+
   /* free working space */
   ierr = VecDestroy(&v);CHKERRQ(ierr);
-
-  ierr = PetscFree(alpha);CHKERRQ(ierr);
-  ierr = PetscFree(beta);CHKERRQ(ierr);
-  ierr = PetscFree(b);CHKERRQ(ierr);
-  ierr = PetscFree(Q);CHKERRQ(ierr);
-  ierr = PetscFree(PT);CHKERRQ(ierr);
+  ierr = PetscFree(w);CHKERRQ(ierr);
   ierr = PetscFree(swork);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDSetFromOptions_TRLanczos"
 PetscErrorCode SVDSetFromOptions_TRLanczos(SVD svd)
 {
   PetscErrorCode ierr;
   PetscBool      set,val;
-  SVD_TRLANCZOS  *lanczos = (SVD_TRLANCZOS *)svd->data;
+  SVD_TRLANCZOS  *lanczos = (SVD_TRLANCZOS*)svd->data;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead("SVD TRLanczos Options");CHKERRQ(ierr);
@@ -372,23 +360,21 @@ PetscErrorCode SVDSetFromOptions_TRLanczos(SVD svd)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDTRLanczosSetOneSide_TRLanczos"
-PetscErrorCode SVDTRLanczosSetOneSide_TRLanczos(SVD svd,PetscBool oneside)
+static PetscErrorCode SVDTRLanczosSetOneSide_TRLanczos(SVD svd,PetscBool oneside)
 {
-  SVD_TRLANCZOS *lanczos = (SVD_TRLANCZOS *)svd->data;
+  SVD_TRLANCZOS *lanczos = (SVD_TRLANCZOS*)svd->data;
 
   PetscFunctionBegin;
   lanczos->oneside = oneside;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
 #undef __FUNCT__
 #define __FUNCT__ "SVDTRLanczosSetOneSide"
 /*@
-   SVDTRLanczosSetOneSide - Indicate if the variant of the Lanczos method 
+   SVDTRLanczosSetOneSide - Indicate if the variant of the Lanczos method
    to be used is one-sided or two-sided.
 
    Logically Collective on SVD
@@ -402,8 +388,8 @@ EXTERN_C_END
 
    Note:
    By default, a two-sided variant is selected, which is sometimes slightly
-   more robust. However, the one-sided variant is faster because it avoids 
-   the orthogonalization associated to left singular vectors. 
+   more robust. However, the one-sided variant is faster because it avoids
+   the orthogonalization associated to left singular vectors.
 
    Level: advanced
 
@@ -423,7 +409,7 @@ PetscErrorCode SVDTRLanczosSetOneSide(SVD svd,PetscBool oneside)
 #undef __FUNCT__
 #define __FUNCT__ "SVDTRLanczosGetOneSide"
 /*@
-   SVDTRLanczosGetOneSide - Gets if the variant of the Lanczos method 
+   SVDTRLanczosGetOneSide - Gets if the variant of the Lanczos method
    to be used is one-sided or two-sided.
 
    Not Collective
@@ -449,20 +435,18 @@ PetscErrorCode SVDTRLanczosGetOneSide(SVD svd,PetscBool *oneside)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDTRLanczosGetOneSide_TRLanczos"
-PetscErrorCode SVDTRLanczosGetOneSide_TRLanczos(SVD svd,PetscBool *oneside)
+static PetscErrorCode SVDTRLanczosGetOneSide_TRLanczos(SVD svd,PetscBool *oneside)
 {
-  SVD_TRLANCZOS    *lanczos = (SVD_TRLANCZOS *)svd->data;
+  SVD_TRLANCZOS    *lanczos = (SVD_TRLANCZOS*)svd->data;
 
   PetscFunctionBegin;
   *oneside = lanczos->oneside;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDReset_TRLanczos"
 PetscErrorCode SVDReset_TRLanczos(SVD svd)
 {
@@ -473,7 +457,7 @@ PetscErrorCode SVDReset_TRLanczos(SVD svd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDDestroy_TRLanczos"
 PetscErrorCode SVDDestroy_TRLanczos(SVD svd)
 {
@@ -481,27 +465,26 @@ PetscErrorCode SVDDestroy_TRLanczos(SVD svd)
 
   PetscFunctionBegin;
   ierr = PetscFree(svd->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDTRLanczosSetOneSide_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDTRLanczosGetOneSide_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDTRLanczosSetOneSide_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDTRLanczosGetOneSide_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDView_TRLanczos"
 PetscErrorCode SVDView_TRLanczos(SVD svd,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  SVD_TRLANCZOS  *lanczos = (SVD_TRLANCZOS *)svd->data;
+  SVD_TRLANCZOS  *lanczos = (SVD_TRLANCZOS*)svd->data;
 
   PetscFunctionBegin;
   ierr = PetscViewerASCIIPrintf(viewer,"  TRLanczos: %s-sided reorthogonalization\n",lanczos->oneside? "one": "two");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDCreate_TRLanczos"
-PetscErrorCode SVDCreate_TRLanczos(SVD svd)
+PETSC_EXTERN PetscErrorCode SVDCreate_TRLanczos(SVD svd)
 {
   PetscErrorCode ierr;
 
@@ -513,9 +496,8 @@ PetscErrorCode SVDCreate_TRLanczos(SVD svd)
   svd->ops->reset          = SVDReset_TRLanczos;
   svd->ops->setfromoptions = SVDSetFromOptions_TRLanczos;
   svd->ops->view           = SVDView_TRLanczos;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDTRLanczosSetOneSide_C","SVDTRLanczosSetOneSide_TRLanczos",SVDTRLanczosSetOneSide_TRLanczos);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDTRLanczosGetOneSide_C","SVDTRLanczosGetOneSide_TRLanczos",SVDTRLanczosGetOneSide_TRLanczos);CHKERRQ(ierr);
-  if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
+  ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDTRLanczosSetOneSide_C",SVDTRLanczosSetOneSide_TRLanczos);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDTRLanczosGetOneSide_C",SVDTRLanczosGetOneSide_TRLanczos);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
+
