@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,15 +21,15 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/svdimpl.h>      /*I "slepcsvd.h" I*/
+#include <slepc-private/svdimpl.h>      /*I "slepcsvd.h" I*/
 
-PetscFList       SVDList = 0;
-PetscBool        SVDRegisterAllCalled = PETSC_FALSE;
-PetscClassId     SVD_CLASSID = 0;
-PetscLogEvent    SVD_SetUp = 0,SVD_Solve = 0,SVD_Dense = 0;
-static PetscBool SVDPackageInitialized = PETSC_FALSE;
+PetscFunctionList SVDList = 0;
+PetscBool         SVDRegisterAllCalled = PETSC_FALSE;
+PetscClassId      SVD_CLASSID = 0;
+PetscLogEvent     SVD_SetUp = 0,SVD_Solve = 0;
+static PetscBool  SVDPackageInitialized = PETSC_FALSE;
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDFinalizePackage"
 /*@C
    SVDFinalizePackage - This function destroys everything in the Slepc interface
@@ -39,30 +39,29 @@ static PetscBool SVDPackageInitialized = PETSC_FALSE;
 
 .seealso: SlepcFinalize()
 @*/
-PetscErrorCode SVDFinalizePackage(void) 
+PetscErrorCode SVDFinalizePackage(void)
 {
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
+  ierr = PetscFunctionListDestroy(&SVDList);CHKERRQ(ierr);
   SVDPackageInitialized = PETSC_FALSE;
-  SVDList               = 0;
   SVDRegisterAllCalled  = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDInitializePackage"
 /*@C
    SVDInitializePackage - This function initializes everything in the SVD package. It is called
    from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to SVDCreate()
    when using static libraries.
 
-   Input Parameter:
-.  path - The dynamic library path, or PETSC_NULL
-
    Level: developer
 
 .seealso: SlepcInitialize()
 @*/
-PetscErrorCode SVDInitializePackage(const char *path)
+PetscErrorCode SVDInitializePackage(void)
 {
   char           logList[256];
   char           *className;
@@ -75,13 +74,12 @@ PetscErrorCode SVDInitializePackage(const char *path)
   /* Register Classes */
   ierr = PetscClassIdRegister("Singular Value Solver",&SVD_CLASSID);CHKERRQ(ierr);
   /* Register Constructors */
-  ierr = SVDRegisterAll(path);CHKERRQ(ierr);
+  ierr = SVDRegisterAll();CHKERRQ(ierr);
   /* Register Events */
   ierr = PetscLogEventRegister("SVDSetUp",SVD_CLASSID,&SVD_SetUp);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("SVDSolve",SVD_CLASSID,&SVD_Solve);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("SVDDense",SVD_CLASSID,&SVD_Dense);CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
     ierr = PetscStrstr(logList,"svd",&className);CHKERRQ(ierr);
     if (className) {
@@ -89,7 +87,7 @@ PetscErrorCode SVDInitializePackage(const char *path)
     }
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL,"-log_summary_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,"-log_summary_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
     ierr = PetscStrstr(logList,"svd",&className);CHKERRQ(ierr);
     if (className) {
@@ -100,7 +98,7 @@ PetscErrorCode SVDInitializePackage(const char *path)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDView"
 /*@C
    SVDView - Prints the SVD data structure.
@@ -119,8 +117,8 @@ PetscErrorCode SVDInitializePackage(const char *path)
 +     PETSC_VIEWER_STDOUT_SELF - standard output (default)
 -     PETSC_VIEWER_STDOUT_WORLD - synchronized standard
          output where only the first processor opens
-         the file.  All other processors send their 
-         data to the first processor to print. 
+         the file.  All other processors send their
+         data to the first processor to print.
 
    The user can open an alternative visualization context with
    PetscViewerASCIIOpen() - output to a specified file.
@@ -132,15 +130,15 @@ PetscErrorCode SVDInitializePackage(const char *path)
 PetscErrorCode SVDView(SVD svd,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  PetscBool      isascii;
+  PetscBool      isascii,isshell;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(((PetscObject)svd)->comm);
+  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)svd));
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(svd,1,viewer,2);
 
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)svd,viewer,"SVD Object");CHKERRQ(ierr);
     if (svd->ops->view) {
@@ -162,22 +160,32 @@ PetscErrorCode SVDView(SVD svd,PetscViewer viewer)
       ierr = PetscViewerASCIIPrintf(viewer,"  selected portion of the spectrum: largest\n");CHKERRQ(ierr);
     } else {
       ierr = PetscViewerASCIIPrintf(viewer,"  selected portion of the spectrum: smallest\n");CHKERRQ(ierr);
-    }  
+    }
     ierr = PetscViewerASCIIPrintf(viewer,"  number of singular values (nsv): %D\n",svd->nsv);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  number of column vectors (ncv): %D\n",svd->ncv);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  maximum dimension of projected problem (mpd): %D\n",svd->mpd);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of iterations: %D\n",svd->max_it);
+    ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of iterations: %D\n",svd->max_it);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  tolerance: %G\n",svd->tol);CHKERRQ(ierr);
-    if (svd->nini!=0) {
+    if (svd->nini) {
       ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial space: %D\n",PetscAbs(svd->nini));CHKERRQ(ierr);
     }
+    if (svd->ninil) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial left space: %D\n",PetscAbs(svd->ninil));CHKERRQ(ierr);
+    }
   } else {
     if (svd->ops->view) {
       ierr = (*svd->ops->view)(svd,viewer);CHKERRQ(ierr);
     }
   }
-  if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
-  ierr = IPView(svd->ip,viewer);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompareAny((PetscObject)svd,&isshell,SVDCROSS,SVDCYCLIC,"");CHKERRQ(ierr);
+  if (!isshell) {
+    if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
+    ierr = IPView(svd->ip,viewer);CHKERRQ(ierr);
+    if (!svd->ds) { ierr = SVDGetDS(svd,&svd->ds);CHKERRQ(ierr); }
+    ierr = PetscViewerPushFormat(viewer,PETSC_VIEWER_ASCII_INFO);CHKERRQ(ierr);
+    ierr = DSView(svd->ds,viewer);CHKERRQ(ierr);
+    ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
@@ -192,7 +200,7 @@ PetscErrorCode SVDView(SVD svd,PetscViewer viewer)
 +  svd - the singular value solver context
 -  viewer - optional visualization context
 
-   Options Database:
+   Options Database Key:
 .  -svd_terse - print only minimal information
 
    Note:
@@ -212,16 +220,14 @@ PetscErrorCode SVDPrintSolution(SVD svd,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(((PetscObject)svd)->comm);
+  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)svd));
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(svd,1,viewer,2);
-  if (!svd->sigma) { 
-    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_WRONGSTATE,"SVDSolve must be called first"); 
-  }
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  if (!svd->sigma) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_WRONGSTATE,"SVDSolve must be called first");
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (!isascii) PetscFunctionReturn(0);
 
-  ierr = PetscOptionsHasName(PETSC_NULL,"-svd_terse",&terse);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,"-svd_terse",&terse);CHKERRQ(ierr);
   if (terse) {
     if (svd->nconv<svd->nsv) {
       ierr = PetscViewerASCIIPrintf(viewer," Problem: less than %D singular values converged\n\n",svd->nsv);CHKERRQ(ierr);
@@ -236,8 +242,8 @@ PetscErrorCode SVDPrintSolution(SVD svd,PetscViewer viewer)
         for (i=0;i<=(svd->nsv-1)/8;i++) {
           ierr = PetscViewerASCIIPrintf(viewer,"\n     ");CHKERRQ(ierr);
           for (j=0;j<PetscMin(8,svd->nsv-8*i);j++) {
-            ierr = SVDGetSingularTriplet(svd,8*i+j,&sigma,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
-            ierr = PetscViewerASCIIPrintf(viewer,"%.5F",sigma);CHKERRQ(ierr); 
+            ierr = SVDGetSingularTriplet(svd,8*i+j,&sigma,NULL,NULL);CHKERRQ(ierr);
+            ierr = PetscViewerASCIIPrintf(viewer,"%.5F",sigma);CHKERRQ(ierr);
             if (8*i+j+1<svd->nsv) { ierr = PetscViewerASCIIPrintf(viewer,", ");CHKERRQ(ierr); }
           }
         }
@@ -253,9 +259,9 @@ PetscErrorCode SVDPrintSolution(SVD svd,PetscViewer viewer)
            "          sigma            relative error\n"
            "   --------------------- ------------------\n");CHKERRQ(ierr);
       for (i=0;i<svd->nconv;i++) {
-        ierr = SVDGetSingularTriplet(svd,i,&sigma,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+        ierr = SVDGetSingularTriplet(svd,i,&sigma,NULL,NULL);CHKERRQ(ierr);
         ierr = SVDComputeRelativeError(svd,i,&error);CHKERRQ(ierr);
-        ierr = PetscViewerASCIIPrintf(viewer,"       % 6F          %12G\n",sigma,error);CHKERRQ(ierr); 
+        ierr = PetscViewerASCIIPrintf(viewer,"       % 6F          %12G\n",sigma,error);CHKERRQ(ierr);
       }
       ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
     }
@@ -263,7 +269,7 @@ PetscErrorCode SVDPrintSolution(SVD svd,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDCreate"
 /*@C
    SVDCreate - Creates the default SVD context.
@@ -291,32 +297,40 @@ PetscErrorCode SVDCreate(MPI_Comm comm,SVD *outsvd)
   PetscFunctionBegin;
   PetscValidPointer(outsvd,2);
   *outsvd = 0;
-  ierr = PetscHeaderCreate(svd,_p_SVD,struct _SVDOps,SVD_CLASSID,-1,"SVD","Singular Value Decomposition","SVD",comm,SVDDestroy,SVDView);CHKERRQ(ierr);
+#if !defined(PETSC_USE_DYNAMIC_LIBRARIES)
+  ierr = SVDInitializePackage();CHKERRQ(ierr);
+#endif
 
-  svd->OP             = PETSC_NULL;
-  svd->A              = PETSC_NULL;
-  svd->AT             = PETSC_NULL;
+  ierr = SlepcHeaderCreate(svd,_p_SVD,struct _SVDOps,SVD_CLASSID,"SVD","Singular Value Decomposition","SVD",comm,SVDDestroy,SVDView);CHKERRQ(ierr);
+
+  svd->OP             = NULL;
+  svd->ip             = NULL;
+  svd->ds             = NULL;
+  svd->A              = NULL;
+  svd->AT             = NULL;
   svd->transmode      = (SVDTransposeMode)PETSC_DECIDE;
-  svd->sigma          = PETSC_NULL;
-  svd->perm           = PETSC_NULL;
-  svd->U              = PETSC_NULL;
-  svd->V              = PETSC_NULL;
-  svd->IS             = PETSC_NULL;
-  svd->tl             = PETSC_NULL;
-  svd->tr             = PETSC_NULL;
-  svd->rand           = PETSC_NULL;
+  svd->sigma          = NULL;
+  svd->perm           = NULL;
+  svd->U              = NULL;
+  svd->V              = NULL;
+  svd->IS             = NULL;
+  svd->ISL            = NULL;
+  svd->tl             = NULL;
+  svd->tr             = NULL;
+  svd->rand           = NULL;
   svd->which          = SVD_LARGEST;
   svd->n              = 0;
   svd->nconv          = 0;
-  svd->nsv            = 1;    
-  svd->ncv            = 0;    
-  svd->mpd            = 0;    
+  svd->nsv            = 1;
+  svd->ncv            = 0;
+  svd->mpd            = 0;
   svd->nini           = 0;
+  svd->ninil          = 0;
   svd->its            = 0;
-  svd->max_it         = 0;  
-  svd->tol            = PETSC_DEFAULT;    
-  svd->errest         = PETSC_NULL;
-  svd->data           = PETSC_NULL;
+  svd->max_it         = 0;
+  svd->tol            = PETSC_DEFAULT;
+  svd->errest         = NULL;
+  svd->data           = NULL;
   svd->setupcalled    = 0;
   svd->reason         = SVD_CONVERGED_ITERATING;
   svd->numbermonitors = 0;
@@ -329,8 +343,8 @@ PetscErrorCode SVDCreate(MPI_Comm comm,SVD *outsvd)
   *outsvd = svd;
   PetscFunctionReturn(0);
 }
- 
-#undef __FUNCT__  
+
+#undef __FUNCT__
 #define __FUNCT__ "SVDReset"
 /*@
    SVDReset - Resets the SVD context to the setupcalled=0 state and removes any
@@ -353,12 +367,13 @@ PetscErrorCode SVDReset(SVD svd)
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   if (svd->ops->reset) { ierr = (svd->ops->reset)(svd);CHKERRQ(ierr); }
   if (svd->ip) { ierr = IPReset(svd->ip);CHKERRQ(ierr); }
+  if (svd->ds) { ierr = DSReset(svd->ds);CHKERRQ(ierr); }
   ierr = MatDestroy(&svd->OP);CHKERRQ(ierr);
   ierr = MatDestroy(&svd->A);CHKERRQ(ierr);
   ierr = MatDestroy(&svd->AT);CHKERRQ(ierr);
   ierr = VecDestroy(&svd->tl);CHKERRQ(ierr);
   ierr = VecDestroy(&svd->tr);CHKERRQ(ierr);
-  if (svd->n) { 
+  if (svd->n) {
     ierr = PetscFree(svd->sigma);CHKERRQ(ierr);
     ierr = PetscFree(svd->perm);CHKERRQ(ierr);
     ierr = PetscFree(svd->errest);CHKERRQ(ierr);
@@ -371,7 +386,7 @@ PetscErrorCode SVDReset(SVD svd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDDestroy"
 /*@C
    SVDDestroy - Destroys the SVD context.
@@ -388,25 +403,28 @@ PetscErrorCode SVDReset(SVD svd)
 PetscErrorCode SVDDestroy(SVD *svd)
 {
   PetscErrorCode ierr;
-  
+
   PetscFunctionBegin;
   if (!*svd) PetscFunctionReturn(0);
   PetscValidHeaderSpecific(*svd,SVD_CLASSID,1);
   if (--((PetscObject)(*svd))->refct > 0) { *svd = 0; PetscFunctionReturn(0); }
   ierr = SVDReset(*svd);CHKERRQ(ierr);
-  ierr = PetscObjectDepublish(*svd);CHKERRQ(ierr);
   if ((*svd)->ops->destroy) { ierr = (*(*svd)->ops->destroy)(*svd);CHKERRQ(ierr); }
   ierr = IPDestroy(&(*svd)->ip);CHKERRQ(ierr);
+  ierr = DSDestroy(&(*svd)->ds);CHKERRQ(ierr);
+  /* just in case the initial vectors have not been used */
+  ierr = SlepcBasisDestroy_Private(&(*svd)->nini,&(*svd)->IS);CHKERRQ(ierr);
+  ierr = SlepcBasisDestroy_Private(&(*svd)->ninil,&(*svd)->ISL);CHKERRQ(ierr);
   ierr = PetscRandomDestroy(&(*svd)->rand);CHKERRQ(ierr);
   ierr = SVDMonitorCancel(*svd);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(svd);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDSetType"
 /*@C
-   SVDSetType - Selects the particular solver to be used in the SVD object. 
+   SVDSetType - Selects the particular solver to be used in the SVD object.
 
    Logically Collective on SVD
 
@@ -415,10 +433,10 @@ PetscErrorCode SVDDestroy(SVD *svd)
 -  type     - a known method
 
    Options Database Key:
-.  -svd_type <method> - Sets the method; use -help for a list 
-    of available methods 
-    
-   Notes:  
+.  -svd_type <method> - Sets the method; use -help for a list
+    of available methods
+
+   Notes:
    See "slepc/include/slepcsvd.h" for available methods. The default
    is SVDCROSS.
 
@@ -428,13 +446,13 @@ PetscErrorCode SVDDestroy(SVD *svd)
    maximum flexibility in evaluating the different available methods.
    The SVDSetType() routine is provided for those situations where it
    is necessary to set the iterative solver independently of the command
-   line or options database. 
+   line or options database.
 
    Level: intermediate
 
 .seealso: SVDType
 @*/
-PetscErrorCode SVDSetType(SVD svd,const SVDType type)
+PetscErrorCode SVDSetType(SVD svd,SVDType type)
 {
   PetscErrorCode ierr,(*r)(SVD);
   PetscBool      match;
@@ -443,11 +461,11 @@ PetscErrorCode SVDSetType(SVD svd,const SVDType type)
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidCharPointer(type,2);
 
-  ierr = PetscTypeCompare((PetscObject)svd,type,&match);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)svd,type,&match);CHKERRQ(ierr);
   if (match) PetscFunctionReturn(0);
 
-  ierr = PetscFListFind(SVDList,((PetscObject)svd)->comm,type,PETSC_TRUE,(void (**)(void)) &r);CHKERRQ(ierr);
-  if (!r) SETERRQ1(((PetscObject)svd)->comm,PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown SVD type given: %s",type);
+  ierr = PetscFunctionListFind(SVDList,type,&r);CHKERRQ(ierr);
+  if (!r) SETERRQ1(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown SVD type given: %s",type);
 
   if (svd->ops->destroy) { ierr = (*svd->ops->destroy)(svd);CHKERRQ(ierr); }
   ierr = PetscMemzero(svd->ops,sizeof(struct _SVDOps));CHKERRQ(ierr);
@@ -458,7 +476,7 @@ PetscErrorCode SVDSetType(SVD svd,const SVDType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDGetType"
 /*@C
    SVDGetType - Gets the SVD type as a string from the SVD object.
@@ -466,16 +484,16 @@ PetscErrorCode SVDSetType(SVD svd,const SVDType type)
    Not Collective
 
    Input Parameter:
-.  svd - the singular value solver context 
+.  svd - the singular value solver context
 
    Output Parameter:
-.  name - name of SVD method 
+.  name - name of SVD method
 
    Level: intermediate
 
 .seealso: SVDSetType()
 @*/
-PetscErrorCode SVDGetType(SVD svd,const SVDType *type)
+PetscErrorCode SVDGetType(SVD svd,SVDType *type)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
@@ -484,51 +502,48 @@ PetscErrorCode SVDGetType(SVD svd,const SVDType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDRegister"
 /*@C
-   SVDRegister - See SVDRegisterDynamic()
-
-   Level: advanced
-@*/
-PetscErrorCode SVDRegister(const char *sname,const char *path,const char *name,PetscErrorCode (*function)(SVD))
-{
-  PetscErrorCode ierr;
-  char           fullname[PETSC_MAX_PATH_LEN];
-
-  PetscFunctionBegin;
-  ierr = PetscFListConcat(path,name,fullname);CHKERRQ(ierr);
-  ierr = PetscFListAdd(&SVDList,sname,fullname,(void (*)(void))function);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
-#define __FUNCT__ "SVDRegisterDestroy"
-/*@
-   SVDRegisterDestroy - Frees the list of SVD methods that were
-   registered by SVDRegisterDynamic().
+   SVDRegister - Adds a method to the singular value solver package.
 
    Not Collective
 
+   Input Parameters:
++  name - name of a new user-defined solver
+-  function - routine to create the solver context
+
+   Notes:
+   SVDRegister() may be called multiple times to add several user-defined solvers.
+
+   Sample usage:
+.vb
+   SVDRegister("my_solver",MySolverCreate);
+.ve
+
+   Then, your solver can be chosen with the procedural interface via
+$     SVDSetType(svd,"my_solver")
+   or at runtime via the option
+$     -svd_type my_solver
+
    Level: advanced
 
-.seealso: SVDRegisterDynamic(), SVDRegisterAll()
+.seealso: SVDRegisterAll()
 @*/
-PetscErrorCode SVDRegisterDestroy(void)
+PetscErrorCode SVDRegister(const char *name,PetscErrorCode (*function)(SVD))
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscFListDestroy(&SVDList);CHKERRQ(ierr);
-  SVDRegisterAllCalled = PETSC_FALSE;
+  ierr = PetscFunctionListAdd(&SVDList,name,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDSetIP"
 /*@
    SVDSetIP - Associates an inner product object to the
-   singular value solver. 
+   singular value solver.
 
    Collective on SVD
 
@@ -559,7 +574,7 @@ PetscErrorCode SVDSetIP(SVD svd,IP ip)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDGetIP"
 /*@C
    SVDGetIP - Obtain the inner product object associated
@@ -585,9 +600,77 @@ PetscErrorCode SVDGetIP(SVD svd,IP *ip)
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidPointer(ip,2);
   if (!svd->ip) {
-    ierr = IPCreate(((PetscObject)svd)->comm,&svd->ip);CHKERRQ(ierr);
+    ierr = IPCreate(PetscObjectComm((PetscObject)svd),&svd->ip);CHKERRQ(ierr);
     ierr = PetscLogObjectParent(svd,svd->ip);CHKERRQ(ierr);
   }
   *ip = svd->ip;
   PetscFunctionReturn(0);
 }
+
+#undef __FUNCT__
+#define __FUNCT__ "SVDSetDS"
+/*@
+   SVDSetDS - Associates a direct solver object to the singular value solver.
+
+   Collective on SVD
+
+   Input Parameters:
++  svd - singular value solver context obtained from SVDCreate()
+-  ds  - the direct solver object
+
+   Note:
+   Use SVDGetDS() to retrieve the direct solver context (for example,
+   to free it at the end of the computations).
+
+   Level: advanced
+
+.seealso: SVDGetDS()
+@*/
+PetscErrorCode SVDSetDS(SVD svd,DS ds)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidHeaderSpecific(ds,DS_CLASSID,2);
+  PetscCheckSameComm(svd,1,ds,2);
+  ierr = PetscObjectReference((PetscObject)ds);CHKERRQ(ierr);
+  ierr = DSDestroy(&svd->ds);CHKERRQ(ierr);
+  svd->ds = ds;
+  ierr = PetscLogObjectParent(svd,svd->ds);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "SVDGetDS"
+/*@C
+   SVDGetDS - Obtain the direct solver object associated to the singular value
+   solver object.
+
+   Not Collective
+
+   Input Parameters:
+.  svd - singular value solver context obtained from SVDCreate()
+
+   Output Parameter:
+.  ds - direct solver context
+
+   Level: advanced
+
+.seealso: SVDSetDS()
+@*/
+PetscErrorCode SVDGetDS(SVD svd,DS *ds)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidPointer(ds,2);
+  if (!svd->ds) {
+    ierr = DSCreate(PetscObjectComm((PetscObject)svd),&svd->ds);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(svd,svd->ds);CHKERRQ(ierr);
+  }
+  *ds = svd->ds;
+  PetscFunctionReturn(0);
+}
+
