@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,12 +21,12 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/svdimpl.h>      /*I "slepcsvd.h" I*/
+#include <slepc-private/svdimpl.h>      /*I "slepcsvd.h" I*/
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDSetTransposeMode"
 /*@
-   SVDSetTransposeMode - Sets how to handle the transpose of the matrix 
+   SVDSetTransposeMode - Sets how to handle the transpose of the matrix
    associated with the singular value problem.
 
    Logically Collective on SVD
@@ -37,7 +37,7 @@
           or SVD_TRANSPOSE_IMPLICIT (see notes below)
 
    Options Database Key:
-.  -svd_transpose_mode <mode> - Indicates the mode flag, where <mode> 
+.  -svd_transpose_mode <mode> - Indicates the mode flag, where <mode>
     is one of 'explicit' or 'implicit'.
 
    Notes:
@@ -45,16 +45,17 @@
    explicitly built.
 
    The option SVD_TRANSPOSE_IMPLICIT does not build the transpose, but
-   handles it implicitly via MatMultTranspose() operations. This is 
+   handles it implicitly via MatMultTranspose() (or MatMultHermitianTranspose()
+   in the complex case) operations. This is
    likely to be more inefficient than SVD_TRANSPOSE_EXPLICIT, both in
    sequential and in parallel, but requires less storage.
 
    The default is SVD_TRANSPOSE_EXPLICIT if the matrix has defined the
    MatTranspose operation, and SVD_TRANSPOSE_IMPLICIT otherwise.
-   
+
    Level: advanced
-   
-.seealso: SVDGetTransposeMode(), SVDSolve(), SVDSetOperator(), 
+
+.seealso: SVDGetTransposeMode(), SVDSolve(), SVDSetOperator(),
    SVDGetOperator(), SVDTransposeMode
 @*/
 PetscErrorCode SVDSetTransposeMode(SVD svd,SVDTransposeMode mode)
@@ -72,15 +73,15 @@ PetscErrorCode SVDSetTransposeMode(SVD svd,SVDTransposeMode mode)
       }
       break;
     default:
-      SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid transpose mode"); 
+      SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Invalid transpose mode");
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDGetTransposeMode"
 /*@C
-   SVDGetTransposeMode - Gets the mode used to compute the transpose 
+   SVDGetTransposeMode - Gets the mode used to compute the transpose
    of the matrix associated with the singular value problem.
 
    Not Collective
@@ -91,10 +92,10 @@ PetscErrorCode SVDSetTransposeMode(SVD svd,SVDTransposeMode mode)
    Output paramter:
 .  mode - how to compute the transpose, one of SVD_TRANSPOSE_EXPLICIT
           or SVD_TRANSPOSE_IMPLICIT
-   
+
    Level: advanced
-   
-.seealso: SVDSetTransposeMode(), SVDSolve(), SVDSetOperator(), 
+
+.seealso: SVDSetTransposeMode(), SVDSolve(), SVDSetOperator(),
    SVDGetOperator(), SVDTransposeMode
 @*/
 PetscErrorCode SVDGetTransposeMode(SVD svd,SVDTransposeMode *mode)
@@ -106,11 +107,11 @@ PetscErrorCode SVDGetTransposeMode(SVD svd,SVDTransposeMode *mode)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDSetTolerances"
 /*@
    SVDSetTolerances - Sets the tolerance and maximum
-   iteration count used by the default SVD convergence testers. 
+   iteration count used by the default SVD convergence testers.
 
    Logically Collective on SVD
 
@@ -121,11 +122,11 @@ PetscErrorCode SVDGetTransposeMode(SVD svd,SVDTransposeMode *mode)
 
    Options Database Keys:
 +  -svd_tol <tol> - Sets the convergence tolerance
--  -svd_max_it <maxits> - Sets the maximum number of iterations allowed 
+-  -svd_max_it <maxits> - Sets the maximum number of iterations allowed
    (use PETSC_DECIDE to compute an educated guess based on basis and matrix sizes)
 
    Notes:
-   Use PETSC_IGNORE to retain the previous value of any parameter. 
+   Pass 0 to retain the previous value of any parameter.
 
    Level: intermediate
 
@@ -137,43 +138,43 @@ PetscErrorCode SVDSetTolerances(SVD svd,PetscReal tol,PetscInt maxits)
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidLogicalCollectiveReal(svd,tol,2);
   PetscValidLogicalCollectiveInt(svd,maxits,3);
-  if (tol != PETSC_IGNORE) {
+  if (tol) {
     if (tol == PETSC_DEFAULT) {
       tol = PETSC_DEFAULT;
     } else {
-      if (tol < 0.0) SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
+      if (tol < 0.0) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
       svd->tol = tol;
     }
   }
-  if (maxits != PETSC_IGNORE) {
+  if (maxits) {
     if (maxits == PETSC_DEFAULT || maxits == PETSC_DECIDE) {
       svd->max_it = 0;
       svd->setupcalled = 0;
     } else {
-      if (maxits < 0) SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of maxits. Must be > 0");
+      if (maxits < 0) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of maxits. Must be > 0");
       svd->max_it = maxits;
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDGetTolerances"
 /*@
    SVDGetTolerances - Gets the tolerance and maximum
-   iteration count used by the default SVD convergence tests. 
+   iteration count used by the default SVD convergence tests.
 
    Not Collective
 
    Input Parameter:
 .  svd - the singular value solver context
-  
+
    Output Parameters:
 +  tol - the convergence tolerance
 -  maxits - maximum number of iterations
 
    Notes:
-   The user can specify PETSC_NULL for any parameter that is not needed.
+   The user can specify NULL for any parameter that is not needed.
 
    Level: intermediate
 
@@ -188,7 +189,7 @@ PetscErrorCode SVDGetTolerances(SVD svd,PetscReal *tol,PetscInt *maxits)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDSetDimensions"
 /*@
    SVDSetDimensions - Sets the number of singular values to compute
@@ -208,9 +209,9 @@ PetscErrorCode SVDGetTolerances(SVD svd,PetscReal *tol,PetscInt *maxits)
 -  -svd_mpd <mpd> - Sets the maximum projected dimension
 
    Notes:
-   Use PETSC_IGNORE to retain the previous value of any parameter.
+   Pass 0 to retain the previous value of any parameter.
 
-   Use PETSC_DECIDE for ncv and mpd to assign a reasonably good value, which is 
+   Use PETSC_DECIDE for ncv and mpd to assign a reasonably good value, which is
    dependent on the solution method and the number of singular values required.
 
    The parameters ncv and mpd are intimately related, so that the user is advised
@@ -233,31 +234,31 @@ PetscErrorCode SVDSetDimensions(SVD svd,PetscInt nsv,PetscInt ncv,PetscInt mpd)
   PetscValidLogicalCollectiveInt(svd,nsv,2);
   PetscValidLogicalCollectiveInt(svd,ncv,3);
   PetscValidLogicalCollectiveInt(svd,mpd,4);
-  if (nsv != PETSC_IGNORE) {
-    if (nsv<1) SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of nsv. Must be > 0");
+  if (nsv) {
+    if (nsv<1) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of nsv. Must be > 0");
     svd->nsv = nsv;
   }
-  if (ncv != PETSC_IGNORE) {
+  if (ncv) {
     if (ncv == PETSC_DEFAULT || ncv == PETSC_DECIDE) {
       svd->ncv = 0;
     } else {
-      if (ncv<1) SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of ncv. Must be > 0");
+      if (ncv<1) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of ncv. Must be > 0");
       svd->ncv = ncv;
     }
     svd->setupcalled = 0;
   }
-  if (mpd != PETSC_IGNORE) {
+  if (mpd) {
     if (mpd == PETSC_DECIDE || mpd == PETSC_DEFAULT) {
       svd->mpd = 0;
     } else {
-      if (mpd<1) SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of mpd. Must be > 0");
+      if (mpd<1) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of mpd. Must be > 0");
       svd->mpd = mpd;
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDGetDimensions"
 /*@
    SVDGetDimensions - Gets the number of singular values to compute
@@ -267,14 +268,14 @@ PetscErrorCode SVDSetDimensions(SVD svd,PetscInt nsv,PetscInt ncv,PetscInt mpd)
 
    Input Parameter:
 .  svd - the singular value context
-  
+
    Output Parameters:
 +  nsv - number of singular values to compute
 .  ncv - the maximum dimension of the subspace to be used by the solver
 -  mpd - the maximum dimension allowed for the projected problem
 
    Notes:
-   The user can specify PETSC_NULL for any parameter that is not needed.
+   The user can specify NULL for any parameter that is not needed.
 
    Level: intermediate
 
@@ -290,10 +291,10 @@ PetscErrorCode SVDGetDimensions(SVD svd,PetscInt *nsv,PetscInt *ncv,PetscInt *mp
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDSetWhichSingularTriplets"
 /*@
-    SVDSetWhichSingularTriplets - Specifies which singular triplets are 
+    SVDSetWhichSingularTriplets - Specifies which singular triplets are
     to be sought.
 
     Logically Collective on SVD
@@ -306,14 +307,14 @@ PetscErrorCode SVDGetDimensions(SVD svd,PetscInt *nsv,PetscInt *ncv,PetscInt *mp
 
     Possible values:
     The parameter 'which' can have one of these values:
-    
+
 +     SVD_LARGEST  - largest singular values
 -     SVD_SMALLEST - smallest singular values
 
     Options Database Keys:
 +   -svd_largest  - Sets largest singular values
 -   -svd_smallest - Sets smallest singular values
-    
+
     Level: intermediate
 
 .seealso: SVDGetWhichSingularTriplets(), SVDWhich
@@ -332,12 +333,12 @@ PetscErrorCode SVDSetWhichSingularTriplets(SVD svd,SVDWhich which)
       }
       break;
   default:
-    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'which' parameter");    
+    SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'which' parameter");
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDGetWhichSingularTriplets"
 /*@C
     SVDGetWhichSingularTriplets - Returns which singular triplets are
@@ -358,7 +359,7 @@ PetscErrorCode SVDSetWhichSingularTriplets(SVD svd,SVDWhich which)
 
 .seealso: SVDSetWhichSingularTriplets(), SVDWhich
 @*/
-PetscErrorCode SVDGetWhichSingularTriplets(SVD svd,SVDWhich *which) 
+PetscErrorCode SVDGetWhichSingularTriplets(SVD svd,SVDWhich *which)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
@@ -367,24 +368,24 @@ PetscErrorCode SVDGetWhichSingularTriplets(SVD svd,SVDWhich *which)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDSetFromOptions"
 /*@
    SVDSetFromOptions - Sets SVD options from the options database.
-   This routine must be called before SVDSetUp() if the user is to be 
-   allowed to set the solver type. 
+   This routine must be called before SVDSetUp() if the user is to be
+   allowed to set the solver type.
 
    Collective on SVD
 
    Input Parameters:
 .  svd - the singular value solver context
 
-   Notes:  
+   Notes:
    To see all options, run your program with the -help option.
 
    Level: beginner
 
-.seealso: 
+.seealso:
 @*/
 PetscErrorCode SVDSetFromOptions(SVD svd)
 {
@@ -400,8 +401,8 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   svd->setupcalled = 0;
-  if (!SVDRegisterAllCalled) { ierr = SVDRegisterAll(PETSC_NULL);CHKERRQ(ierr); }
-  ierr = PetscOptionsBegin(((PetscObject)svd)->comm,((PetscObject)svd)->prefix,"Singular Value Solver (SVD) Options","SVD");CHKERRQ(ierr);
+  if (!SVDRegisterAllCalled) { ierr = SVDRegisterAll();CHKERRQ(ierr); }
+  ierr = PetscObjectOptionsBegin((PetscObject)svd);CHKERRQ(ierr);
 
   ierr = PetscOptionsList("-svd_type","Singular Value Solver method","SVDSetType",SVDList,(char*)(((PetscObject)svd)->type_name?((PetscObject)svd)->type_name:SVDCROSS),type,256,&flg);CHKERRQ(ierr);
   if (flg) {
@@ -415,17 +416,17 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
   ierr = PetscOptionsEList("-svd_transpose_mode","Transpose SVD mode","SVDSetTransposeMode",mode_list,2,svd->transmode == PETSC_DECIDE ? "decide" : mode_list[svd->transmode],&i,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = SVDSetTransposeMode(svd,(SVDTransposeMode)i);CHKERRQ(ierr);
-  }   
+  }
 
-  r = i = PETSC_IGNORE;
-  ierr = PetscOptionsInt("-svd_max_it","Maximum number of iterations","SVDSetTolerances",svd->max_it,&i,PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsReal("-svd_tol","Tolerance","SVDSetTolerances",svd->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:svd->tol,&r,PETSC_NULL);CHKERRQ(ierr);
+  r = i = 0;
+  ierr = PetscOptionsInt("-svd_max_it","Maximum number of iterations","SVDSetTolerances",svd->max_it,&i,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsReal("-svd_tol","Tolerance","SVDSetTolerances",svd->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:svd->tol,&r,NULL);CHKERRQ(ierr);
   ierr = SVDSetTolerances(svd,r,i);CHKERRQ(ierr);
 
-  i = j = k = PETSC_IGNORE;
-  ierr = PetscOptionsInt("-svd_nsv","Number of singular values to compute","SVDSetDimensions",svd->nsv,&i,PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-svd_ncv","Number of basis vectors","SVDSetDimensions",svd->ncv,&j,PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-svd_mpd","Maximum dimension of projected problem","SVDSetDimensions",svd->mpd,&k,PETSC_NULL);CHKERRQ(ierr);
+  i = j = k = 0;
+  ierr = PetscOptionsInt("-svd_nsv","Number of singular values to compute","SVDSetDimensions",svd->nsv,&i,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsInt("-svd_ncv","Number of basis vectors","SVDSetDimensions",svd->ncv,&j,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsInt("-svd_mpd","Maximum dimension of projected problem","SVDSetDimensions",svd->mpd,&k,NULL);CHKERRQ(ierr);
   ierr = SVDSetDimensions(svd,i,j,k);CHKERRQ(ierr);
 
   ierr = PetscOptionsBoolGroupBegin("-svd_largest","compute largest singular values","SVDSetWhichSingularTriplets",&flg);CHKERRQ(ierr);
@@ -434,37 +435,37 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
   if (flg) { ierr = SVDSetWhichSingularTriplets(svd,SVD_SMALLEST);CHKERRQ(ierr); }
 
   flg = PETSC_FALSE;
-  ierr = PetscOptionsBool("-svd_monitor_cancel","Remove any hardwired monitor routines","SVDMonitorCancel",flg,&flg,PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-svd_monitor_cancel","Remove any hardwired monitor routines","SVDMonitorCancel",flg,&flg,NULL);CHKERRQ(ierr);
   if (flg) {
-    ierr = SVDMonitorCancel(svd);CHKERRQ(ierr); 
+    ierr = SVDMonitorCancel(svd);CHKERRQ(ierr);
   }
 
-  ierr = PetscOptionsString("-svd_monitor_all","Monitor approximate singular values and error estimates","SVDMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
+  ierr = PetscOptionsString("-svd_monitor_all","Monitor approximate singular values and error estimates","SVDMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
   if (flg) {
-    ierr = PetscViewerASCIIOpen(((PetscObject)svd)->comm,monfilename,&monviewer);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIOpen(PetscObjectComm((PetscObject)svd),monfilename,&monviewer);CHKERRQ(ierr);
     ierr = SVDMonitorSet(svd,SVDMonitorAll,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
     ierr = SVDSetTrackAll(svd,PETSC_TRUE);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsString("-svd_monitor_conv","Monitor approximate singular values and error estimates as they converge","SVDMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
+  ierr = PetscOptionsString("-svd_monitor_conv","Monitor approximate singular values and error estimates as they converge","SVDMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
   if (flg) {
       ierr = PetscNew(struct _n_SlepcConvMonitor,&ctx);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIOpen(((PetscObject)svd)->comm,monfilename,&ctx->viewer);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIOpen(PetscObjectComm((PetscObject)svd),monfilename,&ctx->viewer);CHKERRQ(ierr);
       ierr = SVDMonitorSet(svd,SVDMonitorConverged,ctx,(PetscErrorCode (*)(void**))SlepcConvMonitorDestroy);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsString("-svd_monitor","Monitor first unconverged approximate singular value and error estimate","SVDMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
+  ierr = PetscOptionsString("-svd_monitor","Monitor first unconverged approximate singular value and error estimate","SVDMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
   if (flg) {
-    ierr = PetscViewerASCIIOpen(((PetscObject)svd)->comm,monfilename,&monviewer);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIOpen(PetscObjectComm((PetscObject)svd),monfilename,&monviewer);CHKERRQ(ierr);
     ierr = SVDMonitorSet(svd,SVDMonitorFirst,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
   }
   flg = PETSC_FALSE;
-  ierr = PetscOptionsBool("-svd_monitor_draw","Monitor first unconverged approximate singular value and error estimate graphically","SVDMonitorSet",flg,&flg,PETSC_NULL);CHKERRQ(ierr); 
+  ierr = PetscOptionsBool("-svd_monitor_lg","Monitor first unconverged approximate singular value and error estimate graphically","SVDMonitorSet",flg,&flg,NULL);CHKERRQ(ierr);
   if (flg) {
-    ierr = SVDMonitorSet(svd,SVDMonitorLG,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+    ierr = SVDMonitorSet(svd,SVDMonitorLG,NULL,NULL);CHKERRQ(ierr);
   }
   flg = PETSC_FALSE;
-  ierr = PetscOptionsBool("-svd_monitor_draw_all","Monitor error estimates graphically","SVDMonitorSet",flg,&flg,PETSC_NULL);CHKERRQ(ierr); 
+  ierr = PetscOptionsBool("-svd_monitor_lg_all","Monitor error estimates graphically","SVDMonitorSet",flg,&flg,NULL);CHKERRQ(ierr);
   if (flg) {
-    ierr = SVDMonitorSet(svd,SVDMonitorLGAll,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+    ierr = SVDMonitorSet(svd,SVDMonitorLGAll,NULL,NULL);CHKERRQ(ierr);
     ierr = SVDSetTrackAll(svd,PETSC_TRUE);CHKERRQ(ierr);
   }
   if (svd->ops->setfromoptions) {
@@ -476,11 +477,13 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
 
   if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
   ierr = IPSetFromOptions(svd->ip);CHKERRQ(ierr);
+  if (!svd->ds) { ierr = SVDGetDS(svd,&svd->ds);CHKERRQ(ierr); }
+  ierr = DSSetFromOptions(svd->ds);CHKERRQ(ierr);
   ierr = PetscRandomSetFromOptions(svd->rand);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDSetTrackAll"
 /*@
    SVDSetTrackAll - Specifies if the solver must compute the residual norm of all
@@ -495,10 +498,10 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
    Notes:
    If the user sets trackall=PETSC_TRUE then the solver computes (or estimates)
    the residual norm for each singular value approximation. Computing the residual is
-   usually an expensive operation and solvers commonly compute only the residual 
+   usually an expensive operation and solvers commonly compute only the residual
    associated to the first unconverged singular value.
 
-   The options '-svd_monitor_all' and '-svd_monitor_draw_all' automatically
+   The options '-svd_monitor_all' and '-svd_monitor_lg_all' automatically
    activate this option.
 
    Level: intermediate
@@ -514,7 +517,7 @@ PetscErrorCode SVDSetTrackAll(SVD svd,PetscBool trackall)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDGetTrackAll"
 /*@
    SVDGetTrackAll - Returns the flag indicating whether all residual norms must
@@ -532,7 +535,7 @@ PetscErrorCode SVDSetTrackAll(SVD svd,PetscBool trackall)
 
 .seealso: SVDSetTrackAll()
 @*/
-PetscErrorCode SVDGetTrackAll(SVD svd,PetscBool *trackall) 
+PetscErrorCode SVDGetTrackAll(SVD svd,PetscBool *trackall)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
@@ -542,10 +545,10 @@ PetscErrorCode SVDGetTrackAll(SVD svd,PetscBool *trackall)
 }
 
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDSetOptionsPrefix"
 /*@C
-   SVDSetOptionsPrefix - Sets the prefix used for searching for all 
+   SVDSetOptionsPrefix - Sets the prefix used for searching for all
    SVD options in the database.
 
    Logically Collective on SVD
@@ -580,9 +583,11 @@ PetscErrorCode SVDSetOptionsPrefix(SVD svd,const char *prefix)
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
   ierr = IPSetOptionsPrefix(svd->ip,prefix);CHKERRQ(ierr);
+  if (!svd->ds) { ierr = SVDGetDS(svd,&svd->ds);CHKERRQ(ierr); }
+  ierr = DSSetOptionsPrefix(svd->ds,prefix);CHKERRQ(ierr);
   ierr = PetscObjectSetOptionsPrefix((PetscObject)svd,prefix);CHKERRQ(ierr);
-  ierr = PetscTypeCompare((PetscObject)svd,SVDCROSS,&flg1);CHKERRQ(ierr);
-  ierr = PetscTypeCompare((PetscObject)svd,SVDCYCLIC,&flg2);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)svd,SVDCROSS,&flg1);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)svd,SVDCYCLIC,&flg2);CHKERRQ(ierr);
   if (flg1) {
     ierr = SVDCrossGetEPS(svd,&eps);CHKERRQ(ierr);
   } else if (flg2) {
@@ -592,13 +597,13 @@ PetscErrorCode SVDSetOptionsPrefix(SVD svd,const char *prefix)
     ierr = EPSSetOptionsPrefix(eps,prefix);CHKERRQ(ierr);
     ierr = EPSAppendOptionsPrefix(eps,"svd_");CHKERRQ(ierr);
   }
-  PetscFunctionReturn(0);  
+  PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDAppendOptionsPrefix"
 /*@C
-   SVDAppendOptionsPrefix - Appends to the prefix used for searching for all 
+   SVDAppendOptionsPrefix - Appends to the prefix used for searching for all
    SVD options in the database.
 
    Logically Collective on SVD
@@ -620,14 +625,16 @@ PetscErrorCode SVDAppendOptionsPrefix(SVD svd,const char *prefix)
   PetscErrorCode ierr;
   PetscBool      flg1,flg2;
   EPS            eps;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
   ierr = IPSetOptionsPrefix(svd->ip,prefix);CHKERRQ(ierr);
+  if (!svd->ds) { ierr = SVDGetDS(svd,&svd->ds);CHKERRQ(ierr); }
+  ierr = DSSetOptionsPrefix(svd->ds,prefix);CHKERRQ(ierr);
   ierr = PetscObjectAppendOptionsPrefix((PetscObject)svd,prefix);CHKERRQ(ierr);
-  ierr = PetscTypeCompare((PetscObject)svd,SVDCROSS,&flg1);CHKERRQ(ierr);
-  ierr = PetscTypeCompare((PetscObject)svd,SVDCYCLIC,&flg2);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)svd,SVDCROSS,&flg1);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)svd,SVDCYCLIC,&flg2);CHKERRQ(ierr);
   if (flg1) {
     ierr = SVDCrossGetEPS(svd,&eps);CHKERRQ(ierr);
   } else if (flg2) {
@@ -640,10 +647,10 @@ PetscErrorCode SVDAppendOptionsPrefix(SVD svd,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDGetOptionsPrefix"
 /*@C
-   SVDGetOptionsPrefix - Gets the prefix used for searching for all 
+   SVDGetOptionsPrefix - Gets the prefix used for searching for all
    SVD options in the database.
 
    Not Collective
