@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,9 +21,9 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/svdimpl.h>   /*I "slepcsvd.h" I*/
+#include <slepc-private/svdimpl.h>   /*I "slepcsvd.h" I*/
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDSolve"
 /*@
    SVDSolve - Solves the singular value problem.
@@ -33,33 +33,41 @@
    Input Parameter:
 .  svd - singular value solver context obtained from SVDCreate()
 
-   Options Database:
-.   -svd_view - print information about the solver used
+   Options Database Keys:
++  -svd_view - print information about the solver used
+-  -svd_view_mat binary - save the matrix to the default binary viewer
 
    Level: beginner
 
-.seealso: SVDCreate(), SVDSetUp(), SVDDestroy() 
+.seealso: SVDCreate(), SVDSetUp(), SVDDestroy()
 @*/
-PetscErrorCode SVDSolve(SVD svd) 
+PetscErrorCode SVDSolve(SVD svd)
 {
-  PetscErrorCode ierr;
-  PetscBool      flg;
-  PetscInt       i,*workperm;
-  char           filename[PETSC_MAX_PATH_LEN];
-  PetscViewer    viewer;
+  PetscErrorCode    ierr;
+  PetscBool         flg;
+  PetscInt          i,*workperm;
+  PetscViewer       viewer;
+  PetscViewerFormat format;
+  PetscErrorCode    (*which_func)(PetscScalar,PetscScalar,PetscScalar,PetscScalar,PetscInt*,void*);
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  if (!svd->setupcalled) { ierr = SVDSetUp(svd);CHKERRQ(ierr); }
+  ierr = PetscLogEventBegin(SVD_Solve,svd,0,0,0);CHKERRQ(ierr);
+
+  /* call setup */
+  ierr = SVDSetUp(svd);CHKERRQ(ierr);
   svd->its = 0;
   svd->nconv = 0;
-  svd->reason = SVD_CONVERGED_ITERATING;
-  for (i=0;i<svd->ncv;i++) svd->sigma[i]=svd->errest[i]=0.0;
+  for (i=0;i<svd->ncv;i++) {
+    svd->sigma[i]  = 0.0;
+    svd->errest[i] = 0.0;
+  }
   ierr = SVDMonitor(svd,svd->its,svd->nconv,svd->sigma,svd->errest,svd->ncv);CHKERRQ(ierr);
 
-  ierr = PetscLogEventBegin(SVD_Solve,svd,0,0,0);CHKERRQ(ierr);
+  which_func = (svd->which==SVD_LARGEST)? SlepcCompareLargestReal: SlepcCompareSmallestReal;
+  ierr = DSSetEigenvalueComparison(svd->ds,which_func,NULL);CHKERRQ(ierr);
+
   ierr = (*svd->ops->solve)(svd);CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(SVD_Solve,svd,0,0,0);CHKERRQ(ierr);
 
   /* sort singular triplets */
   if (svd->which == SVD_SMALLEST) {
@@ -73,10 +81,16 @@ PetscErrorCode SVDSolve(SVD svd)
     ierr = PetscFree(workperm);CHKERRQ(ierr);
   }
 
-  ierr = PetscOptionsGetString(((PetscObject)svd)->prefix,"-svd_view",filename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(SVD_Solve,svd,0,0,0);CHKERRQ(ierr);
+
+  /* various viewers */
+  ierr = MatViewFromOptions(svd->OP,((PetscObject)svd)->prefix,"-svd_view_mat");CHKERRQ(ierr);
+
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)svd),((PetscObject)svd)->prefix,"-svd_view",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg && !PetscPreLoadingOn) {
-    ierr = PetscViewerASCIIOpen(((PetscObject)svd)->comm,filename,&viewer);CHKERRQ(ierr);
-    ierr = SVDView(svd,viewer);CHKERRQ(ierr); 
+    ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
+    ierr = SVDView(svd,viewer);CHKERRQ(ierr);
+    ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   }
 
@@ -85,13 +99,13 @@ PetscErrorCode SVDSolve(SVD svd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDGetIterationNumber"
 /*@
-   SVDGetIterationNumber - Gets the current iteration number. If the 
-   call to SVDSolve() is complete, then it returns the number of iterations 
+   SVDGetIterationNumber - Gets the current iteration number. If the
+   call to SVDSolve() is complete, then it returns the number of iterations
    carried out by the solution method.
- 
+
    Not Collective
 
    Input Parameter:
@@ -103,10 +117,10 @@ PetscErrorCode SVDSolve(SVD svd)
    Level: intermediate
 
    Notes:
-      During the i-th iteration this call returns i-1. If SVDSolve() is 
+      During the i-th iteration this call returns i-1. If SVDSolve() is
       complete, then parameter "its" contains either the iteration number at
-      which convergence was successfully reached, or failure was detected.  
-      Call SVDGetConvergedReason() to determine if the solver converged or 
+      which convergence was successfully reached, or failure was detected.
+      Call SVDGetConvergedReason() to determine if the solver converged or
       failed and why.
 
 @*/
@@ -119,10 +133,10 @@ PetscErrorCode SVDGetIterationNumber(SVD svd,PetscInt *its)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDGetConvergedReason"
 /*@C
-   SVDGetConvergedReason - Gets the reason why the SVDSolve() iteration was 
+   SVDGetConvergedReason - Gets the reason why the SVDSolve() iteration was
    stopped.
 
    Not Collective
@@ -154,7 +168,7 @@ PetscErrorCode SVDGetConvergedReason(SVD svd,SVDConvergedReason *reason)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDGetConverged"
 /*@
    SVDGetConverged - Gets the number of converged singular values.
@@ -163,9 +177,9 @@ PetscErrorCode SVDGetConvergedReason(SVD svd,SVDConvergedReason *reason)
 
    Input Parameter:
 .  svd - the singular value solver context
-  
+
    Output Parameter:
-.  nconv - number of converged singular values 
+.  nconv - number of converged singular values
 
    Note:
    This function should be called after SVDSolve() has finished.
@@ -178,24 +192,21 @@ PetscErrorCode SVDGetConverged(SVD svd,PetscInt *nconv)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidIntPointer(nconv,2);
-  if (svd->reason == SVD_CONVERGED_ITERATING) { 
-    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_WRONGSTATE,"SVDSolve must be called first"); 
-  }
   *nconv = svd->nconv;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "SVDGetSingularTriplet" 
+#undef __FUNCT__
+#define __FUNCT__ "SVDGetSingularTriplet"
 /*@
    SVDGetSingularTriplet - Gets the i-th triplet of the singular value decomposition
-   as computed by SVDSolve(). The solution consists in the singular value and its left 
+   as computed by SVDSolve(). The solution consists in the singular value and its left
    and right singular vectors.
 
    Not Collective, but vectors are shared by all processors that share the SVD
 
    Input Parameters:
-+  svd - singular value solver context 
++  svd - singular value solver context
 -  i   - index of the solution
 
    Output Parameters:
@@ -205,7 +216,7 @@ PetscErrorCode SVDGetConverged(SVD svd,PetscInt *nconv)
 
    Note:
    The index i should be a value between 0 and nconv-1 (see SVDGetConverged()).
-   Both U or V can be PETSC_NULL if singular vectors are not required. 
+   Both U or V can be NULL if singular vectors are not required.
 
    Level: beginner
 
@@ -222,21 +233,18 @@ PetscErrorCode SVDGetSingularTriplet(SVD svd,PetscInt i,PetscReal *sigma,Vec u,V
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   if (u) { PetscValidHeaderSpecific(u,VEC_CLASSID,4); PetscCheckSameComm(svd,1,u,4); }
   if (v) { PetscValidHeaderSpecific(v,VEC_CLASSID,5); PetscCheckSameComm(svd,1,v,5); }
-  if (svd->reason == SVD_CONVERGED_ITERATING) { 
-    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_WRONGSTATE,"SVDSolve must be called first"); 
-  }
-  if (i<0 || i>=svd->nconv) { 
-    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range"); 
-  }
+  if (svd->reason == SVD_CONVERGED_ITERATING) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_WRONGSTATE,"SVDSolve must be called first");
+  if (i<0 || i>=svd->nconv) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
   *sigma = svd->sigma[svd->perm[i]];
   ierr = MatGetSize(svd->OP,&M,&N);CHKERRQ(ierr);
   if (M<N) { w = u; u = v; v = w; }
   if (u) {
     if (!svd->U) {
       ierr = VecDuplicateVecs(svd->tl,svd->ncv,&svd->U);CHKERRQ(ierr);
+      ierr = PetscLogObjectParents(svd,svd->ncv,svd->U);CHKERRQ(ierr);
       for (j=0;j<svd->nconv;j++) {
         ierr = SVDMatMult(svd,PETSC_FALSE,svd->V[j],svd->U[j]);CHKERRQ(ierr);
-        ierr = IPOrthogonalize(svd->ip,0,PETSC_NULL,j,PETSC_NULL,svd->U,svd->U[j],PETSC_NULL,&norm,PETSC_NULL);CHKERRQ(ierr);
+        ierr = IPOrthogonalize(svd->ip,0,NULL,j,NULL,svd->U,svd->U[j],NULL,&norm,NULL);CHKERRQ(ierr);
         ierr = VecScale(svd->U[j],1.0/norm);CHKERRQ(ierr);
       }
     }
@@ -248,10 +256,10 @@ PetscErrorCode SVDGetSingularTriplet(SVD svd,PetscInt i,PetscReal *sigma,Vec u,V
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDComputeResidualNorms"
 /*@
-   SVDComputeResidualNorms - Computes the norms of the residual vectors associated with 
+   SVDComputeResidualNorms - Computes the norms of the residual vectors associated with
    the i-th computed singular triplet.
 
    Collective on SVD
@@ -261,13 +269,13 @@ PetscErrorCode SVDGetSingularTriplet(SVD svd,PetscInt i,PetscReal *sigma,Vec u,V
 -  i    - the solution index
 
    Output Parameters:
-+  norm1 - the norm ||A*v-sigma*u||_2 where sigma is the 
-           singular value, u and v are the left and right singular vectors. 
++  norm1 - the norm ||A*v-sigma*u||_2 where sigma is the
+           singular value, u and v are the left and right singular vectors.
 -  norm2 - the norm ||A^T*u-sigma*v||_2 with the same sigma, u and v
 
    Note:
    The index i should be a value between 0 and nconv-1 (see SVDGetConverged()).
-   Both output parameters can be PETSC_NULL on input if not needed.
+   Both output parameters can be NULL on input if not needed.
 
    Level: beginner
 
@@ -276,20 +284,16 @@ PetscErrorCode SVDGetSingularTriplet(SVD svd,PetscInt i,PetscReal *sigma,Vec u,V
 PetscErrorCode SVDComputeResidualNorms(SVD svd,PetscInt i,PetscReal *norm1,PetscReal *norm2)
 {
   PetscErrorCode ierr;
-  Vec            u,v,x = PETSC_NULL,y = PETSC_NULL;
+  Vec            u,v,x = NULL,y = NULL;
   PetscReal      sigma;
   PetscInt       M,N;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidLogicalCollectiveInt(svd,i,2);
-  if (svd->reason == SVD_CONVERGED_ITERATING) { 
-    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_WRONGSTATE,"SVDSolve must be called first"); 
-  }
-  if (i<0 || i>=svd->nconv) { 
-    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range"); 
-  }
-  
+  if (svd->reason == SVD_CONVERGED_ITERATING) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_WRONGSTATE,"SVDSolve must be called first");
+  if (i<0 || i>=svd->nconv) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
+
   ierr = MatGetVecs(svd->OP,&v,&u);CHKERRQ(ierr);
   ierr = SVDGetSingularTriplet(svd,i,&sigma,u,v);CHKERRQ(ierr);
   if (norm1) {
@@ -308,7 +312,11 @@ PetscErrorCode SVDComputeResidualNorms(SVD svd,PetscInt i,PetscReal *norm1,Petsc
         ierr = MatMult(svd->AT,u,y);CHKERRQ(ierr);
       }
     } else {
+#if defined(PETSC_USE_COMPLEX)
+      ierr = MatMultHermitianTranspose(svd->OP,u,y);CHKERRQ(ierr);
+#else
       ierr = MatMultTranspose(svd->OP,u,y);CHKERRQ(ierr);
+#endif
     }
     ierr = VecAXPY(y,-sigma,v);CHKERRQ(ierr);
     ierr = VecNorm(y,NORM_2,norm2);CHKERRQ(ierr);
@@ -321,10 +329,10 @@ PetscErrorCode SVDComputeResidualNorms(SVD svd,PetscInt i,PetscReal *norm1,Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDComputeRelativeError"
 /*@
-   SVDComputeRelativeError - Computes the relative error bound associated 
+   SVDComputeRelativeError - Computes the relative error bound associated
    with the i-th singular triplet.
 
    Collective on SVD
@@ -335,7 +343,7 @@ PetscErrorCode SVDComputeResidualNorms(SVD svd,PetscInt i,PetscReal *norm1,Petsc
 
    Output Parameter:
 .  error - the relative error bound, computed as sqrt(n1^2+n2^2)/sigma
-   where n1 = ||A*v-sigma*u||_2 , n2 = ||A^T*u-sigma*v||_2 , sigma is the singular value, 
+   where n1 = ||A*v-sigma*u||_2 , n2 = ||A^T*u-sigma*v||_2 , sigma is the singular value,
    u and v are the left and right singular vectors.
    If sigma is too small the relative error is computed as sqrt(n1^2+n2^2).
 
@@ -352,17 +360,17 @@ PetscErrorCode SVDComputeRelativeError(SVD svd,PetscInt i,PetscReal *error)
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidLogicalCollectiveInt(svd,i,2);
   PetscValidPointer(error,3);
-  ierr = SVDGetSingularTriplet(svd,i,&sigma,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+  ierr = SVDGetSingularTriplet(svd,i,&sigma,NULL,NULL);CHKERRQ(ierr);
   ierr = SVDComputeResidualNorms(svd,i,&norm1,&norm2);CHKERRQ(ierr);
   *error = PetscSqrtReal(norm1*norm1+norm2*norm2);
   if (sigma>*error) *error /= sigma;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SVDGetOperationCounters"
 /*@
-   SVDGetOperationCounters - Gets the total number of matrix vector and dot 
+   SVDGetOperationCounters - Gets the total number of matrix vector and dot
    products used by the SVD object during the last SVDSolve() call.
 
    Not Collective
@@ -383,10 +391,10 @@ PetscErrorCode SVDComputeRelativeError(SVD svd,PetscInt i,PetscReal *error)
 PetscErrorCode SVDGetOperationCounters(SVD svd,PetscInt* matvecs,PetscInt* dots)
 {
   PetscErrorCode ierr;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  if (matvecs) *matvecs = svd->matvecs; 
+  if (matvecs) *matvecs = svd->matvecs;
   if (dots) {
     if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
     ierr = IPGetOperationCounters(svd->ip,dots);CHKERRQ(ierr);
