@@ -1,17 +1,17 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -19,16 +19,49 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/slepcimpl.h>           /*I "slepcsys.h" I*/
-#include <private/epsimpl.h>
-#include <private/stimpl.h>
-#include <private/svdimpl.h>
-#include <private/qepimpl.h>
-#include <private/ipimpl.h>
-#include <private/vecimplslepc.h>
+#include <slepc-private/slepcimpl.h>           /*I "slepcsys.h" I*/
+#include <slepc-private/epsimpl.h>
+#include <slepc-private/stimpl.h>
+#include <slepc-private/svdimpl.h>
+#include <slepc-private/qepimpl.h>
+#include <slepc-private/nepimpl.h>
+#include <slepc-private/mfnimpl.h>
+#include <slepc-private/ipimpl.h>
+#include <slepc-private/dsimpl.h>
+#include <slepc-private/fnimpl.h>
+#include <slepc-private/vecimplslepc.h>
 #include <stdlib.h>
 
-#undef __FUNCT__  
+#undef __FUNCT__
+#define __FUNCT__ "SlepcGetVersion"
+/*@C
+    SlepcGetVersion - Gets the SLEPc version information in a string.
+
+    Input Parameter:
+.   len - length of the string
+
+    Output Parameter:
+.   version - version string
+
+    Fortran Note:
+    This routine is not supported in Fortran.
+
+    Level: developer
+@*/
+PetscErrorCode SlepcGetVersion(char version[],size_t len)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+#if (SLEPC_VERSION_RELEASE == 1)
+  ierr = PetscSNPrintf(version,len,"SLEPc Release Version %d.%d.%d, %s",SLEPC_VERSION_MAJOR,SLEPC_VERSION_MINOR,SLEPC_VERSION_SUBMINOR,SLEPC_VERSION_DATE);CHKERRQ(ierr);
+#else
+  ierr = PetscSNPrintf(version,len,"SLEPc Development GIT revision: %d  GIT Date: %s",SLEPC_VERSION_GIT,SLEPC_VERSION_DATE_GIT);CHKERRQ(ierr);
+#endif
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
 #define __FUNCT__ "SlepcPrintVersion"
 /*
    SlepcPrintVersion - Prints SLEPc version info.
@@ -39,7 +72,7 @@ PetscErrorCode SlepcPrintVersion(MPI_Comm comm)
 {
   PetscErrorCode ierr;
   char           version[256];
-  
+
   PetscFunctionBegin;
   ierr = SlepcGetVersion(version,256);CHKERRQ(ierr);
   ierr = (*PetscHelpPrintf)(comm,"--------------------------------------------------------------------------\n");CHKERRQ(ierr);
@@ -50,7 +83,7 @@ PetscErrorCode SlepcPrintVersion(MPI_Comm comm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SlepcPrintHelpIntro"
 /*
    SlepcPrintHelpIntro - Prints introductory SLEPc help info.
@@ -60,7 +93,7 @@ PetscErrorCode SlepcPrintVersion(MPI_Comm comm)
 PetscErrorCode SlepcPrintHelpIntro(MPI_Comm comm)
 {
   PetscErrorCode  ierr;
-  
+
   PetscFunctionBegin;
   ierr = (*PetscHelpPrintf)(comm,"SLEPc help information includes that for the PETSc libraries, which provide\n");CHKERRQ(ierr);
   ierr = (*PetscHelpPrintf)(comm,"low-level system infrastructure and linear algebra tools.\n");CHKERRQ(ierr);
@@ -70,23 +103,20 @@ PetscErrorCode SlepcPrintHelpIntro(MPI_Comm comm)
 
 /* ------------------------Nasty global variables -------------------------------*/
 /*
-   Indicates whether SLEPc started PETSc, or whether it was 
+   Indicates whether SLEPc started PETSc, or whether it was
    already started before SLEPc was initialized.
 */
-PetscBool SlepcBeganPetsc = PETSC_FALSE; 
+PetscBool SlepcBeganPetsc = PETSC_FALSE;
 PetscBool SlepcInitializeCalled = PETSC_FALSE;
-extern PetscLogEvent SLEPC_UpdateVectors,SLEPC_VecMAXPBY,SLEPC_SlepcDenseMatProd,SLEPC_SlepcDenseOrth,
-                     SLEPC_SlepcDenseMatInvProd,SLEPC_SlepcDenseNorm,SLEPC_SlepcDenseCopy,SLEPC_VecsMult;
 
 #if defined(PETSC_USE_DYNAMIC_LIBRARIES)
-extern PetscDLLibrary DLLibrariesLoaded;
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SlepcInitialize_DynamicLibraries"
 /*
-    SlepcInitialize_DynamicLibraries - Adds the default dynamic link libraries to the 
+    SlepcInitialize_DynamicLibraries - Adds the default dynamic link libraries to the
     search path.
-*/ 
+*/
 PetscErrorCode SlepcInitialize_DynamicLibraries(void)
 {
   PetscErrorCode ierr;
@@ -98,39 +128,17 @@ PetscErrorCode SlepcInitialize_DynamicLibraries(void)
   ierr = PetscStrcat(libs,"/libslepc");CHKERRQ(ierr);
   ierr = PetscDLLibraryRetrieve(PETSC_COMM_WORLD,libs,dlib,1024,&found);CHKERRQ(ierr);
   if (found) {
-    ierr = PetscDLLibraryAppend(PETSC_COMM_WORLD,&DLLibrariesLoaded,libs);CHKERRQ(ierr);
-  } else {
-    SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc dynamic library\n You cannot move the dynamic libraries!");
-  }
+    ierr = PetscDLLibraryAppend(PETSC_COMM_WORLD,&PetscDLLibrariesLoaded,libs);CHKERRQ(ierr);
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc dynamic library\nYou cannot move the dynamic libraries");
   PetscFunctionReturn(0);
 }
 #endif
 
-#undef __FUNCT__  
-#define __FUNCT__ "SlepcInitialize_Packages"
-/*
-    SlepcInitialize_Packages - Initialize all SLEPc packages at the initialization.
-*/ 
-PetscErrorCode SlepcInitialize_Packages(void)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  ierr = STInitializePackage(PETSC_NULL);CHKERRQ(ierr);
-  ierr = EPSInitializePackage(PETSC_NULL);CHKERRQ(ierr);
-  ierr = SVDInitializePackage(PETSC_NULL);CHKERRQ(ierr);
-  ierr = QEPInitializePackage(PETSC_NULL);CHKERRQ(ierr);
-  ierr = IPInitializePackage(PETSC_NULL);CHKERRQ(ierr);
-  /* New special type of Vec, implemented in SLEPc */
-  ierr = VecRegister_Comp(PETSC_NULL);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SlepcInitialize_LogEvents"
 /*
     SlepcInitialize_LogEvents - Initialize log events not pertaining to any object class.
-*/ 
+*/
 PetscErrorCode SlepcInitialize_LogEvents(void)
 {
   PetscErrorCode ierr;
@@ -139,16 +147,15 @@ PetscErrorCode SlepcInitialize_LogEvents(void)
   ierr = PetscLogEventRegister("UpdateVectors",0,&SLEPC_UpdateVectors);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("VecMAXPBY",0,&SLEPC_VecMAXPBY);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("DenseMatProd",EPS_CLASSID,&SLEPC_SlepcDenseMatProd);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("DenseOrth",EPS_CLASSID,&SLEPC_SlepcDenseOrth);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("DenseMatNorm",EPS_CLASSID,&SLEPC_SlepcDenseNorm);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("DenseCopy",EPS_CLASSID,&SLEPC_SlepcDenseCopy);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("VecsMult",EPS_CLASSID,&SLEPC_VecsMult);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SlepcInitialize"
-/*@C 
+/*@C
    SlepcInitialize - Initializes the SLEPc library. SlepcInitialize() calls
    PetscInitialize() if that has not been called yet, so this routine should
    always be called near the beginning of your program.
@@ -159,36 +166,32 @@ PetscErrorCode SlepcInitialize_LogEvents(void)
 +  argc - count of number of command line arguments
 .  args - the command line arguments
 .  file - [optional] PETSc database file, defaults to ~username/.petscrc
-          (use PETSC_NULL for default)
--  help - [optional] Help message to print, use PETSC_NULL for no message
+          (use NULL for default)
+-  help - [optional] Help message to print, use NULL for no message
 
    Fortran Note:
    Fortran syntax is very similar to that of PetscInitialize()
-   
+
    Level: beginner
 
 .seealso: SlepcFinalize(), PetscInitialize()
 @*/
 PetscErrorCode SlepcInitialize(int *argc,char ***args,const char file[],const char help[])
 {
-  PetscErrorCode ierr,info=0;
+  PetscErrorCode ierr;
   PetscBool      flg;
 
   PetscFunctionBegin;
-  if (SlepcInitializeCalled) {
-    PetscFunctionReturn(0); 
-  }
-  info = PetscSetHelpVersionFunctions(SlepcPrintHelpIntro,SlepcPrintVersion);CHKERRQ(info);
+  if (SlepcInitializeCalled) PetscFunctionReturn(0);
+  ierr = PetscSetHelpVersionFunctions(SlepcPrintHelpIntro,SlepcPrintVersion);CHKERRQ(ierr);
   ierr = PetscInitialized(&flg);CHKERRQ(ierr);
   if (!flg) {
-    info = PetscInitialize(argc,args,file,help);CHKERRQ(info);
+    ierr = PetscInitialize(argc,args,file,help);CHKERRQ(ierr);
     SlepcBeganPetsc = PETSC_TRUE;
   }
 
 #if defined(PETSC_USE_DYNAMIC_LIBRARIES)
   ierr = SlepcInitialize_DynamicLibraries();CHKERRQ(ierr);
-#else
-  ierr = SlepcInitialize_Packages();CHKERRQ(ierr);
 #endif
   ierr = SlepcInitialize_LogEvents();CHKERRQ(ierr);
 
@@ -198,11 +201,11 @@ PetscErrorCode SlepcInitialize(int *argc,char ***args,const char file[],const ch
 #endif
 
   SlepcInitializeCalled = PETSC_TRUE;
-  PetscInfo(0,"SLEPc successfully started\n");
-  PetscFunctionReturn(info);
+  ierr = PetscInfo(0,"SLEPc successfully started\n");CHKERRQ(ierr);
+  PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SlepcFinalize"
 /*@
    SlepcFinalize - Checks for options to be called at the conclusion
@@ -216,22 +219,45 @@ PetscErrorCode SlepcInitialize(int *argc,char ***args,const char file[],const ch
 @*/
 PetscErrorCode SlepcFinalize(void)
 {
-  PetscErrorCode info=0;
-  
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
-  PetscInfo(0,"SLEPc successfully ended!\n");
+  ierr = PetscInfo(0,"SlepcFinalize() called\n");CHKERRQ(ierr);
   if (SlepcBeganPetsc) {
-    info = PetscFinalize();CHKERRQ(info);
+    ierr = PetscFinalize();CHKERRQ(ierr);
   }
   SlepcInitializeCalled = PETSC_FALSE;
-  PetscFunctionReturn(info);
+  PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
+#define __FUNCT__ "SlepcInitializeNoArguments"
+/*@C
+   SlepcInitializeNoArguments - Calls SlepcInitialize() from C/C++ without
+   the command line arguments.
+
+   Collective
+
+   Level: advanced
+
+.seealso: SlepcInitialize(), SlepcInitializeFortran()
+@*/
+PetscErrorCode SlepcInitializeNoArguments(void)
+{
+  PetscErrorCode ierr;
+  int            argc = 0;
+  char           **args = 0;
+
+  PetscFunctionBegin;
+  ierr = SlepcInitialize(&argc,&args,NULL,NULL);
+  PetscFunctionReturn(ierr);
+}
+
+#undef __FUNCT__
 #define __FUNCT__ "SlepcInitialized"
 /*@
    SlepcInitialized - Determine whether SLEPc is initialized.
-  
+
    Level: beginner
 
 .seealso: SlepcInitialize(), SlepcInitializeFortran()
@@ -244,63 +270,39 @@ PetscErrorCode SlepcInitialized(PetscBool *isInitialized)
   PetscFunctionReturn(0);
 }
 
-#if defined(PETSC_HAVE_MATLAB_ENGINE)
-extern PetscBool PetscBeganMPI;
+PETSC_EXTERN PetscBool PetscBeganMPI;
 
-#undef __FUNCT__  
-#define __FUNCT__ "SlepcInitializeMatlab"
+#undef __FUNCT__
+#define __FUNCT__ "SlepcInitializeNoPointers"
 /*
-   SlepcInitializeMatlab - Calls SlepcInitialize() from MATLAB (analogue to
-   PetscInitializeMatlab).
+   SlepcInitializeNoPointers - Calls SlepcInitialize() from C/C++ without the pointers
+   to argc and args (analogue to PetscInitializeNoPointers).
 
    Collective
-  
+
    Level: advanced
 
 .seealso: SlepcInitialize()
 */
-PetscErrorCode SlepcInitializeMatlab(int argc,char **args,const char *filename,const char *help)
+PetscErrorCode SlepcInitializeNoPointers(int argc,char **args,const char *filename,const char *help)
 {
   PetscErrorCode ierr;
   int            myargc = argc;
   char           **myargs = args;
 
   PetscFunctionBegin;
-  ierr = SlepcInitialize(&myargc,&myargs,filename,help);
+  ierr = SlepcInitialize(&myargc,&myargs,filename,help);CHKERRQ(ierr);
   ierr = PetscPopSignalHandler();CHKERRQ(ierr);
   PetscBeganMPI = PETSC_FALSE;
   PetscFunctionReturn(ierr);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "SlepcInitializedMatlab"
-/*
-   SlepcInitializedMatlab - Has SLEPc been initialized already?
+#if defined(PETSC_USE_DYNAMIC_LIBRARIES)
 
-   Not Collective
-  
-   Level: advanced
-
-   Notes: this is called only by the SLEPc MATLAB interface.
-
-.seealso: SlepcInitialize()
-*/
-int SlepcInitializedMatlab(void)
-{
-  PetscBool flg;
-
-  SlepcInitialized(&flg);
-  if (flg) return 1;
-  else return 0;
-}
-#endif
-
-#ifdef PETSC_USE_DYNAMIC_LIBRARIES
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "PetscDLLibraryRegister_slepc"
 /*
-  PetscDLLibraryRegister - This function is called when the dynamic library 
+  PetscDLLibraryRegister - This function is called when the dynamic library
   it is in is opened.
 
   This one registers all the EPS and ST methods in the libslepc.a
@@ -309,18 +311,21 @@ EXTERN_C_BEGIN
   Input Parameter:
   path - library path
  */
-PetscErrorCode PetscDLLibraryRegister_slepc(char *path)
+PETSC_EXTERN PetscErrorCode PetscDLLibraryRegister_slepc(char *path)
 {
   PetscErrorCode ierr;
 
-  ierr = PetscInitializeNoArguments(); if (ierr) return 1;
-
   PetscFunctionBegin;
-  /*
-      If we got here then PETSc was properly loaded
-  */
-  ierr = SlepcInitialize_Packages();CHKERRQ(ierr);
+  ierr = EPSInitializePackage();CHKERRQ(ierr);
+  ierr = SVDInitializePackage();CHKERRQ(ierr);
+  ierr = QEPInitializePackage();CHKERRQ(ierr);
+  ierr = NEPInitializePackage();CHKERRQ(ierr);
+  ierr = MFNInitializePackage();CHKERRQ(ierr);
+  ierr = STInitializePackage();CHKERRQ(ierr);
+  ierr = IPInitializePackage();CHKERRQ(ierr);
+  ierr = DSInitializePackage();CHKERRQ(ierr);
+  ierr = FNInitializePackage();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 #endif /* PETSC_USE_DYNAMIC_LIBRARIES */
+
