@@ -3,17 +3,17 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -21,7 +21,8 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/vecimplslepc.h>            /*I "slepcvec.h" I*/
+#include <slepc-private/vecimplslepc.h>       /*I "slepcvec.h" I*/
+#include <petsc-private/vecimpl.h>            /*I "petscvec.h" I*/
 #include <petscblaslapack.h>
 
 PetscLogEvent SLEPC_UpdateVectors = 0,SLEPC_VecMAXPBY = 0;
@@ -63,20 +64,20 @@ static PetscErrorCode VecDuplicateVecs_Contiguous(Vec v,PetscInt m,Vec *V[])
   ierr = PetscNew(Vecs_Contiguous,&vc);CHKERRQ(ierr);
   vc->nvecs = m;
   vc->array = pV;
-  ierr = PetscContainerCreate(((PetscObject)v)->comm,&container);CHKERRQ(ierr);
+  ierr = PetscContainerCreate(PetscObjectComm((PetscObject)v),&container);CHKERRQ(ierr);
   ierr = PetscContainerSetPointer(container,vc);CHKERRQ(ierr);
   ierr = PetscContainerSetUserDestroy(container,Vecs_ContiguousDestroy);CHKERRQ(ierr);
   /* Create vectors */
   ierr = PetscMalloc(m*sizeof(Vec),V);CHKERRQ(ierr);
   for (i=0;i<m;i++) {
-    ierr = VecCreateMPIWithArray(((PetscObject)v)->comm,nloc,PETSC_DECIDE,pV+i*nloc,*V+i);CHKERRQ(ierr);
+    ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)v),1,nloc,PETSC_DECIDE,pV+i*nloc,*V+i);CHKERRQ(ierr);
     ierr = PetscObjectCompose((PetscObject)*(*V+i),"contiguous",(PetscObject)container);CHKERRQ(ierr);
   }
   ierr = PetscContainerDestroy(&container);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SlepcVecSetTemplate"
 /*@
    SlepcVecSetTemplate - Sets a vector as a template for contiguous storage.
@@ -101,19 +102,19 @@ PetscErrorCode SlepcVecSetTemplate(Vec v)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(v,VEC_CLASSID,1);
-  ierr = PetscTypeCompareAny((PetscObject)v,&flg,VECSEQ,VECMPI,"");CHKERRQ(ierr);
-  if (!flg) SETERRQ(((PetscObject)v)->comm,PETSC_ERR_SUP,"Only available for standard vectors (VECSEQ or VECMPI)");
+  ierr = PetscObjectTypeCompareAny((PetscObject)v,&flg,VECSEQ,VECMPI,"");CHKERRQ(ierr);
+  if (!flg) SETERRQ(PetscObjectComm((PetscObject)v),PETSC_ERR_SUP,"Only available for standard vectors (VECSEQ or VECMPI)");
   v->ops->duplicatevecs = VecDuplicateVecs_Contiguous;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SlepcMatGetVecsTemplate"
 /*@
    SlepcMatGetVecsTemplate - Get vectors compatible with a matrix,
    i.e. with the same parallel layout, and mark them as templates
    for contiguous storage.
-   
+
    Collective on Mat
 
    Input Parameter:
@@ -145,9 +146,9 @@ PetscErrorCode SlepcMatGetVecsTemplate(Mat mat,Vec *right,Vec *left)
   PetscValidType(mat,1);
   ierr = MatGetVecs(mat,right,left);CHKERRQ(ierr);
   v = right? *right: *left;
-  ierr = PetscTypeCompareAny((PetscObject)v,&flg,VECSEQ,VECMPI,"");CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompareAny((PetscObject)v,&flg,VECSEQ,VECMPI,"");CHKERRQ(ierr);
   if (!flg) PetscFunctionReturn(0);
-  ierr = PetscOptionsHasName(PETSC_NULL,"-slepc_non_contiguous",&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,"-slepc_non_contiguous",&flg);CHKERRQ(ierr);
   if (!flg) {
     if (right) { ierr = SlepcVecSetTemplate(*right);CHKERRQ(ierr); }
     if (left) { ierr = SlepcVecSetTemplate(*left);CHKERRQ(ierr); }
@@ -155,7 +156,7 @@ PetscErrorCode SlepcMatGetVecsTemplate(Mat mat,Vec *right,Vec *left)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SlepcUpdateVectors_Noncontiguous_Inplace"
 /*
    SlepcUpdateVectors_Noncontiguous_Inplace - V = V*Q for regular vectors
@@ -171,9 +172,9 @@ static PetscErrorCode SlepcUpdateVectors_Noncontiguous_Inplace(PetscInt m_,Vec *
   PetscFunctionBegin;
   ierr = PetscLogEventBegin(SLEPC_UpdateVectors,0,0,0,0);CHKERRQ(ierr);
   ierr = VecGetLocalSize(V[0],&l);CHKERRQ(ierr);
-  ls = PetscBLASIntCast(l);  
-  m = PetscBLASIntCast(m_);
-  ldq = PetscBLASIntCast(ldq_);
+  ierr = PetscBLASIntCast(l,&ls);CHKERRQ(ierr);
+  ierr = PetscBLASIntCast(m_,&m);CHKERRQ(ierr);
+  ierr = PetscBLASIntCast(ldq_,&ldq);CHKERRQ(ierr);
   ierr = PetscMalloc(sizeof(PetscScalar)*2*bs*m,&work);CHKERRQ(ierr);
   out = work+m*bs;
   k = ls % bs;
@@ -181,26 +182,26 @@ static PetscErrorCode SlepcUpdateVectors_Noncontiguous_Inplace(PetscInt m_,Vec *
     for (j=0;j<m;j++) {
       ierr = VecGetArray(V[j],&pv);CHKERRQ(ierr);
       ierr = PetscMemcpy(work+j*bs,pv,k*sizeof(PetscScalar));CHKERRQ(ierr);
-      ierr = VecRestoreArray(V[j],&pv);CHKERRQ(ierr);    
+      ierr = VecRestoreArray(V[j],&pv);CHKERRQ(ierr);
     }
-    BLASgemm_("N",qtrans?"C":"N",&k,&m,&m,&one,work,&bs,pq,&ldq,&zero,out,&bs);
+    PetscStackCallBLAS("BLASgemm",BLASgemm_("N",qtrans?"C":"N",&k,&m,&m,&one,work,&bs,pq,&ldq,&zero,out,&bs));
     for (j=0;j<m;j++) {
       ierr = VecGetArray(V[j],&pv);CHKERRQ(ierr);
       ierr = PetscMemcpy(pv,out+j*bs,k*sizeof(PetscScalar));CHKERRQ(ierr);
-      ierr = VecRestoreArray(V[j],&pv);CHKERRQ(ierr);    
+      ierr = VecRestoreArray(V[j],&pv);CHKERRQ(ierr);
     }
   }
   for (;k<ls;k+=bs) {
     for (j=0;j<m;j++) {
       ierr = VecGetArray(V[j],&pv);CHKERRQ(ierr);
       ierr = PetscMemcpy(work+j*bs,pv+k,bs*sizeof(PetscScalar));CHKERRQ(ierr);
-      ierr = VecRestoreArray(V[j],&pv);CHKERRQ(ierr);    
+      ierr = VecRestoreArray(V[j],&pv);CHKERRQ(ierr);
     }
-    BLASgemm_("N",qtrans?"C":"N",&bs,&m,&m,&one,work,&bs,pq,&ldq,&zero,out,&bs);
+    PetscStackCallBLAS("BLASgemm",BLASgemm_("N",qtrans?"C":"N",&bs,&m,&m,&one,work,&bs,pq,&ldq,&zero,out,&bs));
     for (j=0;j<m;j++) {
       ierr = VecGetArray(V[j],&pv);CHKERRQ(ierr);
       ierr = PetscMemcpy(pv+k,out+j*bs,bs*sizeof(PetscScalar));CHKERRQ(ierr);
-      ierr = VecRestoreArray(V[j],&pv);CHKERRQ(ierr);    
+      ierr = VecRestoreArray(V[j],&pv);CHKERRQ(ierr);
     }
   }
   ierr = PetscFree(work);CHKERRQ(ierr);
@@ -209,10 +210,10 @@ static PetscErrorCode SlepcUpdateVectors_Noncontiguous_Inplace(PetscInt m_,Vec *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SlepcUpdateVectors_Noncontiguous"
 /*
-   SlepcUpdateVectors_Noncontiguous - V(:,s:e-1) = V*Q(:,s:e-1) for 
+   SlepcUpdateVectors_Noncontiguous - V(:,s:e-1) = V*Q(:,s:e-1) for
    regular vectors (non-contiguous).
 
    Writing V = [ V1 V2 V3 ] and Q = [ Q1 Q2 Q3 ], where the V2 and Q2
@@ -261,7 +262,7 @@ static PetscErrorCode SlepcUpdateVectors_Noncontiguous(PetscInt n,Vec *V,PetscIn
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SlepcUpdateVectors"
 /*@
    SlepcUpdateVectors - Update a set of vectors V as V(:,s:e-1) = V*Q(:,s:e-1).
@@ -279,7 +280,7 @@ static PetscErrorCode SlepcUpdateVectors_Noncontiguous(PetscInt n,Vec *V,PetscIn
    Input/Output parameter:
 .  V      - set of vectors
 
-   Notes: 
+   Notes:
    This function computes V(:,s:e-1) = V*Q(:,s:e-1), that is, given a set of
    vectors V, columns from s to e-1 are overwritten with columns from s to
    e-1 of the matrix-matrix product V*Q.
@@ -290,12 +291,13 @@ static PetscErrorCode SlepcUpdateVectors_Noncontiguous(PetscInt n,Vec *V,PetscIn
 
    If qtrans=PETSC_TRUE, the operation is V*Q'.
 
-   This routine is implemented with a call to BLAS, therefore V is an array 
+   This routine is implemented with a call to BLAS, therefore V is an array
    of Vec which have the data stored contiguously in memory as a Fortran matrix.
    PETSc does not create such arrays by default.
 
    Level: developer
 
+.seealso: SlepcUpdateStrideVectors()
 @*/
 PetscErrorCode SlepcUpdateVectors(PetscInt n,Vec *V,PetscInt s,PetscInt e,const PetscScalar *Q,PetscInt ldq,PetscBool qtrans)
 {
@@ -304,7 +306,7 @@ PetscErrorCode SlepcUpdateVectors(PetscInt n,Vec *V,PetscInt s,PetscInt e,const
 
   PetscFunctionBegin;
   if (n<0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Number of vectors (given %D) cannot be negative",n);
-  if (n==0 || s>=e) PetscFunctionReturn(0);
+  if (!n || s>=e) PetscFunctionReturn(0);
   PetscValidPointer(V,2);
   PetscValidHeaderSpecific(*V,VEC_CLASSID,2);
   PetscValidType(*V,2);
@@ -320,7 +322,7 @@ PetscErrorCode SlepcUpdateVectors(PetscInt n,Vec *V,PetscInt s,PetscInt e,const
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SlepcUpdateStrideVectors"
 /*@
    SlepcUpdateStrideVectors - Update a set of vectors V as
@@ -340,7 +342,7 @@ PetscErrorCode SlepcUpdateVectors(PetscInt n,Vec *V,PetscInt s,PetscInt e,const
    Input/Output parameter:
 .  V      - set of vectors
 
-   Notes: 
+   Notes:
    This function computes V(:,s:d:e-1) = V*Q(:,s:e-1), that is, given a set
    of vectors V, columns from s to e-1 are overwritten with columns from s to
    e-1 of the matrix-matrix product V*Q.
@@ -351,12 +353,13 @@ PetscErrorCode SlepcUpdateVectors(PetscInt n,Vec *V,PetscInt s,PetscInt e,const
 
    If qtrans=PETSC_TRUE, the operation is V*Q'.
 
-   This routine is implemented with a call to BLAS, therefore V is an array 
+   This routine is implemented with a call to BLAS, therefore V is an array
    of Vec which have the data stored contiguously in memory as a Fortran matrix.
    PETSc does not create such arrays by default.
 
    Level: developer
 
+.seealso: SlepcUpdateVectors()
 @*/
 PetscErrorCode SlepcUpdateStrideVectors(PetscInt n_,Vec *V,PetscInt s,PetscInt d,PetscInt e,const PetscScalar *Q,PetscInt ldq_,PetscBool qtrans)
 {
@@ -367,16 +370,16 @@ PetscErrorCode SlepcUpdateStrideVectors(PetscInt n_,Vec *V,PetscInt s,PetscInt d
   const char     *qt;
 
   PetscFunctionBegin;
-  n = PetscBLASIntCast(n_/d);
-  ldq = PetscBLASIntCast(ldq_);
+  ierr = PetscBLASIntCast(n_/d,&n);CHKERRQ(ierr);
+  ierr = PetscBLASIntCast(ldq_,&ldq);CHKERRQ(ierr);
   m = (e-s)/d;
-  if (m==0) PetscFunctionReturn(0);
+  if (!m) PetscFunctionReturn(0);
   PetscValidIntPointer(Q,5);
-  if (m<0 || n<0 || s<0 || m>n) SETERRQ(((PetscObject)*V)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Index argument out of range");
+  if (m<0 || n<0 || s<0 || m>n) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Index argument out of range");
   ierr = PetscLogEventBegin(SLEPC_UpdateVectors,0,0,0,0);CHKERRQ(ierr);
   ierr = VecGetLocalSize(V[0],&l);CHKERRQ(ierr);
-  ls = PetscBLASIntCast(l);
-  ld = ls*PetscBLASIntCast(d);
+  ierr = PetscBLASIntCast(l,&ls);CHKERRQ(ierr);
+  ierr = PetscBLASIntCast(ls*d,&ld);CHKERRQ(ierr);
   ierr = VecGetArray(V[0],&pv);CHKERRQ(ierr);
   if (qtrans) {
     pq = (PetscScalar*)Q+s;
@@ -388,17 +391,17 @@ PetscErrorCode SlepcUpdateStrideVectors(PetscInt n_,Vec *V,PetscInt s,PetscInt d
   ierr = PetscMalloc(sizeof(PetscScalar)*bs*m,&work);CHKERRQ(ierr);
   k = ls % bs;
   if (k) {
-    BLASgemm_("N",qt,&k,&m,&n,&one,pv,&ld,pq,&ldq,&zero,work,&k);
+    PetscStackCallBLAS("BLASgemm",BLASgemm_("N",qt,&k,&m,&n,&one,pv,&ld,pq,&ldq,&zero,work,&k));
     for (j=0;j<m;j++) {
       pw = pv+(s+j)*ld;
       pwork = work+j*k;
       for (i=0;i<k;i++) {
         *pw++ = *pwork++;
       }
-    }        
+    }
   }
   for (;k<ls;k+=bs) {
-    BLASgemm_("N",qt,&bs,&m,&n,&one,pv+k,&ld,pq,&ldq,&zero,work,&bs);
+    PetscStackCallBLAS("BLASgemm",BLASgemm_("N",qt,&bs,&m,&n,&one,pv+k,&ld,pq,&ldq,&zero,work,&bs));
     for (j=0;j<m;j++) {
       pw = pv+(s+j)*ld+k;
       pwork = work+j*bs;
@@ -414,7 +417,7 @@ PetscErrorCode SlepcUpdateStrideVectors(PetscInt n_,Vec *V,PetscInt s,PetscInt d
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "SlepcVecMAXPBY"
 /*@
    SlepcVecMAXPBY - Computes y = beta*y + sum alpha*a[j]*x[j]
@@ -450,8 +453,8 @@ PetscErrorCode SlepcVecMAXPBY(Vec y,PetscScalar beta,PetscScalar alpha,PetscInt
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(y,VEC_CLASSID,1);
-  if (!nv || !(y)->map->n) PetscFunctionReturn(0);
-  if (nv < 0) SETERRQ1(((PetscObject)y)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Number of vectors (given %D) cannot be negative",nv);
+  if (!nv) PetscFunctionReturn(0);
+  if (nv < 0) SETERRQ1(PetscObjectComm((PetscObject)y),PETSC_ERR_ARG_OUTOFRANGE,"Number of vectors (given %D) cannot be negative",nv);
   PetscValidLogicalCollectiveScalar(y,alpha,2);
   PetscValidLogicalCollectiveScalar(y,beta,3);
   PetscValidLogicalCollectiveInt(y,nv,4);
@@ -461,8 +464,8 @@ PetscErrorCode SlepcVecMAXPBY(Vec y,PetscScalar beta,PetscScalar alpha,PetscInt
   PetscValidType(y,1);
   PetscValidType(*x,6);
   PetscCheckSameTypeAndComm(y,1,*x,6);
-  if ((*x)->map->N != (y)->map->N) SETERRQ(((PetscObject)y)->comm,PETSC_ERR_ARG_INCOMP,"Incompatible vector global lengths");
-  if ((*x)->map->n != (y)->map->n) SETERRQ(((PetscObject)y)->comm,PETSC_ERR_ARG_INCOMP,"Incompatible vector local lengths");
+  if ((*x)->map->N != (y)->map->N) SETERRQ(PetscObjectComm((PetscObject)y),PETSC_ERR_ARG_INCOMP,"Incompatible vector global lengths");
+  if ((*x)->map->n != (y)->map->n) SETERRQ(PetscObjectComm((PetscObject)y),PETSC_ERR_ARG_INCOMP,"Incompatible vector local lengths");
 
   ierr = PetscObjectQuery((PetscObject)(x[0]),"contiguous",(PetscObject*)&container);CHKERRQ(ierr);
   if (container) {
@@ -470,9 +473,9 @@ PetscErrorCode SlepcVecMAXPBY(Vec y,PetscScalar beta,PetscScalar alpha,PetscInt
     ierr = PetscLogEventBegin(SLEPC_VecMAXPBY,*x,y,0,0);CHKERRQ(ierr);
     ierr = VecGetArray(y,&py);CHKERRQ(ierr);
     ierr = VecGetArrayRead(*x,&px);CHKERRQ(ierr);
-    n = PetscBLASIntCast(nv);
-    m = PetscBLASIntCast((y)->map->n);
-    BLASgemv_("N",&m,&n,&alpha,px,&m,a,&one,&beta,py,&one);
+    ierr = PetscBLASIntCast(nv,&n);CHKERRQ(ierr);
+    ierr = PetscBLASIntCast((y)->map->n,&m);CHKERRQ(ierr);
+    if (m>0) PetscStackCallBLAS("BLASgemv",BLASgemv_("N",&m,&n,&alpha,px,&m,a,&one,&beta,py,&one));
     ierr = VecRestoreArray(y,&py);CHKERRQ(ierr);
     ierr = VecRestoreArrayRead(*x,&px);CHKERRQ(ierr);
     ierr = PetscLogFlops(nv*2*(y)->map->n);CHKERRQ(ierr);
