@@ -1,17 +1,17 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-      
+
    SLEPc is free software: you can redistribute it and/or modify it under  the
    terms of version 3 of the GNU Lesser General Public License as published by
    the Free Software Foundation.
 
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
    more details.
 
    You  should have received a copy of the GNU Lesser General  Public  License
@@ -19,14 +19,118 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <private/vecimplslepc.h>     /*I "slepcvec.h" I*/
+#include <slepc-private/vecimplslepc.h>     /*I "slepcvec.h" I*/
+
+/* Private MPI datatypes and operators */
+static MPI_Datatype MPIU_NORM2=0, MPIU_NORM1_AND_2=0;
+static MPI_Op MPIU_NORM2_SUM=0;
+#if !defined(PETSC_USE_DYNAMIC_LIBRARIES)
+static PetscBool VecCompInitialized = PETSC_FALSE;
+#endif
+
+/* Private inline functions */
+PETSC_STATIC_INLINE void SumNorm2(PetscReal *,PetscReal *,PetscReal *,PetscReal *);
+PETSC_STATIC_INLINE PetscReal GetNorm2(PetscReal,PetscReal);
+PETSC_STATIC_INLINE void AddNorm2(PetscReal *,PetscReal *,PetscReal);
 
 #include "veccomp0.h"
 
 #define __WITH_MPI__
 #include "veccomp0.h"
 
-#undef __FUNCT__  
+PETSC_STATIC_INLINE void SumNorm2(PetscReal *ssq0,PetscReal *scale0,PetscReal *ssq1,PetscReal *scale1)
+{
+  PetscReal q;
+  if (*scale0 > *scale1) {
+    q = *scale1/(*scale0);
+    *ssq1 = *ssq0 + q*q*(*ssq1);
+    *scale1 = *scale0;
+  } else {
+    q = *scale0/(*scale1);
+    *ssq1 += q*q*(*ssq0);
+  }
+}
+
+PETSC_STATIC_INLINE PetscReal GetNorm2(PetscReal ssq,PetscReal scale)
+{
+  return scale*PetscSqrtReal(ssq);
+}
+
+PETSC_STATIC_INLINE void AddNorm2(PetscReal *ssq,PetscReal *scale,PetscReal x)
+{
+  PetscReal absx,q;
+  if (x != 0.0) {
+    absx = PetscAbs(x);
+    if (*scale < absx) {
+      q = *scale/absx;
+      *ssq = 1.0 + *ssq*q*q;
+      *scale = absx;
+    } else {
+      q = absx/(*scale);
+      *ssq += q*q;
+    }
+  }
+}
+
+#if !defined(PETSC_USE_DYNAMIC_LIBRARIES)
+
+#undef __FUNCT__
+#define __FUNCT__ "SlepcSumNorm2_Local"
+static void SlepcSumNorm2_Local(void *in,void *out,PetscMPIInt *cnt,MPI_Datatype *datatype)
+{
+  PetscInt       i,count = *cnt;
+
+  PetscFunctionBegin;
+  if (*datatype == MPIU_NORM2) {
+    PetscReal *xin = (PetscReal*)in,*xout = (PetscReal*)out;
+    for (i=0; i<count; i++) {
+      SumNorm2(&xin[i*2],&xin[i*2+1],&xout[i*2],&xout[i*2+1]);
+    }
+  } else if (*datatype == MPIU_NORM1_AND_2) {
+    PetscReal *xin = (PetscReal*)in,*xout = (PetscReal*)out;
+    for (i=0; i<count; i++) {
+      xout[i*3]+= xin[i*3];
+      SumNorm2(&xin[i*3+1],&xin[i*3+2],&xout[i*3+1],&xout[i*3+2]);
+    }
+  } else {
+    (*PetscErrorPrintf)("Can only handle MPIU_NORM* data types");
+    MPI_Abort(MPI_COMM_WORLD,1);
+  }
+  PetscFunctionReturnVoid();
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "VecNormCompEnd"
+static PetscErrorCode VecNormCompEnd(void)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = MPI_Type_free(&MPIU_NORM2);CHKERRQ(ierr);
+  ierr = MPI_Type_free(&MPIU_NORM1_AND_2);CHKERRQ(ierr);
+  ierr = MPI_Op_free(&MPIU_NORM2_SUM);CHKERRQ(ierr);
+  VecCompInitialized = PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "VecNormCompInit"
+static PetscErrorCode VecNormCompInit()
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = MPI_Type_contiguous(sizeof(PetscReal)*2,MPI_BYTE,&MPIU_NORM2);CHKERRQ(ierr);
+  ierr = MPI_Type_commit(&MPIU_NORM2);CHKERRQ(ierr);
+  ierr = MPI_Type_contiguous(sizeof(PetscReal)*3,MPI_BYTE,&MPIU_NORM1_AND_2);CHKERRQ(ierr);
+  ierr = MPI_Type_commit(&MPIU_NORM1_AND_2);CHKERRQ(ierr);
+  ierr = MPI_Op_create(SlepcSumNorm2_Local,1,&MPIU_NORM2_SUM);CHKERRQ(ierr);
+  ierr = PetscRegisterFinalize(VecNormCompEnd);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+#endif
+
+#undef __FUNCT__
 #define __FUNCT__ "VecDestroy_Comp"
 PetscErrorCode VecDestroy_Comp(Vec v)
 {
@@ -35,16 +139,13 @@ PetscErrorCode VecDestroy_Comp(Vec v)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  /* if memory was published with AMS then destroy it */
-  ierr = PetscObjectDepublish(v);CHKERRQ(ierr);
-
 #if defined(PETSC_USE_LOG)
   PetscLogObjectState((PetscObject)v,"Length=%D",v->map->n);
 #endif
-  for(i=0; i<vs->nx; i++) {
+  for (i=0;i<vs->nx;i++) {
     ierr = VecDestroy(&vs->x[i]);CHKERRQ(ierr);
   }
-  if(--vs->n->friends <= 0) {
+  if (--vs->n->friends <= 0) {
     ierr = PetscFree(vs->n);CHKERRQ(ierr);
   }
   ierr = PetscFree(vs->x);CHKERRQ(ierr);
@@ -53,13 +154,13 @@ PetscErrorCode VecDestroy_Comp(Vec v)
 }
 
 static struct _VecOps DvOps = {VecDuplicate_Comp, /* 1 */
-            VecDuplicateVecs_Default,
-            VecDestroyVecs_Default,
+            VecDuplicateVecs_Comp,
+            VecDestroyVecs_Comp,
             VecDot_Comp_MPI,
             VecMDot_Comp_MPI,
             VecNorm_Comp_MPI,
-            0,
-            0,
+            VecTDot_Comp_MPI,
+            VecMTDot_Comp_MPI,
             VecScale_Comp,
             VecCopy_Comp, /* 10 */
             VecSet_Comp,
@@ -71,7 +172,7 @@ static struct _VecOps DvOps = {VecDuplicate_Comp, /* 1 */
             VecWAXPY_Comp,
             VecAXPBYPCZ_Comp,
             VecPointwiseMult_Comp,
-            VecPointwiseDivide_Comp, 
+            VecPointwiseDivide_Comp,
             0, /* 20 */
             0,0,
             0 /*VecGetArray_Seq*/,
@@ -114,7 +215,38 @@ static struct _VecOps DvOps = {VecDuplicate_Comp, /* 1 */
             VecDotNorm2_Comp_MPI
           };
 
-#undef __FUNCT__  
+#undef __FUNCT__
+#define __FUNCT__ "VecDuplicateVecs_Comp"
+PetscErrorCode VecDuplicateVecs_Comp(Vec w,PetscInt m,Vec *V[])
+{
+  PetscErrorCode ierr;
+  PetscInt       i;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(w,VEC_CLASSID,1);
+  PetscValidPointer(V,3);
+  if (m<=0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"m must be > 0: m = %D",m);
+  ierr = PetscMalloc(m*sizeof(Vec*),V);CHKERRQ(ierr);
+  for (i=0;i<m;i++) { ierr = VecDuplicate(w,*V+i);CHKERRQ(ierr); }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "VecDestroyVecs_Comp"
+PetscErrorCode VecDestroyVecs_Comp(PetscInt m,Vec v[])
+{
+  PetscErrorCode ierr;
+  PetscInt       i;
+
+  PetscFunctionBegin;
+  PetscValidPointer(v,1);
+  if (m<=0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"m must be > 0: m = %D",m);
+  for (i=0;i<m;i++) { ierr = VecDestroy(&v[i]);CHKERRQ(ierr); }
+  ierr = PetscFree(v);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
 #define __FUNCT__ "VecCreate_Comp_Private"
 static PetscErrorCode VecCreate_Comp_Private(Vec v,Vec *x,PetscInt nx,PetscBool x_to_me,Vec_Comp_N *n)
 {
@@ -123,6 +255,13 @@ static PetscErrorCode VecCreate_Comp_Private(Vec v,Vec *x,PetscInt nx,PetscBool
   PetscInt       N=0,lN=0,i,k;
 
   PetscFunctionBegin;
+#if !defined(PETSC_USE_DYNAMIC_LIBRARIES)
+  if (!VecCompInitialized) {
+    VecCompInitialized = PETSC_TRUE;
+    ierr = VecRegister(VECCOMP,VecCreate_Comp);CHKERRQ(ierr);
+    ierr = VecNormCompInit();CHKERRQ(ierr);
+  }
+#endif
   /* Allocate a new Vec_Comp */
   if (v->data) { ierr = PetscFree(v->data);CHKERRQ(ierr); }
   ierr = PetscNewLog(v,Vec_Comp,&s);CHKERRQ(ierr);
@@ -143,7 +282,7 @@ static PetscErrorCode VecCreate_Comp_Private(Vec v,Vec *x,PetscInt nx,PetscBool
     ierr = VecGetLocalSize(x[i],&k);CHKERRQ(ierr);
     lN+= k;
   }
- 
+
   /* Allocate the shared structure, if it is not given */
   if (!n) {
     ierr = PetscNewLog(v,Vec_Comp_N,&n);CHKERRQ(ierr);
@@ -165,32 +304,18 @@ static PetscErrorCode VecCreate_Comp_Private(Vec v,Vec *x,PetscInt nx,PetscBool
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecCreate_Comp"
-PetscErrorCode VecCreate_Comp(Vec V)
+PETSC_EXTERN PetscErrorCode VecCreate_Comp(Vec V)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = VecCreate_Comp_Private(V,PETSC_NULL,0,PETSC_FALSE,PETSC_NULL);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-EXTERN_C_END
-
-#undef __FUNCT__  
-#define __FUNCT__ "VecRegister_Comp"
-PetscErrorCode VecRegister_Comp(const char path[])
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  ierr = VecRegisterDynamic(VECCOMP,path,"VecCreate_Comp",VecCreate_Comp);CHKERRQ(ierr);
-  ierr = VecNormCompInit();CHKERRQ(ierr);
+  ierr = VecCreate_Comp_Private(V,NULL,0,PETSC_FALSE,NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecCreateComp"
 /*@C
    VecCreateComp - Creates a new vector containing several subvectors, each stored separately
@@ -215,7 +340,7 @@ PetscErrorCode VecRegister_Comp(const char path[])
 
 .seealso: VecCreateCompWithVecs(), VecCompSetSubVecs()
 @*/
-PetscErrorCode VecCreateComp(MPI_Comm comm,PetscInt *Nx,PetscInt n,const VecType t,Vec Vparent,Vec *V)
+PetscErrorCode VecCreateComp(MPI_Comm comm,PetscInt *Nx,PetscInt n,VecType t,Vec Vparent,Vec *V)
 {
   PetscErrorCode ierr;
   Vec            *x;
@@ -223,18 +348,19 @@ PetscErrorCode VecCreateComp(MPI_Comm comm,PetscInt *Nx,PetscInt n,const VecType
 
   PetscFunctionBegin;
   ierr = VecCreate(comm,V);CHKERRQ(ierr);
-  ierr = PetscMalloc(sizeof(Vec)*n,&x);CHKERRQ(ierr);
-  for(i=0;i<n;i++) {
+  ierr = PetscMalloc(n*sizeof(Vec),&x);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory(*V,n*sizeof(Vec));CHKERRQ(ierr);
+  for (i=0;i<n;i++) {
     ierr = VecCreate(comm,&x[i]);CHKERRQ(ierr);
     ierr = VecSetSizes(x[i],PETSC_DECIDE,Nx[i]);CHKERRQ(ierr);
     ierr = VecSetType(x[i],t);CHKERRQ(ierr);
   }
   ierr = VecCreate_Comp_Private(*V,x,n,PETSC_TRUE,
-                           Vparent?((Vec_Comp*)Vparent->data)->n:PETSC_NULL);CHKERRQ(ierr);
+                           Vparent?((Vec_Comp*)Vparent->data)->n:NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecCreateCompWithVecs"
 /*@C
    VecCreateCompWithVecs - Creates a new vector containing several subvectors,
@@ -260,16 +386,16 @@ PetscErrorCode VecCreateCompWithVecs(Vec *x,PetscInt n,Vec Vparent,Vec *V)
   PetscInt       i;
 
   PetscFunctionBegin;
-  ierr = VecCreate(((PetscObject)x[0])->comm,V);CHKERRQ(ierr);
-  for(i=0;i<n;i++) {
+  ierr = VecCreate(PetscObjectComm((PetscObject)x[0]),V);CHKERRQ(ierr);
+  for (i=0;i<n;i++) {
     ierr = PetscObjectReference((PetscObject)x[i]);CHKERRQ(ierr);
   }
   ierr = VecCreate_Comp_Private(*V,x,n,PETSC_FALSE,
-                           Vparent?((Vec_Comp*)Vparent->data)->n:PETSC_NULL);CHKERRQ(ierr);
+                           Vparent?((Vec_Comp*)Vparent->data)->n:NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecDuplicate_Comp"
 PetscErrorCode VecDuplicate_Comp(Vec win,Vec *V)
 {
@@ -279,9 +405,10 @@ PetscErrorCode VecDuplicate_Comp(Vec win,Vec *V)
   Vec_Comp       *s = (Vec_Comp*)win->data;
 
   PetscFunctionBegin;
-  PetscValidVecComp(win);
-  ierr = VecCreate(((PetscObject)win)->comm,V);CHKERRQ(ierr);
-  ierr = PetscMalloc(sizeof(Vec)*s->nx,&x);CHKERRQ(ierr);
+  SlepcValidVecComp(win);
+  ierr = VecCreate(PetscObjectComm((PetscObject)win),V);CHKERRQ(ierr);
+  ierr = PetscMalloc(s->nx*sizeof(Vec),&x);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory(*V,s->nx*sizeof(Vec));CHKERRQ(ierr);
   for (i=0;i<s->nx;i++) {
     ierr = VecDuplicate(s->x[i],&x[i]);CHKERRQ(ierr);
   }
@@ -289,7 +416,7 @@ PetscErrorCode VecDuplicate_Comp(Vec win,Vec *V)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecCompGetSubVecs"
 /*@C
    VecCompGetSubVecs - Returns the entire array of vectors defining a compound vector
@@ -312,13 +439,13 @@ PetscErrorCode VecCompGetSubVecs(Vec win,PetscInt *n,const Vec **x)
   Vec_Comp *s = (Vec_Comp*)win->data;
 
   PetscFunctionBegin;
-  PetscValidVecComp(win);
-  if(x) *x = s->x;
-  if(n) *n = s->nx;
+  SlepcValidVecComp(win);
+  if (x) *x = s->x;
+  if (n) *n = s->nx;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecCompSetSubVecs"
 /*@C
    VecCompSetSubVecs - Resets the number of subvectors defining a compound vector,
@@ -341,8 +468,8 @@ PetscErrorCode VecCompSetSubVecs(Vec win,PetscInt n,Vec *x)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidVecComp(win);
-  if(x) {
+  SlepcValidVecComp(win);
+  if (x) {
     if (n > s->nx) {
       ierr = PetscFree(s->x);CHKERRQ(ierr);
       ierr = PetscMalloc(sizeof(Vec)*n,&s->x);CHKERRQ(ierr);
@@ -354,7 +481,7 @@ PetscErrorCode VecCompSetSubVecs(Vec win,PetscInt n,Vec *x)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecAXPY_Comp"
 PetscErrorCode VecAXPY_Comp(Vec v,PetscScalar alpha,Vec w)
 {
@@ -363,15 +490,15 @@ PetscErrorCode VecAXPY_Comp(Vec v,PetscScalar alpha,Vec w)
   PetscInt       i;
 
   PetscFunctionBegin;
-  PetscValidVecComp(v);
-  PetscValidVecComp(w);
+  SlepcValidVecComp(v);
+  SlepcValidVecComp(w);
   for (i=0;i<vs->n->n;i++) {
     ierr = VecAXPY(vs->x[i],alpha,ws->x[i]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecAYPX_Comp"
 PetscErrorCode VecAYPX_Comp(Vec v,PetscScalar alpha,Vec w)
 {
@@ -380,15 +507,15 @@ PetscErrorCode VecAYPX_Comp(Vec v,PetscScalar alpha,Vec w)
   PetscInt       i;
 
   PetscFunctionBegin;
-  PetscValidVecComp(v);
-  PetscValidVecComp(w);
+  SlepcValidVecComp(v);
+  SlepcValidVecComp(w);
   for (i=0;i<vs->n->n;i++) {
     ierr = VecAYPX(vs->x[i],alpha,ws->x[i]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecAXPBY_Comp"
 PetscErrorCode VecAXPBY_Comp(Vec v,PetscScalar alpha,PetscScalar beta,Vec w)
 {
@@ -397,15 +524,15 @@ PetscErrorCode VecAXPBY_Comp(Vec v,PetscScalar alpha,PetscScalar beta,Vec w)
   PetscInt       i;
 
   PetscFunctionBegin;
-  PetscValidVecComp(v);
-  PetscValidVecComp(w);
+  SlepcValidVecComp(v);
+  SlepcValidVecComp(w);
   for (i=0;i<vs->n->n;i++) {
     ierr = VecAXPBY(vs->x[i],alpha,beta,ws->x[i]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecMAXPY_Comp"
 PetscErrorCode VecMAXPY_Comp(Vec v,PetscInt n,const PetscScalar *alpha,Vec *w)
 {
@@ -415,8 +542,8 @@ PetscErrorCode VecMAXPY_Comp(Vec v,PetscInt n,const PetscScalar *alpha,Vec *w)
   PetscInt       i,j;
 
   PetscFunctionBegin;
-  PetscValidVecComp(v);
-  for (i=0;i<n;i++) PetscValidVecComp(w[i]);
+  SlepcValidVecComp(v);
+  for (i=0;i<n;i++) SlepcValidVecComp(w[i]);
 
   ierr = PetscMalloc(sizeof(Vec)*n,&wx);CHKERRQ(ierr);
 
@@ -429,7 +556,7 @@ PetscErrorCode VecMAXPY_Comp(Vec v,PetscInt n,const PetscScalar *alpha,Vec *w)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecWAXPY_Comp"
 PetscErrorCode VecWAXPY_Comp(Vec v,PetscScalar alpha,Vec w,Vec z)
 {
@@ -438,16 +565,16 @@ PetscErrorCode VecWAXPY_Comp(Vec v,PetscScalar alpha,Vec w,Vec z)
   PetscInt       i;
 
   PetscFunctionBegin;
-  PetscValidVecComp(v);
-  PetscValidVecComp(w);
-  PetscValidVecComp(z);
+  SlepcValidVecComp(v);
+  SlepcValidVecComp(w);
+  SlepcValidVecComp(z);
   for (i=0;i<vs->n->n;i++) {
     ierr = VecWAXPY(vs->x[i],alpha,ws->x[i],zs->x[i]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecAXPBYPCZ_Comp"
 PetscErrorCode VecAXPBYPCZ_Comp(Vec v,PetscScalar alpha,PetscScalar beta,PetscScalar gamma,Vec w,Vec z)
 {
@@ -456,42 +583,42 @@ PetscErrorCode VecAXPBYPCZ_Comp(Vec v,PetscScalar alpha,PetscScalar beta,PetscSc
   PetscInt        i;
 
   PetscFunctionBegin;
-  PetscValidVecComp(v);
-  PetscValidVecComp(w);
-  PetscValidVecComp(z);
+  SlepcValidVecComp(v);
+  SlepcValidVecComp(w);
+  SlepcValidVecComp(z);
   for (i=0;i<vs->n->n;i++) {
     ierr = VecAXPBYPCZ(vs->x[i],alpha,beta,gamma,ws->x[i],zs->x[i]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecGetSize_Comp"
 PetscErrorCode VecGetSize_Comp(Vec v,PetscInt *size)
 {
   Vec_Comp *vs = (Vec_Comp*)v->data;
 
   PetscFunctionBegin;
-  PetscValidVecComp(v);
+  SlepcValidVecComp(v);
   PetscValidIntPointer(size,2);
   *size = vs->n->N;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecGetLocalSize_Comp"
 PetscErrorCode VecGetLocalSize_Comp(Vec v,PetscInt *size)
 {
   Vec_Comp *vs = (Vec_Comp*)v->data;
 
   PetscFunctionBegin;
-  PetscValidVecComp(v);
+  SlepcValidVecComp(v);
   PetscValidIntPointer(size,2);
   *size = vs->n->lN;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecMax_Comp"
 PetscErrorCode VecMax_Comp(Vec v,PetscInt *idx,PetscReal *z)
 {
@@ -502,16 +629,16 @@ PetscErrorCode VecMax_Comp(Vec v,PetscInt *idx,PetscReal *z)
   PetscInt       i;
 
   PetscFunctionBegin;
-  PetscValidVecComp(v);
+  SlepcValidVecComp(v);
   if (!idx && !z) PetscFunctionReturn(0);
 
   if (vs->n->n > 0) {
-    ierr = VecMax(vs->x[0],idx?&idxp:PETSC_NULL,&zp);CHKERRQ(ierr);
+    ierr = VecMax(vs->x[0],idx?&idxp:NULL,&zp);CHKERRQ(ierr);
   }
   for (i=1;i<vs->n->n;i++) {
     ierr = VecGetSize(vs->x[i-1],&s0);CHKERRQ(ierr);
     s+= s0;
-    ierr = VecMax(vs->x[i],idx?&idxp:PETSC_NULL,&z0);CHKERRQ(ierr);
+    ierr = VecMax(vs->x[i],idx?&idxp:NULL,&z0);CHKERRQ(ierr);
     if (zp < z0) {
       if (idx) *idx = s+idxp;
       zp = z0;
@@ -521,7 +648,7 @@ PetscErrorCode VecMax_Comp(Vec v,PetscInt *idx,PetscReal *z)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecMin_Comp"
 PetscErrorCode VecMin_Comp(Vec v,PetscInt *idx,PetscReal *z)
 {
@@ -532,18 +659,18 @@ PetscErrorCode VecMin_Comp(Vec v,PetscInt *idx,PetscReal *z)
   PetscInt       i;
 
   PetscFunctionBegin;
-  PetscValidVecComp(v);
+  SlepcValidVecComp(v);
   if (!idx && !z) PetscFunctionReturn(0);
 
   if (vs->n->n > 0) {
-    ierr = VecMin(vs->x[0],idx?&idxp:PETSC_NULL,&zp);CHKERRQ(ierr);
+    ierr = VecMin(vs->x[0],idx?&idxp:NULL,&zp);CHKERRQ(ierr);
   }
   for (i=1;i<vs->n->n;i++) {
     ierr = VecGetSize(vs->x[i-1],&s0);CHKERRQ(ierr);
     s+= s0;
-    ierr = VecMin(vs->x[i],idx?&idxp:PETSC_NULL,&z0);CHKERRQ(ierr);
-    if(zp > z0) {
-      if(idx) *idx = s+idxp;
+    ierr = VecMin(vs->x[i],idx?&idxp:NULL,&z0);CHKERRQ(ierr);
+    if (zp > z0) {
+      if (idx) *idx = s+idxp;
       zp = z0;
     }
   }
@@ -551,7 +678,7 @@ PetscErrorCode VecMin_Comp(Vec v,PetscInt *idx,PetscReal *z)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecMaxPointwiseDivide_Comp"
 PetscErrorCode VecMaxPointwiseDivide_Comp(Vec v,Vec w,PetscReal *m)
 {
@@ -561,8 +688,8 @@ PetscErrorCode VecMaxPointwiseDivide_Comp(Vec v,Vec w,PetscReal *m)
   PetscInt       i;
 
   PetscFunctionBegin;
-  PetscValidVecComp(v);
-  PetscValidVecComp(w);
+  SlepcValidVecComp(v);
+  SlepcValidVecComp(w);
   if (!m || vs->n->n == 0) PetscFunctionReturn(0);
   ierr = VecMaxPointwiseDivide(vs->x[0],ws->x[0],m);CHKERRQ(ierr);
   for (i=1;i<vs->n->n;i++) {
@@ -585,34 +712,34 @@ PetscErrorCode __COMPOSE3__(Vec,NAME,_Comp)(Vec v) \
   PetscInt        i; \
 \
   PetscFunctionBegin; \
-  PetscValidVecComp(v); \
+  SlepcValidVecComp(v); \
   for (i=0;i<vs->n->n;i++) { \
     ierr = __COMPOSE2__(Vec,NAME)(vs->x[i]);CHKERRQ(ierr); \
   } \
   PetscFunctionReturn(0);\
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecConjugate_Comp"
 __FUNC_TEMPLATE1__(Conjugate)
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecReciprocal_Comp"
 __FUNC_TEMPLATE1__(Reciprocal)
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecSqrtAbs_Comp"
 __FUNC_TEMPLATE1__(SqrtAbs)
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecAbs_Comp"
 __FUNC_TEMPLATE1__(Abs)
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecExp_Comp"
 __FUNC_TEMPLATE1__(Exp)
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecLog_Comp"
 __FUNC_TEMPLATE1__(Log)
 
@@ -625,30 +752,30 @@ PetscErrorCode __COMPOSE3__(Vec,NAME,_Comp)(Vec v,T0 __a) \
   PetscInt        i; \
 \
   PetscFunctionBegin; \
-  PetscValidVecComp(v); \
+  SlepcValidVecComp(v); \
   for (i=0;i<vs->n->n;i++) { \
     ierr = __COMPOSE2__(Vec,NAME)(vs->x[i],__a);CHKERRQ(ierr); \
   } \
   PetscFunctionReturn(0);\
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecSet_Comp"
 __FUNC_TEMPLATE2__(Set,PetscScalar)
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecView_Comp"
 __FUNC_TEMPLATE2__(View,PetscViewer)
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecScale_Comp"
 __FUNC_TEMPLATE2__(Scale,PetscScalar)
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecSetRandom_Comp"
 __FUNC_TEMPLATE2__(SetRandom,PetscRandom)
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecShift_Comp"
 __FUNC_TEMPLATE2__(Shift,PetscScalar)
 
@@ -662,19 +789,19 @@ PetscErrorCode __COMPOSE3__(Vec,NAME,_Comp)(Vec v,Vec w) \
   PetscInt        i; \
 \
   PetscFunctionBegin; \
-  PetscValidVecComp(v); \
-  PetscValidVecComp(w); \
+  SlepcValidVecComp(v); \
+  SlepcValidVecComp(w); \
   for (i=0;i<vs->n->n;i++) { \
     ierr = __COMPOSE2__(Vec,NAME)(vs->x[i],ws->x[i]);CHKERRQ(ierr); \
   } \
   PetscFunctionReturn(0);\
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecCopy_Comp"
 __FUNC_TEMPLATE3__(Copy)
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecSwap_Comp"
 __FUNC_TEMPLATE3__(Swap)
 
@@ -689,31 +816,31 @@ PetscErrorCode __COMPOSE3__(Vec,NAME,_Comp)(Vec v,Vec w,Vec z) \
   PetscInt        i; \
 \
   PetscFunctionBegin; \
-  PetscValidVecComp(v); \
-  PetscValidVecComp(w); \
-  PetscValidVecComp(z); \
+  SlepcValidVecComp(v); \
+  SlepcValidVecComp(w); \
+  SlepcValidVecComp(z); \
   for (i=0;i<vs->n->n;i++) { \
     ierr = __COMPOSE2__(Vec,NAME)(vs->x[i],ws->x[i],zs->x[i]);CHKERRQ(ierr); \
   } \
   PetscFunctionReturn(0);\
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecPointwiseMax_Comp"
 __FUNC_TEMPLATE4__(PointwiseMax)
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecPointwiseMaxAbs_Comp"
 __FUNC_TEMPLATE4__(PointwiseMaxAbs)
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecPointwiseMin_Comp"
 __FUNC_TEMPLATE4__(PointwiseMin)
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecPointwiseMult_Comp"
 __FUNC_TEMPLATE4__(PointwiseMult)
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "VecPointwiseDivide_Comp"
 __FUNC_TEMPLATE4__(PointwiseDivide)
